@@ -770,11 +770,12 @@ struct __CUDA_ALIGN__(1) __nv_fp8_e5m2 {
         unsigned long retval;
         /* Suppress VS warning: warning C4127: conditional expression is constant */
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
+#pragma warning (push)
 #pragma warning (disable: 4127)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         if (sizeof(unsigned long) == sizeof(unsigned long long))
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
-#pragma warning (default: 4127)
+#pragma warning (pop)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         {
             retval = static_cast<unsigned long>(__half2ull_rz(__half(*this)));
@@ -838,11 +839,12 @@ struct __CUDA_ALIGN__(1) __nv_fp8_e5m2 {
         char value;
         /* Suppress VS warning: warning C4127: conditional expression is constant */
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
+#pragma warning (push)
 #pragma warning (disable: 4127)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         if (((char)-1) < (char)0)
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
-#pragma warning (default: 4127)
+#pragma warning (pop)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         {
             value = static_cast<char>(static_cast<signed char>(*this));
@@ -883,11 +885,12 @@ struct __CUDA_ALIGN__(1) __nv_fp8_e5m2 {
         long retval;
         /* Suppress VS warning: warning C4127: conditional expression is constant */
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
+#pragma warning (push)
 #pragma warning (disable: 4127)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         if (sizeof(long) == sizeof(long long))
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
-#pragma warning (default: 4127)
+#pragma warning (pop)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         {
             retval = static_cast<long>(__half2ll_rz(__half(*this)));
@@ -1382,11 +1385,12 @@ struct __CUDA_ALIGN__(1) __nv_fp8_e4m3 {
         unsigned long retval;
         /* Suppress VS warning: warning C4127: conditional expression is constant */
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
+#pragma warning (push)
 #pragma warning (disable: 4127)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         if (sizeof(unsigned long) == sizeof(unsigned long long))
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
-#pragma warning (default: 4127)
+#pragma warning (pop)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         {
             retval = static_cast<unsigned long>(__half2ull_rz(__half(*this)));
@@ -1450,11 +1454,12 @@ struct __CUDA_ALIGN__(1) __nv_fp8_e4m3 {
         char value;
         /* Suppress VS warning: warning C4127: conditional expression is constant */
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
+#pragma warning (push)
 #pragma warning (disable: 4127)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         if (((char)-1) < (char)0)
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
-#pragma warning (default: 4127)
+#pragma warning (pop)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         {
             value = static_cast<char>(static_cast<signed char>(*this));
@@ -1493,11 +1498,12 @@ struct __CUDA_ALIGN__(1) __nv_fp8_e4m3 {
         long retval;
         /* Suppress VS warning: warning C4127: conditional expression is constant */
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
+#pragma warning (push)
 #pragma warning (disable: 4127)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         if (sizeof(long) == sizeof(long long))
 #if defined(_MSC_VER) && !defined(__CUDA_ARCH__)
-#pragma warning (default: 4127)
+#pragma warning (pop)
 #endif /* _MSC_VER && !defined(__CUDA_ARCH__) */
         {
             retval = static_cast<long>(__half2ll_rz(__half(*this)));
