@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2020 NVIDIA Corporation.  All rights reserved.
+ * Copyright 2010-2021 NVIDIA Corporation.  All rights reserved.
  *
  * NOTICE TO LICENSEE:
  *
@@ -334,13 +334,13 @@ typedef enum {
    */
   CUPTI_EVENT_GROUP_ATTR_INSTANCE_COUNT               = 5,
   /**
-   * Event group scope can be set to CUPTI_EVENT_PROFILING_SCOPE_DEVICE or 
+   * Event group scope can be set to CUPTI_EVENT_PROFILING_SCOPE_DEVICE or
    * CUPTI_EVENT_PROFILING_SCOPE_CONTEXT for an eventGroup, before
    * adding any event.
-   * Sets the scope of eventgroup as CUPTI_EVENT_PROFILING_SCOPE_DEVICE or 
+   * Sets the scope of eventgroup as CUPTI_EVENT_PROFILING_SCOPE_DEVICE or
    * CUPTI_EVENT_PROFILING_SCOPE_CONTEXT when the scope of the events
-   * that will be added is CUPTI_EVENT_PROFILING_SCOPE_BOTH. 
-   * If profiling scope of event is either 
+   * that will be added is CUPTI_EVENT_PROFILING_SCOPE_BOTH.
+   * If profiling scope of event is either
    * CUPTI_EVENT_PROFILING_SCOPE_DEVICE or CUPTI_EVENT_PROFILING_SCOPE_CONTEXT
    * then setting this attribute will not affect the default scope.
    * It is not allowed to add events of different scope to same eventgroup.
@@ -589,25 +589,6 @@ CUptiResult CUPTIAPI cuptiDeviceGetAttribute(CUdevice device,
                                              size_t *valueSize,
                                              void *value);
 
-/**
- * \brief Read a device timestamp.
- *
- * Returns the device timestamp in \p *timestamp. The timestamp is
- * reported in nanoseconds and indicates the time since the device was
- * last reset.
- * \note \b Thread-safety: this function is thread safe.
- *
- * \param context A context on the device from which to get the timestamp
- * \param timestamp Returns the device timestamp
- *
- * \retval CUPTI_SUCCESS
- * \retval CUPTI_ERROR_NOT_INITIALIZED
- * \retval CUPTI_ERROR_INVALID_CONTEXT
- * \retval CUPTI_ERROR_INVALID_PARAMETER is \p timestamp is NULL
- */
-CUptiResult CUPTIAPI cuptiDeviceGetTimestamp(CUcontext context,
-                                             uint64_t *timestamp);
-
 /**
  * \brief Get the number of domains for a device.
  *
