@@ -97,6 +97,12 @@
 # if defined(__CUDA_ARCH__) && (__CUDA_ARCH__ >= 900)
 #define __CUDA_AND_AT_LEAST_SM_90__
 #endif /* defined(__CUDA_ARCH__) && (__CUDA_ARCH__ >= 900) */
+# if defined(__CUDA_ARCH__) && (__CUDA_ARCH__ >= 700)
+#define __CUDA_AND_AT_LEAST_SM_70__
+#endif /* defined(__CUDA_ARCH__) && (__CUDA_ARCH__ >= 700) */
+# if defined(__CUDA_ARCH__) && (__CUDA_ARCH__ >= 750)
+#define __CUDA_AND_AT_LEAST_SM_75__
+#endif /* defined(__CUDA_ARCH__) && (__CUDA_ARCH__ >= 750) */
 #endif /* __CUDACC__ */
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimax_s32_relu(const int a, const int b){
@@ -113,10 +119,11 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimax_s32_relu(const int a, const in
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimax_s16x2_relu(const unsigned int a, const unsigned int b){
-#ifdef __CUDA_AND_AT_LEAST_SM_90__
   unsigned int res;
+#ifdef __CUDA_AND_AT_LEAST_SM_90__
   asm("{max.s16x2.relu %0, %1, %2;}" : "=r"(res) : "r"(a), "r"(b));
-  return res;
+#elif defined(__CUDA_ARCH__)
+  res = __vmaxs2(__vmaxs2(a, b), 0U);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -146,10 +153,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimax_s16x2_relu(const unsi
   unsigned short ansU_hi = *(unsigned short*)& ansS_hi;
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimin_s32_relu(const int a, const int b){
@@ -166,10 +173,11 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimin_s32_relu(const int a, const in
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimin_s16x2_relu(const unsigned int a, const unsigned int b){
-#ifdef __CUDA_AND_AT_LEAST_SM_90__
   unsigned int res;
+#ifdef __CUDA_AND_AT_LEAST_SM_90__
   asm("{min.s16x2.relu %0, %1, %2;}" : "=r"(res) : "r"(a), "r"(b));
-  return res;
+#elif defined(__CUDA_ARCH__)
+  res = __vmaxs2(__vmins2(a, b), 0U);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -199,10 +207,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimin_s16x2_relu(const unsi
   unsigned short ansU_hi = *(unsigned short*)& ansS_hi;
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimax3_s32(const int a, const int b, const int c){
@@ -220,14 +228,15 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimax3_s32(const int a, const int b,
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimax3_s16x2(const unsigned int a, const unsigned int b, const unsigned int c){
+  unsigned int res;
 #ifdef __CUDA_AND_AT_LEAST_SM_90__
   // Future asm code (naming/syntax may change):
-  unsigned int res;
   asm ("{.reg .b32 t1; \n\t"
       "max.s16x2 t1, %1, %2; \n\t"
       "max.s16x2 %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_AND_AT_LEAST_SM_70__)
+  res = __vmaxs2(__vmaxs2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -259,10 +268,9 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimax3_s16x2(const unsigned
   unsigned short ansU_hi = *(unsigned short*)& ansS_hi;
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimax3_u32(const unsigned int a, const unsigned int b, const unsigned int c){
@@ -280,13 +288,14 @@ int res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimax3_u16x2(const unsigned int a, const unsigned int b, const unsigned int c){
+  unsigned int res;
 #ifdef __CUDA_AND_AT_LEAST_SM_90__
-  int res;
   asm ("{.reg .b32 t1; \n\t"
       "max.u16x2 t1, %1, %2; \n\t"
       "max.u16x2 %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_ARCH__)
+  res = __vmaxu2(__vmaxu2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -304,10 +313,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimax3_u16x2(const unsigned
   unsigned short ansU_hi = (unsigned short)max(max(aU_hi, bU_hi), cU_hi);
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimin3_s32(const int a, const int b, const int c){
@@ -325,13 +334,14 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimin3_s32(const int a, const int b,
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimin3_s16x2(const unsigned int a, const unsigned int b, const unsigned int c){
+  unsigned int res;
 #ifdef __CUDA_AND_AT_LEAST_SM_90__
-  int res;
   asm ("{.reg .b32 t1; \n\t"
       "min.s16x2 t1, %1, %2; \n\t"
       "min.s16x2 %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_AND_AT_LEAST_SM_70__)
+  res = __vmins2(__vmins2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -363,10 +373,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimin3_s16x2(const unsigned
   unsigned short ansU_hi = *(unsigned short*)& ansS_hi;
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimin3_u32(const unsigned int a, const unsigned int b, const unsigned int c){
@@ -384,13 +394,14 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimin3_u32(const unsigned i
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimin3_u16x2(const unsigned int a, const unsigned int b, const unsigned int c){
+  unsigned int res;
 #ifdef __CUDA_AND_AT_LEAST_SM_90__
-  int res;
   asm ("{.reg .b32 t1; \n\t"
       "min.u16x2 t1, %1, %2; \n\t"
       "min.u16x2 %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_ARCH__)
+  res = __vminu2(__vminu2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -408,10 +419,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimin3_u16x2(const unsigned
   unsigned short ansU_hi = (unsigned short)min(min(aU_hi, bU_hi), cU_hi);
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimax3_s32_relu(const int a, const int b, const int c){
@@ -431,13 +442,14 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimax3_s32_relu(const int a, const i
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimax3_s16x2_relu(const unsigned int a, const unsigned int b, const unsigned int c){
+  unsigned int res;
 #ifdef __CUDA_AND_AT_LEAST_SM_90__
-  int res;
   asm ("{.reg .b32 t1; \n\t"
       "max.s16x2.relu t1, %1, %2; \n\t"
       "max.s16x2.relu %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_AND_AT_LEAST_SM_75__)
+  res = __vimax_s16x2_relu(__vmaxs2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -473,10 +485,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimax3_s16x2_relu(const uns
   unsigned short ansU_hi = *(unsigned short*)& ansS_hi;
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimin3_s32_relu(const int a, const int b, const int c){
@@ -496,13 +508,14 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __vimin3_s32_relu(const int a, const i
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimin3_s16x2_relu(const unsigned int a, const unsigned int b, const unsigned int c){
+  unsigned res;
 #ifdef __CUDA_AND_AT_LEAST_SM_90__
-  int res;
   asm ("{.reg .b32 t1; \n\t"
       "min.s16x2.relu t1, %1, %2; \n\t"
       "min.s16x2.relu %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_AND_AT_LEAST_SM_75__)
+  res = __vimin_s16x2_relu(__vmins2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -538,10 +551,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __vimin3_s16x2_relu(const uns
   unsigned short ansU_hi = *(unsigned short*)& ansS_hi;
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __viaddmax_s32(const int a, const int b, const int c){
@@ -559,13 +572,14 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __viaddmax_s32(const int a, const int
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmax_s16x2(const unsigned int a, const unsigned int b, const unsigned int c){
-#ifdef __CUDA_AND_AT_LEAST_SM_90__
   unsigned int res;
+#ifdef __CUDA_AND_AT_LEAST_SM_90__
   asm ("{.reg .b32 t1; \n\t"
       "add.s16x2 t1, %1, %2; \n\t"
       "max.s16x2 %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_ARCH__)
+  res = __vmaxs2(__vadd2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -597,10 +611,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmax_s16x2(const unsign
   unsigned short ansU_hi = *(unsigned short*)& ansS_hi;
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;  
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmax_u32(const unsigned int a, const unsigned int b, const unsigned int c){
@@ -618,13 +632,14 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmax_u32(const unsigned
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmax_u16x2(const unsigned int a, const unsigned int b, const unsigned int c){
-#ifdef __CUDA_AND_AT_LEAST_SM_90__
   unsigned int res;
+#ifdef __CUDA_AND_AT_LEAST_SM_90__
   asm ("{.reg .b32 t1; \n\t"
       "add.u16x2 t1, %1, %2; \n\t"
       "max.u16x2 %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_ARCH__)
+  res = __vmaxu2(__vadd2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -642,10 +657,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmax_u16x2(const unsign
   unsigned short ansU_hi = (unsigned short)max((unsigned short)(aU_hi + bU_hi), cU_hi);
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __viaddmin_s32(const int a, const int b, const int c){
@@ -663,13 +678,14 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __viaddmin_s32(const int a, const int
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmin_s16x2(const unsigned int a, const unsigned int b, const unsigned int c){
-#ifdef __CUDA_AND_AT_LEAST_SM_90__
   unsigned int res;
+#ifdef __CUDA_AND_AT_LEAST_SM_90__
   asm ("{.reg .b32 t1; \n\t"
       "add.s16x2 t1, %1, %2; \n\t"
       "min.s16x2 %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_ARCH__)
+  res = __vmins2(__vadd2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -701,10 +717,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmin_s16x2(const unsign
   unsigned short ansU_hi = *(unsigned short*)& ansS_hi;
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmin_u32(const unsigned int a, const unsigned int b, const unsigned int c){
@@ -722,13 +738,14 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmin_u32(const unsigned
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmin_u16x2(const unsigned int a, const unsigned int b, const unsigned int c){
-#ifdef __CUDA_AND_AT_LEAST_SM_90__
   unsigned int res;
+#ifdef __CUDA_AND_AT_LEAST_SM_90__
   asm ("{.reg .b32 t1; \n\t"
       "add.u16x2 t1, %1, %2; \n\t"
       "min.u16x2 %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_ARCH__)
+  res = __vminu2(__vadd2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -746,10 +763,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmin_u16x2(const unsign
   unsigned short ansU_hi = (unsigned short)min((unsigned short)(aU_hi + bU_hi), cU_hi);
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __viaddmax_s32_relu(const int a, const int b, const int c){
@@ -769,13 +786,14 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __viaddmax_s32_relu(const int a, const
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmax_s16x2_relu(const unsigned int a, const unsigned int b, const unsigned int c){
-#ifdef __CUDA_AND_AT_LEAST_SM_90__
   unsigned int res;
+#ifdef __CUDA_AND_AT_LEAST_SM_90__
   asm ("{.reg .b32 t1; \n\t"
       "add.s16x2 t1, %1, %2; \n\t"
       "max.s16x2.relu %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_ARCH__)
+  res = __vimax_s16x2_relu(__vadd2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -810,10 +828,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmax_s16x2_relu(const u
   unsigned short ansU_hi = *(unsigned short*)& ansS_hi;
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __viaddmin_s32_relu(const int a, const int b, const int c){
@@ -833,13 +851,14 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ int __viaddmin_s32_relu(const int a, const
 }
 
 __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmin_s16x2_relu(const unsigned int a, const unsigned int b, const unsigned int c){
-#ifdef __CUDA_AND_AT_LEAST_SM_90__
   unsigned int res;
+#ifdef __CUDA_AND_AT_LEAST_SM_90__
   asm ("{.reg .b32 t1; \n\t"
       "add.s16x2 t1, %1, %2; \n\t"
       "min.s16x2.relu %0, t1, %3;}\n\t"
       : "=r"(res) : "r"(a), "r"(b), "r"(c));
-  return res;
+#elif defined(__CUDA_ARCH__)
+  res = __vimin_s16x2_relu(__vadd2(a, b), c);
 #else
   // Host and older architecture code
   // Separate our high and low bit:
@@ -874,10 +893,10 @@ __DEVICE_HOST_FUNCTIONS_STATIC_DECL__ unsigned int __viaddmin_s16x2_relu(const u
   unsigned short ansU_hi = *(unsigned short*)& ansS_hi;
 
   // Put answer back together:
-  unsigned int ans = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
-
-  return ans;
+  res = ((unsigned int) ansU_lo) | (((unsigned int) ansU_hi) << 16);
 #endif
+
+  return res;
 }
 
 // vimax vimin with predicate
