@@ -186,6 +186,11 @@ typedef struct cuCtxGetFlags_params_st {
     unsigned int *flags;
 } cuCtxGetFlags_params;
 
+typedef struct cuCtxGetId_params_st {
+    CUcontext ctx;
+    unsigned long long *ctxId;
+} cuCtxGetId_params;
+
 typedef struct cuCtxSetLimit_params_st {
     CUlimit limit;
     size_t value;
@@ -329,6 +334,88 @@ typedef struct cuModuleGetSurfRef_params_st {
     const char *name;
 } cuModuleGetSurfRef_params;
 
+typedef struct cuLibraryLoadData_params_st {
+    CUlibrary *library;
+    const void *code;
+    CUjit_option *jitOptions;
+    void **jitOptionsValues;
+    unsigned int numJitOptions;
+    CUlibraryOption *libraryOptions;
+    void **libraryOptionValues;
+    unsigned int numLibraryOptions;
+} cuLibraryLoadData_params;
+
+typedef struct cuLibraryLoadFromFile_params_st {
+    CUlibrary *library;
+    const char *fileName;
+    CUjit_option *jitOptions;
+    void **jitOptionsValues;
+    unsigned int numJitOptions;
+    CUlibraryOption *libraryOptions;
+    void **libraryOptionValues;
+    unsigned int numLibraryOptions;
+} cuLibraryLoadFromFile_params;
+
+typedef struct cuLibraryUnload_params_st {
+    CUlibrary library;
+} cuLibraryUnload_params;
+
+typedef struct cuLibraryGetKernel_params_st {
+    CUkernel *pKernel;
+    CUlibrary library;
+    const char *name;
+} cuLibraryGetKernel_params;
+
+typedef struct cuLibraryGetModule_params_st {
+    CUmodule *pMod;
+    CUlibrary library;
+} cuLibraryGetModule_params;
+
+typedef struct cuKernelGetFunction_params_st {
+    CUfunction *pFunc;
+    CUkernel kernel;
+} cuKernelGetFunction_params;
+
+typedef struct cuLibraryGetGlobal_params_st {
+    CUdeviceptr *dptr;
+    size_t *bytes;
+    CUlibrary library;
+    const char *name;
+} cuLibraryGetGlobal_params;
+
+typedef struct cuLibraryGetManaged_params_st {
+    CUdeviceptr *dptr;
+    size_t *bytes;
+    CUlibrary library;
+    const char *name;
+} cuLibraryGetManaged_params;
+
+typedef struct cuLibraryGetUnifiedFunction_params_st {
+    void **fptr;
+    CUlibrary library;
+    const char *symbol;
+} cuLibraryGetUnifiedFunction_params;
+
+typedef struct cuKernelGetAttribute_params_st {
+    int *pi;
+    CUfunction_attribute attrib;
+    CUkernel kernel;
+    CUdevice dev;
+} cuKernelGetAttribute_params;
+
+typedef struct cuKernelSetAttribute_params_st {
+    CUfunction_attribute attrib;
+    int val;
+    CUkernel kernel;
+    CUdevice dev;
+} cuKernelSetAttribute_params;
+
+typedef struct cuKernelSetCacheConfig_params_st {
+    CUkernel kernel;
+    CUfunc_cache config;
+    CUdevice dev;
+} cuKernelSetCacheConfig_params;
+
 typedef struct cuMemGetInfo_v2_params_st {
     size_t *free;
     size_t *total;
@@ -745,6 +832,14 @@ typedef struct cuMipmappedArrayDestroy_params_st {
     CUmipmappedArray hMipmappedArray;
 } cuMipmappedArrayDestroy_params;
 
+typedef struct cuMemGetHandleForAddressRange_params_st {
+    void *handle;
+    CUdeviceptr dptr;
+    size_t size;
+    CUmemRangeHandleType handleType;
+    unsigned long long flags;
+} cuMemGetHandleForAddressRange_params;
+
 typedef struct cuMemAddressReserve_params_st {
     CUdeviceptr *ptr;
     size_t size;
@@ -982,6 +1077,11 @@ typedef struct cuStreamGetFlags_ptsz_params_st {
     unsigned int *flags;
 } cuStreamGetFlags_ptsz_params;
 
+typedef struct cuStreamGetId_ptsz_params_st {
+    CUstream hStream;
+    unsigned long long *streamId;
+} cuStreamGetId_ptsz_params;
+
 typedef struct cuStreamGetCtx_ptsz_params_st {
     CUstream hStream;
     CUcontext *pctx;
@@ -1673,6 +1773,17 @@ typedef struct cuGraphInstantiateWithFlags_params_st {
     unsigned long long flags;
 } cuGraphInstantiateWithFlags_params;
 
+typedef struct cuGraphInstantiateWithParams_ptsz_params_st {
+    CUgraphExec *phGraphExec;
+    CUgraph hGraph;
+    CUDA_GRAPH_INSTANTIATE_PARAMS *instantiateParams;
+} cuGraphInstantiateWithParams_ptsz_params;
+
+typedef struct cuGraphExecGetFlags_params_st {
+    CUgraphExec hGraphExec;
+    cuuint64_t *flags;
+} cuGraphExecGetFlags_params;
+
 typedef struct cuGraphExecKernelNodeSetParams_params_st {
     CUgraphExec hGraphExec;
     CUgraphNode hNode;
@@ -2130,6 +2241,14 @@ typedef struct cuGraphicsUnmapResources_ptsz_params_st {
     CUstream hStream;
 } cuGraphicsUnmapResources_ptsz_params;
 
+typedef struct cuGetProcAddress_v2_params_st {
+    const char *symbol;
+    void **pfn;
+    int cudaVersion;
+    cuuint64_t flags;
+    CUdriverProcAddressQueryResult *symbolStatus;
+} cuGetProcAddress_v2_params;
+
 typedef struct cuCoredumpGetAttribute_params_st {
     CUcoredumpSettings attrib;
     void *value;
@@ -2741,6 +2860,11 @@ typedef struct cuStreamGetPriority_params_st {
     int *priority;
 } cuStreamGetPriority_params;
 
+typedef struct cuStreamGetId_params_st {
+    CUstream hStream;
+    unsigned long long *streamId;
+} cuStreamGetId_params;
+
 typedef struct cuStreamGetFlags_params_st {
     CUstream hStream;
     unsigned int *flags;
@@ -2971,6 +3095,12 @@ typedef struct cuStreamGetCaptureInfo_v2_params_st {
     size_t *numDependencies_out;
 } cuStreamGetCaptureInfo_v2_params;
 
+typedef struct cuGraphInstantiateWithParams_params_st {
+    CUgraphExec *phGraphExec;
+    CUgraph hGraph;
+    CUDA_GRAPH_INSTANTIATE_PARAMS *instantiateParams;
+} cuGraphInstantiateWithParams_params;
+
 typedef struct cuGraphUpload_params_st {
     CUgraphExec hGraph;
     CUstream hStream;
@@ -3042,3 +3172,10 @@ typedef struct cuStreamUpdateCaptureDependencies_params_st {
     size_t numDependencies;
     unsigned int flags;
 } cuStreamUpdateCaptureDependencies_params;
+
+typedef struct cuGetProcAddress_params_st {
+    const char *symbol;
+    void **pfn;
+    int cudaVersion;
+    cuuint64_t flags;
+} cuGetProcAddress_params;
