@@ -206,6 +206,11 @@ typedef struct cudaStreamGetFlags_ptsz_v7000_params_st {
     unsigned int *flags;
 } cudaStreamGetFlags_ptsz_v7000_params;
 
+typedef struct cudaStreamGetId_ptsz_v12000_params_st {
+    cudaStream_t hStream;
+    unsigned long long *streamId;
+} cudaStreamGetId_ptsz_v12000_params;
+
 typedef struct cudaStreamCopyAttributes_ptsz_v11000_params_st {
     cudaStream_t dst;
     cudaStream_t src;
@@ -289,12 +294,12 @@ typedef struct cudaStreamGetCaptureInfo_v2_ptsz_v11030_params_st {
     size_t *numDependencies_out;
 } cudaStreamGetCaptureInfo_v2_ptsz_v11030_params;
 
-typedef struct cudaStreamUpdateCaptureDependencies_v11030_params_st {
+typedef struct cudaStreamUpdateCaptureDependencies_ptsz_v11030_params_st {
     cudaStream_t stream;
     cudaGraphNode_t *dependencies;
     size_t numDependencies;
     unsigned int flags;
-} cudaStreamUpdateCaptureDependencies_v11030_params;
+} cudaStreamUpdateCaptureDependencies_ptsz_v11030_params;
 
 typedef struct cudaEventCreate_v3020_params_st {
     cudaEvent_t *event;
@@ -1535,6 +1540,17 @@ typedef struct cudaGraphInstantiateWithFlags_v11040_params_st {
     unsigned long long flags;
 } cudaGraphInstantiateWithFlags_v11040_params;
 
+typedef struct cudaGraphInstantiateWithParams_ptsz_v12000_params_st {
+    cudaGraphExec_t *pGraphExec;
+    cudaGraph_t graph;
+    cudaGraphInstantiateParams *instantiateParams;
+} cudaGraphInstantiateWithParams_ptsz_v12000_params;
+
+typedef struct cudaGraphExecGetFlags_v12000_params_st {
+    cudaGraphExec_t graphExec;
+    unsigned long long *flags;
+} cudaGraphExecGetFlags_v12000_params;
+
 typedef struct cudaGraphExecKernelNodeSetParams_v10010_params_st {
     cudaGraphExec_t hGraphExec;
     cudaGraphNode_t node;
@@ -1696,6 +1712,7 @@ typedef struct cudaGetDriverEntryPoint_ptsz_v11030_params_st {
     const char *symbol;
     void **funcPtr;
     unsigned long long flags;
+    enum cudaDriverEntryPointQueryResult *driverStatus;
 } cudaGetDriverEntryPoint_ptsz_v11030_params;
 
 typedef struct cudaGetFuncBySymbol_v11000_params_st {
@@ -1950,6 +1967,11 @@ typedef struct cudaStreamGetFlags_v5050_params_st {
     unsigned int *flags;
 } cudaStreamGetFlags_v5050_params;
 
+typedef struct cudaStreamGetId_v12000_params_st {
+    cudaStream_t hStream;
+    unsigned long long *streamId;
+} cudaStreamGetId_v12000_params;
+
 typedef struct cudaStreamGetPriority_v5050_params_st {
     cudaStream_t hStream;
     int *priority;
@@ -2069,6 +2091,12 @@ typedef struct cudaWaitExternalSemaphoresAsync_v2_v11020_params_st {
     cudaStream_t stream;
 } cudaWaitExternalSemaphoresAsync_v2_v11020_params;
 
+typedef struct cudaGraphInstantiateWithParams_v12000_params_st {
+    cudaGraphExec_t *pGraphExec;
+    cudaGraph_t graph;
+    cudaGraphInstantiateParams *instantiateParams;
+} cudaGraphInstantiateWithParams_v12000_params;
+
 typedef struct cudaGraphUpload_v10000_params_st {
     cudaGraphExec_t graphExec;
     cudaStream_t stream;
@@ -2109,12 +2137,12 @@ typedef struct cudaStreamGetCaptureInfo_v2_v11030_params_st {
     size_t *numDependencies_out;
 } cudaStreamGetCaptureInfo_v2_v11030_params;
 
-typedef struct cudaStreamUpdateCaptureDependencies_ptsz_v11030_params_st {
+typedef struct cudaStreamUpdateCaptureDependencies_v11030_params_st {
     cudaStream_t stream;
     cudaGraphNode_t *dependencies;
     size_t numDependencies;
     unsigned int flags;
-} cudaStreamUpdateCaptureDependencies_ptsz_v11030_params;
+} cudaStreamUpdateCaptureDependencies_v11030_params;
 
 typedef struct cudaStreamCopyAttributes_v11000_params_st {
     cudaStream_t dstStream;
@@ -2155,6 +2183,7 @@ typedef struct cudaGetDriverEntryPoint_v11030_params_st {
     const char *symbol;
     void **funcPtr;
     unsigned long long flags;
+    enum cudaDriverEntryPointQueryResult *driverStatus;
 } cudaGetDriverEntryPoint_v11030_params;
 
 // Parameter trace structures for removed functions 
