@@ -378,6 +378,26 @@ typedef enum {
      */
     SANITIZER_CBID_RESOURCE_FUNCTIONS_LAZY_LOADED         = 24,
 
+    /**
+     * CUDA lazily loaded functions were patched.
+     */
+    SANITIZER_CBID_RESOURCE_FUNCTIONS_LAZY_PATCHED        = 25,
+
+    /**
+     * The CUDA driver reserved a virtual address range.
+     */
+    SANITIZER_CBID_RESOURCE_VIRTUAL_RESERVE               = 26,
+
+    /**
+     * The CUDA driver released a virtual address range.
+     */
+    SANITIZER_CBID_RESOURCE_VIRTUAL_RELEASE               = 27,
+
+    /**
+     * A memory pool allocation was imported.
+     */
+    SANITIZER_CBID_RESOURCE_MEMPOOL_IMPORT_POINTER       = 28,
+
     SANITIZER_CBID_RESOURCE_SIZE,
     SANITIZER_CBID_RESOURCE_FORCE_INT                     = 0x7fffffff
 } Sanitizer_CallbackIdResource;
@@ -636,7 +656,8 @@ typedef enum {
  * SANITIZER_CBID_RESOURCE_HOST_MEMORY_FREE,
  * SANITIZER_CBID_RESOURCE_MEMORY_ALLOC_ASYNC,
  * SANITIZER_CBID_RESOURCE_MEMORY_FREE_ASYNC or
- * SANITIZER_CBID_RESOURCE_MEMORY_FREE_ASYNC_DONE.
+ * SANITIZER_CBID_RESOURCE_MEMORY_FREE_ASYNC_DONE or
+ * SANITIZER_CBID_RESOURCE_MEMPOOL_IMPORT_POINTER.
  * The callback data is only valid within the invocation of the
  * callback function that is passed the data. If you need to
  * retain some data for use outside of the callback, you must
@@ -787,7 +808,8 @@ typedef struct {
  * \p cbdata argument to \ref Sanitizer_CallbackFunc. The
  * \p cbdata will be this type for \p domain equal tp
  * SANITIZER_CB_DOMAIN_RESOURCE and \p cbid equal to
- * SANITIZER_CBID_RESOURCE_FUNCTIONS_LAZY_LOADED.
+ * SANITIZER_CBID_RESOURCE_FUNCTIONS_LAZY_LOADED or
+ * SANITIZER_CBID_RESOURCE_FUNCTIONS_LAZY_PATCHED.
  * The callback data is only valid within the invocation of the
  * callback function that is passed the data. If you need to
  * retain some data for use outside of the callback, you must
@@ -815,6 +837,34 @@ typedef struct {
     uint32_t numFunctions;
 } Sanitizer_ResourceFunctionsLazyLoadedData;
 
+/**
+ *
+ * \brief Data passed into a VA reservation callback function.
+ *
+ * Data passed into a VA reservation callback function as the
+ * \p cbdata argument to \ref Sanitizer_CallbackFunc. The
+ * \p cbdata will be this type for \p domain equal tp
+ * SANITIZER_CB_DOMAIN_RESOURCE and \p cbid equal to
+ * SANITIZER_CBID_RESOURCE_VIRTUAL_RESERVE.
+ * or SANITIZER_CBID_RESOURCE_VIRTUAL_RELEASE.
+ * The callback data is only valid within the invocation of the
+ * callback function that is passed the data. If you need to
+ * retain some data for use outside of the callback, you must
+ * make a copy of it.
+ * Available with a driver version of 535 or newer.
+ */
+typedef struct {
+    /**
+     * Address of the VA range being reserved or released.
+     */
+    uint64_t address;
+
+    /**
+     * Size of the VA range being reserved or released.
+     */
+    uint64_t size;
+} Sanitizer_ResourceVirtualRange;
+
 
 /**
  * \brief Callback IDs for synchronization domain.
@@ -1140,6 +1190,21 @@ typedef struct {
      * The direction of the transfer
      */
     Sanitizer_MemcpyDirection direction;
+
+    /**
+     * The context on which the operation was requested
+     */
+    CUcontext apiContext;
+
+    /**
+     * The stream on which the operation was requested
+     */
+    CUstream apiStream;
+
+    /**
+     * Unique handle for the API stream.
+     */
+    Sanitizer_StreamHandle hApiStream;
 } Sanitizer_MemcpyData;
 
 /**
