@@ -729,7 +729,43 @@ typedef enum SANITIZER_CBID_DRIVER_API_enum {
    SANITIZER_CBID_DRIVER_API_cuMemAdvise_v2                                               = 715,
    SANITIZER_CBID_DRIVER_API_cuMemPrefetchAsync_v2                                        = 716,
    SANITIZER_CBID_DRIVER_API_cuMemPrefetchAsync_v2_ptsz                                   = 717,
-   SANITIZER_CBID_DRIVER_API_SIZE                                                         = 718,
+   SANITIZER_CBID_DRIVER_API_cuFuncGetName                                                = 718,
+   SANITIZER_CBID_DRIVER_API_cuKernelGetName                                              = 719,
+   SANITIZER_CBID_DRIVER_API_cuStreamBeginCaptureToGraph                                  = 720,
+   SANITIZER_CBID_DRIVER_API_cuStreamBeginCaptureToGraph_ptsz                             = 721,
+   SANITIZER_CBID_DRIVER_API_cuGraphConditionalHandleCreate                               = 722,
+   SANITIZER_CBID_DRIVER_API_cuGraphAddNode_v2                                            = 723,
+   SANITIZER_CBID_DRIVER_API_cuGraphGetEdges_v2                                           = 724,
+   SANITIZER_CBID_DRIVER_API_cuGraphNodeGetDependencies_v2                                = 725,
+   SANITIZER_CBID_DRIVER_API_cuGraphNodeGetDependentNodes_v2                              = 726,
+   SANITIZER_CBID_DRIVER_API_cuGraphAddDependencies_v2                                    = 727,
+   SANITIZER_CBID_DRIVER_API_cuGraphRemoveDependencies_v2                                 = 728,
+   SANITIZER_CBID_DRIVER_API_cuStreamGetCaptureInfo_v3                                    = 729,
+   SANITIZER_CBID_DRIVER_API_cuStreamGetCaptureInfo_v3_ptsz                               = 730,
+   SANITIZER_CBID_DRIVER_API_cuStreamUpdateCaptureDependencies_v2                         = 731,
+   SANITIZER_CBID_DRIVER_API_cuStreamUpdateCaptureDependencies_v2_ptsz                    = 732,
+   SANITIZER_CBID_DRIVER_API_cuFuncGetParamInfo                                           = 733,
+   SANITIZER_CBID_DRIVER_API_cuKernelGetParamInfo                                         = 734,
+   SANITIZER_CBID_DRIVER_API_cuDeviceRegisterAsyncNotification                            = 735,
+   SANITIZER_CBID_DRIVER_API_cuDeviceUnregisterAsyncNotification                          = 736,
+   SANITIZER_CBID_DRIVER_API_cuModuleGetFunctionCount                                     = 737,
+   SANITIZER_CBID_DRIVER_API_cuModuleEnumerateFunctions                                   = 738,
+   SANITIZER_CBID_DRIVER_API_cuLibraryGetKernelCount                                      = 739,
+   SANITIZER_CBID_DRIVER_API_cuLibraryEnumerateKernels                                    = 740,
+   SANITIZER_CBID_DRIVER_API_cuFuncIsLoaded                                               = 741,
+   SANITIZER_CBID_DRIVER_API_cuFuncLoad                                                   = 742,
+   SANITIZER_CBID_DRIVER_API_cuGreenCtxCreate                                             = 743,
+   SANITIZER_CBID_DRIVER_API_cuGreenCtxDestroy                                            = 744,
+   SANITIZER_CBID_DRIVER_API_cuDeviceGetDevResource                                       = 745,
+   SANITIZER_CBID_DRIVER_API_cuCtxGetDevResource                                          = 746,
+   SANITIZER_CBID_DRIVER_API_cuGreenCtxGetDevResource                                     = 747,
+   SANITIZER_CBID_DRIVER_API_cuDevResourceGenerateDesc                                    = 748,
+   SANITIZER_CBID_DRIVER_API_cuGreenCtxRecordEvent                                        = 749,
+   SANITIZER_CBID_DRIVER_API_cuGreenCtxWaitEvent                                          = 750,
+   SANITIZER_CBID_DRIVER_API_cuDevSmResourceSplitByCount                                  = 751,
+   SANITIZER_CBID_DRIVER_API_cuStreamGetGreenCtx                                          = 752,
+   SANITIZER_CBID_DRIVER_API_cuCtxFromGreenCtx                                            = 753,
+   SANITIZER_CBID_DRIVER_API_SIZE                                                         = 754,
    SANITIZER_CBID_DRIVER_API_FORCE_INT                                                    = 0x7fffffff,
 } SANITIZER_CBID_DRIVER_API;
 
