@@ -219,10 +219,13 @@ typedef enum {
  * 8, and 16.
  * \p flags contains information about the type of access. See
  * Sanitizer_DeviceMemoryFlags to interpret this value.
- * \p pNewValue is a pointer to the new value being written if the access is a
- * write. If the access is a read or an atomic, the pointer will be NULL.
+ * \p pData is a pointer which value depends on the type of access:
+ * - If the access is a write, \p pData points to the new value being written.
+ * - If the access is a read and \p pData is not \p NULL, then it points to a
+ *   32-bit mask of loaded bytes being used (padding bytes will not appear).
+ * - If the access is an atomic, the pointer will be \p NULL.
  */
-typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackMemoryAccess)(void* userdata, uint64_t pc, void* ptr, uint32_t accessSize, uint32_t flags, const void* pNewValue);
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackMemoryAccess)(void* userdata, uint64_t pc, void* ptr, uint32_t accessSize, uint32_t flags, const void* pData);
 
 /**
  * \brief Flags describing a barrier
@@ -605,6 +608,18 @@ typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackAsyncStore)(void* u
  */
 typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackAsyncReduction)(void* userdata, uint64_t pc, uint32_t address, uint32_t mbarAddress, uint32_t accessSize);
 
+/**
+ * \brief Function type for setting the shared memory size allocated to a block
+ *
+ * This can be generated by a setsmemsize.sync instruction
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule.
+ * \p pc is the program counter of the patched instruction.
+ * \p size is the requested size in bytes.
+ *
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackSetSmemSize)(void* userdata, uint64_t pc, uint32_t size);
+
 /**
  * \brief Instrumentation.
  *
@@ -615,98 +630,98 @@ typedef enum {
     /**
      * Invalid instruction ID.
      */
-    SANITIZER_INSTRUCTION_INVALID              = 0,
+    SANITIZER_INSTRUCTION_INVALID                     = 0,
 
     /**
      * CUDA block enter. This is called prior to any user code. The type of the
      * callback must be SanitizerCallbackBlockEnter.
      */
-    SANITIZER_INSTRUCTION_BLOCK_ENTER          = 1,
+    SANITIZER_INSTRUCTION_BLOCK_ENTER                 = 1,
 
     /**
      * CUDA block exit. This is called after all user code has executed. The type of
      * the callback must be SanitizerCallbackBlockExit.
      */
-    SANITIZER_INSTRUCTION_BLOCK_EXIT           = 2,
+    SANITIZER_INSTRUCTION_BLOCK_EXIT                  = 2,
 
     /**
      * Global Memory Access. This can be a store, load or atomic operation. The type
      * of the callback must be SanitizerCallbackMemoryAccess.
      */
-    SANITIZER_INSTRUCTION_GLOBAL_MEMORY_ACCESS = 3,
+    SANITIZER_INSTRUCTION_GLOBAL_MEMORY_ACCESS        = 3,
 
     /**
      * Shared Memory Access. This can be a store, load or atomic operation. The type
      * of the callback must be SanitizerCallbackMemoryAccess.
      */
-    SANITIZER_INSTRUCTION_SHARED_MEMORY_ACCESS = 4,
+    SANITIZER_INSTRUCTION_SHARED_MEMORY_ACCESS        = 4,
 
     /**
      * Local Memory Access. This can be a store or load operation. The type
      * of the callback must be SanitizerCallbackMemoryAccess.
      */
-    SANITIZER_INSTRUCTION_LOCAL_MEMORY_ACCESS  = 5,
+    SANITIZER_INSTRUCTION_LOCAL_MEMORY_ACCESS         = 5,
 
     /**
      * Barrier. The type of the callback must be SanitizerCallbackBarrier.
      */
-    SANITIZER_INSTRUCTION_BARRIER              = 6,
+    SANITIZER_INSTRUCTION_BARRIER                     = 6,
 
     /**
      * Syncwarp. The type of the callback must be SanitizerCallbackSyncwarp.
      */
-    SANITIZER_INSTRUCTION_SYNCWARP             = 7,
+    SANITIZER_INSTRUCTION_SYNCWARP                    = 7,
 
     /**
      * Shfl. The type of the callback must be SanitizerCallbackShfl.
      */
-    SANITIZER_INSTRUCTION_SHFL                 = 8,
+    SANITIZER_INSTRUCTION_SHFL                        = 8,
 
     /**
      * Function call. The type of the callback must be SanitizerCallbackCall.
      */
-    SANITIZER_INSTRUCTION_CALL                 = 9,
+    SANITIZER_INSTRUCTION_CALL                        = 9,
 
     /**
      * Function return. The type of the callback must be SanitizerCallbackRet.
      */
-    SANITIZER_INSTRUCTION_RET                  = 10,
+    SANITIZER_INSTRUCTION_RET                         = 10,
 
     /**
      * Device-side malloc. The type of the callback must be
      * SanitizerCallbackDeviceSideMalloc.
      */
-    SANITIZER_INSTRUCTION_DEVICE_SIDE_MALLOC   = 11,
+    SANITIZER_INSTRUCTION_DEVICE_SIDE_MALLOC          = 11,
 
     /**
      * Device-side free. The type of the callback must be
      * SanitizerCallbackDeviceSideFree.
      */
-    SANITIZER_INSTRUCTION_DEVICE_SIDE_FREE     = 12,
+    SANITIZER_INSTRUCTION_DEVICE_SIDE_FREE            = 12,
 
     /**
      * CUDA Barrier operation. The type of the callback must be
      * SanitizerCallbackCudaBarrier.
      */
-    SANITIZER_INSTRUCTION_CUDA_BARRIER         = 13,
+    SANITIZER_INSTRUCTION_CUDA_BARRIER                = 13,
 
     /**
      * Global to shared memory asynchronous copy. The type of the
      * callback must be SanitizerCallbackMemcpyAsync.
      */
-    SANITIZER_INSTRUCTION_MEMCPY_ASYNC         = 14,
+    SANITIZER_INSTRUCTION_MEMCPY_ASYNC                = 14,
 
     /**
      * Pipeline commit. The type of the callback must be
      * SanitizerCallbackPipelineCommit.
      */
-    SANITIZER_INSTRUCTION_PIPELINE_COMMIT      = 15,
+    SANITIZER_INSTRUCTION_PIPELINE_COMMIT             = 15,
 
     /**
      * Pipeline wait. The type of the callback must be
      * SanitizerCallbackPipelineWait.
      */
-    SANITIZER_INSTRUCTION_PIPELINE_WAIT        = 16,
+    SANITIZER_INSTRUCTION_PIPELINE_WAIT               = 16,
 
     /**
      * Remote Shared Memory Access. This can be a store or load operation. The
@@ -724,57 +739,69 @@ typedef enum {
      * Matrix shared memory access. The type of the callback must
      * be SanitizerCallbackMatrixMemoryAccess.
      */
-    SANITIZER_INSTRUCTION_MATRIX_MEMORY_ACCESS = 19,
+    SANITIZER_INSTRUCTION_MATRIX_MEMORY_ACCESS        = 19,
 
     /**
      * Cache control instruction. The type of the callback must
      * be SanitizerCallbackCacheControl.
      */
-    SANITIZER_INSTRUCTION_CACHE_CONTROL        = 20,
+    SANITIZER_INSTRUCTION_CACHE_CONTROL               = 20,
 
     /**
      * Cluster barrier arrive instruction. The type of the callback must
      * be SanitizerCallbackClusterBarrierArrive.
      */
-    SANITIZER_INSTRUCTION_CLUSTER_BARRIER_ARRIVE = 21,
+    SANITIZER_INSTRUCTION_CLUSTER_BARRIER_ARRIVE      = 21,
 
     /**
      * Cluster barrier wait instruction. The type of the callback must
      * be SanitizerCallbackClusterBarrierWait.
      */
-    SANITIZER_INSTRUCTION_CLUSTER_BARRIER_WAIT = 22,
+    SANITIZER_INSTRUCTION_CLUSTER_BARRIER_WAIT        = 22,
 
     /**
      * Warpgroup aligned async MMA instruction. The type of the callback must
      * be SanitizerCallbackWarpgroupMMAAsync.
      */
-    SANITIZER_INSTRUCTION_WARPGROUP_MMA_ASYNC = 23,
+    SANITIZER_INSTRUCTION_WARPGROUP_MMA_ASYNC         = 23,
 
     /**
      * Warpgroup wait MMA group instruction. The type of the callback must
      * be SanitizerCallbackWarpgroupWaitGroup.
      */
-    SANITIZER_INSTRUCTION_WARPGROUP_WAIT_GROUP = 24,
+    SANITIZER_INSTRUCTION_WARPGROUP_WAIT_GROUP        = 24,
 
     /**
      * Warpgroup fence instruction. The type of the callback must
      * be SanitizerCallbackWarpgroupFence.
      */
-    SANITIZER_INSTRUCTION_WARPGROUP_FENCE = 25,
+    SANITIZER_INSTRUCTION_WARPGROUP_FENCE             = 25,
 
     /**
      * Asynchronous store instruction. The type of the callback must
      * be SanitizerCallbackAsyncStore.
      */
-    SANITIZER_INSTRUCTION_ASYNC_STORE          = 26,
+    SANITIZER_INSTRUCTION_ASYNC_STORE                 = 26,
 
     /**
      * Asynchronous reduction instruction. The type of the callback must
      * be SanitizerCallbackAsyncReduction.
      */
-    SANITIZER_INSTRUCTION_ASYNC_REDUCTION      = 27,
+    SANITIZER_INSTRUCTION_ASYNC_REDUCTION             = 27,
 
-    SANITIZER_INSTRUCTION_FORCE_INT            = 0x7fffffff
+    /**
+     * Set the shared memory size allocated to a block instruction.
+     * The type of the callback must SanitizerCallbackSetSmemSize
+     */
+    SANITIZER_INSTRUCTION_SET_SHARED_MEMORY_SIZE      = 28,
+
+    /**
+     * Barrier after it is released. The type of the callback must be
+     * SanitizerCallbackBarrier.
+     */
+    SANITIZER_INSTRUCTION_BARRIER_RELEASE             = 29,
+
+    SANITIZER_INSTRUCTION_FORCE_INT                   = 0x7fffffff
 } Sanitizer_InstructionId;
 
 /**
