@@ -444,7 +444,27 @@ typedef enum SANITIZER_CBID_RUNTIME_API_enum {
     SANITIZER_CBID_RUNTIME_API_cudaGraphAddNode                                            = 445,
     SANITIZER_CBID_RUNTIME_API_cudaGraphNodeSetParams                                      = 446,
     SANITIZER_CBID_RUNTIME_API_cudaGraphExecNodeSetParams                                  = 447,
-    SANITIZER_CBID_RUNTIME_API_SIZE                                                        = 448,
+    SANITIZER_CBID_RUNTIME_API_cudaMemAdvise_v2                                            = 448,
+    SANITIZER_CBID_RUNTIME_API_cudaMemPrefetchAsync_v2                                     = 449,
+    SANITIZER_CBID_RUNTIME_API_cudaMemPrefetchAsync_v2_ptsz                                = 450,
+    SANITIZER_CBID_RUNTIME_API_cudaFuncGetName                                             = 451,
+    SANITIZER_CBID_RUNTIME_API_cudaStreamBeginCaptureToGraph                               = 452,
+    SANITIZER_CBID_RUNTIME_API_cudaStreamBeginCaptureToGraph_ptsz                          = 453,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphConditionalHandleCreate                            = 454,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphGetEdges_v2                                        = 455,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphNodeGetDependencies_v2                             = 456,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphNodeGetDependentNodes_v2                           = 457,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphAddDependencies_v2                                 = 458,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphRemoveDependencies_v2                              = 459,
+    SANITIZER_CBID_RUNTIME_API_cudaGraphAddNode_v2                                         = 460,
+    SANITIZER_CBID_RUNTIME_API_cudaStreamGetCaptureInfo_v3                                 = 461,
+    SANITIZER_CBID_RUNTIME_API_cudaStreamGetCaptureInfo_v3_ptsz                            = 462,
+    SANITIZER_CBID_RUNTIME_API_cudaStreamUpdateCaptureDependencies_v2                      = 463,
+    SANITIZER_CBID_RUNTIME_API_cudaStreamUpdateCaptureDependencies_v2_ptsz                 = 464,
+    SANITIZER_CBID_RUNTIME_API_cudaDeviceRegisterAsyncNotification                         = 465,
+    SANITIZER_CBID_RUNTIME_API_cudaDeviceUnregisterAsyncNotification                       = 466,
+    SANITIZER_CBID_RUNTIME_API_cudaFuncGetParamInfo                                        = 467,
+    SANITIZER_CBID_RUNTIME_API_SIZE                                                        = 468,
     SANITIZER_CBID_RUNTIME_API_FORCE_INT                                                   = 0x7fffffff
 } SANITIZER_CBID_RUNTIME_API;
 
