@@ -1,6 +1,6 @@
 /* Auxiliary vector support for GDB, the GNU debugger.
 
-   Copyright (C) 2004-2022 Free Software Foundation, Inc.
+   Copyright (C) 2004-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -89,7 +89,7 @@ ld_so_xfer_auxv (gdb_byte *readbuf,
   struct bound_minimal_symbol msym;
   CORE_ADDR data_address, pointer_address;
   struct type *ptr_type = builtin_type (target_gdbarch ())->builtin_data_ptr;
-  size_t ptr_size = TYPE_LENGTH (ptr_type);
+  size_t ptr_size = ptr_type->length ();
   size_t auxv_pair_size = 2 * ptr_size;
   gdb_byte *ptr_buf = (gdb_byte *) alloca (ptr_size);
   LONGEST retval;
@@ -99,14 +99,14 @@ ld_so_xfer_auxv (gdb_byte *readbuf,
   if (msym.minsym == NULL)
     return TARGET_XFER_E_IO;
 
-  if (MSYMBOL_SIZE (msym.minsym) != ptr_size)
+  if (msym.minsym->size () != ptr_size)
     return TARGET_XFER_E_IO;
 
   /* POINTER_ADDRESS is a location where the `_dl_auxv' variable
      resides.  DATA_ADDRESS is the inferior value present in
      `_dl_auxv', therefore the real inferior AUXV address.  */
 
-  pointer_address = BMSYMBOL_VALUE_ADDRESS (msym);
+  pointer_address = msym.value_address ();
 
   /* The location of the _dl_auxv symbol may no longer be correct if
      ld.so runs at a different address than the one present in the
@@ -239,7 +239,7 @@ memory_xfer_auxv (struct target_ops *ops,
       this function only when attaching to a process.
       */
 
-  if (current_inferior ()->attach_flag != 0)
+  if (current_inferior ()->attach_flag)
     {
       enum target_xfer_status ret;
 
@@ -255,14 +255,14 @@ memory_xfer_auxv (struct target_ops *ops,
    the auxv type field as a parameter.  */
 
 static int
-generic_auxv_parse (struct gdbarch *gdbarch, gdb_byte **readptr,
-		    gdb_byte *endptr, CORE_ADDR *typep, CORE_ADDR *valp,
+generic_auxv_parse (struct gdbarch *gdbarch, const gdb_byte **readptr,
+		    const gdb_byte *endptr, CORE_ADDR *typep, CORE_ADDR *valp,
 		    int sizeof_auxv_type)
 {
   struct type *ptr_type = builtin_type (gdbarch)->builtin_data_ptr;
-  const int sizeof_auxv_val = TYPE_LENGTH (ptr_type);
+  const int sizeof_auxv_val = ptr_type->length ();
   enum bfd_endian byte_order = gdbarch_byte_order (gdbarch);
-  gdb_byte *ptr = *readptr;
+  const gdb_byte *ptr = *readptr;
 
   if (endptr == ptr)
     return 0;
@@ -286,12 +286,12 @@ generic_auxv_parse (struct gdbarch *gdbarch, gdb_byte **readptr,
 /* See auxv.h.  */
 
 int
-default_auxv_parse (struct target_ops *ops, gdb_byte **readptr,
-		    gdb_byte *endptr, CORE_ADDR *typep, CORE_ADDR *valp)
+default_auxv_parse (struct target_ops *ops, const gdb_byte **readptr,
+		    const gdb_byte *endptr, CORE_ADDR *typep, CORE_ADDR *valp)
 {
   struct gdbarch *gdbarch = target_gdbarch ();
   struct type *ptr_type = builtin_type (gdbarch)->builtin_data_ptr;
-  const int sizeof_auxv_type = TYPE_LENGTH (ptr_type);
+  const int sizeof_auxv_type = ptr_type->length ();
 
   return generic_auxv_parse (gdbarch, readptr, endptr, typep, valp,
 			     sizeof_auxv_type);
@@ -300,31 +300,33 @@ default_auxv_parse (struct target_ops *ops, gdb_byte **readptr,
 /* See auxv.h.  */
 
 int
-svr4_auxv_parse (struct gdbarch *gdbarch, gdb_byte **readptr,
-		 gdb_byte *endptr, CORE_ADDR *typep, CORE_ADDR *valp)
+svr4_auxv_parse (struct gdbarch *gdbarch, const gdb_byte **readptr,
+		 const gdb_byte *endptr, CORE_ADDR *typep, CORE_ADDR *valp)
 {
   struct type *int_type = builtin_type (gdbarch)->builtin_int;
-  const int sizeof_auxv_type = TYPE_LENGTH (int_type);
+  const int sizeof_auxv_type = int_type->length ();
 
   return generic_auxv_parse (gdbarch, readptr, endptr, typep, valp,
 			     sizeof_auxv_type);
 }
 
 /* Read one auxv entry from *READPTR, not reading locations >= ENDPTR.
+
+   Use the auxv_parse method from GDBARCH, if defined, else use the auxv_parse
+   method of OPS.
+
    Return 0 if *READPTR is already at the end of the buffer.
    Return -1 if there is insufficient buffer for a whole entry.
    Return 1 if an entry was read into *TYPEP and *VALP.  */
-int
-target_auxv_parse (gdb_byte **readptr,
-		   gdb_byte *endptr, CORE_ADDR *typep, CORE_ADDR *valp)
-{
-  struct gdbarch *gdbarch = target_gdbarch();
 
+static int
+parse_auxv (target_ops *ops, gdbarch *gdbarch, const gdb_byte **readptr,
+	    const gdb_byte *endptr, CORE_ADDR *typep, CORE_ADDR *valp)
+{
   if (gdbarch_auxv_parse_p (gdbarch))
     return gdbarch_auxv_parse (gdbarch, readptr, endptr, typep, valp);
 
-  return current_inferior ()->top_target ()->auxv_parse (readptr, endptr,
-							 typep, valp);
+  return ops->auxv_parse (readptr, endptr, typep, valp);
 }
 
 
@@ -337,7 +339,7 @@ struct auxv_info
 };
 
 /* Per-inferior data key for auxv.  */
-static const struct inferior_key<auxv_info> auxv_inferior_data;
+static const registry<inferior>::key<auxv_info> auxv_inferior_data;
 
 /* Invalidate INF's auxv cache.  */
 
@@ -355,45 +357,44 @@ invalidate_auxv_cache (void)
   invalidate_auxv_cache_inf (current_inferior ());
 }
 
-/* Fetch the auxv object from inferior INF.  If auxv is cached already,
-   return a pointer to the cache.  If not, fetch the auxv object from the
-   target and cache it.  This function always returns a valid INFO pointer.  */
+/* See auxv.h.  */
 
-static struct auxv_info *
-get_auxv_inferior_data (struct target_ops *ops)
+gdb::optional<gdb::byte_vector>
+target_read_auxv ()
 {
-  struct auxv_info *info;
-  struct inferior *inf = current_inferior ();
+  inferior *inf = current_inferior ();
+  auxv_info *info = auxv_inferior_data.get (inf);
 
-  info = auxv_inferior_data.get (inf);
-  if (info == NULL)
+  if (info == nullptr)
     {
       info = auxv_inferior_data.emplace (inf);
-      info->data = target_read_alloc (ops, TARGET_OBJECT_AUXV, NULL);
+      info->data = target_read_auxv_raw (inf->top_target ());
     }
 
-  return info;
+  return info->data;
 }
 
-/* Extract the auxiliary vector entry with a_type matching MATCH.
-   Return zero if no such entry was found, or -1 if there was
-   an error getting the information.  On success, return 1 after
-   storing the entry's value field in *VALP.  */
+/* See auxv.h.  */
+
+gdb::optional<gdb::byte_vector>
+target_read_auxv_raw (target_ops *ops)
+{
+  return target_read_alloc (ops, TARGET_OBJECT_AUXV, NULL);
+}
+
+/* See auxv.h.  */
+
 int
-target_auxv_search (struct target_ops *ops, CORE_ADDR match, CORE_ADDR *valp)
+target_auxv_search (const gdb::byte_vector &auxv, target_ops *ops,
+		    gdbarch *gdbarch, CORE_ADDR match, CORE_ADDR *valp)
 {
   CORE_ADDR type, val;
-  auxv_info *info = get_auxv_inferior_data (ops);
-
-  if (!info->data)
-    return -1;
-
-  gdb_byte *data = info->data->data ();
-  gdb_byte *ptr = data;
-  size_t len = info->data->size ();
+  const gdb_byte *data = auxv.data ();
+  const gdb_byte *ptr = data;
+  size_t len = auxv.size ();
 
   while (1)
-    switch (target_auxv_parse (&ptr, data + len, &type, &val))
+    switch (parse_auxv (ops, gdbarch, &ptr, data + len, &type, &val))
       {
       case 1:			/* Here's an entry, check it.  */
 	if (type == match)
@@ -407,10 +408,21 @@ target_auxv_search (struct target_ops *ops, CORE_ADDR match, CORE_ADDR *valp)
       default:			/* Bogosity.  */
 	return -1;
       }
-
-  /*NOTREACHED*/
 }
 
+/* See auxv.h.  */
+
+int
+target_auxv_search (CORE_ADDR match, CORE_ADDR *valp)
+{
+  gdb::optional<gdb::byte_vector> auxv = target_read_auxv ();
+
+  if (!auxv.has_value ())
+    return -1;
+
+  return target_auxv_search (*auxv, current_inferior ()->top_target (),
+			     current_inferior ()->gdbarch, match, valp);
+}
 
 /* Print the description of a single AUXV entry on the specified file.  */
 
@@ -419,15 +431,15 @@ fprint_auxv_entry (struct ui_file *file, const char *name,
 		   const char *description, enum auxv_format format,
 		   CORE_ADDR type, CORE_ADDR val)
 {
-  fprintf_filtered (file, ("%-4s %-20s %-30s "),
-		    plongest (type), name, description);
+  gdb_printf (file, ("%-4s %-20s %-30s "),
+	      plongest (type), name, description);
   switch (format)
     {
     case AUXV_FORMAT_DEC:
-      fprintf_filtered (file, ("%s\n"), plongest (val));
+      gdb_printf (file, ("%s\n"), plongest (val));
       break;
     case AUXV_FORMAT_HEX:
-      fprintf_filtered (file, ("%s\n"), paddress (target_gdbarch (), val));
+      gdb_printf (file, ("%s\n"), paddress (target_gdbarch (), val));
       break;
     case AUXV_FORMAT_STR:
       {
@@ -435,10 +447,10 @@ fprint_auxv_entry (struct ui_file *file, const char *name,
 
 	get_user_print_options (&opts);
 	if (opts.addressprint)
-	  fprintf_filtered (file, ("%s "), paddress (target_gdbarch (), val));
+	  gdb_printf (file, ("%s "), paddress (target_gdbarch (), val));
 	val_print_string (builtin_type (target_gdbarch ())->builtin_char,
 			  NULL, val, -1, file, &opts);
-	fprintf_filtered (file, ("\n"));
+	gdb_printf (file, ("\n"));
       }
       break;
     }
@@ -566,22 +578,24 @@ default_print_auxv_entry (struct gdbarch *gdbarch, struct ui_file *file,
 
 /* Print the contents of the target's AUXV on the specified file.  */
 
-int
-fprint_target_auxv (struct ui_file *file, struct target_ops *ops)
+static int
+fprint_target_auxv (struct ui_file *file)
 {
   struct gdbarch *gdbarch = target_gdbarch ();
   CORE_ADDR type, val;
   int ents = 0;
-  auxv_info *info = get_auxv_inferior_data (ops);
+  gdb::optional<gdb::byte_vector> auxv = target_read_auxv ();
 
-  if (!info->data)
+  if (!auxv.has_value ())
     return -1;
 
-  gdb_byte *data = info->data->data ();
-  gdb_byte *ptr = data;
-  size_t len = info->data->size ();
+  const gdb_byte *data = auxv->data ();
+  const gdb_byte *ptr = data;
+  size_t len = auxv->size ();
 
-  while (target_auxv_parse (&ptr, data + len, &type, &val) > 0)
+  while (parse_auxv (current_inferior ()->top_target (),
+		     current_inferior ()->gdbarch,
+		     &ptr, data + len, &type, &val) > 0)
     {
       gdbarch_print_auxv_entry (gdbarch, file, type, val);
       ++ents;
@@ -599,8 +613,7 @@ info_auxv_command (const char *cmd, int from_tty)
     error (_("The program has no auxiliary information now."));
   else
     {
-      int ents = fprint_target_auxv (gdb_stdout,
-				     current_inferior ()->top_target ());
+      int ents = fprint_target_auxv (gdb_stdout);
 
       if (ents < 0)
 	error (_("No auxiliary vector found, or failed reading it."));
