@@ -2,16 +2,16 @@
  * NVIDIA CUDA Debugger CUDA-GDB
  * Copyright (C) 2007-2023 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
  * published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
@@ -42,8 +42,8 @@ cuda_latest_launched_kernel_id (void)
 }
 
 /* forward declaration */
-static void
-kernels_add_parent_kernel (uint32_t dev_id, uint64_t grid_id, uint64_t *parent_grid_id);
+static void kernels_add_parent_kernel (uint32_t dev_id, uint64_t grid_id,
+                                       uint64_t *parent_grid_id);
 
 /******************************************************************************
  *
@@ -51,29 +51,30 @@ kernels_add_parent_kernel (uint32_t dev_id, uint64_t grid_id, uint64_t *parent_g
  *
  *****************************************************************************/
 
-struct kernel_st {
-  bool                          grid_status_p;
-  bool                          cluster_dim_p;
-  uint64_t                      id;              /* unique kernel id per GDB session */
-  uint32_t                      dev_id;          /* device where the kernel was launched */
-  uint64_t                      grid_id;         /* unique kernel id per device */
-  CUDBGGridStatus               grid_status;     /* current grid status of the kernel */
-  kernel_t                      parent;          /* the kernel that launched this grid */
-  kernel_t                      children;        /* list of children */
-  kernel_t                      siblings;        /* next sibling when traversing the list of children */
-  gdb::unique_xmalloc_ptr<char> name;            /* name of the kernel if available */
-  gdb::unique_xmalloc_ptr<char> args;            /* kernel arguments in string format */
-  uint64_t                      virt_code_base;  /* virtual address of the kernel entry point */
-  module_t                      module;          /* CUmodule handle of the kernel */
-  bool                          launched;        /* Has the kernel been seen on the hw? */
-  CuDim3                        grid_dim;        /* The grid dimensions of the kernel. */
-  CuDim3                        cluster_dim;     /* The cluster dimensions of the kernel. */
-  CuDim3                        block_dim;       /* The block dimensions of the kernel. */
-  char                          dimensions[128]; /* A string repr. of the kernel dimensions. */
-  CUDBGKernelType               type;            /* The kernel type: system or application. */
-  CUDBGKernelOrigin             origin;          /* The kernel origin: CPU or GPU */
-  kernel_t                      next;            /* next kernel on the same device */
-  unsigned int                  depth;           /* kernel nest level (0 - host launched kernel) */
+struct kernel_st
+{
+  bool grid_status_p;
+  bool cluster_dim_p;
+  uint64_t id;                 /* unique kernel id per GDB session */
+  uint32_t dev_id;             /* device where the kernel was launched */
+  uint64_t grid_id;            /* unique kernel id per device */
+  CUDBGGridStatus grid_status; /* current grid status of the kernel */
+  kernel_t parent;             /* the kernel that launched this grid */
+  kernel_t children;           /* list of children */
+  kernel_t siblings; /* next sibling when traversing the list of children */
+  gdb::unique_xmalloc_ptr<char> name; /* name of the kernel if available */
+  gdb::unique_xmalloc_ptr<char> args; /* kernel arguments in string format */
+  uint64_t virt_code_base;  /* virtual address of the kernel entry point */
+  module_t module;          /* CUmodule handle of the kernel */
+  bool launched;            /* Has the kernel been seen on the hw? */
+  CuDim3 grid_dim;          /* The grid dimensions of the kernel. */
+  CuDim3 cluster_dim;       /* The cluster dimensions of the kernel. */
+  CuDim3 block_dim;         /* The block dimensions of the kernel. */
+  char dimensions[128];     /* A string repr. of the kernel dimensions. */
+  CUDBGKernelType type;     /* The kernel type: system or application. */
+  CUDBGKernelOrigin origin; /* The kernel origin: CPU or GPU */
+  kernel_t next;            /* next kernel on the same device */
+  unsigned int depth;       /* kernel nest level (0 - host launched kernel) */
 };
 
 static void
@@ -104,8 +105,7 @@ kernel_remove_child (kernel_t parent, kernel_t child)
       return;
     }
 
-  for (prev = parent->children, cur = parent->children->siblings;
-       cur != NULL;
+  for (prev = parent->children, cur = parent->children->siblings; cur != NULL;
        prev = cur, cur = cur->siblings)
     if (cur == child)
       {
@@ -122,60 +122,64 @@ should_print_kernel_event (kernel_t kernel)
   if (depth_or_disabled && kernel->depth > depth_or_disabled - 1)
     return false;
 
-  return (kernel->type == CUDBG_KNL_TYPE_SYSTEM && cuda_options_show_kernel_events_system ()) ||
-         (kernel->type == CUDBG_KNL_TYPE_APPLICATION && cuda_options_show_kernel_events_application ());
+  return (kernel->type == CUDBG_KNL_TYPE_SYSTEM
+          && cuda_options_show_kernel_events_system ())
+         || (kernel->type == CUDBG_KNL_TYPE_APPLICATION
+             && cuda_options_show_kernel_events_application ());
 }
 
 static kernel_t
 kernel_new (uint32_t dev_id, uint64_t grid_id, uint64_t virt_code_base,
             gdb::unique_xmalloc_ptr<char> name, module_t module,
-            CuDim3 grid_dim, CuDim3 block_dim,
-            CUDBGKernelType type, uint64_t parent_grid_id,
-            CUDBGKernelOrigin origin, bool has_cluster_dim, CuDim3 cluster_dim)
+            CuDim3 grid_dim, CuDim3 block_dim, CUDBGKernelType type,
+            uint64_t parent_grid_id, CUDBGKernelOrigin origin,
+            bool has_cluster_dim, CuDim3 cluster_dim)
 {
-  kernel_t   kernel;
-  kernel_t   parent_kernel;
+  kernel_t kernel;
+  kernel_t parent_kernel;
 
   parent_kernel = kernels_find_kernel_by_grid_id (dev_id, parent_grid_id);
   if (!parent_kernel && origin == CUDBG_KNL_ORIGIN_GPU)
     {
-      kernels_add_parent_kernel (dev_id, grid_id, &parent_grid_id); 
+      kernels_add_parent_kernel (dev_id, grid_id, &parent_grid_id);
       parent_kernel = kernels_find_kernel_by_grid_id (dev_id, parent_grid_id);
     }
 
   kernel = new struct kernel_st;
 
-  kernel->grid_status_p            = false;
-  kernel->cluster_dim_p            = has_cluster_dim;
-  kernel->id                       = next_kernel_id++;
-  kernel->dev_id                   = dev_id;
-  kernel->grid_id                  = grid_id;
-  kernel->parent                   = parent_kernel;
-  kernel->children                 = NULL;
-  kernel->siblings                 = NULL;
-  kernel->virt_code_base           = virt_code_base;
-  kernel->name                     = name ? std::move (name) : make_unique_xstrdup ("??");
-  kernel->args                     = make_unique_xstrdup ("");
-  kernel->module                   = module;
-  kernel->grid_dim                 = grid_dim;
-  kernel->cluster_dim              = cluster_dim;
-  kernel->block_dim                = block_dim;
-  kernel->type                     = type;
-  kernel->origin                   = origin;
-  kernel->next                     = NULL;
-  kernel->depth                    = !parent_kernel ? 0 : parent_kernel->depth + 1;
+  kernel->grid_status_p = false;
+  kernel->cluster_dim_p = has_cluster_dim;
+  kernel->id = next_kernel_id++;
+  kernel->dev_id = dev_id;
+  kernel->grid_id = grid_id;
+  kernel->parent = parent_kernel;
+  kernel->children = NULL;
+  kernel->siblings = NULL;
+  kernel->virt_code_base = virt_code_base;
+  kernel->name = name ? std::move (name) : make_unique_xstrdup ("??");
+  kernel->args = make_unique_xstrdup ("");
+  kernel->module = module;
+  kernel->grid_dim = grid_dim;
+  kernel->cluster_dim = cluster_dim;
+  kernel->block_dim = block_dim;
+  kernel->type = type;
+  kernel->origin = origin;
+  kernel->next = NULL;
+  kernel->depth = !parent_kernel ? 0 : parent_kernel->depth + 1;
 
-  snprintf (kernel->dimensions, sizeof (kernel->dimensions), "<<<(%d,%d,%d),(%d,%d,%d)>>>",
-            grid_dim.x, grid_dim.y, grid_dim.z, block_dim.x, block_dim.y, block_dim.z);
+  snprintf (kernel->dimensions, sizeof (kernel->dimensions),
+            "<<<(%d,%d,%d),(%d,%d,%d)>>>", grid_dim.x, grid_dim.y, grid_dim.z,
+            block_dim.x, block_dim.y, block_dim.z);
 
   kernel->launched = false;
 
   kernel_add_child (parent_kernel, kernel);
 
-  if (should_print_kernel_event(kernel))
-    printf_unfiltered (_("[Launch of CUDA Kernel %llu (%s%s) on Device %u, level %u]\n"),
-                       (unsigned long long)kernel->id, kernel->name.get (), kernel->dimensions,
-                       kernel->dev_id, kernel->depth);
+  if (should_print_kernel_event (kernel))
+    printf_unfiltered (
+        _ ("[Launch of CUDA Kernel %llu (%s%s) on Device %u, level %u]\n"),
+        (unsigned long long)kernel->id, kernel->name.get (),
+        kernel->dimensions, kernel->dev_id, kernel->depth);
 
   return kernel;
 }
@@ -187,10 +191,11 @@ kernel_delete (kernel_t kernel)
 
   kernel_remove_child (kernel->parent, kernel);
 
-  if (should_print_kernel_event(kernel))
-    printf_unfiltered (_("[Termination of CUDA Kernel %llu (%s%s) on Device %u, level %u]\n"),
-                       (unsigned long long)kernel->id, kernel->name.get (), kernel->dimensions,
-                       kernel->dev_id, kernel->depth);
+  if (should_print_kernel_event (kernel))
+    printf_unfiltered (_ ("[Termination of CUDA Kernel %llu (%s%s) on Device "
+                          "%u, level %u]\n"),
+                       (unsigned long long)kernel->id, kernel->name.get (),
+                       kernel->dimensions, kernel->dev_id, kernel->depth);
 
   delete kernel;
 }
@@ -221,37 +226,39 @@ kernel_get_name (kernel_t kernel)
 static void
 kernel_populate_args (kernel_t kernel)
 {
-  cuda_coords_t *coords, requested;
-  cuda_coords_t candidates[CK_MAX] = {CUDA_INVALID_COORDS};
-  struct frame_info *prev_frame, *frame;
-  cuda_focus_t focus;
-
-  cuda_focus_init (&focus);
-
-  string_file stream;
-
   /* Find an active lane for the kernel */
-  requested = CUDA_WILDCARD_COORDS;
-  requested.kernelId = kernel_get_id (kernel);
-  cuda_coords_find_valid (requested, candidates);
-  coords = &candidates[CK_EXACT_LOGICAL];
-  if (!coords->valid || !cuda_coords_equal (&requested, coords)) {
+  cuda_coords filter{ CUDA_WILDCARD,          CUDA_WILDCARD,
+                      CUDA_WILDCARD,          CUDA_WILDCARD,
+                      kernel_get_id (kernel), CUDA_WILDCARD,
+                      CUDA_WILDCARD_DIM,      CUDA_WILDCARD_DIM,
+                      CUDA_WILDCARD_DIM };
+  cuda_iterator<cuda_iterator_type::lanes, select_valid | select_sngl> coord{
+    filter
+  };
+  /* Cannot populate args if we didn't find an active lane */
+  if (!coord.size ())
     return;
-  }
 
   /* Save environment */
-  cuda_focus_save (&focus);
+  string_file stream;
   current_uiout->redirect (&stream);
 
+  /* Make sure we switch back to the current focus when done. */
+  cuda_focus_restore r;
+
   try
     {
       /* Switch focus to that lane/kernel, temporarily */
-      switch_to_cuda_thread (coords);
+      switch_to_cuda_thread (*coord.cbegin ());
 
       /* Find the outermost frame */
-      frame = get_current_frame ();
-      while ((prev_frame = get_prev_frame (frame)))
-	frame = prev_frame;
+      frame_info_ptr frame = get_current_frame ();
+      frame_info_ptr prev_frame = get_prev_frame (frame);
+      while (prev_frame)
+        {
+          frame = prev_frame;
+          prev_frame = get_prev_frame (frame);
+        }
 
       /* Print the arguments */
       print_args_frame (frame);
@@ -264,7 +271,6 @@ kernel_populate_args (kernel_t kernel)
 
   /* Restore environment */
   current_uiout->redirect (NULL);
-  cuda_focus_restore (&focus);
 }
 
 const char *
@@ -348,10 +354,11 @@ kernel_get_cluster_dim (kernel_t kernel)
 {
   gdb_assert (kernel);
   if (!kernel->cluster_dim_p)
-  {
-    cuda_api_get_cluster_dim (kernel->dev_id, kernel->grid_id, &kernel->cluster_dim);
-    kernel->cluster_dim_p = CACHED;
-  }
+    {
+      cuda_debugapi::get_cluster_dim (kernel->dev_id, kernel->grid_id,
+                                      &kernel->cluster_dim);
+      kernel->cluster_dim_p = CACHED;
+    }
   return kernel->cluster_dim;
 }
 
@@ -362,7 +369,7 @@ kernel_get_block_dim (kernel_t kernel)
   return kernel->block_dim;
 }
 
-const char*
+const char *
 kernel_get_dimensions (kernel_t kernel)
 {
   gdb_assert (kernel);
@@ -383,7 +390,8 @@ kernel_get_status (kernel_t kernel)
 
   if (!kernel->grid_status_p)
     {
-      cuda_api_get_grid_status (kernel->dev_id, kernel->grid_id, &kernel->grid_status);
+      cuda_debugapi::get_grid_status (kernel->dev_id, kernel->grid_id,
+                                      &kernel->grid_status);
       kernel->grid_status_p = CACHED;
     }
 
@@ -441,34 +449,31 @@ kernel_is_present (kernel_t kernel)
   gdb_assert (kernel);
 
   status = kernel_get_status (kernel);
-  present = (status == CUDBG_GRID_STATUS_ACTIVE ||
-             status == CUDBG_GRID_STATUS_SLEEPING);
+  present = (status == CUDBG_GRID_STATUS_ACTIVE
+             || status == CUDBG_GRID_STATUS_SLEEPING);
 
   return present;
 }
 
 void
-kernel_compute_sms_mask (kernel_t kernel, std::bitset<CUDBG_MAX_SMS> &sms_mask)
+kernel_compute_sms_mask (kernel_t kernel, cuda_bitset &sms_mask)
 {
-  cuda_coords_t current;
-  cuda_coords_t filter;
-  cuda_iterator itr;
-
   gdb_assert (kernel);
 
-  filter        = CUDA_WILDCARD_COORDS;
-  filter.dev    = kernel->dev_id;
-  filter.gridId = kernel->grid_id;
-  itr = cuda_iterator_create (CUDA_ITERATOR_TYPE_WARPS, &filter, CUDA_SELECT_VALID);
+  cuda_coords filter{
+    kernel->dev_id,    CUDA_WILDCARD,     CUDA_WILDCARD,
+    CUDA_WILDCARD,     kernel->id,        kernel->grid_id,
+    CUDA_WILDCARD_DIM, CUDA_WILDCARD_DIM, CUDA_WILDCARD_DIM
+  };
+  cuda_iterator<cuda_iterator_type::sms, select_valid,
+                cuda_compare_type::physical>
+      coords{ filter };
 
   // Reset the bitset passed in
-  sms_mask.reset ();
+  sms_mask.resize (cuda_state::device_get_num_sms (kernel->dev_id));
 
-  for (cuda_iterator_start (itr); !cuda_iterator_end (itr); cuda_iterator_next (itr))
-    {
-      current = cuda_iterator_get_current (itr);
-      sms_mask.set (current.sm);
-    }
+  for (const auto &coord : coords)
+    sms_mask.set (coord.physical ().sm ());
 }
 
 void
@@ -480,16 +485,20 @@ kernel_print (kernel_t kernel)
   fprintf (stderr, "        name        : %s\n", kernel->name.get ());
   fprintf (stderr, "        device id   : %u\n", kernel->dev_id);
   fprintf (stderr, "        grid id     : %lld\n", (long long)kernel->grid_id);
-  fprintf (stderr, "        module id   : 0x%llx\n", (unsigned long long)module_get_id (kernel->module));
-  fprintf (stderr, "        entry point : 0x%llx\n", (unsigned long long)kernel->virt_code_base);
+  fprintf (stderr, "        module id   : 0x%llx\n",
+           (unsigned long long)module_get_id (kernel->module));
+  fprintf (stderr, "        entry point : 0x%llx\n",
+           (unsigned long long)kernel->virt_code_base);
   fprintf (stderr, "        dimensions  : %s\n", kernel->dimensions);
-  fprintf (stderr, "        launched    : %s\n", kernel->launched ? "yes" : "no");
-  fprintf (stderr, "        present     : %s\n", kernel_is_present (kernel)? "yes" : "no");
-  fprintf (stderr, "        next        : 0x%llx\n", (unsigned long long)(uintptr_t)kernel->next);
+  fprintf (stderr, "        launched    : %s\n",
+           kernel->launched ? "yes" : "no");
+  fprintf (stderr, "        present     : %s\n",
+           kernel_is_present (kernel) ? "yes" : "no");
+  fprintf (stderr, "        next        : 0x%llx\n",
+           (unsigned long long)(uintptr_t)kernel->next);
   fflush (stderr);
 }
 
-
 /******************************************************************************
  *
  *                                   Kernels
@@ -511,52 +520,66 @@ kernels_print (void)
 void
 kernels_start_kernel (uint32_t dev_id, uint64_t grid_id,
                       uint64_t virt_code_base, uint64_t context_id,
-                      uint64_t module_id, CuDim3 grid_dim,
-                      CuDim3 block_dim, CUDBGKernelType type,
-                      uint64_t parent_grid_id, CUDBGKernelOrigin origin,
-                      bool has_cluster_dim, CuDim3 cluster_dim)
+                      uint64_t module_id, CuDim3 grid_dim, CuDim3 block_dim,
+                      CUDBGKernelType type, uint64_t parent_grid_id,
+                      CUDBGKernelOrigin origin, bool has_cluster_dim,
+                      CuDim3 cluster_dim)
 {
   context_t context;
   modules_t modules;
-  module_t  module;
-  kernel_t  kernel;
+  module_t module;
+  kernel_t kernel;
 
-  context = device_find_context_by_id (dev_id, context_id);
+  context = cuda_state::device_find_context_by_id (dev_id, context_id);
   modules = context_get_modules (context);
   module = modules_find_module_by_id (modules, module_id);
 
+  if (!module)
+    warning (
+        "Could not find CUDA module for context_id 0x%llx module_id 0x%llx",
+        (unsigned long long)context_id, (unsigned long long)module_id);
   gdb_assert (module);
 
   if (context)
     set_current_context (context);
 
-  gdb::unique_xmalloc_ptr<char> kernel_name = cuda_find_function_name_from_pc (virt_code_base, true);
-
-  kernel = kernel_new (dev_id, grid_id, virt_code_base, std::move (kernel_name), module,
-                       grid_dim, block_dim, type, parent_grid_id,
-                       origin, has_cluster_dim, cluster_dim);
+  gdb::unique_xmalloc_ptr<char> kernel_name
+      = cuda_find_function_name_from_pc (virt_code_base, true);
+  if (kernel_name.get () == nullptr)
+    // NOTE: Not having an entry function is a normal situation, this means
+    // an internal kernel contained in a public module was launched.
+    kernel_name = make_unique_xstrdup ("<internal>");
 
+  kernel
+      = kernel_new (dev_id, grid_id, virt_code_base, std::move (kernel_name),
+                    module, grid_dim, block_dim, type, parent_grid_id, origin,
+                    has_cluster_dim, cluster_dim);
 
   kernel->next = kernels;
   kernels = kernel;
 }
 
 static void
-kernels_add_parent_kernel (uint32_t dev_id, uint64_t grid_id, uint64_t *parent_grid_id)
+kernels_add_parent_kernel (uint32_t dev_id, uint64_t grid_id,
+                           uint64_t *parent_grid_id)
 {
   CUDBGGridInfo grid_info;
   CUDBGGridInfo parent_grid_info;
   CUDBGGridStatus grid_status;
 
-  cuda_api_get_grid_status (dev_id, grid_id, &grid_status);
-  if (grid_status == CUDBG_GRID_STATUS_INVALID) return;
+  cuda_debugapi::get_grid_status (dev_id, grid_id, &grid_status);
+  if (grid_status == CUDBG_GRID_STATUS_INVALID)
+    return;
 
-  cuda_api_get_grid_info (dev_id, grid_id, &grid_info);
+  cuda_debugapi::get_grid_info (dev_id, grid_id, &grid_info);
 
-  cuda_api_get_grid_status (dev_id, grid_info.parentGridId, &grid_status);
-  if (grid_status == CUDBG_GRID_STATUS_INVALID) return;
+  cuda_debugapi::get_grid_status (dev_id, grid_info.parentGridId,
+                                  &grid_status);
+  if (grid_status == CUDBG_GRID_STATUS_INVALID)
+    return;
 
-  cuda_api_get_grid_info (dev_id, grid_info.parentGridId, &parent_grid_info);
+  cuda_debugapi::get_grid_info (dev_id, grid_info.parentGridId,
+                                &parent_grid_info);
   *parent_grid_id = parent_grid_info.gridId64;
   kernels_start_kernel (parent_grid_info.dev, parent_grid_info.gridId64,
                         parent_grid_info.functionEntry,
@@ -569,7 +592,7 @@ kernels_add_parent_kernel (uint32_t dev_id, uint64_t grid_id, uint64_t *parent_g
 void
 kernels_terminate_kernel (kernel_t kernel)
 {
-  kernel_t  prev, ker;
+  kernel_t prev, ker;
 
   if (!kernel)
     return;
@@ -578,8 +601,7 @@ kernels_terminate_kernel (kernel_t kernel)
   if (kernel->children)
     return;
 
-  for (ker = kernels, prev = NULL;
-       ker && ker != kernel;
+  for (ker = kernels, prev = NULL; ker && ker != kernel;
        prev = ker, ker = kernels_get_next_kernel (ker))
     ;
   gdb_assert (ker);
@@ -648,12 +670,25 @@ kernels_find_kernel_by_kernel_id (uint64_t kernel_id)
   return NULL;
 }
 
+void
+kernels_invalidate (uint32_t dev_id)
+{
+
+  for (auto kernel = kernels_get_first_kernel (); kernel; )
+    {
+      if (kernel->dev_id == dev_id)
+	kernel_invalidate (kernel);
+      kernel = kernels_get_next_kernel (kernel);
+    }
+}
+
 void
 kernels_update_args (void)
 {
   kernel_t kernel;
 
-  for (kernel = kernels_get_first_kernel (); kernel; kernel = kernels_get_next_kernel (kernel))
+  for (kernel = kernels_get_first_kernel (); kernel;
+       kernel = kernels_get_next_kernel (kernel))
     if (!kernel->args && kernel_is_present (kernel))
       kernel_populate_args (kernel);
 }
@@ -661,8 +696,15 @@ kernels_update_args (void)
 void
 kernels_update_terminated (void)
 {
-  kernel_t      kernel;
-  kernel_t      next_kernel;
+  kernel_t kernel;
+  kernel_t next_kernel;
+
+  /* Make sure we have up-to-date information about running kernels.
+   * TODO: This feels like we are relying on a side-effect by creating an
+   * iterator. */
+  cuda_iterator<cuda_iterator_type::kernels, select_valid> coord{
+    cuda_coords::wild ()
+  };
 
   /* rediscover the kernels currently running on the hardware */
   kernel = kernels_get_first_kernel ();
