@@ -26,6 +26,8 @@
 #include "cuda-elf-image.h"
 #include "cuda-options.h"
 #include "cuda-tdep.h"
+#include "cuda-modules.h"
+#include "cuda-kernel.h"
 
 
 /******************************************************************************
