@@ -104,18 +104,21 @@ cuda_options_initialize_debug_cuda_prefix (void)
  * set debug cuda trace
  */
 static std::string cuda_debug_trace_string;
-static int cuda_debug_trace_flags  = 0;
+static uint32_t cuda_debug_trace_flags = 0;
 static struct {
   const char *name;
   cuda_trace_domain_t domain;
   const char *description;
 } cuda_debug_trace_names[] = {
   {"general", CUDA_TRACE_GENERAL, "show/hide general debug trace of the internal CUDA-specific functions"},
+  {"disasm", CUDA_TRACE_DISASSEMBLER, "When enabled trace GPU disassembly operations"},
+  {"elf", CUDA_TRACE_ELF, "show/hide CUDA ELF file processing trace messages"},
   {"event", CUDA_TRACE_EVENT, "show/hide CUDA event trace messages"},
   {"breakpoint", CUDA_TRACE_BREAKPOINT, "show/hide CUDA-specific breakpoint handling trace messages"},
   {"api", CUDA_TRACE_API, "show/hide CUDA Debugger API trace messages"},
   {"siginfo", CUDA_TRACE_SIGINFO, "When enabled, update $_siginfo if the application is signalled by a CUDA exception"},
-  {"disasm", CUDA_TRACE_DISASSEMBLER, "When enabled trace GPU disassembly operations"},
+  {"state", CUDA_TRACE_STATE, "show/hide CUDA state trace messages"},
+  {"decode", CUDA_TRACE_STATE_DECODE, "show/hide CUDA state device update decoding messages"},
   {NULL, (cuda_trace_domain_t) 0, NULL},
 };
 
@@ -125,7 +128,13 @@ cuda_options_trace_domain_enabled (cuda_trace_domain_t domain)
   return (cuda_debug_trace_flags>>domain)&1;
 }
 
-static void
+uint32_t
+cuda_options_trace_domain_enabled_flags ()
+{
+  return cuda_debug_trace_flags;
+}
+
+void
 cuda_options_set_trace_domain (cuda_trace_domain_t domain, bool enabled)
 {
   if (enabled)
@@ -134,20 +143,19 @@ cuda_options_set_trace_domain (cuda_trace_domain_t domain, bool enabled)
     cuda_debug_trace_flags &= ~(1<<domain);
 }
 
-
 static void
 cuda_show_debug_trace (struct ui_file *file, int from_tty,
                          struct cmd_list_element *c, const char *value)
 {
   int cnt;
 
-  fprintf_filtered (file, _("CUDA general debug trace:"));
+  gdb_printf (file, _("CUDA general debug trace:"));
   for (cnt = 0; cuda_debug_trace_names[cnt].name; cnt++)
-    fprintf_filtered (file, "%s %s: %s",
+    gdb_printf (file, "%s %s: %s",
             cnt ? "," : "",
             cuda_debug_trace_names[cnt].name,
             cuda_options_trace_domain_enabled (cuda_debug_trace_names[cnt].domain) ? "on" : "off" );
-  fprintf_filtered (file, "\n");
+  gdb_printf (file, "\n");
 }
 
 static void
@@ -254,7 +262,7 @@ static void
 cuda_show_debug_notifications (struct ui_file *file, int from_tty,
                                struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("CUDA notifications debug trace is %s.\n"), value);
+  gdb_printf (file, _("CUDA notifications debug trace is %s.\n"), value);
 }
 
 static void
@@ -292,7 +300,7 @@ static void
 cuda_show_debug_libcudbg (struct ui_file *file, int from_tty,
                           struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("CUDA libcudbg debug trace is %s.\n"), value);
+  gdb_printf (file, _("CUDA libcudbg debug trace is %s.\n"), value);
 }
 
 static void
@@ -329,7 +337,7 @@ static void
 cuda_show_debug_convenience_vars (struct ui_file *file, int from_tty,
                                   struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("The following extra convenience variables are used: %s.\n"),
+  gdb_printf (file, _("The following extra convenience variables are used: %s.\n"),
                     value && *value != 0 ? value : "none");
 }
 
@@ -394,7 +402,7 @@ static void
 cuda_show_debug_strict (struct ui_file *file, int from_tty,
                         struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("The debugger strict execution mode is %s.\n"), value);
+  gdb_printf (file, _("The debugger strict execution mode is %s.\n"), value);
 }
 
 static void
@@ -435,7 +443,7 @@ static void
 cuda_show_cuda_memcheck (struct ui_file *file, int from_tty,
                          struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Support for CUDA Memory Checker has been removed. Use the standalone Compute Sanitizer instead.\n"));
+  gdb_printf (file, _("Support for CUDA Memory Checker has been removed. Use the standalone Compute Sanitizer instead.\n"));
 }
 
 static void
@@ -460,7 +468,7 @@ static enum auto_boolean cuda_coalescing = AUTO_BOOLEAN_AUTO;
 static void
 cuda_set_coalescing (const char *args, int from_tty, struct cmd_list_element *c)
 {
-  printf_filtered ("Coalescing of the CUDA commands output is %s.\n",
+  gdb_printf ("Coalescing of the CUDA commands output is %s.\n",
                    cuda_options_coalescing () ? "on" : "off");
 }
 
@@ -468,7 +476,7 @@ static void
 cuda_show_coalescing (struct ui_file *file, int from_tty,
                       struct cmd_list_element *c, const char *value)
 {
-  printf_filtered ("Coalescing of the CUDA commands output is %s.\n",
+  gdb_printf ("Coalescing of the CUDA commands output is %s.\n",
                    cuda_options_coalescing () ? "on" : "off");
 }
 
@@ -508,7 +516,7 @@ static void
 cuda_show_notify (struct ui_file *file, int from_tty,
                   struct cmd_list_element *c, const char *value)
 {
-  printf_filtered ("CUDA notifications will be sent by default to thread: %s.\n", value);
+  gdb_printf ("CUDA notifications will be sent by default to thread: %s.\n", value);
 }
 
 static void
@@ -566,7 +574,7 @@ static void
 cuda_show_break_on_launch (struct ui_file *file, int from_tty,
                            struct cmd_list_element *c, const char *value)
 {
-  printf_filtered ("Break on every kernel launch is set to '%s'.\n", value);
+  gdb_printf ("Break on every kernel launch is set to '%s'.\n", value);
 }
 
 static void
@@ -634,13 +642,13 @@ static void
 cuda_show_disassemble_from (struct ui_file *file, int from_tty,
                             struct cmd_list_element *c, const char *value)
 {
-  printf_filtered ("CUDA code is disassembled from %s.\n", value);
+  gdb_printf ("CUDA code is disassembled from %s.\n", value);
 }
 
 static void
 cuda_set_disassemble_from (const char *args, int from_tty, struct cmd_list_element *c)
 {
-  cuda_system_flush_disasm_cache ();
+  cuda_state::flush_disasm_caches ();
 }
 
 static void
@@ -688,7 +696,7 @@ static void
 cuda_show_disassemble_per (struct ui_file *file, int from_tty,
                           struct cmd_list_element *c, const char *value)
 {
-  printf_filtered ("CUDA code is disassembled per %s.\n", value);
+  gdb_printf ("CUDA code is disassembled per %s.\n", value);
 }
 
 static void
@@ -739,7 +747,7 @@ static void
 cuda_show_hide_internal_frames (struct ui_file *file, int from_tty,
                                 struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Hiding of CUDA internal frames is %s.\n"), value);
+  gdb_printf (file, _("Hiding of CUDA internal frames is %s.\n"), value);
 }
 
 static void
@@ -783,7 +791,7 @@ static void
 cuda_show_show_kernel_events (struct ui_file *file, int from_tty,
                               struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Show CUDA kernel events is set to '%s'.\n"), value);
+  gdb_printf (file, _("Show CUDA kernel events is set to '%s'.\n"), value);
 }
 
 void
@@ -794,9 +802,9 @@ cuda_options_force_set_launch_notification_update (void)
   if ((cuda_options_auto_breakpoints_needed () ||
        (cuda_show_kernel_events != cuda_show_kernel_events_none)) &&
       (!cuda_options_auto_breakpoints_forced_needed ()))
-    cuda_api_set_kernel_launch_notification_mode (CUDBG_KNL_LAUNCH_NOTIFY_EVENT);
+    cuda_debugapi::set_kernel_launch_notification_mode (CUDBG_KNL_LAUNCH_NOTIFY_EVENT);
   else
-    cuda_api_set_kernel_launch_notification_mode (CUDBG_KNL_LAUNCH_NOTIFY_DEFER);
+    cuda_debugapi::set_kernel_launch_notification_mode (CUDBG_KNL_LAUNCH_NOTIFY_DEFER);
 }
 
 static void
@@ -812,7 +820,7 @@ static void
 cuda_show_show_kernel_events_depth (struct ui_file *file, int from_tty,
                               struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Show CUDA kernel events depth is set to %s.\n"), value);
+  gdb_printf (file, _("Show CUDA kernel events depth is set to %s.\n"), value);
 }
 
 static void
@@ -888,7 +896,7 @@ static void
 cuda_show_show_context_events (struct ui_file *file, int from_tty,
                                struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Show CUDA context events is %s.\n"), value);
+  gdb_printf (file, _("Show CUDA context events is %s.\n"), value);
 }
 
 static void
@@ -918,9 +926,9 @@ static void
 cuda_set_launch_blocking (const char *args, int from_tty, struct cmd_list_element *c)
 {
   if (cuda_launch_blocking)
-      printf_filtered ("On the next run, the CUDA kernel launches will be blocking.\n");
+      gdb_printf ("On the next run, the CUDA kernel launches will be blocking.\n");
   else
-      printf_filtered ("On the next run, the CUDA kernel launches will be non-blocking.\n");
+      gdb_printf ("On the next run, the CUDA kernel launches will be non-blocking.\n");
 }
 
 static void
@@ -928,9 +936,9 @@ cuda_show_launch_blocking (struct ui_file *file, int from_tty,
                            struct cmd_list_element *c, const char *value)
 {
   if (cuda_launch_blocking)
-    fprintf_filtered (file, _("On the next run, the CUDA kernel launches will be blocking.\n"));
+    gdb_printf (file, _("On the next run, the CUDA kernel launches will be blocking.\n"));
   else
-    fprintf_filtered (file, _("On the next run, the CUDA kernel launches will be non-blocking.\n"));
+    gdb_printf (file, _("On the next run, the CUDA kernel launches will be non-blocking.\n"));
 }
 
 static void
@@ -967,7 +975,7 @@ static void
 show_cuda_thread_selection_policy (struct ui_file *file, int from_tty,
                                    struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("CUDA thread selection policy is %s.\n"), value);
+  gdb_printf (file, _("CUDA thread selection policy is %s.\n"), value);
 }
 
 static void
@@ -999,7 +1007,7 @@ static void
 show_cuda_copyright_command (const char *ignore, int from_tty)
 {
   print_gdb_version (gdb_stdout, false);
-  printf_filtered ("\n");
+  gdb_printf ("\n");
 }
 
 static void
@@ -1034,7 +1042,7 @@ static void
 cuda_show_api_failures (struct ui_file *file, int from_tty,
                           struct cmd_list_element *c, const char *value)
 {
-  printf_filtered ("api_failures is set to '%s'.\n", value);
+  gdb_printf ("api_failures is set to '%s'.\n", value);
 }
 
 static void
@@ -1096,7 +1104,7 @@ static void
 cuda_show_cuda_software_preemption (struct ui_file *file, int from_tty,
                          struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Software preemption debugging is %s.\n"), value);
+  gdb_printf (file, _("Software preemption debugging is %s.\n"), value);
 }
 
 static void
@@ -1133,7 +1141,7 @@ static void
 cuda_show_cuda_variable_value_cache_enabled (struct ui_file *file, int from_tty,
                          struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Variable value cache enabled is %s.\n"), value);
+  gdb_printf (file, _("Variable value cache enabled is %s.\n"), value);
 }
 
 static void
@@ -1217,7 +1225,7 @@ static void
 cuda_show_cuda_gpu_collect_stats (struct ui_file *file, int from_tty,
                          struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("CUDA debugger API statistics collection is %s.\n"), value);
+  gdb_printf (file, _("CUDA debugger API statistics collection is %s.\n"), value);
 }
 
 
@@ -1251,7 +1259,7 @@ static void
 cuda_show_cuda_value_extrapolation (struct ui_file *file, int from_tty,
                          struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("CUDA value extrapolation is %s.\n"), value);
+  gdb_printf (file, _("CUDA value extrapolation is %s.\n"), value);
 }
 
 bool
@@ -1300,7 +1308,7 @@ static void
 cuda_show_single_stepping_optimizations (struct ui_file *file, int from_tty,
                          struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("CUDA single stepping optimizations are %s.\n"), value);
+  gdb_printf (file, _("CUDA single stepping optimizations are %s.\n"), value);
 }
 
 bool
@@ -1339,7 +1347,7 @@ static void
 cuda_show_stop_signal (struct ui_file *file, int from_tty,
                        struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("CUDA stop signal is %s\n"), value);
+  gdb_printf (file, _("CUDA stop signal is %s\n"), value);
 }
 
 static void
@@ -1380,7 +1388,7 @@ static void
 cuda_show_no_device_resume_on_cpu_dynamic_function_call (struct ui_file *file, int from_tty,
                                                          struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("CUDA device resume during dynamic function call on host is %s.\n"), value);
+  gdb_printf (file, _("CUDA device resume during dynamic function call on host is %s.\n"), value);
 }
 
 static void
