@@ -2,16 +2,16 @@
  * NVIDIA CUDA Debugger CUDA-GDB
  * Copyright (C) 2007-2023 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
  * published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
@@ -22,45 +22,46 @@
 #include "server.h"
 #else
 #include "defs.h"
-#include "inferior.h"
-#include "gdb/signals.h"
-#include "stack.h"
-#include "gdbthread.h"
+
 #include "cuda-options.h"
-#include "objfiles.h"
 #include "exceptions.h"
+#include "gdb/signals.h"
+#include "gdbthread.h"
+#include "inferior.h"
+#include "objfiles.h"
+#include "stack.h"
 #include "utils.h"
 #endif
 
 #include <ctype.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/param.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 #ifdef __QNXHOST__
-# include <ftw.h>
+#include <ftw.h>
 #else
-# include <dirent.h>
+#include <dirent.h>
 #endif
-#include <unistd.h>
 #include <string.h>
+#include <unistd.h>
 
-#include <fcntl.h>
-#include "cuda-utils.h"
 #include "cuda-defs.h"
+#include "cuda-utils.h"
+#include <fcntl.h>
 
-#define RECORD_FORMAT_MASTER   "LOCK:%10d\n"
-#define RECORD_FORMAT_DEVICE    "%4d:%10d\n"
-#define RECORD_SIZE                       16
-#define RECORD_MASTER                      0
-#define RECORD_DEVICE(i)           ((i) + 1)
-#define DEVICE_RECORD(i)           ((i) - 1)
+#define RECORD_FORMAT_MASTER "LOCK:%10d\n"
+#define RECORD_FORMAT_DEVICE "%4d:%10d\n"
+#define RECORD_SIZE 16
+#define RECORD_MASTER 0
+#define RECORD_DEVICE(i) ((i) + 1)
+#define DEVICE_RECORD(i) ((i)-1)
 
 bool cuda_use_lockfile = false;
 
 static const char cuda_gdb_lock_file[] = "cuda-gdb.lock";
-static char cuda_gdb_tmp_basedir[CUDA_GDB_TMP_BUF_SIZE] = {0};
+static char cuda_gdb_tmp_basedir[CUDA_GDB_TMP_BUF_SIZE] = { 0 };
 static int cuda_gdb_lock_fd = -1;
-static char* cuda_gdb_tmp_dir = NULL;
+static char *cuda_gdb_tmp_dir = NULL;
 static uint64_t dev_mask = 0;
 
 int
@@ -106,11 +107,10 @@ cuda_gdb_tmpdir_create_basedir (void)
     snprintf (cuda_gdb_tmp_basedir, sizeof (cuda_gdb_tmp_basedir),
               "/tmp/cuda-dbg");
 
-  ret = cuda_gdb_dir_create (cuda_gdb_tmp_basedir,
-                             S_IRWXU | S_IRWXG | S_IRWXO,
+  ret = cuda_gdb_dir_create (cuda_gdb_tmp_basedir, S_IRWXU | S_IRWXG | S_IRWXO,
                              override_umask, &dir_exists);
   if (ret)
-    error (_("Error creating temporary directory %s\n"),
+    error (_ ("Error creating temporary directory %s\n"),
            cuda_gdb_tmp_basedir);
 }
 
@@ -125,7 +125,8 @@ cuda_gdb_get_tmp_basedir (void)
 
 #ifdef __QNXHOST__
 static int
-fn_cleanup_dir (const char *path, const struct stat *stat, int flag, struct FTW *ftw)
+fn_cleanup_dir (const char *path, const struct stat *stat, int flag,
+                struct FTW *ftw)
 {
   switch (flag)
     {
@@ -152,28 +153,28 @@ cuda_gdb_tmpdir_cleanup_dir (const char *dirpath)
   nftw (dirpath, fn_cleanup_dir, maxDepth, FTW_DEPTH);
 }
 
-#else /* __QNXHOST__*/
+#else  /* __QNXHOST__*/
 
 void
 cuda_gdb_dir_cleanup_files (const char *dirpath)
 {
   char path[CUDA_GDB_TMP_BUF_SIZE];
-  DIR* dir = opendir (dirpath);
-  struct dirent* dir_ent = NULL;
+  DIR *dir = opendir (dirpath);
+  struct dirent *dir_ent = NULL;
 
   if (!dir)
     return;
 
   while ((dir_ent = readdir (dir)))
     {
-      if (!strcmp(dir_ent->d_name,".") ||
-          !strcmp(dir_ent->d_name, ".."))
+      if (!strcmp (dir_ent->d_name, ".") || !strcmp (dir_ent->d_name, ".."))
         continue;
       snprintf (path, sizeof (path), "%s/%s", dirpath, dir_ent->d_name);
-      if (dir_ent->d_type == DT_DIR) {
-        cuda_gdb_dir_cleanup_files (path);
-        rmdir (path);
-      }
+      if (dir_ent->d_type == DT_DIR)
+        {
+          cuda_gdb_dir_cleanup_files (path);
+          rmdir (path);
+        }
       else
         unlink (path);
     };
@@ -207,8 +208,7 @@ cuda_gdb_record_write (int record_idx, int pid)
   int res;
 
   if (record_idx == 0)
-    snprintf (record, CUDA_GDB_TMP_BUF_SIZE, RECORD_FORMAT_MASTER,
-              pid);
+    snprintf (record, CUDA_GDB_TMP_BUF_SIZE, RECORD_FORMAT_MASTER, pid);
   else
     snprintf (record, CUDA_GDB_TMP_BUF_SIZE, RECORD_FORMAT_DEVICE,
               DEVICE_RECORD (record_idx), pid);
@@ -226,7 +226,7 @@ static int
 cuda_gdb_record_read (int record_idx)
 {
 
-  char record[CUDA_GDB_TMP_BUF_SIZE] = {0};
+  char record[CUDA_GDB_TMP_BUF_SIZE] = { 0 };
   int res;
   char *colon = NULL;
   int rc = -1;
@@ -243,21 +243,22 @@ cuda_gdb_record_read (int record_idx)
   if (!colon || colon[1] == 0)
     return -1;
 
-  if (sscanf(colon+1, "%d", &rc) != 1)
+  if (sscanf (colon + 1, "%d", &rc) != 1)
     return -1;
 
   return rc;
 }
 
-/* Returns true if lock was acquired and false if user decided not to acquire further locks */
+/* Returns true if lock was acquired and false if user decided not to acquire
+ * further locks */
 static bool
 cuda_gdb_record_set_lock (int record_idx, bool enable_lock)
 {
-  struct flock lock = {0};
+  struct flock lock = { 0 };
   int e = 0;
   int pid = -1;
 
-  lock.l_type = enable_lock ? F_WRLCK: F_UNLCK;
+  lock.l_type = enable_lock ? F_WRLCK : F_UNLCK;
   lock.l_whence = SEEK_SET;
   lock.l_start = record_idx * RECORD_SIZE;
   lock.l_len = RECORD_SIZE;
@@ -270,7 +271,8 @@ cuda_gdb_record_set_lock (int record_idx, bool enable_lock)
 
   /* Raise an error if received an unexpected errno code */
   if (errno != EACCES && errno != EAGAIN)
-    error (_("Internal error with the cuda-gdb lock file (errno=%d).\n"), errno);
+    error (_ ("Internal error with the cuda-gdb lock file (errno=%d).\n"),
+           errno);
 
   /* Ask the user if he want to continue */
   pid = cuda_gdb_record_read (record_idx);
@@ -278,7 +280,8 @@ cuda_gdb_record_set_lock (int record_idx, bool enable_lock)
   current_inferior ()->top_target ()->terminal_ours ();
   if (nquery ("cuda-gdb failed to grab the lock file %s/%s.\n"
               "Another CUDA debug session (pid %d) could be in progress.\n"
-              "Are you sure you want to continue? ", cuda_gdb_get_tmp_basedir (), cuda_gdb_lock_file, pid))
+              "Are you sure you want to continue? ",
+              cuda_gdb_get_tmp_basedir (), cuda_gdb_lock_file, pid))
     {
       current_inferior ()->top_target ()->terminal_inferior ();
       return false;
@@ -286,13 +289,16 @@ cuda_gdb_record_set_lock (int record_idx, bool enable_lock)
 #endif
 
   if (record_idx != RECORD_MASTER)
-    error (_("An instance of cuda-gdb(pid %d) is already using device %d.\n"
-             "If you believe you are seeing this message in error, try "
-             "deleting %s/%s.\n"), pid, DEVICE_RECORD (record_idx),
-           cuda_gdb_get_tmp_basedir (), cuda_gdb_lock_file);
+    error (_ ("An instance of cuda-gdb(pid %d) is already using device %d.\n"
+              "If you believe you are seeing this message in error, try "
+              "deleting %s/%s.\n"),
+           pid, DEVICE_RECORD (record_idx), cuda_gdb_get_tmp_basedir (),
+           cuda_gdb_lock_file);
   else
-    error (_("Another cuda-gdb instance is working with the lock file. Try again.\n"
-             "If you believe you are seeing this message in error, try deleting %s/%s.\n"),
+    error (_ ("Another cuda-gdb instance is working with the lock file. Try "
+              "again.\n"
+              "If you believe you are seeing this message in error, try "
+              "deleting %s/%s.\n"),
            cuda_gdb_get_tmp_basedir (), cuda_gdb_lock_file);
   return false;
 }
@@ -304,7 +310,7 @@ cuda_gdb_lock_file_initialize (void)
 
   for (i = 0; i < CUDBG_MAX_DEVICES; i++)
     {
-      cuda_gdb_record_write (RECORD_DEVICE(i), 0);
+      cuda_gdb_record_write (RECORD_DEVICE (i), 0);
     }
 }
 
@@ -317,8 +323,8 @@ cuda_gdb_record_remove_all (void *unused)
     {
       if (dev_mask & (1 << i))
         {
-          cuda_gdb_record_write (RECORD_DEVICE(i), 0);
-          cuda_gdb_record_set_lock (RECORD_DEVICE(i), false);
+          cuda_gdb_record_write (RECORD_DEVICE (i), 0);
+          cuda_gdb_record_set_lock (RECORD_DEVICE (i), false);
           dev_mask &= ~(1 << i);
         }
     }
@@ -343,33 +349,32 @@ cuda_gdb_lock_file_create (void)
   bool initialize_lock_file = false;
   bool grab_lock = true;
   mode_t old_umask;
-  int my_pid = (int) getpid();
+  int my_pid = (int)getpid ();
 
   /* Default == true, can be overriden via a command-line option */
   if (!cuda_use_lockfile)
     return;
 
-  snprintf (buf, sizeof (buf), "%s/%s",
-              cuda_gdb_get_tmp_basedir (), cuda_gdb_lock_file);
+  snprintf (buf, sizeof (buf), "%s/%s", cuda_gdb_get_tmp_basedir (),
+            cuda_gdb_lock_file);
 
   visible_devices = getenv ("CUDA_VISIBLE_DEVICES");
 
-  if (stat (buf, &st) || !(S_ISREG(st.st_mode)))
+  if (stat (buf, &st) || !(S_ISREG (st.st_mode)))
     initialize_lock_file = true;
 
   /* Save the old umask and reset it */
   old_umask = umask (0);
-  cuda_gdb_lock_fd = open (buf, O_CREAT | O_RDWR,
-                           S_IRWXU | S_IRWXG | S_IRWXO);
+  cuda_gdb_lock_fd = open (buf, O_CREAT | O_RDWR, S_IRWXU | S_IRWXG | S_IRWXO);
   /* Restore the old umask */
   umask (old_umask);
 
   if (cuda_gdb_lock_fd == -1)
-    error (_("Cannot open %s. \n"), buf);
+    error (_ ("Cannot open %s. \n"), buf);
 
-  /* Register cleanup routine */
-  /* No final cleanup chain at server side,
-     cleanup function is called explicitly when server quits */
+    /* Register cleanup routine */
+    /* No final cleanup chain at server side,
+       cleanup function is called explicitly when server quits */
 #ifndef GDBSERVER
   make_final_cleanup (cuda_gdb_record_remove_all, NULL);
 #endif
@@ -389,9 +394,10 @@ cuda_gdb_lock_file_create (void)
       /* Lock all devices */
       for (i = 0; i < CUDBG_MAX_DEVICES; i++)
         {
-          grab_lock = cuda_gdb_record_set_lock (RECORD_DEVICE(i), true);
-          if (!grab_lock) break;
-          cuda_gdb_record_write (RECORD_DEVICE(i), my_pid);
+          grab_lock = cuda_gdb_record_set_lock (RECORD_DEVICE (i), true);
+          if (!grab_lock)
+            break;
+          cuda_gdb_record_write (RECORD_DEVICE (i), my_pid);
           dev_mask |= 1 << i;
         }
     }
@@ -407,28 +413,31 @@ cuda_gdb_lock_file_create (void)
           if (*visible_devices == ',')
             visible_devices++;
 
-          if ((sscanf (visible_devices, "%u,", &dev_id) > 0) &&
-              (++num_devices < CUDBG_MAX_DEVICES) &&
-              (dev_id < CUDBG_MAX_DEVICES))
+          if ((sscanf (visible_devices, "%u,", &dev_id) > 0)
+              && (++num_devices < CUDBG_MAX_DEVICES)
+              && (dev_id < CUDBG_MAX_DEVICES))
             {
-              grab_lock = cuda_gdb_record_set_lock (RECORD_DEVICE(dev_id), true);
-              if (!grab_lock) break;
-              cuda_gdb_record_write (RECORD_DEVICE(dev_id), my_pid);
+              grab_lock
+                  = cuda_gdb_record_set_lock (RECORD_DEVICE (dev_id), true);
+              if (!grab_lock)
+                break;
+              cuda_gdb_record_write (RECORD_DEVICE (dev_id), my_pid);
               dev_mask |= 1 << dev_id;
             }
           else
             break;
-        } while ((visible_devices = strstr (visible_devices, ",")));
+        }
+      while ((visible_devices = strstr (visible_devices, ",")));
     }
 
-    cuda_gdb_record_write (RECORD_MASTER, 0);
-    cuda_gdb_record_set_lock (RECORD_MASTER, false);
+  cuda_gdb_record_write (RECORD_MASTER, 0);
+  cuda_gdb_record_set_lock (RECORD_MASTER, false);
 }
 
 static void
 cuda_gdb_tmpdir_setup (void)
 {
-  char dirpath [CUDA_GDB_TMP_BUF_SIZE];
+  char dirpath[CUDA_GDB_TMP_BUF_SIZE];
   int ret;
   bool dir_exists = false;
   bool override_umask = false;
@@ -437,19 +446,20 @@ cuda_gdb_tmpdir_setup (void)
             getpid ());
 
 #ifdef __QNXHOST__
-  cuda_gdb_tmpdir_cleanup_dir (dirpath); /* Try to remove the dir if it exists */
-#endif /* __QNXHOST__ */
-  ret = cuda_gdb_dir_create (dirpath, S_IRWXU | S_IRWXG | S_IXOTH, override_umask,
-                             &dir_exists);
+  cuda_gdb_tmpdir_cleanup_dir (
+      dirpath); /* Try to remove the dir if it exists */
+#endif          /* __QNXHOST__ */
+  ret = cuda_gdb_dir_create (dirpath, S_IRWXU | S_IRWXG | S_IXOTH,
+                             override_umask, &dir_exists);
   if (ret)
-    error (_("Error creating temporary directory %s\n"), dirpath);
+    error (_ ("Error creating temporary directory %s\n"), dirpath);
 
 #ifndef __QNXHOST__
   if (dir_exists)
     cuda_gdb_dir_cleanup_files (dirpath);
 #endif /* __QNXHOST__ */
 
-  cuda_gdb_tmp_dir = (char *) xmalloc (strlen (dirpath) + 1);
+  cuda_gdb_tmp_dir = (char *)xmalloc (strlen (dirpath) + 1);
   strncpy (cuda_gdb_tmp_dir, dirpath, strlen (dirpath) + 1);
 
   /* No final cleanup chain at server side,
@@ -459,7 +469,7 @@ cuda_gdb_tmpdir_setup (void)
 #endif
 }
 
-const char*
+const char *
 cuda_gdb_tmpdir_getdir (void)
 {
   if (!cuda_gdb_tmp_dir)
@@ -467,7 +477,8 @@ cuda_gdb_tmpdir_getdir (void)
   return cuda_gdb_tmp_dir;
 }
 
-static cuda_clock_t cuda_clock_ = 0;
+// Reserve 0 as a starting point for timestamp comparisons
+static cuda_clock_t cuda_clock_ = 1;
 
 cuda_clock_t
 cuda_clock (void)
@@ -480,7 +491,8 @@ cuda_clock_increment (void)
 {
   ++cuda_clock_;
   if (cuda_clock_ == 0)
-    warning (_("The internal clock counter used for cuda debugging wrapped around.\n"));
+    warning (_ ("The internal clock counter used for cuda debugging wrapped "
+                "around.\n"));
 }
 
 #ifndef GDBSERVER
@@ -488,15 +500,16 @@ static unsigned char *
 cuda_nat_save_gdb_signal_handlers (void)
 {
   unsigned char *sigs;
-  int i,j;
-  static int (*sighand_savers[])(int) =
-    {signal_stop_state, signal_print_state, signal_pass_state};
+  int i, j;
+  static int (*sighand_savers[]) (int)
+      = { signal_stop_state, signal_print_state, signal_pass_state };
 
-  sigs = (unsigned char *) xmalloc (GDB_SIGNAL_LAST*ARRAY_SIZE(sighand_savers));
+  sigs = (unsigned char *)xmalloc (GDB_SIGNAL_LAST
+                                   * ARRAY_SIZE (sighand_savers));
 
-  for (i=0; i < ARRAY_SIZE(sighand_savers); i++)
-    for (j=0; j < GDB_SIGNAL_LAST; j++)
-      sigs[i*GDB_SIGNAL_LAST+j] = sighand_savers[i](j);
+  for (i = 0; i < ARRAY_SIZE (sighand_savers); i++)
+    for (j = 0; j < GDB_SIGNAL_LAST; j++)
+      sigs[i * GDB_SIGNAL_LAST + j] = sighand_savers[i](j);
 
   return sigs;
 }
@@ -504,16 +517,17 @@ cuda_nat_save_gdb_signal_handlers (void)
 static void
 cuda_nat_restore_gdb_signal_handlers (unsigned char *sigs)
 {
-  int i,j;
-  static int (*sighand_updaters[])(int,int) =
-    {signal_stop_update, signal_print_update, signal_pass_update};
+  int i, j;
+  static int (*sighand_updaters[]) (int, int)
+      = { signal_stop_update, signal_print_update, signal_pass_update };
 
-  for (i=0; i < ARRAY_SIZE(sighand_updaters); i++)
-    for (j=0; j < GDB_SIGNAL_LAST; j++)
-      sighand_updaters[i] (j, sigs[i*GDB_SIGNAL_LAST+j]);
+  for (i = 0; i < ARRAY_SIZE (sighand_updaters); i++)
+    for (j = 0; j < GDB_SIGNAL_LAST; j++)
+      sighand_updaters[i](j, sigs[i * GDB_SIGNAL_LAST + j]);
 }
 
-void cuda_nat_bypass_signals_cleanup (unsigned char *sigs)
+void
+cuda_nat_bypass_signals_cleanup (unsigned char *sigs)
 {
   cuda_nat_restore_gdb_signal_handlers (sigs);
   xfree (sigs);
@@ -523,18 +537,16 @@ unsigned char *
 cuda_gdb_bypass_signals (void)
 {
   unsigned char *sigs;
-  unsigned cuda_stop_signal = cuda_options_stop_signal();
+  unsigned cuda_stop_signal = cuda_options_stop_signal ();
   int i;
 
   sigs = cuda_nat_save_gdb_signal_handlers ();
-  for (i=0;i< GDB_SIGNAL_LAST; i++)
+  for (i = 0; i < GDB_SIGNAL_LAST; i++)
     {
-      if ( i == cuda_stop_signal ||
-           i == GDB_SIGNAL_TRAP  ||
-           i == GDB_SIGNAL_KILL  ||
-           i == GDB_SIGNAL_STOP  ||
-           i == GDB_SIGNAL_CHLD  ||
-           i >= GDB_SIGNAL_CUDA_UNKNOWN_EXCEPTION ) continue;
+      if (i == cuda_stop_signal || i == GDB_SIGNAL_TRAP || i == GDB_SIGNAL_KILL
+          || i == GDB_SIGNAL_STOP || i == GDB_SIGNAL_CHLD
+          || i >= GDB_SIGNAL_CUDA_UNKNOWN_EXCEPTION)
+        continue;
       signal_stop_update (i, 0);
       signal_pass_update (i, 1);
       signal_print_update (i, 1);
@@ -544,9 +556,10 @@ cuda_gdb_bypass_signals (void)
 }
 
 /* CUDA PTX registers cache */
-struct cuda_ptx_cache_element {
+struct cuda_ptx_cache_element
+{
   struct frame_id frame_id;
-  cuda_coords_t coords;
+  cuda_coords coords;
   int dwarf_regnum;
   char data[16];
   int len;
@@ -555,44 +568,52 @@ typedef struct cuda_ptx_cache_element cuda_ptx_cache_element_t;
 
 static std::vector<cuda_ptx_cache_element_t> cuda_ptx_register_cache;
 
-/* Searches for cache entry containing given dwarf register for a given frame */
+/* Searches for cache entry containing given dwarf register for a given frame
+ */
 static std::vector<cuda_ptx_cache_element_t>::iterator
 cuda_ptx_cache_find_element (struct frame_id frame_id, int dwarf_regnum)
 {
-  cuda_coords_t coords;
 
-  if (cuda_coords_get_current (&coords)) return cuda_ptx_register_cache.end ();
+  if (!cuda_current_focus::isDevice ())
+    return cuda_ptx_register_cache.end ();
 
-  for (auto it = cuda_ptx_register_cache.begin (); it != cuda_ptx_register_cache.end (); ++it)
-    if (cuda_coords_equal (&coords, &it->coords) &&
-	frame_id_eq (frame_id, it->frame_id) &&
-	it->dwarf_regnum == dwarf_regnum) return it;
+  const auto &coords = cuda_current_focus::get ();
+
+  for (auto it = cuda_ptx_register_cache.begin ();
+       it != cuda_ptx_register_cache.end (); ++it)
+    if ((coords == it->coords) && (frame_id == it->frame_id)
+        && (dwarf_regnum == it->dwarf_regnum))
+      return it;
 
   return cuda_ptx_register_cache.end ();
 }
 
-
 /**
  *  Adds PTX register to a cache.
- *  PTX register cache is considered valid only for given lane within given frame
+ *  PTX register cache is considered valid only for given lane within given
+ * frame
  */
 void
-cuda_ptx_cache_store_register (struct frame_info *frame, int dwarf_regnum, struct value *value)
+cuda_ptx_cache_store_register (frame_info_ptr frame, int dwarf_regnum,
+                               struct value *value)
 {
   struct cuda_ptx_cache_element new_elem;
 
   /* If focus is not on device - return */
-  if (cuda_coords_get_current (&new_elem.coords))
+  if (!cuda_current_focus::isDevice ())
     return;
 
+  /* Store the current focus */
+  new_elem.coords = cuda_current_focus::get ();
+
   /* Store information about cached register in temporary cache element*/
   new_elem.frame_id = get_frame_id (frame);
   new_elem.dwarf_regnum = dwarf_regnum;
-  new_elem.len = TYPE_LENGTH(value_type(value));
+  new_elem.len = value_type (value)->length ();
   /* If element can not be cached - return */
-  if (new_elem.len > sizeof(new_elem.data))
+  if (new_elem.len > sizeof (new_elem.data))
     return;
-  memcpy (new_elem.data, value_contents_raw(value).data (), new_elem.len);
+  memcpy (new_elem.data, value_contents_raw (value).data (), new_elem.len);
 
   auto elem = cuda_ptx_cache_find_element (new_elem.frame_id, dwarf_regnum);
   if (elem != cuda_ptx_register_cache.end ())
@@ -613,33 +634,37 @@ cuda_ptx_cache_store_register (struct frame_info *frame, int dwarf_regnum, struc
  */
 
 struct value *
-cuda_ptx_cache_get_register (struct frame_info *frame, int dwarf_regnum, struct type *type)
+cuda_ptx_cache_get_register (frame_info_ptr frame, int dwarf_regnum,
+                             struct type *type)
 {
   struct value *retval;
 
   retval = allocate_value (type);
 
   auto elem = cuda_ptx_cache_find_element (get_frame_id (frame), dwarf_regnum);
-  if (elem == cuda_ptx_register_cache.end () || elem->len != TYPE_LENGTH(type) ||
-      !cuda_options_variable_value_cache_enabled ())
+  if ((elem == cuda_ptx_register_cache.end ()) || (elem->len != type->length ())
+      || !cuda_options_variable_value_cache_enabled ())
     {
       VALUE_LVAL (retval) = not_lval;
-      mark_value_bytes_optimized_out (retval, 0, TYPE_LENGTH (type));
+      mark_value_bytes_optimized_out (retval, 0, type->length ());
       return retval;
     }
 
   mark_value_bytes_optimized_out (retval, 0, 0);
   set_value_cached (retval, 1);
-  memcpy (value_contents_raw(retval).data (), elem->data, elem->len);
+  memcpy (value_contents_raw (retval).data (), elem->data, elem->len);
   return retval;
 }
 
 struct cuda_ptx_cache_update_local_vars_data
 {
-  struct frame_info *m_frame;
+  frame_info_ptr m_frame;
 
-  void operator() (const char *print_name, struct symbol *sym)
-  { cuda_ptx_cache_local_vars_iterator (print_name, sym, m_frame);  }
+  void
+  operator() (const char *print_name, struct symbol *sym)
+  {
+    cuda_ptx_cache_local_vars_iterator (print_name, sym, m_frame);
+  }
 };
 
 static void
@@ -648,12 +673,18 @@ cuda_ptx_cache_update_local_vars (void)
   struct cuda_ptx_cache_update_local_vars_data cb_data;
   const struct block *block = NULL;
 
-  if (!cuda_focus_is_device() || find_thread_ptid (current_inferior ()->process_target (), inferior_ptid)->executing ()) return;
-  cb_data.m_frame = get_current_frame();
-  if (!cb_data.m_frame) return;
+  if (!cuda_current_focus::isDevice ()
+      || find_thread_ptid (current_inferior ()->process_target (),
+                           inferior_ptid)
+             ->executing ())
+    return;
+  cb_data.m_frame = get_current_frame ();
+  if (!cb_data.m_frame)
+    return;
 
   block = get_frame_block (cb_data.m_frame, 0);
-  if (!block) return;
+  if (!block)
+    return;
 
   iterate_over_block_local_vars (block, cb_data);
 }
@@ -666,45 +697,49 @@ cuda_ptx_cache_update_local_vars (void)
 void
 cuda_ptx_cache_refresh (void)
 {
-  cuda_coords_t coords;
-
-  if (!cuda_ptx_register_cache.empty ())
+  if (!cuda_ptx_register_cache.empty () && cuda_current_focus::isDevice ())
     {
       auto elem = cuda_ptx_register_cache.begin ();
-     /* If focus is still on the same lane - keep the cache intact */
-     if ( (!cuda_coords_get_current (&coords) &&
-          !cuda_coords_equal (&coords, &elem->coords)) ||
-          !cuda_options_variable_value_cache_enabled ())
-       {
-	 cuda_ptx_register_cache.clear ();	       
-       }
+      const auto &coords = cuda_current_focus::get ();
+      /* If focus is still on the same lane - keep the cache intact */
+      if (coords != elem->coords
+          || !cuda_options_variable_value_cache_enabled ())
+        {
+          cuda_ptx_register_cache.clear ();
+        }
     }
 
-  if (!cuda_options_variable_value_cache_enabled()) return;
+  if (!cuda_options_variable_value_cache_enabled ())
+    return;
   cuda_ptx_cache_update_local_vars ();
 }
 
-
 bool
 cuda_managed_msymbol_p (struct bound_minimal_symbol &bmsym)
 {
   struct obj_section *section = bmsym.obj_section ();
-  struct objfile *obj= section ? section->objfile : NULL;
+  struct objfile *obj = section ? section->objfile : NULL;
   struct gdbarch *arch = obj ? obj->arch () : NULL;
 
-  if (arch == NULL) return false;
+  if (arch == NULL)
+    return false;
 
-  return gdbarch_bfd_arch_info (arch)->arch == bfd_arch_m68k ?
-  /* If this a device symbol MSYMBOL_TARGET_FLAG indicates if it is managed */
-          (MSYMBOL_TARGET_FLAG_1 (bmsym.minsym) != 0) :
-  /* Managed host symbols must be located in __nv_managed_data__ section */
-          (section->the_bfd_section != NULL &&
-          strcmp(section->the_bfd_section->name, "__nv_managed_data__") == 0 );
+  return gdbarch_bfd_arch_info (arch)->arch == bfd_arch_m68k
+             ?
+             /* If this a device symbol MSYMBOL_TARGET_FLAG indicates if it is
+                managed */
+             (bmsym.minsym->target_flag_1 () != 0)
+             :
+             /* Managed host symbols must be located in __nv_managed_data__
+                section */
+             (section->the_bfd_section != NULL
+              && strcmp (section->the_bfd_section->name, "__nv_managed_data__")
+                     == 0);
 }
 
-
 /* CUDA managed memory region list */
-typedef struct {
+typedef struct
+{
   CORE_ADDR begin;
   CORE_ADDR end;
 } memory_region_t;
@@ -723,7 +758,7 @@ cuda_managed_memory_clean_regions (void)
 void
 cuda_managed_memory_add_region (CORE_ADDR begin, CORE_ADDR end)
 {
-  memory_region_t new_reg = {begin, end};
+  memory_region_t new_reg = { begin, end };
 
   cuda_managed_memory_regions.emplace_back (std::move (new_reg));
 }
@@ -741,18 +776,22 @@ cuda_managed_memory_populate_regions (void)
   if (cuda_managed_memory_regions_populated)
     return;
 
-  do {
-    cuda_api_get_managed_memory_region_info (start_addr, regions, ARRAY_SIZE(regions), &regions_returned);
-    if (regions_returned == 0)
-      return;
-    /* Add fetched queries to the list and updated start address*/
-    for (cnt = 0; cnt < regions_returned; cnt++)
-      {
-        end = regions[cnt].startAddress + regions[cnt].size;
-        if (start_addr < end) start_addr = end;
-        cuda_managed_memory_add_region ( regions[cnt].startAddress, end);
-      }
-  } while (regions_returned == ARRAY_SIZE (regions));
+  do
+    {
+      cuda_debugapi::get_managed_memory_region_info (
+          start_addr, regions, ARRAY_SIZE (regions), &regions_returned);
+      if (regions_returned == 0)
+        return;
+      /* Add fetched queries to the list and updated start address*/
+      for (cnt = 0; cnt < regions_returned; cnt++)
+        {
+          end = regions[cnt].startAddress + regions[cnt].size;
+          if (start_addr < end)
+            start_addr = end;
+          cuda_managed_memory_add_region (regions[cnt].startAddress, end);
+        }
+    }
+  while (regions_returned == ARRAY_SIZE (regions));
   cuda_managed_memory_regions_populated = true;
 }
 
@@ -762,7 +801,8 @@ cuda_managed_address_p (CORE_ADDR addr)
   cuda_managed_memory_populate_regions ();
 
   for (auto &elem : cuda_managed_memory_regions)
-    if (elem.begin <= addr && elem.end > addr) return true;
+    if (elem.begin <= addr && elem.end > addr)
+      return true;
   return false;
 }
 
@@ -778,12 +818,13 @@ cuda_is_value_managed_pointer (struct value *value)
 
   try
     {
-      result = cuda_managed_address_p (unpack_pointer (type, value_contents_for_printing (value).data ()));
+      result = cuda_managed_address_p (
+          unpack_pointer (type, value_contents_for_printing (value).data ()));
     }
   catch (const gdb_exception_error &e)
     {
       if (e.reason != 0)
-	return false;
+        return false;
     }
 
   return result;
@@ -807,8 +848,8 @@ cuda_gdb_uid_from_pid (int pid)
     {
       if (strncmp (buffer, "Uid:\t", 5) != 0)
         continue;
-      if (sscanf (buffer+5, "%d", &uid) != 1)
-          uid = -1;
+      if (sscanf (buffer + 5, "%d", &uid) != 1)
+        uid = -1;
       break;
     }
   fclose (procfile);
@@ -837,7 +878,8 @@ cuda_utils_initialize (void)
   static bool utils_initialized = false;
 
   /* Check if cuda utils were already initialized*/
-  if (utils_initialized) return;
+  if (utils_initialized)
+    return;
 
   /* Create the base temporary directory */
   cuda_gdb_tmpdir_create_basedir ();
