@@ -68,14 +68,14 @@ void cuda_gdb_tmpdir_cleanup_self (void *unused);
 void cuda_gdb_record_remove_all (void *unused);
 
 /* CUDA register cache */
-struct frame_info;
+class frame_info_ptr;
 struct value;
 struct type;
 struct symbol;
-void cuda_ptx_cache_store_register (struct frame_info *, int dwarf_regnum, struct value *);
-struct value *cuda_ptx_cache_get_register (struct frame_info *, int dwarf_regnum, struct type *);
+void cuda_ptx_cache_store_register (frame_info_ptr, int dwarf_regnum, struct value *);
+struct value *cuda_ptx_cache_get_register (frame_info_ptr, int dwarf_regnum, struct type *);
 void cuda_ptx_cache_refresh (void);
-void cuda_ptx_cache_local_vars_iterator (const char *, struct symbol *, struct frame_info *);
+void cuda_ptx_cache_local_vars_iterator (const char *, struct symbol *, frame_info_ptr);
 
 /* CUDA - managed variables */
 struct minimal_symbol;
