@@ -26,13 +26,11 @@
 #include "defs.h"
 #endif
 
-#include "cuda.h"
 #include "cuda-version.h"
 
-static const char nvidia_copyright_currrent_year[] = "2023";
-
 /* Globals */
-cuda_api_version cuda_backend_api_version;
+
+static const char nvidia_copyright_currrent_year[] = "2023";
 
 const int cuda_major_version (void)
 {
