@@ -25,6 +25,7 @@
 #include <ansidecl.h>
 #else
 #include "defs.h"
+#include "cuda/cuda-api.h"
 #include "cuda/cuda-options.h"
 #include "cuda/cuda-tdep.h"
 #endif
@@ -44,9 +45,12 @@
 /*Forward declarations */
 static void cudbg_trace(const char *fmt, ...) ATTRIBUTE_PRINTF(1, 2);
 
+#ifdef GDBSERVER
+extern void cuda_gdbserver_set_api_version (uint32_t major, uint32_t minor, uint32_t revision);
+#endif
+
 /* Globals */
 extern CUDBGNotifyNewEventCallback cudbgDebugClientCallback;
-extern cuda_api_version cuda_backend_api_version;
 
 static CUDBGResult
 cudbgInitialize(void)
@@ -107,7 +111,11 @@ cudbgInitialize(void)
         }
     }
 
-    cuda_backend_api_version = cuda_api_version (major, minor, revision);
+#ifdef GDBSERVER
+    cuda_gdbserver_set_api_version (major, minor, revision);
+#else
+    cuda_debugapi::set_api_version (major, minor, revision);
+#endif
 
     return result;
 }
@@ -2089,16 +2097,15 @@ cudbgGetDeviceName (uint32_t dev, char *buf, uint32_t buf_size)
     return result;
 }
 
-#if CUDBG_API_VERSION_REVISION >= 132
 static CUDBGResult
-cudbgGetLoadedFunctionInfo (uint32_t dev, uint64_t handle, CUDBGLoadedFunctionInfo *info, uint32_t numEntries)
+cudbgGetLoadedFunctionInfo118 (uint32_t dev, uint64_t handle, CUDBGLoadedFunctionInfo *info, uint32_t numEntries)
 {
     char *ipc_buf;
     CUDBGResult result;
 
     CUDBG_IPC_PROFILE_START();
 
-    CUDBG_IPC_BEGIN(CUDBGAPIREQ_getLoadedFunctionInfo);
+    CUDBG_IPC_BEGIN(CUDBGAPIREQ_getLoadedFunctionInfo118);
     CUDBG_IPC_APPEND(&dev, sizeof(dev));
     CUDBG_IPC_APPEND(&handle, sizeof(handle));
     CUDBG_IPC_APPEND(&numEntries, sizeof(numEntries));
@@ -2107,21 +2114,20 @@ cudbgGetLoadedFunctionInfo (uint32_t dev, uint64_t handle, CUDBGLoadedFunctionIn
     CUDBG_IPC_RECEIVE(&result, &ipc_buf);
     CUDBG_IPC_RECEIVE_ARRAY(info, numEntries, &ipc_buf);
 
-    CUDBG_IPC_PROFILE_END(CUDBGAPIREQ_getLoadedFunctionInfo, "getLoadedFunctionInfo");
+    CUDBG_IPC_PROFILE_END(CUDBGAPIREQ_getLoadedFunctionInfo118, "getLoadedFunctionInfo118");
 
     return result;
 }
-#endif
 
 static CUDBGResult
-cudbgSingleStepWarp (uint32_t dev, uint32_t sm, uint32_t wp, uint32_t nsteps, uint64_t *warpMask)
+cudbgSingleStepWarp65 (uint32_t dev, uint32_t sm, uint32_t wp, uint32_t nsteps, uint64_t *warpMask)
 {
     char *ipc_buf;
     CUDBGResult result;
 
     CUDBG_IPC_PROFILE_START();
 
-    CUDBG_IPC_BEGIN(CUDBGAPIREQ_singleStepWarp);
+    CUDBG_IPC_BEGIN(CUDBGAPIREQ_singleStepWarp65);
     CUDBG_IPC_APPEND(&dev,sizeof(dev));
     CUDBG_IPC_APPEND(&sm,sizeof(sm));
     CUDBG_IPC_APPEND(&wp,sizeof(wp));
@@ -2131,7 +2137,7 @@ cudbgSingleStepWarp (uint32_t dev, uint32_t sm, uint32_t wp, uint32_t nsteps, ui
     CUDBG_IPC_RECEIVE(&result, &ipc_buf);
     CUDBG_IPC_RECEIVE(warpMask, &ipc_buf);
 
-    CUDBG_IPC_PROFILE_END(CUDBGAPIREQ_singleStepWarp, "singleStepWarp");
+    CUDBG_IPC_PROFILE_END(CUDBGAPIREQ_singleStepWarp65, "singleStepWarp65");
 
     return result;
 }
@@ -2259,6 +2265,149 @@ cudbgGetErrorStringEx (char *buf, uint32_t buf_size, uint32_t *msg_size)
     return result;
 }
 
+static CUDBGResult
+cudbgGetLoadedFunctionInfo (uint32_t dev, uint64_t handle, CUDBGLoadedFunctionInfo *info, uint32_t startIndex, uint32_t numEntries)
+{
+    char *ipc_buf;
+    CUDBGResult result;
+
+    CUDBG_IPC_PROFILE_START();
+
+    CUDBG_IPC_BEGIN(CUDBGAPIREQ_getLoadedFunctionInfo);
+    CUDBG_IPC_APPEND(&dev, sizeof(dev));
+    CUDBG_IPC_APPEND(&handle, sizeof(handle));
+    CUDBG_IPC_APPEND(&startIndex, sizeof(startIndex));
+    CUDBG_IPC_APPEND(&numEntries, sizeof(numEntries));
+
+    CUDBG_IPC_REQUEST((void **)&ipc_buf);
+    CUDBG_IPC_RECEIVE(&result, &ipc_buf);
+    CUDBG_IPC_RECEIVE_ARRAY(info, numEntries, &ipc_buf);
+
+    CUDBG_IPC_PROFILE_END(CUDBGAPIREQ_getLoadedFunctionInfo, "getLoadedFunctionInfo");
+
+    return result;
+}
+
+static CUDBGResult
+cudbgGetConstBankAddress123 (uint32_t dev, uint32_t sm, uint32_t wp, uint32_t bank, uint32_t offset, uint64_t* address)
+{
+    char *ipc_buf;
+    CUDBGResult result;
+
+    CUDBG_IPC_PROFILE_START();
+
+    CUDBG_IPC_BEGIN(CUDBGAPIREQ_getConstBankAddress123);
+    CUDBG_IPC_APPEND(&dev, sizeof(dev));
+    CUDBG_IPC_APPEND(&sm, sizeof(sm));
+    CUDBG_IPC_APPEND(&wp, sizeof(wp));
+    CUDBG_IPC_APPEND(&bank, sizeof(bank));
+    CUDBG_IPC_APPEND(&offset, sizeof(offset));
+
+    CUDBG_IPC_REQUEST((void **)&ipc_buf);
+    CUDBG_IPC_RECEIVE(&result, &ipc_buf);
+    CUDBG_IPC_RECEIVE(address, &ipc_buf);
+
+    CUDBG_IPC_PROFILE_END(CUDBGAPIREQ_getConstBankAddress, "getConstBankAddress123");
+
+    return result;
+}
+
+static CUDBGResult
+cudbgGetDeviceInfoSizes (uint32_t dev, CUDBGDeviceInfoSizes *sizes)
+{
+    char *ipc_buf;
+    CUDBGResult result;
+
+    CUDBG_IPC_PROFILE_START();
+
+    CUDBG_IPC_BEGIN(CUDBGAPIREQ_getDeviceInfoSizes);
+    CUDBG_IPC_APPEND(&dev, sizeof(dev));
+
+    CUDBG_IPC_REQUEST((void **)&ipc_buf);
+    CUDBG_IPC_RECEIVE(&result, &ipc_buf);
+    if (result == CUDBG_SUCCESS)
+      CUDBG_IPC_RECEIVE(sizes, &ipc_buf);
+
+    CUDBG_IPC_PROFILE_END(CUDBGAPIREQ_getDeviceInfoSizes, "getDeviceInfoSizes");
+
+    return result;
+}
+
+static CUDBGResult
+cudbgGetDeviceInfo (uint32_t dev, CUDBGDeviceInfoQueryType_t type,
+		    void *buffer, uint32_t size, uint32_t *data_length)
+{
+    char *ipc_buf;
+    CUDBGResult result;
+
+    CUDBG_IPC_PROFILE_START();
+
+    CUDBG_IPC_BEGIN(CUDBGAPIREQ_getDeviceInfo);
+    CUDBG_IPC_APPEND(&dev, sizeof(dev));
+    CUDBG_IPC_APPEND(&type, sizeof(type));
+    CUDBG_IPC_APPEND(&size, sizeof(size));
+
+    CUDBG_IPC_REQUEST((void **)&ipc_buf);
+    CUDBG_IPC_RECEIVE(&result, &ipc_buf);
+    if (result == CUDBG_SUCCESS)
+      {
+	CUDBG_IPC_RECEIVE(data_length, &ipc_buf);
+	CUDBG_IPC_RECEIVE_ARRAY((uint8_t *)buffer, *data_length, &ipc_buf);
+      }
+
+    CUDBG_IPC_PROFILE_END(CUDBGAPIREQ_getDeviceInfo, "getDeviceInfo");
+
+    return result;
+}
+
+static CUDBGResult
+cudbgGetConstBankAddress (uint32_t dev, uint64_t gridId64, uint32_t bank, uint64_t* address, uint32_t* size)
+{
+    char *ipc_buf;
+    CUDBGResult result;
+
+    CUDBG_IPC_PROFILE_START();
+
+    CUDBG_IPC_BEGIN(CUDBGAPIREQ_getConstBankAddress);
+    CUDBG_IPC_APPEND(&dev, sizeof(dev));
+    CUDBG_IPC_APPEND(&gridId64, sizeof(gridId64));
+    CUDBG_IPC_APPEND(&bank, sizeof(bank));
+
+    CUDBG_IPC_REQUEST((void **)&ipc_buf);
+    CUDBG_IPC_RECEIVE(&result, &ipc_buf);
+    CUDBG_IPC_RECEIVE(address, &ipc_buf);
+    CUDBG_IPC_RECEIVE(size, &ipc_buf);
+
+    CUDBG_IPC_PROFILE_END(CUDBGAPIREQ_getConstBankAddress, "getConstBankAddress");
+
+    return result;
+}
+
+static CUDBGResult
+cudbgSingleStepWarp (uint32_t dev, uint32_t sm, uint32_t wp, uint32_t laneHint, uint32_t nsteps, uint32_t flags, uint64_t *warpMask)
+{
+    char *ipc_buf;
+    CUDBGResult result;
+
+    CUDBG_IPC_PROFILE_START();
+
+    CUDBG_IPC_BEGIN(CUDBGAPIREQ_singleStepWarp);
+    CUDBG_IPC_APPEND(&dev,sizeof(dev));
+    CUDBG_IPC_APPEND(&sm,sizeof(sm));
+    CUDBG_IPC_APPEND(&wp,sizeof(wp));
+    CUDBG_IPC_APPEND(&laneHint,sizeof(laneHint));
+    CUDBG_IPC_APPEND(&nsteps,sizeof(nsteps));
+    CUDBG_IPC_APPEND(&flags,sizeof(flags));
+
+    CUDBG_IPC_REQUEST((void **)&ipc_buf);
+    CUDBG_IPC_RECEIVE(&result, &ipc_buf);
+    CUDBG_IPC_RECEIVE(warpMask, &ipc_buf);
+
+    CUDBG_IPC_PROFILE_END(CUDBGAPIREQ_singleStepWarp, "singleStepWarp");
+
+    return result;
+}
+
 static const struct CUDBGAPI_st cudbgCurrentApi={
     /* Initialization */
     cudbgInitialize,
@@ -2410,7 +2559,7 @@ static const struct CUDBGAPI_st cudbgCurrentApi={
     cudbgWriteCCRegister,
 
     cudbgGetDeviceName,
-    cudbgSingleStepWarp,
+    cudbgSingleStepWarp65,
 
     /* 9.0 Extensions */
     cudbgReadDeviceExceptionState,
@@ -2423,10 +2572,8 @@ static const struct CUDBGAPI_st cudbgCurrentApi={
     cudbgReadUniformPredicates,
     cudbgWriteUniformPredicates,
 
-#if CUDBG_API_VERSION_REVISION >= 132
     /* 11.8 Extensions */
-    cudbgGetLoadedFunctionInfo,
-#endif
+    cudbgGetLoadedFunctionInfo118,
 
     cudbgGetGridInfo,
     cudbgGetClusterDim,
@@ -2435,6 +2582,17 @@ static const struct CUDBGAPI_st cudbgCurrentApi={
 
     /* 12.2 Extensions */
     cudbgGetErrorStringEx,
+
+    /* 12.3 Extensions */
+    cudbgGetLoadedFunctionInfo,
+    NULL, // generateCoredump
+    cudbgGetConstBankAddress123,
+
+    /* 12.4 Extensions */
+    cudbgGetDeviceInfoSizes,
+    cudbgGetDeviceInfo,
+    cudbgGetConstBankAddress,
+    cudbgSingleStepWarp,
 };
 
 CUDBGResult
