@@ -154,7 +154,7 @@ typedef enum {
     CUDBGAPIREQ_writeCCRegister,
 
     CUDBGAPIREQ_getDeviceName,
-    CUDBGAPIREQ_singleStepWarp,
+    CUDBGAPIREQ_singleStepWarp65,
 
     /* 9.0 Extensions */
     CUDBGAPIREQ_readDeviceExceptionState,
@@ -168,7 +168,7 @@ typedef enum {
     CUDBGAPIREQ_writeUniformPredicates,
 
     /* 11.8 extensions */
-    CUDBGAPIREQ_getLoadedFunctionInfo,
+    CUDBGAPIREQ_getLoadedFunctionInfo118,
 
     /* 12.0 extensions */
     CUDBGAPIREQ_getGridInfo,
@@ -178,6 +178,17 @@ typedef enum {
 
     /* 12.2 extensions */
     CUDBGAPIREQ_getErrorStringEx,
+
+    /* 12.3 Extensions */
+    CUDBGAPIREQ_getLoadedFunctionInfo,
+    CUDBGAPIREQ_generateCoredump,
+    CUDBGAPIREQ_getConstBankAddress123,
+
+    /* 12.4 Extensions */
+    CUDBGAPIREQ_getDeviceInfoSizes,
+    CUDBGAPIREQ_getDeviceInfo,
+    CUDBGAPIREQ_getConstBankAddress,
+    CUDBGAPIREQ_singleStepWarp,
 } CUDBGAPIREQ_t;
 
 typedef enum {
