@@ -1,6 +1,6 @@
 /* DWARF 2 debugging format support for GDB.
 
-   Copyright (C) 1994-2022 Free Software Foundation, Inc.
+   Copyright (C) 1994-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -41,9 +41,10 @@ struct file_entry
 {
   file_entry () = default;
 
-  file_entry (const char *name_, dir_index d_index_,
+  file_entry (const char *name_, file_name_index index_, dir_index d_index_,
 	      unsigned int mod_time_, unsigned int length_)
     : name (name_),
+      index (index_),
       d_index (d_index_),
       mod_time (mod_time_),
       length (length_)
@@ -57,6 +58,9 @@ struct file_entry
      owned by debug_line_buffer.  */
   const char *name {};
 
+  /* The index of this file in the file table.  */
+  file_name_index index {};
+
   /* The directory index (1-based).  */
   dir_index d_index {};
 
@@ -64,9 +68,6 @@ struct file_entry
 
   unsigned int length {};
 
-  /* True if referenced by the Line Number Program.  */
-  bool included_p {};
-
   /* The associated symbol table, if any.  */
   struct symtab *symtab {};
 };
@@ -76,8 +77,18 @@ struct file_entry
    which contains the following information.  */
 struct line_header
 {
-  line_header ()
-    : offset_in_dwz {}
+  /* COMP_DIR is the value of the DW_AT_comp_dir attribute of the compilation
+     unit in the context of which we are reading this line header, or nullptr
+     if unknown or not applicable.  */
+  explicit line_header (const char *comp_dir)
+    : offset_in_dwz {}, m_comp_dir (comp_dir)
+  {}
+
+  /* This constructor should only be used to create line_header intances to do
+     hash table lookups.  */
+  line_header (sect_offset sect_off, bool offset_in_dwz)
+    : sect_off (sect_off),
+      offset_in_dwz (offset_in_dwz)
   {}
 
   /* Add an entry to the include directory table.  */
@@ -144,9 +155,7 @@ struct line_header
   /* OFFSET is for struct dwz_file associated with dwarf2_per_objfile.  */
   unsigned offset_in_dwz : 1; /* Can't initialize bitfields in-class.  */
 
-  unsigned int total_length {};
   unsigned short version {};
-  unsigned int header_length {};
   unsigned char minimum_instruction_length {};
   unsigned char maximum_ops_per_instruction {};
   unsigned char default_is_stmt {};
@@ -167,17 +176,24 @@ struct line_header
      header.  These point into dwarf2_per_objfile->line_buffer.  */
   const gdb_byte *statement_program_start {}, *statement_program_end {};
 
-  /* Return file name relative to the compilation directory of file
-     number I in this object's file name table.  The result is
-     allocated using xmalloc; the caller is responsible for freeing
-     it.  */
-  gdb::unique_xmalloc_ptr<char> file_file_name (int file) const;
+  /* Return the most "complete" file name for FILE possible.
+
+     This means prepending the directory and compilation directory, as needed,
+     until we get an absolute path.  */
+  std::string file_file_name (const file_entry &fe) const;
+
+  /* Return the compilation directory of the compilation unit in the context of
+     which this line header is read.  Return nullptr if non applicable.  */
+  const char *comp_dir () const
+  { return m_comp_dir; }
 
 #ifdef NVIDIA_CUDA_GDB
   /* Offset of CUDA .debug_line inline function information into
      debug_str */
   uint32_t debug_str_offset {};
   const char *debug_str_base {};
+  /* Offset to end line number information in .debug_line section.  */
+  sect_offset next_sect_off {};
 #endif
 
 
@@ -191,6 +207,10 @@ struct line_header
      before, and is 0 in DWARF 5 and later).  So the client should use
      file_name_at method for access.  */
   std::vector<file_entry> m_file_names;
+
+  /* Compilation directory of the compilation unit in the context of which this
+     line header is read.  nullptr if unknown or not applicable.  */
+  const char *m_comp_dir = nullptr;
 };
 
 typedef std::unique_ptr<line_header> line_header_up;
@@ -211,6 +231,7 @@ file_entry::include_dir (const line_header *lh) const
 
 extern line_header_up dwarf_decode_line_header
   (sect_offset sect_off, bool is_dwz, dwarf2_per_objfile *per_objfile,
-   struct dwarf2_section_info *section, const struct comp_unit_head *cu_header);
+   struct dwarf2_section_info *section, const struct comp_unit_head *cu_header,
+   const char *comp_dir);
 
 #endif /* DWARF2_LINE_HEADER_H */
