@@ -1,6 +1,6 @@
 /* Fortran language support routines for GDB, the GNU debugger.
 
-   Copyright (C) 1993-2022 Free Software Foundation, Inc.
+   Copyright (C) 1993-2023 Free Software Foundation, Inc.
 
    Contributed by Motorola.  Adapted from the C parser by Farooq Butt
    (fmbutt@engage.sps.mot.com).
@@ -55,8 +55,8 @@ static void
 show_repack_array_slices (struct ui_file *file, int from_tty,
 			  struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Repacking of Fortran array slices is %s.\n"),
-		    value);
+  gdb_printf (file, _("Repacking of Fortran array slices is %s.\n"),
+	      value);
 }
 
 /* Debugging of Fortran's array slicing.  */
@@ -68,8 +68,8 @@ show_fortran_array_slicing_debug (struct ui_file *file, int from_tty,
 				  struct cmd_list_element *c,
 				  const char *value)
 {
-  fprintf_filtered (file, _("Debugging of Fortran array slicing is %s.\n"),
-		    value);
+  gdb_printf (file, _("Debugging of Fortran array slicing is %s.\n"),
+	      value);
 }
 
 /* Local functions */
@@ -87,7 +87,7 @@ f_language::get_encoding (struct type *type)
 {
   const char *encoding;
 
-  switch (TYPE_LENGTH (type))
+  switch (type->length ())
     {
     case 1:
       encoding = target_charset (type->arch ());
@@ -106,8 +106,6 @@ f_language::get_encoding (struct type *type)
   return encoding;
 }
 
-
-
 /* A helper function for the "bound" intrinsics that checks that TYPE
    is an array.  LBOUND_P is true for lower bound; this is used for
    the error message, if any.  */
@@ -140,15 +138,15 @@ fortran_bounds_all_dims (bool lbound_p,
   /* Allocate a result value of the correct type.  */
   struct type *range
     = create_static_range_type (nullptr,
-				builtin_type (gdbarch)->builtin_int,
+				builtin_f_type (gdbarch)->builtin_integer,
 				1, ndimensions);
-  struct type *elm_type = builtin_type (gdbarch)->builtin_long_long;
+  struct type *elm_type = builtin_f_type (gdbarch)->builtin_integer;
   struct type *result_type = create_array_type (nullptr, elm_type, range);
   struct value *result = allocate_value (result_type);
 
   /* Walk the array dimensions backwards due to the way the array will be
      laid out in memory, the first dimension will be the most inner.  */
-  LONGEST elm_len = TYPE_LENGTH (elm_type);
+  LONGEST elm_len = elm_type->length ();
   for (LONGEST dst_offset = elm_len * (ndimensions - 1);
        dst_offset >= 0;
        dst_offset -= elm_len)
@@ -163,13 +161,13 @@ fortran_bounds_all_dims (bool lbound_p,
 
       /* And copy the value into the result value.  */
       struct value *v = value_from_longest (elm_type, b);
-      gdb_assert (dst_offset + TYPE_LENGTH (value_type (v))
-		  <= TYPE_LENGTH (value_type (result)));
-      gdb_assert (TYPE_LENGTH (value_type (v)) == elm_len);
+      gdb_assert (dst_offset + value_type (v)->length ()
+		  <= value_type (result)->length ());
+      gdb_assert (value_type (v)->length () == elm_len);
       value_contents_copy (result, dst_offset, v, 0, elm_len);
 
       /* Peel another dimension of the array.  */
-      array_type = TYPE_TARGET_TYPE (array_type);
+      array_type = array_type->target_type ();
     }
 
   return result;
@@ -177,13 +175,12 @@ fortran_bounds_all_dims (bool lbound_p,
 
 /* Return the lower bound (when LBOUND_P is true) or the upper bound (when
    LBOUND_P is false) for dimension DIM_VAL (which must be an integer) of
-   ARRAY (which must be an array).  GDBARCH is the current architecture.  */
+   ARRAY (which must be an array).  RESULT_TYPE corresponds to the type kind
+   the function should be evaluated in.  */
 
-static struct value *
-fortran_bounds_for_dimension (bool lbound_p,
-			      struct gdbarch *gdbarch,
-			      struct value *array,
-			      struct value *dim_val)
+static value *
+fortran_bounds_for_dimension (bool lbound_p, value *array, value *dim_val,
+			      type* result_type)
 {
   /* Check the requested dimension is valid for this array.  */
   type *array_type = check_typedef (value_type (array));
@@ -197,9 +194,6 @@ fortran_bounds_for_dimension (bool lbound_p,
 	error (_("UBOUND dimension must be from 1 to %d"), ndimensions);
     }
 
-  /* The type for the result.  */
-  struct type *bound_type = builtin_type (gdbarch)->builtin_long_long;
-
   /* Walk the dimensions backwards, due to the ordering in which arrays are
      laid out the first dimension is the most inner.  */
   for (int i = ndimensions - 1; i >= 0; --i)
@@ -215,16 +209,15 @@ fortran_bounds_for_dimension (bool lbound_p,
 	  else
 	    b = f77_get_upperbound (array_type);
 
-	  return value_from_longest (bound_type, b);
+	  return value_from_longest (result_type, b);
 	}
 
       /* Peel off another dimension of the array.  */
-      array_type = TYPE_TARGET_TYPE (array_type);
+      array_type = array_type->target_type ();
     }
 
   gdb_assert_not_reached ("failed to find matching dimension");
 }
-
 
 /* Return the number of dimensions for a Fortran array or string.  */
 
@@ -242,7 +235,7 @@ calc_f77_array_dims (struct type *array_type)
 
   tmp_type = array_type;
 
-  while ((tmp_type = TYPE_TARGET_TYPE (tmp_type)))
+  while ((tmp_type = tmp_type->target_type ()))
     {
       if (tmp_type->code () == TYPE_CODE_ARRAY)
 	++ndimen;
@@ -295,8 +288,8 @@ protected:
   void copy_element_to_dest (struct value *elt)
   {
     value_contents_copy (m_dest, m_dest_offset, elt, 0,
-			 TYPE_LENGTH (value_type (elt)));
-    m_dest_offset += TYPE_LENGTH (value_type (elt));
+			 value_type (elt)->length ());
+    m_dest_offset += value_type (elt)->length ();
   }
 
   /* The value being written to.  */
@@ -447,20 +440,20 @@ fortran_associated (struct gdbarch *gdbarch, const language_defn *lang,
 
   struct type *pointer_target_type;
   if (pointer_type->code () == TYPE_CODE_PTR)
-    pointer_target_type = TYPE_TARGET_TYPE (pointer_type);
+    pointer_target_type = pointer_type->target_type ();
   else
     pointer_target_type = pointer_type;
 
   struct type *target_target_type;
   if (target_type->code () == TYPE_CODE_PTR)
-    target_target_type = TYPE_TARGET_TYPE (target_type);
+    target_target_type = target_type->target_type ();
   else
     target_target_type = target_type;
 
   if (pointer_target_type->code () != target_target_type->code ()
       || (pointer_target_type->code () != TYPE_CODE_ARRAY
-	  && (TYPE_LENGTH (pointer_target_type)
-	      != TYPE_LENGTH (target_target_type))))
+	  && (pointer_target_type->length ()
+	      != target_target_type->length ())))
     error (_("arguments to associated must be of same type and kind"));
 
   /* If TARGET is not in memory, or the original pointer is specifically
@@ -542,12 +535,12 @@ fortran_associated (struct gdbarch *gdbarch, const language_defn *lang,
 	  if (pointer_stride == 0)
 	    pointer_stride
 	      = type_length_units (check_typedef
-				     (TYPE_TARGET_TYPE (pointer_type))) * 8;
+				     (pointer_type->target_type ())) * 8;
 	  target_stride = target_range->bounds ()->bit_stride ();
 	  if (target_stride == 0)
 	    target_stride
 	      = type_length_units (check_typedef
-				     (TYPE_TARGET_TYPE (target_type))) * 8;
+				     (target_type->target_type ())) * 8;
 	  if (pointer_stride != target_stride)
 	    break;
 
@@ -586,8 +579,8 @@ eval_op_f_associated (struct type *expect_type,
 }
 
 /* Implement FORTRAN_ARRAY_SIZE expression, this corresponds to the 'SIZE'
-   keyword.  Both GDBARCH and LANG are extracted from the expression being
-   evaluated.  ARRAY is the value that should be an array, though this will
+   keyword.  RESULT_TYPE corresponds to the type kind the function should be
+   evaluated in, ARRAY is the value that should be an array, though this will
    not have been checked before calling this function.  DIM is optional, if
    present then it should be an integer identifying a dimension of the
    array to ask about.  As with ARRAY the validity of DIM is not checked
@@ -596,9 +589,8 @@ eval_op_f_associated (struct type *expect_type,
    Return either the total number of elements in ARRAY (when DIM is
    nullptr), or the number of elements in dimension DIM.  */
 
-static struct value *
-fortran_array_size (struct gdbarch *gdbarch, const language_defn *lang,
-		    struct value *array, struct value *dim_val = nullptr)
+static value *
+fortran_array_size (value *array, value *dim_val, type *result_type)
 {
   /* Check that ARRAY is the correct type.  */
   struct type *array_type = check_typedef (value_type (array));
@@ -647,11 +639,9 @@ fortran_array_size (struct gdbarch *gdbarch, const language_defn *lang,
 	}
 
       /* Peel off another dimension of the array.  */
-      array_type = TYPE_TARGET_TYPE (array_type);
+      array_type = array_type->target_type ();
     }
 
-  struct type *result_type
-    = builtin_f_type (gdbarch)->builtin_integer;
   return value_from_longest (result_type, result);
 }
 
@@ -665,7 +655,9 @@ eval_op_f_array_size (struct type *expect_type,
 		      struct value *arg1)
 {
   gdb_assert (opcode == FORTRAN_ARRAY_SIZE);
-  return fortran_array_size (exp->gdbarch, exp->language_defn, arg1);
+
+  type *result_type = builtin_f_type (exp->gdbarch)->builtin_integer;
+  return fortran_array_size (arg1, nullptr, result_type);
 }
 
 /* See f-exp.h.  */
@@ -679,7 +671,21 @@ eval_op_f_array_size (struct type *expect_type,
 		      struct value *arg2)
 {
   gdb_assert (opcode == FORTRAN_ARRAY_SIZE);
-  return fortran_array_size (exp->gdbarch, exp->language_defn, arg1, arg2);
+
+  type *result_type = builtin_f_type (exp->gdbarch)->builtin_integer;
+  return fortran_array_size (arg1, arg2, result_type);
+}
+
+/* See f-exp.h.  */
+
+value *eval_op_f_array_size (type *expect_type, expression *exp, noside noside,
+			     exp_opcode opcode, value *arg1, value *arg2,
+			     type *kind_arg)
+{
+  gdb_assert (opcode == FORTRAN_ARRAY_SIZE);
+  gdb_assert (kind_arg->code () == TYPE_CODE_INT);
+
+  return fortran_array_size (arg1, arg2, kind_arg);
 }
 
 /* Implement UNOP_FORTRAN_SHAPE expression.  Both GDBARCH and LANG are
@@ -720,7 +726,7 @@ fortran_array_shape (struct gdbarch *gdbarch, const language_defn *lang,
   struct type *elm_type = builtin_f_type (gdbarch)->builtin_integer;
   struct type *result_type = create_array_type (nullptr, elm_type, range);
   struct value *result = allocate_value (result_type);
-  LONGEST elm_len = TYPE_LENGTH (elm_type);
+  LONGEST elm_len = elm_type->length ();
 
   /* Walk the array dimensions backwards due to the way the array will be
      laid out in memory, the first dimension will be the most inner.
@@ -740,13 +746,13 @@ fortran_array_shape (struct gdbarch *gdbarch, const language_defn *lang,
 
       /* And copy the value into the result value.  */
       struct value *v = value_from_longest (elm_type, dim_size);
-      gdb_assert (dst_offset + TYPE_LENGTH (value_type (v))
-		  <= TYPE_LENGTH (value_type (result)));
-      gdb_assert (TYPE_LENGTH (value_type (v)) == elm_len);
+      gdb_assert (dst_offset + value_type (v)->length ()
+		  <= value_type (result)->length ());
+      gdb_assert (value_type (v)->length () == elm_len);
       value_contents_copy (result, dst_offset, v, 0, elm_len);
 
       /* Peel another dimension of the array.  */
-      val_type = TYPE_TARGET_TYPE (val_type);
+      val_type = val_type->target_type ();
     }
 
   return result;
@@ -828,7 +834,22 @@ eval_op_f_mod (struct type *expect_type, struct expression *exp,
   error (_("MOD of type %s not supported"), TYPE_SAFE_NAME (type));
 }
 
-/* A helper function for UNOP_FORTRAN_CEILING.  */
+/* A helper function for the different FORTRAN_CEILING overloads.  Calculates
+   CEILING for ARG1 (a float type) and returns it in the requested kind type
+   RESULT_TYPE.  */
+
+static value *
+fortran_ceil_operation (value *arg1, type *result_type)
+{
+  if (value_type (arg1)->code () != TYPE_CODE_FLT)
+    error (_("argument to CEILING must be of type float"));
+  double val = target_float_to_host_double (value_contents (arg1).data (),
+					    value_type (arg1));
+  val = ceil (val);
+  return value_from_longest (result_type, val);
+}
+
+/* A helper function for FORTRAN_CEILING.  */
 
 struct value *
 eval_op_f_ceil (struct type *expect_type, struct expression *exp,
@@ -836,32 +857,59 @@ eval_op_f_ceil (struct type *expect_type, struct expression *exp,
 		enum exp_opcode opcode,
 		struct value *arg1)
 {
-  struct type *type = value_type (arg1);
-  if (type->code () != TYPE_CODE_FLT)
-    error (_("argument to CEILING must be of type float"));
-  double val
-    = target_float_to_host_double (value_contents (arg1).data (),
-				   value_type (arg1));
-  val = ceil (val);
-  return value_from_host_double (type, val);
+  gdb_assert (opcode == FORTRAN_CEILING);
+  type *result_type = builtin_f_type (exp->gdbarch)->builtin_integer;
+  return fortran_ceil_operation (arg1, result_type);
 }
 
-/* A helper function for UNOP_FORTRAN_FLOOR.  */
+/* A helper function for FORTRAN_CEILING.  */
+
+value *
+eval_op_f_ceil (type *expect_type, expression *exp, noside noside,
+		exp_opcode opcode, value *arg1, type *kind_arg)
+{
+  gdb_assert (opcode == FORTRAN_CEILING);
+  gdb_assert (kind_arg->code () == TYPE_CODE_INT);
+  return fortran_ceil_operation (arg1, kind_arg);
+}
+
+/* A helper function for the different FORTRAN_FLOOR overloads.  Calculates
+   FLOOR for ARG1 (a float type) and returns it in the requested kind type
+   RESULT_TYPE.  */
+
+static value *
+fortran_floor_operation (value *arg1, type *result_type)
+{
+  if (value_type (arg1)->code () != TYPE_CODE_FLT)
+    error (_("argument to FLOOR must be of type float"));
+  double val = target_float_to_host_double (value_contents (arg1).data (),
+					    value_type (arg1));
+  val = floor (val);
+  return value_from_longest (result_type, val);
+}
+
+/* A helper function for FORTRAN_FLOOR.  */
 
 struct value *
 eval_op_f_floor (struct type *expect_type, struct expression *exp,
-		 enum noside noside,
-		 enum exp_opcode opcode,
-		 struct value *arg1)
+		enum noside noside,
+		enum exp_opcode opcode,
+		struct value *arg1)
 {
-  struct type *type = value_type (arg1);
-  if (type->code () != TYPE_CODE_FLT)
-    error (_("argument to FLOOR must be of type float"));
-  double val
-    = target_float_to_host_double (value_contents (arg1).data (),
-				   value_type (arg1));
-  val = floor (val);
-  return value_from_host_double (type, val);
+  gdb_assert (opcode == FORTRAN_FLOOR);
+  type *result_type = builtin_f_type (exp->gdbarch)->builtin_integer;
+  return fortran_floor_operation (arg1, result_type);
+}
+
+/* A helper function for FORTRAN_FLOOR.  */
+
+struct value *
+eval_op_f_floor (type *expect_type, expression *exp, noside noside,
+		 exp_opcode opcode, value *arg1, type *kind_arg)
+{
+  gdb_assert (opcode == FORTRAN_FLOOR);
+  gdb_assert (kind_arg->code () == TYPE_CODE_INT);
+  return fortran_floor_operation (arg1, kind_arg);
 }
 
 /* A helper function for BINOP_FORTRAN_MODULO.  */
@@ -904,7 +952,25 @@ eval_op_f_modulo (struct type *expect_type, struct expression *exp,
   error (_("MODULO of type %s not supported"), TYPE_SAFE_NAME (type));
 }
 
-/* A helper function for BINOP_FORTRAN_CMPLX.  */
+/* A helper function for FORTRAN_CMPLX.  */
+
+value *
+eval_op_f_cmplx (type *expect_type, expression *exp, noside noside,
+		 exp_opcode opcode, value *arg1)
+{
+  gdb_assert (opcode == FORTRAN_CMPLX);
+
+  type *result_type = builtin_f_type (exp->gdbarch)->builtin_complex;
+
+  if (value_type (arg1)->code () == TYPE_CODE_COMPLEX)
+    return value_cast (result_type, arg1);
+  else
+    return value_literal_complex (arg1,
+				  value_zero (value_type (arg1), not_lval),
+				  result_type);
+}
+
+/* A helper function for FORTRAN_CMPLX.  */
 
 struct value *
 eval_op_f_cmplx (struct type *expect_type, struct expression *exp,
@@ -912,8 +978,28 @@ eval_op_f_cmplx (struct type *expect_type, struct expression *exp,
 		 enum exp_opcode opcode,
 		 struct value *arg1, struct value *arg2)
 {
-  struct type *type = builtin_f_type(exp->gdbarch)->builtin_complex_s16;
-  return value_literal_complex (arg1, arg2, type);
+  if (value_type (arg1)->code () == TYPE_CODE_COMPLEX
+      || value_type (arg2)->code () == TYPE_CODE_COMPLEX)
+    error (_("Types of arguments for CMPLX called with more then one argument "
+	     "must be REAL or INTEGER"));
+
+  type *result_type = builtin_f_type (exp->gdbarch)->builtin_complex;
+  return value_literal_complex (arg1, arg2, result_type);
+}
+
+/* A helper function for FORTRAN_CMPLX.  */
+
+value *
+eval_op_f_cmplx (type *expect_type, expression *exp, noside noside,
+		 exp_opcode opcode, value *arg1, value *arg2, type *kind_arg)
+{
+  gdb_assert (kind_arg->code () == TYPE_CODE_COMPLEX);
+  if (value_type (arg1)->code () == TYPE_CODE_COMPLEX
+      || value_type (arg2)->code () == TYPE_CODE_COMPLEX)
+    error (_("Types of arguments for CMPLX called with more then one argument "
+	     "must be REAL or INTEGER"));
+
+  return value_literal_complex (arg1, arg2, kind_arg);
 }
 
 /* A helper function for UNOP_FORTRAN_KIND.  */
@@ -935,11 +1021,11 @@ eval_op_f_kind (struct type *expect_type, struct expression *exp,
       error (_("argument to kind must be an intrinsic type"));
     }
 
-  if (!TYPE_TARGET_TYPE (type))
+  if (!type->target_type ())
     return value_from_longest (builtin_type (exp->gdbarch)->builtin_int,
-			       TYPE_LENGTH (type));
+			       type->length ());
   return value_from_longest (builtin_type (exp->gdbarch)->builtin_int,
-			     TYPE_LENGTH (TYPE_TARGET_TYPE (type)));
+			     type->target_type ()->length ());
 }
 
 /* A helper function for UNOP_FORTRAN_ALLOCATED.  */
@@ -1058,7 +1144,7 @@ fortran_undetermined::value_subarray (value *array,
     for (int i = 0; i < ndimensions; ++i)
       {
 	dim_types.push_back (type);
-	type = TYPE_TARGET_TYPE (type);
+	type = type->target_type ();
       }
     /* TYPE is now the inner element type of the array, we start the new
        array slice off as this type, then as we process the requested slice
@@ -1072,7 +1158,7 @@ fortran_undetermined::value_subarray (value *array,
      of an element at each dimension of the new slice array.  Initially the
      elements of the inner most dimension of the array are the same inner
      most elements as the original ARRAY.  */
-  LONGEST slice_element_size = TYPE_LENGTH (inner_element_type);
+  LONGEST slice_element_size = inner_element_type->length ();
 
   /* Start off assuming all data is contiguous, this will be set to false
      if access to any dimension results in non-contiguous data.  */
@@ -1147,13 +1233,13 @@ fortran_undetermined::value_subarray (value *array,
 	    error (_("stride must not be 0"));
 
 	  /* Get information about this dimension in the original ARRAY.  */
-	  struct type *target_type = TYPE_TARGET_TYPE (dim_type);
+	  struct type *target_type = dim_type->target_type ();
 	  struct type *index_type = dim_type->index_type ();
 	  LONGEST lb = f77_get_lowerbound (dim_type);
 	  LONGEST ub = f77_get_upperbound (dim_type);
 	  LONGEST sd = index_type->bit_stride ();
 	  if (sd == 0)
-	    sd = TYPE_LENGTH (target_type) * 8;
+	    sd = target_type->length () * 8;
 
 	  if (fortran_array_slicing_debug)
 	    {
@@ -1166,9 +1252,9 @@ fortran_undetermined::value_subarray (value *array,
 	      debug_printf ("|   |   |-> Bit stride: %s\n", plongest (sd));
 	      debug_printf ("|   |   |-> Byte stride: %s\n", plongest (sd / 8));
 	      debug_printf ("|   |   |-> Type size: %s\n",
-			    pulongest (TYPE_LENGTH (dim_type)));
+			    pulongest (dim_type->length ()));
 	      debug_printf ("|   |   '-> Target type size: %s\n",
-			    pulongest (TYPE_LENGTH (target_type)));
+			    pulongest (target_type->length ()));
 	      debug_printf ("|   |-> Accessing:\n");
 	      debug_printf ("|   |   |-> Low bound: %s\n",
 			    plongest (low));
@@ -1201,7 +1287,7 @@ fortran_undetermined::value_subarray (value *array,
 	  LONGEST remainder = high - last_elem;
 	  if (low > high)
 	    {
-	      offset += std::abs (remainder) * TYPE_LENGTH (target_type);
+	      offset += std::abs (remainder) * target_type->length ();
 	      if (stride > 0)
 		error (_("incorrect stride and boundary combination"));
 	    }
@@ -1249,13 +1335,13 @@ fortran_undetermined::value_subarray (value *array,
 	    = value_as_long (ops[i]->evaluate_with_coercion (exp, noside));
 
 	  /* Get information about this dimension in the original ARRAY.  */
-	  struct type *target_type = TYPE_TARGET_TYPE (dim_type);
+	  struct type *target_type = dim_type->target_type ();
 	  struct type *index_type = dim_type->index_type ();
 	  LONGEST lb = f77_get_lowerbound (dim_type);
 	  LONGEST ub = f77_get_upperbound (dim_type);
 	  LONGEST sd = index_type->bit_stride () / 8;
 	  if (sd == 0)
-	    sd = TYPE_LENGTH (target_type);
+	    sd = target_type->length ();
 
 	  if (fortran_array_slicing_debug)
 	    {
@@ -1267,9 +1353,9 @@ fortran_undetermined::value_subarray (value *array,
 	      debug_printf ("|   |   |-> High bound: %s\n", plongest (ub));
 	      debug_printf ("|   |   |-> Byte stride: %s\n", plongest (sd));
 	      debug_printf ("|   |   |-> Type size: %s\n",
-			    pulongest (TYPE_LENGTH (dim_type)));
+			    pulongest (dim_type->length ()));
 	      debug_printf ("|   |   '-> Target type size: %s\n",
-			    pulongest (TYPE_LENGTH (target_type)));
+			    pulongest (target_type->length ()));
 	      debug_printf ("|   '-> Accessing:\n");
 	      debug_printf ("|       '-> Index: %s\n",
 			    plongest (index));
@@ -1314,7 +1400,7 @@ fortran_undetermined::value_subarray (value *array,
 
       struct type *new_range
 	= create_range_type_with_stride ((struct type *) NULL,
-					 TYPE_TARGET_TYPE (d.index),
+					 d.index->target_type (),
 					 &p_low, &p_high, 0, &p_stride,
 					 true);
       array_slice_type
@@ -1346,11 +1432,11 @@ fortran_undetermined::value_subarray (value *array,
 
 	  p_low.set_const_val (d.low);
 	  p_high.set_const_val (d.high);
-	  p_stride.set_const_val (TYPE_LENGTH (repacked_array_type));
+	  p_stride.set_const_val (repacked_array_type->length ());
 
 	  struct type *new_range
 	    = create_range_type_with_stride ((struct type *) NULL,
-					     TYPE_TARGET_TYPE (d.index),
+					     d.index->target_type (),
 					     &p_low, &p_high, 0, &p_stride,
 					     true);
 	  repacked_array_type
@@ -1361,8 +1447,8 @@ fortran_undetermined::value_subarray (value *array,
 	 array value DEST.  */
       struct value *dest = allocate_value (repacked_array_type);
       if (value_lazy (array)
-	  || (total_offset + TYPE_LENGTH (array_slice_type)
-	      > TYPE_LENGTH (check_typedef (value_type (array)))))
+	  || (total_offset + array_slice_type->length ()
+	      > check_typedef (value_type (array))->length ()))
 	{
 	  fortran_array_walker<fortran_lazy_array_repacker_impl> p
 	    (array_slice_type, value_address (array) + total_offset, dest);
@@ -1386,8 +1472,8 @@ fortran_undetermined::value_subarray (value *array,
 	     just create a new lazy value pointing at the memory where the
 	     contents we're looking for exist.  */
 	  if (value_lazy (array)
-	      || (total_offset + TYPE_LENGTH (array_slice_type)
-		  > TYPE_LENGTH (check_typedef (value_type (array)))))
+	      || (total_offset + array_slice_type->length ()
+		  > check_typedef (value_type (array))->length ()))
 	    array = value_at_lazy (array_slice_type,
 				   value_address (array) + total_offset);
 	  else
@@ -1422,7 +1508,7 @@ fortran_undetermined::evaluate (struct type *expect_type,
 	 So we need to look into its target type to see if it is
 	 array, string or function.  If it is, we need to switch
 	 to the target value the original one points to.  */
-      struct type *target_type = check_typedef (TYPE_TARGET_TYPE (type));
+      struct type *target_type = check_typedef (type->target_type ());
 
       if (target_type->code () == TYPE_CODE_ARRAY
 	  || target_type->code () == TYPE_CODE_STRING
@@ -1486,8 +1572,8 @@ fortran_bound_2arg::evaluate (struct type *expect_type,
 
   /* User asked for the bounds of a specific dimension of the array.  */
   value *arg2 = std::get<2> (m_storage)->evaluate (nullptr, exp, noside);
-  struct type *type = check_typedef (value_type (arg2));
-  if (type->code () != TYPE_CODE_INT)
+  type *type_arg2 = check_typedef (value_type (arg2));
+  if (type_arg2->code () != TYPE_CODE_INT)
     {
       if (lbound_p)
 	error (_("LBOUND second argument should be an integer"));
@@ -1495,7 +1581,34 @@ fortran_bound_2arg::evaluate (struct type *expect_type,
 	error (_("UBOUND second argument should be an integer"));
     }
 
-  return fortran_bounds_for_dimension (lbound_p, exp->gdbarch, arg1, arg2);
+  type *result_type = builtin_f_type (exp->gdbarch)->builtin_integer;
+  return fortran_bounds_for_dimension (lbound_p, arg1, arg2, result_type);
+}
+
+value *
+fortran_bound_3arg::evaluate (type *expect_type,
+			      expression *exp,
+			      noside noside)
+{
+  const bool lbound_p = std::get<0> (m_storage) == FORTRAN_LBOUND;
+  value *arg1 = std::get<1> (m_storage)->evaluate (nullptr, exp, noside);
+  fortran_require_array (value_type (arg1), lbound_p);
+
+  /* User asked for the bounds of a specific dimension of the array.  */
+  value *arg2 = std::get<2> (m_storage)->evaluate (nullptr, exp, noside);
+  type *type_arg2 = check_typedef (value_type (arg2));
+  if (type_arg2->code () != TYPE_CODE_INT)
+    {
+      if (lbound_p)
+	error (_("LBOUND second argument should be an integer"));
+      else
+	error (_("UBOUND second argument should be an integer"));
+    }
+
+  type *kind_arg = std::get<3> (m_storage);
+  gdb_assert (kind_arg->code () == TYPE_CODE_INT);
+
+  return fortran_bounds_for_dimension (lbound_p, arg1, arg2, kind_arg);
 }
 
 /* Implement STRUCTOP_STRUCT for Fortran.  See operation::evaluate in
@@ -1526,7 +1639,7 @@ fortran_structop_operation::evaluate (struct type *expect_type,
 	  const gdb_byte *valaddr = value_contents_for_printing (elt).data ();
 	  CORE_ADDR address = value_address (elt);
 	  gdb::array_view<const gdb_byte> view
-	    = gdb::make_array_view (valaddr, TYPE_LENGTH (elt_type));
+	    = gdb::make_array_view (valaddr, elt_type->length ());
 	  elt_type = resolve_dynamic_type (elt_type, view, address);
 	}
       elt = value_zero (elt_type, VALUE_LVAL (elt));
@@ -1546,9 +1659,9 @@ f_language::print_array_index (struct type *index_type, LONGEST index,
 {
   struct value *index_value = value_from_longest (index_type, index);
 
-  fprintf_filtered (stream, "(");
+  gdb_printf (stream, "(");
   value_print (index_value, stream, options);
-  fprintf_filtered (stream, ") = ");
+  gdb_printf (stream, ") = ");
 }
 
 /* See language.h.  */
@@ -1573,12 +1686,12 @@ f_language::language_arch_info (struct gdbarch *gdbarch,
   add (builtin->builtin_real);
   add (builtin->builtin_real_s8);
   add (builtin->builtin_real_s16);
+  add (builtin->builtin_complex);
   add (builtin->builtin_complex_s8);
-  add (builtin->builtin_complex_s16);
   add (builtin->builtin_void);
 
   lai->set_string_char_type (builtin->builtin_character);
-  lai->set_bool_type (builtin->builtin_logical_s2, "logical");
+  lai->set_bool_type (builtin->builtin_logical, "logical");
 }
 
 /* See language.h.  */
@@ -1612,11 +1725,10 @@ f_language::get_symbol_name_matcher_inner
 
 static f_language f_language_defn;
 
-static void *
+static struct builtin_f_type *
 build_fortran_types (struct gdbarch *gdbarch)
 {
-  struct builtin_f_type *builtin_f_type
-    = GDBARCH_OBSTACK_ZALLOC (gdbarch, struct builtin_f_type);
+  struct builtin_f_type *builtin_f_type = new struct builtin_f_type;
 
   builtin_f_type->builtin_void
     = arch_type (gdbarch, TYPE_CODE_VOID, TARGET_CHAR_BIT, "void");
@@ -1627,36 +1739,37 @@ build_fortran_types (struct gdbarch *gdbarch)
   builtin_f_type->builtin_logical_s1
     = arch_boolean_type (gdbarch, TARGET_CHAR_BIT, 1, "logical*1");
 
-  builtin_f_type->builtin_integer_s2
-    = arch_integer_type (gdbarch, gdbarch_short_bit (gdbarch), 0,
-			 "integer*2");
-
-  builtin_f_type->builtin_integer_s8
-    = arch_integer_type (gdbarch, gdbarch_long_long_bit (gdbarch), 0,
-			 "integer*8");
-
   builtin_f_type->builtin_logical_s2
-    = arch_boolean_type (gdbarch, gdbarch_short_bit (gdbarch), 1,
-			 "logical*2");
+    = arch_boolean_type (gdbarch, gdbarch_short_bit (gdbarch), 1, "logical*2");
+
+  builtin_f_type->builtin_logical
+    = arch_boolean_type (gdbarch, gdbarch_int_bit (gdbarch), 1, "logical*4");
 
   builtin_f_type->builtin_logical_s8
     = arch_boolean_type (gdbarch, gdbarch_long_long_bit (gdbarch), 1,
 			 "logical*8");
 
-  builtin_f_type->builtin_integer
-    = arch_integer_type (gdbarch, gdbarch_int_bit (gdbarch), 0,
-			 "integer");
+  builtin_f_type->builtin_integer_s1
+    = arch_integer_type (gdbarch, TARGET_CHAR_BIT, 0, "integer*1");
 
-  builtin_f_type->builtin_logical
-    = arch_boolean_type (gdbarch, gdbarch_int_bit (gdbarch), 1,
-			 "logical*4");
+  builtin_f_type->builtin_integer_s2
+    = arch_integer_type (gdbarch, gdbarch_short_bit (gdbarch), 0, "integer*2");
+
+  builtin_f_type->builtin_integer
+    = arch_integer_type (gdbarch, gdbarch_int_bit (gdbarch), 0, "integer*4");
+
+  builtin_f_type->builtin_integer_s8
+    = arch_integer_type (gdbarch, gdbarch_long_long_bit (gdbarch), 0,
+			 "integer*8");
 
   builtin_f_type->builtin_real
     = arch_float_type (gdbarch, gdbarch_float_bit (gdbarch),
-		       "real", gdbarch_float_format (gdbarch));
+		       "real*4", gdbarch_float_format (gdbarch));
+
   builtin_f_type->builtin_real_s8
     = arch_float_type (gdbarch, gdbarch_double_bit (gdbarch),
 		       "real*8", gdbarch_double_format (gdbarch));
+
   auto fmt = gdbarch_floatformat_for_type (gdbarch, "real(kind=16)", 128);
   if (fmt != nullptr)
     builtin_f_type->builtin_real_s16
@@ -1669,27 +1782,35 @@ build_fortran_types (struct gdbarch *gdbarch)
     builtin_f_type->builtin_real_s16
       = arch_type (gdbarch, TYPE_CODE_ERROR, 128, "real*16");
 
+  builtin_f_type->builtin_complex
+    = init_complex_type ("complex*4", builtin_f_type->builtin_real);
+
   builtin_f_type->builtin_complex_s8
-    = init_complex_type ("complex*8", builtin_f_type->builtin_real);
-  builtin_f_type->builtin_complex_s16
-    = init_complex_type ("complex*16", builtin_f_type->builtin_real_s8);
+    = init_complex_type ("complex*8", builtin_f_type->builtin_real_s8);
 
   if (builtin_f_type->builtin_real_s16->code () == TYPE_CODE_ERROR)
-    builtin_f_type->builtin_complex_s32
-      = arch_type (gdbarch, TYPE_CODE_ERROR, 256, "complex*32");
+    builtin_f_type->builtin_complex_s16
+      = arch_type (gdbarch, TYPE_CODE_ERROR, 256, "complex*16");
   else
-    builtin_f_type->builtin_complex_s32
-      = init_complex_type ("complex*32", builtin_f_type->builtin_real_s16);
+    builtin_f_type->builtin_complex_s16
+      = init_complex_type ("complex*16", builtin_f_type->builtin_real_s16);
 
   return builtin_f_type;
 }
 
-static struct gdbarch_data *f_type_data;
+static const registry<gdbarch>::key<struct builtin_f_type> f_type_data;
 
 const struct builtin_f_type *
 builtin_f_type (struct gdbarch *gdbarch)
 {
-  return (const struct builtin_f_type *) gdbarch_data (gdbarch, f_type_data);
+  struct builtin_f_type *result = f_type_data.get (gdbarch);
+  if (result == nullptr)
+    {
+      result = build_fortran_types (gdbarch);
+      f_type_data.set (gdbarch, result);
+    }
+
+  return result;
 }
 
 /* Command-list for the "set/show fortran" prefix command.  */
@@ -1700,8 +1821,6 @@ void _initialize_f_language ();
 void
 _initialize_f_language ()
 {
-  f_type_data = gdbarch_data_register_post_init (build_fortran_types);
-
   add_setshow_prefix_cmd
     ("fortran", no_class,
      _("Prefix command for changing Fortran-specific settings."),
@@ -1761,7 +1880,7 @@ fortran_argument_convert (struct value *value, bool is_artificial)
       if (VALUE_LVAL (value) != lval_memory)
 	{
 	  struct type *type = value_type (value);
-	  const int length = TYPE_LENGTH (type);
+	  const int length = type->length ();
 	  const CORE_ADDR addr
 	    = value_as_long (value_allocate_space_in_inferior (length));
 	  write_memory (addr, value_contents (value).data (), length);
@@ -1875,7 +1994,7 @@ fortran_adjust_dynamic_array_base_address_hack (struct type *type,
 	error ("failed to get range bounds");
 
       /* Figure out the stride for this dimension.  */
-      struct type *elt_type = check_typedef (TYPE_TARGET_TYPE (tmp_type));
+      struct type *elt_type = check_typedef (tmp_type->target_type ());
       stride = tmp_type->index_type ()->bounds ()->bit_stride ();
       if (stride == 0)
 	stride = type_length_units (elt_type);
@@ -1893,7 +2012,7 @@ fortran_adjust_dynamic_array_base_address_hack (struct type *type,
       if (stride < 0 && lowerbound < upperbound)
 	offset = (upperbound - lowerbound) * stride;
       total_offset += offset;
-      tmp_type = TYPE_TARGET_TYPE (tmp_type);
+      tmp_type = tmp_type->target_type ();
     }
 
   /* Adjust the address of this object and return it.  */
