@@ -1,6 +1,6 @@
 /* Support for printing Fortran values for GDB, the GNU debugger.
 
-   Copyright (C) 1993-2022 Free Software Foundation, Inc.
+   Copyright (C) 1993-2023 Free Software Foundation, Inc.
 
    Contributed by Motorola.  Adapted from the C definitions by Farooq Butt
    (fmbutt@engage.sps.mot.com), additionally worked over by Stan Shebs.
@@ -87,19 +87,17 @@ f77_get_dynamic_length_of_aggregate (struct type *type)
      This function also works for strings which behave very 
      similarly to arrays.  */
 
-  if (TYPE_TARGET_TYPE (type)->code () == TYPE_CODE_ARRAY
-      || TYPE_TARGET_TYPE (type)->code () == TYPE_CODE_STRING)
-    f77_get_dynamic_length_of_aggregate (TYPE_TARGET_TYPE (type));
+  if (type->target_type ()->code () == TYPE_CODE_ARRAY
+      || type->target_type ()->code () == TYPE_CODE_STRING)
+    f77_get_dynamic_length_of_aggregate (type->target_type ());
 
   /* Recursion ends here, start setting up lengths.  */
   lower_bound = f77_get_lowerbound (type);
   upper_bound = f77_get_upperbound (type);
 
   /* Patch in a valid length value.  */
-
-  TYPE_LENGTH (type) =
-    (upper_bound - lower_bound + 1)
-    * TYPE_LENGTH (check_typedef (TYPE_TARGET_TYPE (type)));
+  type->set_length ((upper_bound - lower_bound + 1)
+		    * check_typedef (type->target_type ())->length ());
 }
 
 /* Per-dimension statistics.  */
@@ -149,7 +147,7 @@ public:
   {
     bool cont = should_continue && (m_elts < m_options->print_max);
     if (!cont && should_continue)
-      fputs_filtered ("...", m_stream);
+      gdb_puts ("...", m_stream);
     return cont;
   }
 
@@ -167,7 +165,7 @@ public:
 	m_stats[dim_indx].nelts = nelts;
       }
 
-    fputs_filtered ("(", m_stream);
+    gdb_puts ("(", m_stream);
   }
 
   /* Called when we finish processing a batch of items within a dimension
@@ -176,9 +174,9 @@ public:
      separators between elements, and dimensions of the array.  */
   void finish_dimension (bool inner_p, bool last_p)
   {
-    fputs_filtered (")", m_stream);
+    gdb_puts (")", m_stream);
     if (!last_p)
-      fputs_filtered (" ", m_stream);
+      gdb_puts (" ", m_stream);
 
     m_dimension--;
   }
@@ -217,13 +215,13 @@ public:
 	if (nrepeats >= m_options->repeat_count_threshold)
 	  {
 	    annotate_elt_rep (nrepeats + 1);
-	    fprintf_filtered (m_stream, "%p[<repeats %s times>%p]",
-			      metadata_style.style ().ptr (),
-			      plongest (nrepeats + 1),
-			      nullptr);
+	    gdb_printf (m_stream, "%p[<repeats %s times>%p]",
+			metadata_style.style ().ptr (),
+			plongest (nrepeats + 1),
+			nullptr);
 	    annotate_elt_rep_end ();
 	    if (!repeated)
-	      fputs_filtered (" ", m_stream);
+	      gdb_puts (" ", m_stream);
 	    m_elts += nrepeats * m_stats[dim_indx + 1].nelts;
 	  }
 	else
@@ -250,7 +248,7 @@ public:
 		nrepeats++;
 	      }
 	    else if (last_p)
-	      fputs_filtered ("...", m_stream);
+	      gdb_puts ("...", m_stream);
 	  }
       }
 
@@ -271,7 +269,7 @@ public:
     bool repeated = (m_options->repeat_count_threshold < UINT_MAX
 		     && elt_type_prev != nullptr
 		     && value_contents_eq (m_val, elt_off_prev, m_val, elt_off,
-					   TYPE_LENGTH (elt_type)));
+					   elt_type->length ()));
 
     if (repeated)
       m_nrepeats++;
@@ -286,10 +284,10 @@ public:
 	    if (nrepeats >= m_options->repeat_count_threshold)
 	      {
 		annotate_elt_rep (nrepeats + 1);
-		fprintf_filtered (m_stream, "%p[<repeats %s times>%p]",
-				  metadata_style.style ().ptr (),
-				  plongest (nrepeats + 1),
-				  nullptr);
+		gdb_printf (m_stream, "%p[<repeats %s times>%p]",
+			    metadata_style.style ().ptr (),
+			    plongest (nrepeats + 1),
+			    nullptr);
 		annotate_elt_rep_end ();
 	      }
 	    else
@@ -306,7 +304,7 @@ public:
 		    common_val_print (e_val, m_stream, m_recurse, m_options,
 				      current_language);
 		    if (i > 1)
-		      fputs_filtered (", ", m_stream);
+		      gdb_puts (", ", m_stream);
 		  }
 	      }
 	    printed = true;
@@ -319,14 +317,14 @@ public:
 	      = value_from_component (m_val, elt_type, elt_off);
 
 	    if (printed)
-	      fputs_filtered (", ", m_stream);
+	      gdb_puts (", ", m_stream);
 	    maybe_print_array_index (m_stats[dim_indx].index_type, index,
 				     m_stream, m_options);
 	    common_val_print (e_val, m_stream, m_recurse, m_options,
 			      current_language);
 	  }
 	if (!last_p)
-	  fputs_filtered (", ", m_stream);
+	  gdb_puts (", ", m_stream);
       }
 
     m_elt_type_prev = elt_type;
@@ -344,7 +342,7 @@ private:
 			 LONGEST offset1, LONGEST offset2)
   {
     if (type->code () == TYPE_CODE_ARRAY
-	&& TYPE_TARGET_TYPE (type)->code () != TYPE_CODE_CHAR)
+	&& type->target_type ()->code () != TYPE_CODE_CHAR)
       {
 	/* Extract the range, and get lower and upper bounds.  */
 	struct type *range_type = check_typedef (type)->index_type ();
@@ -355,7 +353,7 @@ private:
 	/* CALC is used to calculate the offsets for each element.  */
 	fortran_array_offset_calculator calc (type);
 
-	struct type *subarray_type = check_typedef (TYPE_TARGET_TYPE (type));
+	struct type *subarray_type = check_typedef (type->target_type ());
 	for (LONGEST i = lowerbound; i < upperbound + 1; i++)
 	  {
 	    /* Use the index and the stride to work out a new offset.  */
@@ -370,7 +368,7 @@ private:
       }
     else
       return value_contents_eq (val, offset1, val, offset2,
-				TYPE_LENGTH (type));
+				type->length ());
   }
 
   /* The number of elements printed so far.  */
@@ -453,19 +451,19 @@ f_language::value_print_inner (struct value *val, struct ui_file *stream,
     case TYPE_CODE_STRING:
       f77_get_dynamic_length_of_aggregate (type);
       printstr (stream, builtin_type (gdbarch)->builtin_char, valaddr,
-		TYPE_LENGTH (type), NULL, 0, options);
+		type->length (), NULL, 0, options);
       break;
 
     case TYPE_CODE_ARRAY:
-      if (TYPE_TARGET_TYPE (type)->code () != TYPE_CODE_CHAR)
+      if (type->target_type ()->code () != TYPE_CODE_CHAR)
 	fortran_print_array (type, address, stream, recurse, val, options);
       else
 	{
-	  struct type *ch_type = TYPE_TARGET_TYPE (type);
+	  struct type *ch_type = type->target_type ();
 
 	  f77_get_dynamic_length_of_aggregate (type);
 	  printstr (stream, ch_type, valaddr,
-		    TYPE_LENGTH (type) / TYPE_LENGTH (ch_type), NULL, 0,
+		    type->length () / ch_type->length (), NULL, 0,
 		    options);
 	}
       break;
@@ -485,11 +483,11 @@ f_language::value_print_inner (struct value *val, struct ui_file *stream,
 	  if (addr < 65536)
 	    {
 	      /* Assume if the pointer is < 65536 it is probably not associated or associated with a bogus target.  */
-	      fprintf_unfiltered (stream, "<not associated>");
+	      gdb_printf (stream, "<not associated>");
 	      break;
 	    }
 #endif
-	  elttype = check_typedef (TYPE_TARGET_TYPE (type));
+	  elttype = check_typedef (type->target_type ());
 
 	  if (elttype->code () == TYPE_CODE_FUNC)
 	    {
@@ -503,13 +501,13 @@ f_language::value_print_inner (struct value *val, struct ui_file *stream,
 						 stream, demangle);
 	  else if (options->addressprint && options->format != 's')
 	    {
-	      fputs_filtered (paddress (gdbarch, addr), stream);
+	      gdb_puts (paddress (gdbarch, addr), stream);
 	      want_space = 1;
 	    }
 
 	  /* For a pointer to char or unsigned char, also print the string
 	     pointed to, unless pointer is null.  */
-	  if (TYPE_LENGTH (elttype) == 1
+	  if (elttype->length () == 1
 #ifdef NVIDIA_CUDA_GDB
 	      && (elttype->code () == TYPE_CODE_INT
 		  || elttype->code () == TYPE_CODE_CHAR)
@@ -520,8 +518,8 @@ f_language::value_print_inner (struct value *val, struct ui_file *stream,
 	      && addr != 0)
 	    {
 	      if (want_space)
-		fputs_filtered (" ", stream);
-	      val_print_string (TYPE_TARGET_TYPE (type), NULL, addr, -1,
+		gdb_puts (" ", stream);
+	      val_print_string (type->target_type (), NULL, addr, -1,
 				stream, options);
 	    }
 	  return;
@@ -533,7 +531,7 @@ f_language::value_print_inner (struct value *val, struct ui_file *stream,
     case TYPE_CODE_NAMELIST:
       /* Starting from the Fortran 90 standard, Fortran supports derived
 	 types.  */
-      fprintf_filtered (stream, "( ");
+      gdb_printf (stream, "( ");
       for (index = 0; index < type->num_fields (); index++)
 	{
 	  struct type *field_type
@@ -560,13 +558,13 @@ f_language::value_print_inner (struct value *val, struct ui_file *stream,
 		field = value_field (val, index);
 
 	      if (printed_field > 0)
-		fputs_filtered (", ", stream);
+		gdb_puts (", ", stream);
 
 	      if (field_name != NULL)
 		{
 		  fputs_styled (field_name, variable_name_style.style (),
 				stream);
-		  fputs_filtered (" = ", stream);
+		  gdb_puts (" = ", stream);
 		}
 
 	      common_val_print (field, stream, recurse + 1,
@@ -575,7 +573,7 @@ f_language::value_print_inner (struct value *val, struct ui_file *stream,
 	      ++printed_field;
 	    }
 	 }
-      fprintf_filtered (stream, " )");
+      gdb_printf (stream, " )");
       break;     
 
     case TYPE_CODE_BOOL:
@@ -593,9 +591,9 @@ f_language::value_print_inner (struct value *val, struct ui_file *stream,
 	     represented.  Different compilers use different non zero
 	     values to represent logical true.  */
 	  if (longval == 0)
-	    fputs_filtered (f_decorations.false_name, stream);
+	    gdb_puts (f_decorations.false_name, stream);
 	  else
-	    fputs_filtered (f_decorations.true_name, stream);
+	    gdb_puts (f_decorations.true_name, stream);
 	}
       break;
 
@@ -629,7 +627,7 @@ info_common_command_for_block (const struct block *block, const char *comname,
   ALL_BLOCK_SYMBOLS (block, iter, sym)
     if (sym->domain () == COMMON_BLOCK_DOMAIN)
       {
-	const struct common_block *common = SYMBOL_VALUE_COMMON_BLOCK (sym);
+	const struct common_block *common = sym->value_common_block ();
 	size_t index;
 
 	gdb_assert (sym->aclass () == LOC_COMMON_BLOCK);
@@ -639,21 +637,21 @@ info_common_command_for_block (const struct block *block, const char *comname,
 	  continue;
 
 	if (*any_printed)
-	  putchar_filtered ('\n');
+	  gdb_putc ('\n');
 	else
 	  *any_printed = 1;
 	if (sym->print_name ())
-	  printf_filtered (_("Contents of F77 COMMON block '%s':\n"),
-			   sym->print_name ());
+	  gdb_printf (_("Contents of F77 COMMON block '%s':\n"),
+		      sym->print_name ());
 	else
-	  printf_filtered (_("Contents of blank COMMON block:\n"));
+	  gdb_printf (_("Contents of blank COMMON block:\n"));
 	
 	for (index = 0; index < common->n_entries; index++)
 	  {
 	    struct value *val = NULL;
 
-	    printf_filtered ("%s = ",
-			     common->contents[index]->print_name ());
+	    gdb_printf ("%s = ",
+			common->contents[index]->print_name ());
 
 	    try
 	      {
@@ -668,7 +666,7 @@ info_common_command_for_block (const struct block *block, const char *comname,
 				except.what ());
 	      }
 
-	    putchar_filtered ('\n');
+	    gdb_putc ('\n');
 	  }
       }
 }
@@ -680,7 +678,7 @@ info_common_command_for_block (const struct block *block, const char *comname,
 static void
 info_common_command (const char *comname, int from_tty)
 {
-  struct frame_info *fi;
+  frame_info_ptr fi;
   const struct block *block;
   int values_printed = 0;
 
@@ -697,7 +695,7 @@ info_common_command (const char *comname, int from_tty)
   block = get_frame_block (fi, 0);
   if (block == NULL)
     {
-      printf_filtered (_("No symbol table info available.\n"));
+      gdb_printf (_("No symbol table info available.\n"));
       return;
     }
 
@@ -706,17 +704,17 @@ info_common_command (const char *comname, int from_tty)
       info_common_command_for_block (block, comname, &values_printed);
       /* After handling the function's top-level block, stop.  Don't
 	 continue to its superblock, the block of per-file symbols.  */
-      if (BLOCK_FUNCTION (block))
+      if (block->function ())
 	break;
-      block = BLOCK_SUPERBLOCK (block);
+      block = block->superblock ();
     }
 
   if (!values_printed)
     {
       if (comname)
-	printf_filtered (_("No common block '%s'.\n"), comname);
+	gdb_printf (_("No common block '%s'.\n"), comname);
       else
-	printf_filtered (_("No common blocks.\n"));
+	gdb_printf (_("No common blocks.\n"));
     }
 }
 
