@@ -1,6 +1,6 @@
 # Dynamic architecture support for GDB, the GNU debugger.
 
-# Copyright (C) 1998-2022 Free Software Foundation, Inc.
+# Copyright (C) 1998-2023 Free Software Foundation, Inc.
 
 # This file is part of GDB.
 
@@ -104,6 +104,20 @@
 # argument, and NAME is the name.  Note that while the names could be
 # auto-generated, this approach lets the "comment" field refer to
 # arguments in a nicer way.  It is also just nicer for users.
+#
+# * "param_checks" - optional, a list of strings.  Each string is an
+# expression that is placed within a gdb_assert before the call is
+# made to the Function/Method implementation.  Each expression is
+# something that should be true, and it is expected that the
+# expression will make use of the parameters named in 'params' (though
+# this is not required).
+#
+# * "result_checks" - optional, a list of strings.  Each string is an
+# expression that is placed within a gdb_assert after the call to the
+# Function/Method implementation.  Within each expression the variable
+# 'result' can be used to reference the result of the function/method
+# implementation.  The 'result_checks' can only be used if the 'type'
+# of this Function/Method is not 'void'.
 
 Info(
     type="const struct bfd_arch_info *",
@@ -142,7 +156,7 @@ Number of bits in a short or unsigned short for the target machine.
     invalid=False,
 )
 
-Value(
+int_bit = Value(
     comment="""
 Number of bits in an int or unsigned int for the target machine.
 """,
@@ -152,7 +166,7 @@ Number of bits in an int or unsigned int for the target machine.
     invalid=False,
 )
 
-Value(
+long_bit = Value(
     comment="""
 Number of bits in a long or unsigned long for the target machine.
 """,
@@ -169,7 +183,7 @@ machine.
 """,
     type="int",
     name="long_long_bit",
-    predefault="2*gdbarch->long_bit",
+    predefault="2*" + long_bit.predefault,
     invalid=False,
 )
 
@@ -193,7 +207,7 @@ Value(
     name="nv_fp8_e5m2_format",
     postdefault="floatformats_nv_fp8_e5m2",
     invalid=True,
-    printer="pformat (gdbarch->nv_fp8_e5m2_format)",
+    printer="pformat (gdbarch, gdbarch->nv_fp8_e5m2_format)",
     cudaonly=True,
 )
 
@@ -210,7 +224,7 @@ Value(
     name="nv_fp8_e4m3_format",
     postdefault="floatformats_nv_fp8_e4m3",
     invalid=True,
-    printer="pformat (gdbarch->nv_fp8_e4m3_format)",
+    printer="pformat (gdbarch, gdbarch->nv_fp8_e4m3_format)",
     cudaonly=True,
 )
 
@@ -226,7 +240,7 @@ Value(
     name="bfloat16_format",
     postdefault="floatformats_bfloat16",
     invalid=True,
-    printer="pformat (gdbarch->bfloat16_format)",
+    printer="pformat (gdbarch, gdbarch->bfloat16_format)",
 )
 
 Value(
@@ -241,7 +255,7 @@ Value(
     name="half_format",
     postdefault="floatformats_ieee_half",
     invalid=True,
-    printer="pformat (gdbarch->half_format)",
+    printer="pformat (gdbarch, gdbarch->half_format)",
 )
 
 Value(
@@ -256,7 +270,7 @@ Value(
     name="float_format",
     postdefault="floatformats_ieee_single",
     invalid=True,
-    printer="pformat (gdbarch->float_format)",
+    printer="pformat (gdbarch, gdbarch->float_format)",
 )
 
 Value(
@@ -271,7 +285,7 @@ Value(
     name="double_format",
     postdefault="floatformats_ieee_double",
     invalid=True,
-    printer="pformat (gdbarch->double_format)",
+    printer="pformat (gdbarch, gdbarch->double_format)",
 )
 
 Value(
@@ -286,7 +300,7 @@ Value(
     name="long_double_format",
     postdefault="floatformats_ieee_double",
     invalid=True,
-    printer="pformat (gdbarch->long_double_format)",
+    printer="pformat (gdbarch, gdbarch->long_double_format)",
 )
 
 Value(
@@ -339,7 +353,7 @@ ptr_bit is the size of a pointer on the target
 """,
     type="int",
     name="ptr_bit",
-    predefault="gdbarch->int_bit",
+    predefault=int_bit.predefault,
     invalid=False,
 )
 
@@ -597,9 +611,18 @@ Return -1 for bad REGNUM.  Note: Several targets get this wrong.
 )
 
 Method(
+    comment="""
+Return the name of register REGNR for the specified architecture.
+REGNR can be any value greater than, or equal to zero, and less than
+'gdbarch_num_cooked_regs (GDBARCH)'.  If REGNR is not supported for
+GDBARCH, then this function will return an empty string, this function
+should never return nullptr.
+""",
     type="const char *",
     name="register_name",
     params=[("int", "regnr")],
+    param_checks=["regnr >= 0", "regnr < gdbarch_num_cooked_regs (gdbarch)"],
+    result_checks=["result != nullptr"],
     predefault="0",
     invalid=True,
 )
@@ -629,7 +652,7 @@ frame.
 """,
     type="struct frame_id",
     name="dummy_id",
-    params=[("struct frame_info *", "this_frame")],
+    params=[("frame_info_ptr", "this_frame")],
     predefault="default_dummy_id",
     invalid=False,
 )
@@ -663,7 +686,7 @@ Method(
 )
 
 Value(
-    type="int",
+    type="enum call_dummy_location_type",
     name="call_dummy_location",
     predefault="AT_ENTRY_POINT",
     invalid=False,
@@ -692,7 +715,7 @@ Return true if the code of FRAME is writable.
 """,
     type="int",
     name="code_of_frame_writable",
-    params=[("struct frame_info *", "frame")],
+    params=[("frame_info_ptr", "frame")],
     predefault="default_code_of_frame_writable",
     invalid=False,
 )
@@ -702,7 +725,7 @@ Method(
     name="print_registers_info",
     params=[
         ("struct ui_file *", "file"),
-        ("struct frame_info *", "frame"),
+        ("frame_info_ptr", "frame"),
         ("int", "regnum"),
         ("int", "all"),
     ],
@@ -715,7 +738,7 @@ Method(
     name="print_float_info",
     params=[
         ("struct ui_file *", "file"),
-        ("struct frame_info *", "frame"),
+        ("frame_info_ptr", "frame"),
         ("const char *", "args"),
     ],
     predefault="default_print_float_info",
@@ -727,7 +750,7 @@ Method(
     name="print_vector_info",
     params=[
         ("struct ui_file *", "file"),
-        ("struct frame_info *", "frame"),
+        ("frame_info_ptr", "frame"),
         ("const char *", "args"),
     ],
     predicate=True,
@@ -771,7 +794,7 @@ FRAME corresponds to the longjmp frame.
 """,
     type="int",
     name="get_longjmp_target",
-    params=[("struct frame_info *", "frame"), ("CORE_ADDR *", "pc")],
+    params=[("frame_info_ptr", "frame"), ("CORE_ADDR *", "pc")],
     predicate=True,
     invalid=True,
 )
@@ -794,7 +817,7 @@ Function(
     type="int",
     name="register_to_value",
     params=[
-        ("struct frame_info *", "frame"),
+        ("frame_info_ptr", "frame"),
         ("int", "regnum"),
         ("struct type *", "type"),
         ("gdb_byte *", "buf"),
@@ -808,7 +831,7 @@ Function(
     type="void",
     name="value_to_register",
     params=[
-        ("struct frame_info *", "frame"),
+        ("frame_info_ptr", "frame"),
         ("int", "regnum"),
         ("struct type *", "type"),
         ("const gdb_byte *", "buf"),
@@ -884,6 +907,21 @@ for instance).
     invalid=True,
 )
 
+Function(
+    comment="""
+Return the address at which the value being returned from
+the current function will be stored.  This routine is only
+called if the current function uses the the "struct return
+convention".
+
+May return 0 when unable to determine that address.""",
+    type="CORE_ADDR",
+    name="get_return_buf_addr",
+    params=[("struct type *", "val_type"), ("frame_info_ptr", "cur_frame")],
+    predefault="default_get_return_buf_addr",
+    invalid=False,
+)
+
 Method(
     comment="""
 Return true if the return value of function is stored in the first hidden
@@ -1081,7 +1119,7 @@ Value(
 Method(
     type="CORE_ADDR",
     name="unwind_pc",
-    params=[("struct frame_info *", "next_frame")],
+    params=[("frame_info_ptr", "next_frame")],
     predefault="default_unwind_pc",
     invalid=False,
 )
@@ -1089,7 +1127,7 @@ Method(
 Method(
     type="CORE_ADDR",
     name="unwind_sp",
-    params=[("struct frame_info *", "next_frame")],
+    params=[("frame_info_ptr", "next_frame")],
     predefault="default_unwind_sp",
     invalid=False,
 )
@@ -1101,7 +1139,7 @@ frame-base.  Enable frame-base before frame-unwind.
 """,
     type="int",
     name="frame_num_args",
-    params=[("struct frame_info *", "frame")],
+    params=[("frame_info_ptr", "frame")],
     predicate=True,
     invalid=True,
 )
@@ -1155,15 +1193,23 @@ possible it should be in TARGET_READ_PC instead).
     invalid=False,
 )
 
-Value(
+Method(
     comment="""
-On some machines, not all bits of an address word are significant.
-For example, on AArch64, the top bits of an address known as the "tag"
-are ignored by the kernel, the hardware, etc. and can be regarded as
-additional data associated with the address.
+On some architectures, not all bits of a pointer are significant.
+On AArch64, for example, the top bits of a pointer may carry a "tag", which
+can be ignored by the kernel and the hardware.  The "tag" can be regarded as
+additional data associated with the pointer, but it is not part of the address.
+
+Given a pointer for the architecture, this hook removes all the
+non-significant bits and sign-extends things as needed.  It gets used to remove
+non-address bits from data pointers (for example, removing the AArch64 MTE tag
+bits from a pointer) and from code pointers (removing the AArch64 PAC signature
+from a pointer containing the return address).
 """,
-    type="int",
-    name="significant_addr_bit",
+    type="CORE_ADDR",
+    name="remove_non_address_bits",
+    params=[("CORE_ADDR", "pointer")],
+    predefault="default_remove_non_address_bits",
     invalid=False,
 )
 
@@ -1276,7 +1322,7 @@ further single-step is needed before the instruction finishes.
 """,
     type="int",
     name="single_step_through_delay",
-    params=[("struct frame_info *", "frame")],
+    params=[("frame_info_ptr", "frame")],
     predicate=True,
     invalid=True,
 )
@@ -1296,11 +1342,19 @@ disassembler.  Perhaps objdump can handle it?
 Function(
     type="CORE_ADDR",
     name="skip_trampoline_code",
-    params=[("struct frame_info *", "frame"), ("CORE_ADDR", "pc")],
+    params=[("frame_info_ptr", "frame"), ("CORE_ADDR", "pc")],
     predefault="generic_skip_trampoline_code",
     invalid=False,
 )
 
+Value(
+    comment="Vtable of solib operations functions.",
+    type="const struct target_so_ops *",
+    name="so_ops",
+    postdefault="&solib_target_so_ops",
+    printer="host_address_to_string (gdbarch->so_ops)",
+)
+
 Method(
     comment="""
 If in_solib_dynsym_resolve_code() returns true, and SKIP_SOLIB_RESOLVER
@@ -1499,7 +1553,7 @@ Is a register in a group
 """,
     type="int",
     name="register_reggroup_p",
-    params=[("int", "regnum"), ("struct reggroup *", "reggroup")],
+    params=[("int", "regnum"), ("const struct reggroup *", "reggroup")],
     predefault="default_register_reggroup_p",
     invalid=False,
 )
@@ -1511,7 +1565,7 @@ Fetch the pointer to the ith function argument.
     type="CORE_ADDR",
     name="fetch_pointer_argument",
     params=[
-        ("struct frame_info *", "frame"),
+        ("frame_info_ptr", "frame"),
         ("int", "argi"),
         ("struct type *", "type"),
     ],
@@ -1561,6 +1615,46 @@ Find core file memory regions
     invalid=True,
 )
 
+Method(
+    comment="""
+Given a bfd OBFD, segment ADDRESS and SIZE, create a memory tag section to be dumped to a core file
+""",
+    type="asection *",
+    name="create_memtag_section",
+    params=[("bfd *", "obfd"), ("CORE_ADDR", "address"), ("size_t", "size")],
+    predicate=True,
+    invalid=True,
+)
+
+Method(
+    comment="""
+Given a memory tag section OSEC, fill OSEC's contents with the appropriate tag data
+""",
+    type="bool",
+    name="fill_memtag_section",
+    params=[("asection *", "osec")],
+    predicate=True,
+    invalid=True,
+)
+
+Method(
+    comment="""
+Decode a memory tag SECTION and return the tags of type TYPE contained in
+the memory range [ADDRESS, ADDRESS + LENGTH).
+If no tags were found, return an empty vector.
+""",
+    type="gdb::byte_vector",
+    name="decode_memtag_section",
+    params=[
+        ("bfd_section *", "section"),
+        ("int", "type"),
+        ("CORE_ADDR", "address"),
+        ("size_t", "length"),
+    ],
+    predicate=True,
+    invalid=True,
+)
+
 Method(
     comment="""
 Read offset OFFSET of TARGET_OBJECT_LIBRARIES formatted shared libraries list from
@@ -2401,13 +2495,8 @@ Method(
 Iterate over all objfiles in the order that makes the most sense
 for the architecture to make global symbol searches.
 
-CB is a callback function where OBJFILE is the objfile to be searched,
-and CB_DATA a pointer to user-defined data (the same data that is passed
-when calling this gdbarch method).  The iteration stops if this function
-returns nonzero.
-
-CB_DATA is a pointer to some user-defined data to be passed to
-the callback.
+CB is a callback function passed an objfile to be searched.  The iteration stops
+if this function returns nonzero.
 
 If not NULL, CURRENT_OBJFILE corresponds to the objfile being
 inspected when the symbol search was requested.
@@ -2415,8 +2504,7 @@ inspected when the symbol search was requested.
     type="void",
     name="iterate_over_objfiles_in_search_order",
     params=[
-        ("iterate_over_objfiles_in_search_order_cb_ftype *", "cb"),
-        ("void *", "cb_data"),
+        ("iterate_over_objfiles_in_search_order_cb_ftype", "cb"),
         ("struct objfile *", "current_objfile"),
     ],
     predefault="default_iterate_over_objfiles_in_search_order",
@@ -2489,8 +2577,8 @@ Return 1 if an entry was read into *TYPEP and *VALP.
     type="int",
     name="auxv_parse",
     params=[
-        ("gdb_byte **", "readptr"),
-        ("gdb_byte *", "endptr"),
+        ("const gdb_byte **", "readptr"),
+        ("const gdb_byte *", "endptr"),
         ("CORE_ADDR *", "typep"),
         ("CORE_ADDR *", "valp"),
     ],
@@ -2638,7 +2726,7 @@ Return a string containing any flags for the given PC in the given FRAME.
 """,
     type="std::string",
     name="get_pc_address_flags",
-    params=[("frame_info *", "frame"), ("CORE_ADDR", "pc")],
+    params=[("frame_info_ptr", "frame"), ("CORE_ADDR", "pc")],
     predefault="default_get_pc_address_flags",
     invalid=False,
 )
