@@ -1,5 +1,5 @@
 /* Multi-process/thread control defs for GDB, the GNU debugger.
-   Copyright (C) 1987-2022 Free Software Foundation, Inc.
+   Copyright (C) 1987-2023 Free Software Foundation, Inc.
    Contributed by Lynx Real-Time Systems, Inc.  Los Gatos, CA.
    
 
@@ -549,7 +549,7 @@ public:
   /* CUDA: Used when we need to update the cuda coordinates during
    * cuda_wait (). Handled during context_switch (). */
   bool need_cuda_context_switch = false;
-  cuda_coords_t new_cuda_coords = CUDA_INVALID_COORDS;
+  cuda_coords new_cuda_coords {};
 #endif
   /* Step-over chain.  A thread is in the step-over queue if this node is
      linked.  */
@@ -895,13 +895,16 @@ private:
      language auto".  */
   enum language m_lang;
 #ifdef NVIDIA_CUDA_GDB
-  cuda_coords_t m_cuda_coords;
+  cuda_coords m_cuda_coords;
 #endif
 };
 
 /* Returns a pointer into the thread_info corresponding to
    INFERIOR_PTID.  INFERIOR_PTID *must* be in the thread list.  */
 extern struct thread_info* inferior_thread (void);
+#ifdef NVIDIA_BUGFIX
+extern bool has_inferior_thread (void);
+#endif
 
 extern void update_thread_list (void);
 
