@@ -1,6 +1,6 @@
 /* Top level stuff for GDB, the GNU debugger.
 
-   Copyright (C) 1986-2022 Free Software Foundation, Inc.
+   Copyright (C) 1986-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -71,14 +71,8 @@
 extern void python_print_library ();
 #endif
 #endif
-
-/* The selected interpreter.  This will be used as a set command
-   variable, so it should always be malloc'ed - since
-   do_setshow_command will free it.  */
-char *interpreter_p;
-
-/* Whether dbx commands will be handled.  */
-int dbx_commands = 0;
+/* The selected interpreter.  */
+std::string interpreter_p;
 
 /* System root path, used to find libraries etc.  */
 std::string gdb_sysroot;
@@ -138,7 +132,7 @@ set_gdb_data_directory (const char *new_datadir)
     {
       int save_errno = errno;
 
-      fprintf_unfiltered (gdb_stderr, "Warning: ");
+      gdb_printf (gdb_stderr, "Warning: ");
       print_sys_errmsg (new_datadir, save_errno);
     }
   else if (!S_ISDIR (st.st_mode))
@@ -152,12 +146,7 @@ set_gdb_data_directory (const char *new_datadir)
      "../foo" and "../foo" doesn't exist then we'll record $(pwd)/../foo which
      isn't canonical, but that's ok.  */
   if (!IS_ABSOLUTE_PATH (gdb_datadir.c_str ()))
-    {
-      gdb::unique_xmalloc_ptr<char> abs_datadir
-	= gdb_abspath (gdb_datadir.c_str ());
-
-      gdb_datadir = abs_datadir.get ();
-    }
+    gdb_datadir = gdb_abspath (gdb_datadir.c_str ());
 }
 
 /* Relocate a file or directory.  PROGNAME is the name by which gdb
@@ -723,8 +712,9 @@ captured_main_1 (struct captured_main_args *context)
   main_ui = new ui (stdin, stdout, stderr);
   current_ui = main_ui;
 
-  gdb_stdtargerr = gdb_stderr;	/* for moment */
-  gdb_stdtargin = gdb_stdin;	/* for moment */
+  gdb_stdtarg = gdb_stderr;
+  gdb_stdtargerr = gdb_stderr;
+  gdb_stdtargin = gdb_stdin;
 
   if (bfd_init () != BFD_INIT_MAGIC)
     error (_("fatal error: libbfd ABI mismatch"));
@@ -774,7 +764,7 @@ captured_main_1 (struct captured_main_args *context)
      this captured main, or one specified by the user at start up, or
      the console.  Initialize the interpreter to the one requested by 
      the application.  */
-  interpreter_p = xstrdup (context->interpreter_p);
+  interpreter_p = context->interpreter_p;
 
   /* Parse arguments and options.  */
   {
@@ -811,7 +801,6 @@ captured_main_1 (struct captured_main_args *context)
     static struct option long_options[] =
     {
       {"tui", no_argument, 0, OPT_TUI},
-      {"dbx", no_argument, &dbx_commands, 1},
       {"readnow", no_argument, NULL, OPT_READNOW},
       {"readnever", no_argument, NULL, OPT_READNEVER},
       {"r", no_argument, NULL, OPT_READNOW},
@@ -926,8 +915,7 @@ captured_main_1 (struct captured_main_args *context)
 	  case OPT_TUI:
 	    /* --tui is equivalent to -i=tui.  */
 #ifdef TUI
-	    xfree (interpreter_p);
-	    interpreter_p = xstrdup (INTERP_TUI);
+	    interpreter_p = INTERP_TUI;
 #else
 	    error (_("%s: TUI mode is not supported"), gdb_program_name);
 #endif
@@ -937,14 +925,12 @@ captured_main_1 (struct captured_main_args *context)
 	       actually useful, and if it is, what it should do.  */
 #ifdef GDBTK
 	    /* --windows is equivalent to -i=insight.  */
-	    xfree (interpreter_p);
-	    interpreter_p = xstrdup (INTERP_INSIGHT);
+	    interpreter_p = INTERP_INSIGHT;
 #endif
 	    break;
 	  case OPT_NOWINDOWS:
 	    /* -nw is equivalent to -i=console.  */
-	    xfree (interpreter_p);
-	    interpreter_p = xstrdup (INTERP_CONSOLE);
+	    interpreter_p = INTERP_CONSOLE;
 	    break;
 #ifdef NVIDIA_CUDA_GDB
 	  case OPT_CUDA_USE_LOCKFILE:
@@ -1028,8 +1014,7 @@ captured_main_1 (struct captured_main_args *context)
 	    }
 #endif /* GDBTK */
 	  case 'i':
-	    xfree (interpreter_p);
-	    interpreter_p = xstrdup (optarg);
+	    interpreter_p = optarg;
 	    break;
 	  case 'd':
 	    dirarg.push_back (optarg);
@@ -1097,9 +1082,6 @@ captured_main_1 (struct captured_main_args *context)
       }
   }
 
-  if (dbx_commands)
-    warning (_("--dbx mode is deprecated and will be removed"));
-
   save_original_signals_state (quiet);
 
   /* Try to set up an alternate signal stack for SIGSEGV handlers.  */
@@ -1170,11 +1152,11 @@ captured_main_1 (struct captured_main_args *context)
       /* Any argument left on the command line is unexpected and
 	 will be ignored.  Inform the user.  */
       if (optind < argc)
-	fprintf_unfiltered (gdb_stderr,
-			    _("Excess command line "
-			      "arguments ignored. (%s%s)\n"),
-			    argv[optind],
-			    (optind == argc - 1) ? "" : " ...");
+	gdb_printf (gdb_stderr,
+		    _("Excess command line "
+		      "arguments ignored. (%s%s)\n"),
+		    argv[optind],
+		    (optind == argc - 1) ? "" : " ...");
     }
 
   /* Lookup gdbinit files.  Note that the gdbinit file name may be
@@ -1193,8 +1175,7 @@ captured_main_1 (struct captured_main_args *context)
   if (print_version)
     {
       print_gdb_version (gdb_stdout, false);
-      gdb_stdout->wrap_here (0);
-      printf_filtered ("\n");
+      gdb_printf ("\n");
 #ifdef NVIDIA_CUDA_GDB
 #ifdef HAVE_PYTHON
       python_print_library ();
@@ -1212,8 +1193,7 @@ captured_main_1 (struct captured_main_args *context)
   if (print_configuration)
     {
       print_gdb_configuration (gdb_stdout);
-      gdb_stdout->wrap_here (0);
-      printf_filtered ("\n");
+      gdb_printf ("\n");
 #ifdef NVIDIA_CUDA_GDB
 #ifdef HAVE_PYTHON
       python_print_library ();
@@ -1226,15 +1206,14 @@ captured_main_1 (struct captured_main_args *context)
      GDB retain the old MI1 interpreter startup behavior.  Output the
      copyright message before the interpreter is installed.  That way
      it isn't encapsulated in MI output.  */
-  if (!quiet && strcmp (interpreter_p, INTERP_MI1) == 0)
+  if (!quiet && interpreter_p == INTERP_MI1)
     {
       /* Print all the junk at the top, with trailing "..." if we are
 	 about to read a symbol file (possibly slowly).  */
       print_gdb_version (gdb_stdout, true);
       if (symarg)
-	printf_filtered ("..");
-      gdb_stdout->wrap_here (0);
-      printf_filtered ("\n");
+	gdb_printf ("..");
+      gdb_printf ("\n");
 #ifdef NVIDIA_CUDA_GDB
 #ifdef HAVE_PYTHON
       python_print_library ();
@@ -1246,7 +1225,7 @@ captured_main_1 (struct captured_main_args *context)
 
   /* Install the default UI.  All the interpreters should have had a
      look at things by now.  Initialize the default interpreter.  */
-  set_top_level_interpreter (interpreter_p);
+  set_top_level_interpreter (interpreter_p.c_str ());
 
   /* FIXME: cagney/2003-02-03: The big hack (part 2 of 2) that lets
      GDB retain the old MI1 interpreter startup behavior.  Output the
@@ -1258,9 +1237,8 @@ captured_main_1 (struct captured_main_args *context)
 	 about to read a symbol file (possibly slowly).  */
       print_gdb_version (gdb_stdout, true);
       if (symarg)
-	printf_filtered ("..");
-      gdb_stdout->wrap_here (0);
-      printf_filtered ("\n");
+	gdb_printf ("..");
+      gdb_printf ("\n");
 #ifdef NVIDIA_CUDA_GDB
 #ifdef HAVE_PYTHON
       python_print_library ();
@@ -1389,8 +1367,13 @@ captured_main_1 (struct captured_main_args *context)
 
       if (!inhibit_gdbinit && auto_load_local_gdbinit)
 	{
+#ifdef NVIDIA_CUDA_GDB
+	  auto_load_debug_printf ("Loading .cuda-gdbinit file \"%s\".",
+				  local_gdbinit.c_str ());
+#else
 	  auto_load_debug_printf ("Loading .gdbinit file \"%s\".",
 				  local_gdbinit.c_str ());
+#endif
 
 	  if (file_is_auto_load_safe (local_gdbinit.c_str ()))
 	    {
@@ -1488,19 +1471,19 @@ print_gdb_help (struct ui_file *stream)
      in the alphabetical order, so as to group closely related options
      together.  */
 #ifdef NVIDIA_CUDA_GDB
-  fputs_unfiltered (_("\
+  gdb_puts (_("\
 This is the GNU debugger with CUDA support.  Usage:\n\n\
     cuda-gdb [options] [executable-file [core-file or process-id]]\n\
     cuda-gdb [options] --args executable-file [inferior-arguments ...]\n\n\
 "), stream);
 #else
-  fputs_unfiltered (_("\
+  gdb_puts (_("\
 This is the GNU debugger.  Usage:\n\n\
     gdb [options] [executable-file [core-file or process-id]]\n\
     gdb [options] --args executable-file [inferior-arguments ...]\n\n\
 "), stream);
 #endif
-  fputs_unfiltered (_("\
+  gdb_puts (_("\
 Selection of debuggee and its files:\n\n\
   --args             Arguments after executable-file are passed to inferior.\n\
   --core=COREFILE    Analyze the core dump COREFILE.\n\
@@ -1513,7 +1496,23 @@ Selection of debuggee and its files:\n\n\
   --readnever        Do not read symbol files.\n\
   --write            Set writing into executable and core files.\n\n\
 "), stream);
-  fputs_unfiltered (_("\
+#ifdef NVIDIA_CUDA_GDB
+  gdb_puts (_("\
+Initial commands and command files:\n\n\
+  --command=FILE, -x Execute CUDA-GDB commands from FILE.\n\
+  --init-command=FILE, -ix\n\
+		     Like -x but execute commands before loading inferior.\n\
+  --eval-command=COMMAND, -ex\n\
+		     Execute a single CUDA-GDB command.\n\
+		     May be used multiple times and in conjunction\n\
+		     with --command.\n\
+  --init-eval-command=COMMAND, -iex\n\
+		     Like -ex but before loading inferior.\n\
+  --nh               Do not read ~/.cuda-gdbinit.\n\
+  --nx               Do not read any .cuda-gdbinit files in any directory.\n\n\
+"), stream);
+#else
+  gdb_puts (_("\
 Initial commands and command files:\n\n\
   --command=FILE, -x Execute GDB commands from FILE.\n\
   --init-command=FILE, -ix\n\
@@ -1527,7 +1526,8 @@ Initial commands and command files:\n\n\
   --nh               Do not read ~/.gdbinit.\n\
   --nx               Do not read any .gdbinit files in any directory.\n\n\
 "), stream);
-  fputs_unfiltered (_("\
+#endif
+  gdb_puts (_("\
 Output and user interface control:\n\n\
   --fullname         Output information used by emacs-GDB interface.\n\
   --interpreter=INTERP\n\
@@ -1537,33 +1537,51 @@ Output and user interface control:\n\n\
   --nw               Do not use the GUI interface.\n\
 "), stream);
 #if defined(TUI)
-  fputs_unfiltered (_("\
+  gdb_puts (_("\
   --tui              Use a terminal user interface.\n\
 "), stream);
 #endif
-  fputs_unfiltered (_("\
-  --dbx              DBX compatibility mode.\n\
+  gdb_puts (_("\
   -q, --quiet, --silent\n\
 		     Do not print version number on startup.\n\n\
 "), stream);
 #ifdef NVIDIA_CUDA_GDB
-  fputs_unfiltered (_("\
+  gdb_puts (_("\
 CUDA-specific options:\n\n\
 "), stream);
-  fputs_unfiltered (_("\
+  gdb_puts (_("\
   --cuda-use-lockfile=VALUE\n\
                      If VALUE == 1, create a lock file for cuda-gdb.\n\
                      Default behavior is not to create a lock file.\n\
 "), stream);
 #ifdef HAVE_PYTHON
-  fputs_unfiltered (_("\
+  gdb_puts (_("\
   --disable-python   Disable python integration.\n\n\
 "), stream);
 #else
-  fputs_unfiltered (_("\n"), stream);
+  gdb_puts (_("\n"), stream);
 #endif
 #endif
-  fputs_unfiltered (_("\
+#ifdef NVIDIA_CUDA_GDB
+  gdb_puts (_("\
+Operating modes:\n\n\
+  --batch            Exit after processing options.\n\
+  --batch-silent     Like --batch, but suppress all cuda-gdb stdout output.\n\
+  --return-child-result\n\
+		     CUDA-GDB exit code will be the child's exit code.\n\
+  --configuration    Print details about CUDA-GDB configuration and then exit.\n\
+  --help             Print this message and then exit.\n\
+  --version          Print version information and then exit.\n\n\
+Remote debugging options:\n\n\
+  -b BAUDRATE        Set serial port baud rate used for remote debugging.\n\
+  -l TIMEOUT         Set timeout in seconds for remote debugging.\n\n\
+Other options:\n\n\
+  --cd=DIR           Change current directory to DIR.\n\
+  --data-directory=DIR, -D\n\
+		     Set CUDA-GDB's data-directory to DIR.\n\
+"), stream);
+#else
+  gdb_puts (_("\
 Operating modes:\n\n\
   --batch            Exit after processing options.\n\
   --batch-silent     Like --batch, but suppress all gdb stdout output.\n\
@@ -1580,20 +1598,34 @@ Other options:\n\n\
   --data-directory=DIR, -D\n\
 		     Set GDB's data-directory to DIR.\n\
 "), stream);
-  fputs_unfiltered (_("\n\
+#endif
+#ifdef NVIDIA_CUDA_GDB
+  gdb_puts (_("\n\
+At startup, CUDA-GDB reads the following early init files and executes their\n\
+commands:\n\
+"), stream);
+#else
+  gdb_puts (_("\n\
 At startup, GDB reads the following early init files and executes their\n\
 commands:\n\
 "), stream);
+#endif
   if (!home_gdbearlyinit.empty ())
-    fprintf_unfiltered (stream, _("\
+    gdb_printf (stream, _("\
    * user-specific early init file: %s\n\
 "), home_gdbearlyinit.c_str ());
   if (home_gdbearlyinit.empty ())
-    fprintf_unfiltered (stream, _("\
+    gdb_printf (stream, _("\
    None found.\n"));
-  fputs_unfiltered (_("\n\
+#ifdef NVIDIA_CUDA_GDB
+  gdb_puts (_("\n\
+At startup, CUDA-GDB reads the following init files and executes their commands:\n\
+"), stream);
+#else
+  gdb_puts (_("\n\
 At startup, GDB reads the following init files and executes their commands:\n\
 "), stream);
+#endif
   if (!system_gdbinit.empty ())
     {
       std::string output;
@@ -1603,44 +1635,41 @@ At startup, GDB reads the following init files and executes their commands:\n\
 	  if (idx < system_gdbinit.size () - 1)
 	    output += ", ";
 	}
-      fprintf_unfiltered (stream, _("\
+      gdb_printf (stream, _("\
    * system-wide init files: %s\n\
 "), output.c_str ());
     }
   if (!home_gdbinit.empty ())
-    fprintf_unfiltered (stream, _("\
+    gdb_printf (stream, _("\
    * user-specific init file: %s\n\
 "), home_gdbinit.c_str ());
   if (!local_gdbinit.empty ())
-    fprintf_unfiltered (stream, _("\
+    gdb_printf (stream, _("\
    * local init file (see also 'set auto-load local-gdbinit'): ./%s\n\
 "), local_gdbinit.c_str ());
   if (system_gdbinit.empty () && home_gdbinit.empty ()
       && local_gdbinit.empty ())
-    fprintf_unfiltered (stream, _("\
+    gdb_printf (stream, _("\
    None found.\n"));
 #ifdef NVIDIA_CUDA_GDB
-    fprintf_unfiltered (stream, _("\
-   * local cuda-gdb init file: ./%s\n\
-"), GDBINIT);
-#endif
-  fputs_unfiltered (_("\n\
+  gdb_puts (_("\n\
+For more information, type \"help\" from within CUDA-GDB, or consult the\n\
+CUDA-GDB manual (available as on-line info or a printed manual).\n\
+"), stream);
+#else
+  gdb_puts (_("\n\
 For more information, type \"help\" from within GDB, or consult the\n\
 GDB manual (available as on-line info or a printed manual).\n\
 "), stream);
-#ifdef NVIDIA_CUDA_GDB
-  fputs_unfiltered (_("\n\
-For more information about CUDA-related features, type \"help cuda\"\n\
-or \"help info cuda\" from within GDB, or consult the CUDA-GDB manual.\n\
-Report CUDA-related bugs to \"cuda-debugger-bugs@nvidia.com\".\n\
-"), stream);
 #endif
   if (REPORT_BUGS_TO[0] && stream == gdb_stdout)
-    fprintf_unfiltered (stream, _("\n\
-Report bugs to %s.\n\
-"), REPORT_BUGS_TO);
+    gdb_printf (stream, _("\n\
+Report bugs to %ps.\n\
+"), styled_string (file_name_style.style (), REPORT_BUGS_TO));
+#ifndef NVIDIA_CUDA_GDB
   if (stream == gdb_stdout)
-    fprintf_unfiltered (stream, _("\n\
+    gdb_printf (stream, _("\n\
 You can ask GDB-related questions on the GDB users mailing list\n\
 (gdb@sourceware.org) or on GDB's IRC channel (#gdb on Freenode).\n"));
+#endif
 }
