@@ -1,6 +1,6 @@
 /* MI Interpreter Definitions and Commands for GDB, the GNU debugger.
 
-   Copyright (C) 2002-2022 Free Software Foundation, Inc.
+   Copyright (C) 2002-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -106,7 +106,7 @@ display_mi_prompt (struct mi_interp *mi)
 {
   struct ui *ui = current_ui;
 
-  fputs_unfiltered ("(gdb) \n", mi->raw_stdout);
+  gdb_puts ("(gdb) \n", mi->raw_stdout);
   gdb_flush (mi->raw_stdout);
   ui->prompt_state = PROMPTED;
 }
@@ -149,7 +149,7 @@ mi_interp::init (bool top_level)
   mi->event_channel = new mi_console_file (mi->raw_stdout, "=", 0);
   mi->mi_uiout = mi_out_new (name ());
   gdb_assert (mi->mi_uiout != nullptr);
-  mi->cli_uiout = cli_out_new (mi->out);
+  mi->cli_uiout = new cli_ui_out (mi->out);
 
   if (top_level)
     {
@@ -167,9 +167,9 @@ mi_interp::init (bool top_level)
 	  target_terminal::scoped_restore_terminal_state term_state;
 	  target_terminal::ours_for_output ();
 
-	  fprintf_unfiltered (mi->event_channel,
-			      "thread-group-added,id=\"i%d\"",
-			      inf->num);
+	  gdb_printf (mi->event_channel,
+		      "thread-group-added,id=\"i%d\"",
+		      inf->num);
 
 	  gdb_flush (mi->event_channel);
 	}
@@ -198,10 +198,6 @@ mi_interp::resume ()
   /* Route target error through the MI as well.  */
   gdb_stdtargerr = mi->targ;
 
-  /* Replace all the hooks that we know about.  There really needs to
-     be a better way of doing this... */
-  clear_interpreter_hooks ();
-
   deprecated_show_load_progress = mi_load_progress;
 }
 
@@ -356,9 +352,9 @@ mi_new_thread (struct thread_info *t)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel,
-			  "thread-created,id=\"%d\",group-id=\"i%d\"",
-			  t->global_num, t->inf->num);
+      gdb_printf (mi->event_channel,
+		  "thread-created,id=\"%d\",group-id=\"i%d\"",
+		  t->global_num, t->inf->num);
       gdb_flush (mi->event_channel);
     }
 }
@@ -366,9 +362,6 @@ mi_new_thread (struct thread_info *t)
 static void
 mi_thread_exit (struct thread_info *t, int silent)
 {
-  if (silent)
-    return;
-
   SWITCH_THRU_ALL_UIS ()
     {
       struct mi_interp *mi = as_mi_interp (top_level_interpreter ());
@@ -378,9 +371,9 @@ mi_thread_exit (struct thread_info *t, int silent)
 
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
-      fprintf_unfiltered (mi->event_channel,
-			  "thread-exited,id=\"%d\",group-id=\"i%d\"",
-			  t->global_num, t->inf->num);
+      gdb_printf (mi->event_channel,
+		  "thread-exited,id=\"%d\",group-id=\"i%d\"",
+		  t->global_num, t->inf->num);
       gdb_flush (mi->event_channel);
     }
 }
@@ -405,24 +398,24 @@ mi_record_changed (struct inferior *inferior, int started, const char *method,
 	{
 	  if (format != NULL)
 	    {
-	      fprintf_unfiltered (mi->event_channel,
-				  "record-started,thread-group=\"i%d\","
-				  "method=\"%s\",format=\"%s\"",
-				  inferior->num, method, format);
+	      gdb_printf (mi->event_channel,
+			  "record-started,thread-group=\"i%d\","
+			  "method=\"%s\",format=\"%s\"",
+			  inferior->num, method, format);
 	    }
 	  else
 	    {
-	      fprintf_unfiltered (mi->event_channel,
-				  "record-started,thread-group=\"i%d\","
-				  "method=\"%s\"",
-				  inferior->num, method);
+	      gdb_printf (mi->event_channel,
+			  "record-started,thread-group=\"i%d\","
+			  "method=\"%s\"",
+			  inferior->num, method);
 	    }
 	}
       else
 	{
-	  fprintf_unfiltered (mi->event_channel,
-			      "record-stopped,thread-group=\"i%d\"",
-			      inferior->num);
+	  gdb_printf (mi->event_channel,
+		      "record-stopped,thread-group=\"i%d\"",
+		      inferior->num);
 	}
 
       gdb_flush (mi->event_channel);
@@ -450,9 +443,9 @@ mi_inferior_added (struct inferior *inf)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel,
-			  "thread-group-added,id=\"i%d\"",
-			  inf->num);
+      gdb_printf (mi->event_channel,
+		  "thread-group-added,id=\"i%d\"",
+		  inf->num);
       gdb_flush (mi->event_channel);
     }
 }
@@ -470,9 +463,9 @@ mi_inferior_appeared (struct inferior *inf)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel,
-			  "thread-group-started,id=\"i%d\",pid=\"%d\"",
-			  inf->num, inf->pid);
+      gdb_printf (mi->event_channel,
+		  "thread-group-started,id=\"i%d\",pid=\"%d\"",
+		  inf->num, inf->pid);
       gdb_flush (mi->event_channel);
     }
 }
@@ -491,12 +484,12 @@ mi_inferior_exit (struct inferior *inf)
       target_terminal::ours_for_output ();
 
       if (inf->has_exit_code)
-	fprintf_unfiltered (mi->event_channel,
-			    "thread-group-exited,id=\"i%d\",exit-code=\"%s\"",
-			    inf->num, int_string (inf->exit_code, 8, 0, 0, 1));
+	gdb_printf (mi->event_channel,
+		    "thread-group-exited,id=\"i%d\",exit-code=\"%s\"",
+		    inf->num, int_string (inf->exit_code, 8, 0, 0, 1));
       else
-	fprintf_unfiltered (mi->event_channel,
-			    "thread-group-exited,id=\"i%d\"", inf->num);
+	gdb_printf (mi->event_channel,
+		    "thread-group-exited,id=\"i%d\"", inf->num);
 
       gdb_flush (mi->event_channel);
     }
@@ -515,9 +508,9 @@ mi_inferior_removed (struct inferior *inf)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel,
-			  "thread-group-removed,id=\"i%d\"",
-			  inf->num);
+      gdb_printf (mi->event_channel,
+		  "thread-group-removed,id=\"i%d\"",
+		  inf->num);
       gdb_flush (mi->event_channel);
     }
 }
@@ -682,11 +675,11 @@ mi_on_normal_stop_1 (struct bpstat *bs, int print_frame)
 	mi_uiout->field_signed ("core", core);
     }
   
-  fputs_unfiltered ("*stopped", mi->raw_stdout);
+  gdb_puts ("*stopped", mi->raw_stdout);
   mi_out_put (mi_uiout, mi->raw_stdout);
   mi_out_rewind (mi_uiout);
   mi_print_timing_maybe (mi->raw_stdout);
-  fputs_unfiltered ("\n", mi->raw_stdout);
+  gdb_puts ("\n", mi->raw_stdout);
   gdb_flush (mi->raw_stdout);
 }
 
@@ -721,7 +714,7 @@ mi_about_to_proceed (void)
      have been resumed to collect more information, and it does
      not mean that attach/detach has completed. So we need to
      return early and not send the "running" message to the client. */
-  if (cuda_api_attach_or_detach_in_progress ())
+  if (cuda_debugapi::attach_or_detach_in_progress ())
     return;
 #endif
   mi_proceeded = 1;
@@ -757,11 +750,11 @@ mi_traceframe_changed (int tfnum, int tpnum)
       target_terminal::ours_for_output ();
 
       if (tfnum >= 0)
-	fprintf_unfiltered (mi->event_channel, "traceframe-changed,"
-			    "num=\"%d\",tracepoint=\"%d\"",
-			    tfnum, tpnum);
+	gdb_printf (mi->event_channel, "traceframe-changed,"
+		    "num=\"%d\",tracepoint=\"%d\"",
+		    tfnum, tpnum);
       else
-	fprintf_unfiltered (mi->event_channel, "traceframe-changed,end");
+	gdb_printf (mi->event_channel, "traceframe-changed,end");
 
       gdb_flush (mi->event_channel);
     }
@@ -782,9 +775,9 @@ mi_tsv_created (const struct trace_state_variable *tsv)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel, "tsv-created,"
-			  "name=\"%s\",initial=\"%s\"",
-			  tsv->name.c_str (), plongest (tsv->initial_value));
+      gdb_printf (mi->event_channel, "tsv-created,"
+		  "name=\"%s\",initial=\"%s\"",
+		  tsv->name.c_str (), plongest (tsv->initial_value));
 
       gdb_flush (mi->event_channel);
     }
@@ -806,10 +799,10 @@ mi_tsv_deleted (const struct trace_state_variable *tsv)
       target_terminal::ours_for_output ();
 
       if (tsv != NULL)
-	fprintf_unfiltered (mi->event_channel, "tsv-deleted,"
-			    "name=\"%s\"", tsv->name.c_str ());
+	gdb_printf (mi->event_channel, "tsv-deleted,"
+		    "name=\"%s\"", tsv->name.c_str ());
       else
-	fprintf_unfiltered (mi->event_channel, "tsv-deleted");
+	gdb_printf (mi->event_channel, "tsv-deleted");
 
       gdb_flush (mi->event_channel);
     }
@@ -833,19 +826,17 @@ mi_tsv_modified (const struct trace_state_variable *tsv)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel,
-			  "tsv-modified");
+      gdb_printf (mi->event_channel,
+		  "tsv-modified");
 
-      mi_uiout->redirect (mi->event_channel);
+      ui_out_redirect_pop redir (mi_uiout, mi->event_channel);
 
       mi_uiout->field_string ("name", tsv->name);
       mi_uiout->field_string ("initial",
-			   plongest (tsv->initial_value));
+			      plongest (tsv->initial_value));
       if (tsv->value_known)
 	mi_uiout->field_string ("current", plongest (tsv->value));
 
-      mi_uiout->redirect (NULL);
-
       gdb_flush (mi->event_channel);
     }
 }
@@ -864,7 +855,7 @@ mi_print_breakpoint_for_event (struct mi_interp *mi, breakpoint *bp)
      break if anything is output to mi_uiout prior to calling the
      breakpoint_created notifications.  So, we use
      ui_out_redirect.  */
-  mi_uiout->redirect (mi->event_channel);
+  ui_out_redirect_pop redir (mi_uiout, mi->event_channel);
 
   try
     {
@@ -877,8 +868,6 @@ mi_print_breakpoint_for_event (struct mi_interp *mi, breakpoint *bp)
     {
       exception_print (gdb_stderr, ex);
     }
-
-  mi_uiout->redirect (NULL);
 }
 
 /* Emit notification about a created breakpoint.  */
@@ -902,8 +891,8 @@ mi_breakpoint_created (struct breakpoint *b)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel,
-			  "breakpoint-created");
+      gdb_printf (mi->event_channel,
+		  "breakpoint-created");
       mi_print_breakpoint_for_event (mi, b);
 
       gdb_flush (mi->event_channel);
@@ -931,8 +920,8 @@ mi_breakpoint_deleted (struct breakpoint *b)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel, "breakpoint-deleted,id=\"%d\"",
-			  b->number);
+      gdb_printf (mi->event_channel, "breakpoint-deleted,id=\"%d\"",
+		  b->number);
 
       gdb_flush (mi->event_channel);
     }
@@ -958,8 +947,8 @@ mi_breakpoint_modified (struct breakpoint *b)
 
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
-      fprintf_unfiltered (mi->event_channel,
-			  "breakpoint-modified");
+      gdb_printf (mi->event_channel,
+		  "breakpoint-modified");
       mi_print_breakpoint_for_event (mi, b);
 
       gdb_flush (mi->event_channel);
@@ -976,9 +965,9 @@ mi_output_running (struct thread_info *thread)
       if (mi == NULL)
 	continue;
 
-      fprintf_unfiltered (mi->raw_stdout,
-			  "*running,thread-id=\"%d\"\n",
-			  thread->global_num);
+      gdb_printf (mi->raw_stdout,
+		  "*running,thread-id=\"%d\"\n",
+		  thread->global_num);
     }
 }
 
@@ -1014,8 +1003,8 @@ mi_on_resume_1 (struct mi_interp *mi,
      In future (MI3), we'll be outputting "^done" here.  */
   if (!running_result_record_printed && mi_proceeded)
     {
-      fprintf_unfiltered (mi->raw_stdout, "%s^running\n",
-			  current_token ? current_token : "");
+      gdb_printf (mi->raw_stdout, "%s^running\n",
+		  current_token ? current_token : "");
     }
 
   /* Backwards compatibility.  If doing a wildcard resume and there's
@@ -1023,7 +1012,7 @@ mi_on_resume_1 (struct mi_interp *mi,
      thread individually.  */
   if ((ptid == minus_one_ptid || ptid.is_pid ())
       && !multiple_inferiors_p ())
-    fprintf_unfiltered (mi->raw_stdout, "*running,thread-id=\"all\"\n");
+    gdb_printf (mi->raw_stdout, "*running,thread-id=\"all\"\n");
   else
     for (thread_info *tp : all_non_exited_threads (targ, ptid))
       mi_output_running (tp);
@@ -1035,7 +1024,7 @@ mi_on_resume_1 (struct mi_interp *mi,
 	 even if it cannot actually accept any input.  This will be
 	 surely removed for MI3, and may be removed even earlier.  */
       if (current_ui->prompt_state == PROMPT_BLOCKED)
-	fputs_unfiltered ("(gdb) \n", mi->raw_stdout);
+	gdb_puts ("(gdb) \n", mi->raw_stdout);
     }
   gdb_flush (mi->raw_stdout);
 }
@@ -1108,14 +1097,12 @@ mi_solib_loaded (struct so_list *solib)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel, "library-loaded");
+      gdb_printf (mi->event_channel, "library-loaded");
 
-      uiout->redirect (mi->event_channel);
+      ui_out_redirect_pop redir (uiout, mi->event_channel);
 
       mi_output_solib_attribs (uiout, solib);
 
-      uiout->redirect (NULL);
-
       gdb_flush (mi->event_channel);
     }
 }
@@ -1136,9 +1123,9 @@ mi_solib_unloaded (struct so_list *solib)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel, "library-unloaded");
+      gdb_printf (mi->event_channel, "library-unloaded");
 
-      uiout->redirect (mi->event_channel);
+      ui_out_redirect_pop redir (uiout, mi->event_channel);
 
       uiout->field_string ("id", solib->so_original_name);
       uiout->field_string ("target-name", solib->so_original_name);
@@ -1148,8 +1135,6 @@ mi_solib_unloaded (struct so_list *solib)
 	  uiout->field_fmt ("thread-group", "i%d", current_inferior ()->num);
 	}
 
-      uiout->redirect (NULL);
-
       gdb_flush (mi->event_channel);
     }
 }
@@ -1175,15 +1160,13 @@ mi_command_param_changed (const char *param, const char *value)
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel, "cmd-param-changed");
+      gdb_printf (mi->event_channel, "cmd-param-changed");
 
-      mi_uiout->redirect (mi->event_channel);
+      ui_out_redirect_pop redir (mi_uiout, mi->event_channel);
 
       mi_uiout->field_string ("param", param);
       mi_uiout->field_string ("value", value);
 
-      mi_uiout->redirect (NULL);
-
       gdb_flush (mi->event_channel);
     }
 }
@@ -1211,9 +1194,9 @@ mi_memory_changed (struct inferior *inferior, CORE_ADDR memaddr,
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
 
-      fprintf_unfiltered (mi->event_channel, "memory-changed");
+      gdb_printf (mi->event_channel, "memory-changed");
 
-      mi_uiout->redirect (mi->event_channel);
+      ui_out_redirect_pop redir (mi_uiout, mi->event_channel);
 
       mi_uiout->field_fmt ("thread-group", "i%d", inferior->num);
       mi_uiout->field_core_addr ("addr", target_gdbarch (), memaddr);
@@ -1230,8 +1213,6 @@ mi_memory_changed (struct inferior *inferior, CORE_ADDR memaddr,
 	    mi_uiout->field_string ("type", "code");
 	}
 
-      mi_uiout->redirect (NULL);
-
       gdb_flush (mi->event_channel);
     }
 }
@@ -1263,8 +1244,7 @@ mi_user_selected_context_changed (user_selected_what selection)
 
       mi_uiout = top_level_interpreter ()->interp_ui_out ();
 
-      mi_uiout->redirect (mi->event_channel);
-      ui_out_redirect_pop redirect_popper (mi_uiout);
+      ui_out_redirect_pop redirect_popper (mi_uiout, mi->event_channel);
 
       target_terminal::scoped_restore_terminal_state term_state;
       target_terminal::ours_for_output ();
@@ -1277,9 +1257,9 @@ mi_user_selected_context_changed (user_selected_what selection)
 	{
 	  print_selected_thread_frame (mi->cli_uiout, selection);
 
-	  fprintf_unfiltered (mi->event_channel,
-			      "thread-selected,id=\"%d\"",
-			      tp->global_num);
+	  gdb_printf (mi->event_channel,
+		      "thread-selected,id=\"%d\"",
+		      tp->global_num);
 
 	  if (tp->state != THREAD_RUNNING)
 	    {
@@ -1312,32 +1292,26 @@ mi_interp::set_logging (ui_file_up logfile, bool logging_redirect,
     {
       mi->saved_raw_stdout = mi->raw_stdout;
 
-      /* If something is being redirected, then grab logfile.  */
-      ui_file *logfile_p = nullptr;
-      if (logging_redirect || debug_redirect)
-	{
-	  logfile_p = logfile.get ();
-	  mi->saved_raw_file_to_delete = logfile_p;
-	}
+      ui_file *logfile_p = logfile.get ();
+      mi->logfile_holder = std::move (logfile);
 
       /* If something is not being redirected, then a tee containing both the
 	 logfile and stdout.  */
       ui_file *tee = nullptr;
       if (!logging_redirect || !debug_redirect)
 	{
-	  tee = new tee_file (mi->raw_stdout, std::move (logfile));
-	  mi->saved_raw_file_to_delete = tee;
+	  tee = new tee_file (mi->raw_stdout, logfile_p);
+	  mi->stdout_holder.reset (tee);
 	}
 
       mi->raw_stdout = logging_redirect ? logfile_p : tee;
-      mi->raw_stdlog = debug_redirect ? logfile_p : tee;
     }
   else
     {
-      delete mi->saved_raw_file_to_delete;
+      mi->logfile_holder.reset ();
+      mi->stdout_holder.reset ();
       mi->raw_stdout = mi->saved_raw_stdout;
       mi->saved_raw_stdout = nullptr;
-      mi->saved_raw_file_to_delete = nullptr;
     }
 
   mi->out->set_raw (mi->raw_stdout);
@@ -1363,6 +1337,7 @@ _initialize_mi_interp ()
   interp_factory_register (INTERP_MI1, mi_interp_factory);
   interp_factory_register (INTERP_MI2, mi_interp_factory);
   interp_factory_register (INTERP_MI3, mi_interp_factory);
+  interp_factory_register (INTERP_MI4, mi_interp_factory);
   interp_factory_register (INTERP_MI, mi_interp_factory);
 
   gdb::observers::signal_received.attach (mi_on_signal_received, "mi-interp");
