@@ -1,6 +1,6 @@
 /* nto-tdep.c - general QNX Neutrino target functionality.
 
-   Copyright (C) 2003-2022 Free Software Foundation, Inc.
+   Copyright (C) 2003-2023 Free Software Foundation, Inc.
 
    Contributed by QNX Software Systems Ltd.
 
@@ -91,12 +91,11 @@ static char default_nto_target[] = "";
 int nto_internal_debugging;
 int nto_stop_on_thread_events;
 static char default_nto_target[] = "";
-static struct gdbarch_data *nto_gdbarch_data_handle;
 #else
 struct nto_target_ops current_nto_target;
 #endif
 
-static const struct inferior_key<struct nto_inferior_data>
+static const registry<inferior>::key<struct nto_inferior_data>
   nto_inferior_data_reg;
 
 #ifdef NVIDIA_CUDA_GDB
@@ -115,15 +114,17 @@ nto_thread_info::nto_thread_info ()
 nto_thread_info::~nto_thread_info ()
 {
 }
-static void *
-init_nto_gdbarch_data (struct gdbarch *gdbarch)
-{
-  return GDBARCH_OBSTACK_ZALLOC (gdbarch, struct nto_gdbarch_data);
-}
-static struct nto_gdbarch_data *
+
+/* Per-architecture data key.  */
+static const registry<gdbarch>::key<nto_gdbarch_data> nto_gdbarch_data_data;
+/* Get or initialize the nto gdbarch data.  */
+static nto_gdbarch_data *
 get_nto_gdbarch_data (struct gdbarch *gdbarch)
 {
-  return (struct nto_gdbarch_data *) gdbarch_data (gdbarch, nto_gdbarch_data_handle);
+  nto_gdbarch_data *result = nto_gdbarch_data_data.get (gdbarch);
+  if (result == nullptr)
+    result = nto_gdbarch_data_data.emplace (gdbarch);
+  return result;
 }
 struct type *
 nto_get_siginfo_type (struct gdbarch *gdbarch)
@@ -149,21 +150,21 @@ nto_get_siginfo_type (struct gdbarch *gdbarch)
   sigval_type->set_name ("union sigval");
   append_composite_type_field (sigval_type, "sival_int", int_type);
   append_composite_type_field_aligned (sigval_type, "sival_ptr",
-				       void_ptr_type, TYPE_LENGTH (long_type));
+				       void_ptr_type, long_type->length ());
   /* pid_t */
   pid_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
-			TYPE_LENGTH (int_type) * TARGET_CHAR_BIT, "pid_t");
-  TYPE_TARGET_TYPE (pid_type) = int_type;
+			int_type->length () * TARGET_CHAR_BIT, "pid_t");
+  pid_type->set_target_type (int_type);
   pid_type->set_is_stub (true);
   /* uid_t */
   uid_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
-			TYPE_LENGTH (uint_type) * TARGET_CHAR_BIT, "uid_t");
-  TYPE_TARGET_TYPE (uid_type) = int_type;
+			uint_type->length () * TARGET_CHAR_BIT, "uid_t");
+  uid_type->set_target_type (int_type);
   uid_type->set_is_stub (true);
   /* clock_t */
   clock_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
-			  TYPE_LENGTH (uint_type) * TARGET_CHAR_BIT, "clock_t");
-  TYPE_TARGET_TYPE (clock_type) = uint_type;
+			  uint_type->length () * TARGET_CHAR_BIT, "clock_t");
+  clock_type->set_target_type (uint_type);
   clock_type->set_is_stub (true);
   /* __data */
   sidata_type = arch_composite_type (gdbarch, NULL, TYPE_CODE_UNION);
@@ -186,22 +187,22 @@ nto_get_siginfo_type (struct gdbarch *gdbarch)
   append_composite_type_field (type, "__status", int_type);
   append_composite_type_field (type, "__stime", clock_type);
   append_composite_type_field_aligned (sipdata_type, "__chld", type,
-				       TYPE_LENGTH (long_type));
+				       long_type->length ());
   append_composite_type_field_aligned (siproc_type, "__pdata", sipdata_type,
-				       TYPE_LENGTH (long_type));
+				       long_type->length ());
   append_composite_type_field_aligned (sidata_type, "__proc", siproc_type,
-				       TYPE_LENGTH (long_type));
+				       long_type->length ());
   /* __data.__fault */
   type = arch_composite_type (gdbarch, NULL, TYPE_CODE_STRUCT);
   append_composite_type_field (type, "__fltno", int_type);
   append_composite_type_field_aligned (type, "__fltip", void_ptr_type,
-				       TYPE_LENGTH (long_type));
+				       long_type->length ());
   append_composite_type_field_aligned (type, "__addr", void_ptr_type,
-				       TYPE_LENGTH (long_type));
+				       long_type->length ());
   append_composite_type_field_aligned (type, "__bdslot", int_type,
-				       TYPE_LENGTH (long_type));
+				       long_type->length ());
   append_composite_type_field_aligned (sidata_type, "__fault", type,
-				       TYPE_LENGTH (long_type));
+				       long_type->length ());
   /* struct siginfo */
   siginfo_type = arch_composite_type (gdbarch, NULL, TYPE_CODE_STRUCT);
   siginfo_type->set_name ("siginfo");
@@ -210,7 +211,7 @@ nto_get_siginfo_type (struct gdbarch *gdbarch)
   append_composite_type_field (siginfo_type, "si_errno", int_type);
   append_composite_type_field_aligned (siginfo_type,
 				       "__data", sidata_type,
-				       TYPE_LENGTH (long_type));
+				       long_type->length ());
   nto_gdbarch_data->siginfo_type = siginfo_type;
   return siginfo_type;
 }
@@ -801,7 +802,7 @@ static void
 show_nto_debug (struct ui_file *file, int from_tty,
                 struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("QNX NTO debug level is %d.\n"), nto_internal_debugging);
+  gdb_printf (file, _("QNX NTO debug level is %d.\n"), nto_internal_debugging);
 }
 static int
 nto_print_tidinfo_callback (struct thread_info *tp, void *data)
@@ -819,7 +820,7 @@ nto_print_tidinfo_callback (struct thread_info *tp, void *data)
 	  state = info->state;
 	  flags = info->flags;
 	}
-      printf_filtered ("%c%d\t%d\t%d\n", star, info->tid, state, flags);
+      gdb_printf ("%c%d\t%d\t%d\n", star, info->tid, state, flags);
     }
   return 0;
 }
@@ -830,7 +831,7 @@ nto_info_tidinfo_command (const char *args, int from_tty)
   nto_trace (0) ("%s (args=%s, from_tty=%d)\n", __func__,
 		  args ? args : "(null)", from_tty);
   target_update_thread_list ();
-  printf_filtered("Threads for pid %d (%s)\nTid:\tState:\tFlags:\n",
+  gdb_printf("Threads for pid %d (%s)\nTid:\tState:\tFlags:\n",
 		  inferior_ptid.pid (), execfile ? execfile : "");
   iterate_over_threads (nto_print_tidinfo_callback, NULL);
 }
@@ -1075,13 +1076,16 @@ nto_solib_added_listener (struct so_list *solib)
       mem_phdr_addr += sizeof_Elf_Phdr;
     }
 }
-struct gdbarch_data *nto_gdbarch_ops;
-static void *
-nto_gdbarch_init (struct obstack *obstack)
+/* Per-architecture data key.  */
+static const registry<gdbarch>::key<nto_target_ops> nto_gdbarch_ops_data;
+/* Get or initialize the nto target ops.  */
+nto_target_ops *
+get_nto_target_ops (struct gdbarch *gdbarch)
 {
-  struct nto_target_ops *ops;
-  ops = OBSTACK_ZALLOC (obstack, struct nto_target_ops);
-  return ops;
+  nto_target_ops *result = nto_gdbarch_ops_data.get (gdbarch);
+  if (result == nullptr)
+    result = nto_gdbarch_ops_data.emplace (gdbarch);
+  return result;
 }
 #endif /* NVIDIA_CUDA_GDB */
 /* Read AUXV from initial_stack.  */
@@ -1278,7 +1282,6 @@ void _initialize_nto_tdep ();
 void
 _initialize_nto_tdep ()
 {
-  nto_gdbarch_ops = gdbarch_data_register_pre_init (nto_gdbarch_init);
   nto_trace (0) ("%s ()\n", __func__);
   add_setshow_zinteger_cmd ("nto-debug", class_maintenance,
                             &nto_internal_debugging, _("\
@@ -1301,7 +1304,5 @@ When set to 1, stop on thread created and thread destroyed events.\n"),
                             &setlist, &showlist);
   add_info ("tidinfo", nto_info_tidinfo_command, "List threads for current process." );
   gdb::observers::solib_loaded.attach (nto_solib_added_listener, "nto-tdep");
-  nto_gdbarch_data_handle =
-    gdbarch_data_register_post_init (init_nto_gdbarch_data);
 }
 #endif
