@@ -1,6 +1,6 @@
 /* GDB parameters implemented in Python
 
-   Copyright (C) 2008-2022 Free Software Foundation, Inc.
+   Copyright (C) 2008-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -28,6 +28,14 @@
 #include "language.h"
 #include "arch-utils.h"
 
+#ifdef NVIDIA_PYTHON_DYNLIB
+/* Redefine NULL to nullptr to avoid warnings due to passing
+ * NULL to long int with the generated wrappers. */
+#undef NULL
+#define NULL nullptr
+#endif
+
+
 /* Parameter constants and their values.  */
 static struct {
   const char *name;
@@ -125,12 +133,8 @@ static PyObject *show_doc_cst;
 static PyObject *
 get_attr (PyObject *obj, PyObject *attr_name)
 {
-  if (gdbpy_StringCheck (attr_name)
-#ifdef IS_PY3K
+  if (PyUnicode_Check (attr_name)
       && ! PyUnicode_CompareWithASCIIString (attr_name, "value"))
-#else
-      && ! strcmp (PyString_AsString (attr_name), "value"))
-#endif
     {
       parmpy_object *self = (parmpy_object *) obj;
 
@@ -155,18 +159,14 @@ set_parameter_value (parmpy_object *self, PyObject *value)
     case var_filename:
       if (! gdbpy_is_string (value)
 	  && (self->type == var_filename
-	      || value != gdbpy_None))
+	      || value != Py_None))
 	{
-	  PyErr_SetString (gdbpyExc_RuntimeError,
+	  PyErr_SetString (PyExc_RuntimeError,
 			   _("String required for filename."));
 
 	  return -1;
 	}
-#ifndef NVIDIA_CUDA_GDB
       if (value == Py_None)
-#else
-      if (value == gdbpy_None)
-#endif
 	self->value.stringval->clear ();
       else
 	{
@@ -185,7 +185,7 @@ set_parameter_value (parmpy_object *self, PyObject *value)
 
 	if (! gdbpy_is_string (value))
 	  {
-	    PyErr_SetString (gdbpyExc_RuntimeError,
+	    PyErr_SetString (PyExc_RuntimeError,
 			     _("ENUM arguments must be a string."));
 	    return -1;
 	  }
@@ -199,7 +199,7 @@ set_parameter_value (parmpy_object *self, PyObject *value)
 	    break;
 	if (! self->enumeration[i])
 	  {
-	    PyErr_SetString (gdbpyExc_RuntimeError,
+	    PyErr_SetString (PyExc_RuntimeError,
 			     _("The value must be member of an enumeration."));
 	    return -1;
 	  }
@@ -208,9 +208,9 @@ set_parameter_value (parmpy_object *self, PyObject *value)
       }
 
     case var_boolean:
-      if (Py_TYPE(value) != gdbpy_BoolType)
+      if (! PyBool_Check (value))
 	{
-	  PyErr_SetString (gdbpyExc_RuntimeError,
+	  PyErr_SetString (PyExc_RuntimeError,
 			   _("A boolean argument is required."));
 	  return -1;
 	}
@@ -221,14 +221,14 @@ set_parameter_value (parmpy_object *self, PyObject *value)
       break;
 
     case var_auto_boolean:
-      if (Py_TYPE(value) != gdbpy_BoolType && value != gdbpy_None)
+      if (! PyBool_Check (value) && value != Py_None)
 	{
-	  PyErr_SetString (gdbpyExc_RuntimeError,
+	  PyErr_SetString (PyExc_RuntimeError,
 			   _("A boolean or None is required"));
 	  return -1;
 	}
 
-      if (value == gdbpy_None)
+      if (value == Py_None)
 	self->value.autoboolval = AUTO_BOOLEAN_AUTO;
       else
 	{
@@ -251,14 +251,18 @@ set_parameter_value (parmpy_object *self, PyObject *value)
 	long l;
 	int ok;
 
-	if (! gdbpy_IntCheck (value))
+	if (value == Py_None
+	    && (self->type == var_uinteger || self->type == var_integer))
+	  l = 0;
+	else if (value == Py_None && self->type == var_zuinteger_unlimited)
+	  l = -1;
+	else if (!PyLong_Check (value))
 	  {
-	    PyErr_SetString (gdbpyExc_RuntimeError,
+	    PyErr_SetString (PyExc_RuntimeError,
 			     _("The value must be integer."));
 	    return -1;
 	  }
-
-	if (! gdb_py_int_as_long (value, &l))
+	else if (! gdb_py_int_as_long (value, &l))
 	  return -1;
 
 	switch (self->type)
@@ -291,7 +295,7 @@ set_parameter_value (parmpy_object *self, PyObject *value)
 
 	if (! ok)
 	  {
-	    PyErr_SetString (gdbpyExc_RuntimeError,
+	    PyErr_SetString (PyExc_RuntimeError,
 			     _("Range exceeded."));
 	    return -1;
 	  }
@@ -304,7 +308,7 @@ set_parameter_value (parmpy_object *self, PyObject *value)
       }
 
     default:
-      PyErr_SetString (gdbpyExc_RuntimeError,
+      PyErr_SetString (PyExc_RuntimeError,
 		       _("Unhandled type in parameter value."));
       return -1;
     }
@@ -316,16 +320,12 @@ set_parameter_value (parmpy_object *self, PyObject *value)
 static int
 set_attr (PyObject *obj, PyObject *attr_name, PyObject *val)
 {
-  if (gdbpy_StringCheck (attr_name)
-#ifdef IS_PY3K
+  if (PyUnicode_Check (attr_name)
       && ! PyUnicode_CompareWithASCIIString (attr_name, "value"))
-#else
-      && ! strcmp (PyString_AsString (attr_name), "value"))
-#endif
     {
       if (!val)
 	{
-	  PyErr_SetString (gdbpyExc_RuntimeError,
+	  PyErr_SetString (PyExc_RuntimeError,
 			   _("Cannot delete a parameter's value."));
 	  return -1;
 	}
@@ -397,6 +397,8 @@ get_doc_string (PyObject *object, enum doc_string_type doc_type,
 	  result = python_string_to_host_string (ds_obj.get ());
 	  if (result == NULL)
 	    gdbpy_print_stack ();
+	  else if (doc_type == doc_string_description)
+	    result = gdbpy_fix_doc_string_indentation (std::move (result));
 	}
     }
 
@@ -425,7 +427,7 @@ static gdb::unique_xmalloc_ptr<char>
 call_doc_function (PyObject *obj, PyObject *method, PyObject *arg)
 {
   gdb::unique_xmalloc_ptr<char> data;
-  gdbpy_ref<> result (gdbpy_PyObject_CallMethodObjArgs (obj, method, arg, NULL));
+  gdbpy_ref<> result (PyObject_CallMethodObjArgs (obj, method, arg, NULL));
 
   if (result == NULL)
     return NULL;
@@ -438,7 +440,7 @@ call_doc_function (PyObject *obj, PyObject *method, PyObject *arg)
     }
   else
     {
-      PyErr_SetString (gdbpyExc_RuntimeError,
+      PyErr_SetString (PyExc_RuntimeError,
 		       _("Parameter must return a string value."));
       return NULL;
     }
@@ -459,7 +461,7 @@ get_set_value (const char *args, int from_tty,
   gdb::unique_xmalloc_ptr<char> set_doc_string;
 
   gdbpy_enter enter_py;
-  gdbpy_ref<> set_doc_func (PyString_FromString ("get_set_string"));
+  gdbpy_ref<> set_doc_func (PyUnicode_FromString ("get_set_string"));
 
   if (set_doc_func == NULL)
     {
@@ -476,7 +478,7 @@ get_set_value (const char *args, int from_tty,
 
   const char *str = set_doc_string.get ();
   if (str != nullptr && str[0] != '\0')
-    printf_filtered ("%s\n", str);
+    gdb_printf ("%s\n", str);
 }
 
 /* A callback function that is registered against the respective
@@ -494,7 +496,7 @@ get_show_value (struct ui_file *file, int from_tty,
   gdb::unique_xmalloc_ptr<char> show_doc_string;
 
   gdbpy_enter enter_py;
-  gdbpy_ref<> show_doc_func (PyString_FromString ("get_show_string"));
+  gdbpy_ref<> show_doc_func (PyUnicode_FromString ("get_show_string"));
 
   if (show_doc_func == NULL)
     {
@@ -504,7 +506,7 @@ get_show_value (struct ui_file *file, int from_tty,
 
   if (PyObject_HasAttr (obj, show_doc_func.get ()))
     {
-      gdbpy_ref<> val_obj (PyString_FromString (value));
+      gdbpy_ref<> val_obj (PyUnicode_FromString (value));
 
       if (val_obj == NULL)
 	{
@@ -520,7 +522,7 @@ get_show_value (struct ui_file *file, int from_tty,
 	  return;
 	}
 
-      fprintf_filtered (file, "%s\n", show_doc_string.get ());
+      gdb_printf (file, "%s\n", show_doc_string.get ());
     }
   else
     {
@@ -531,8 +533,8 @@ get_show_value (struct ui_file *file, int from_tty,
 	 show_doc adjustment is not i18n friendly, so, instead, we just
 	 print this generic string.  */
       std::string cmd_path = full_cmd_name_without_first_prefix (c);
-      fprintf_filtered (file, _("The current value of '%s' is \"%s\".\n"),
-				cmd_path.c_str (), value);
+      gdb_printf (file, _("The current value of '%s' is \"%s\".\n"),
+		  cmd_path.c_str (), value);
     }
 }
 
@@ -668,14 +670,14 @@ compute_enum_values (parmpy_object *self, PyObject *enum_values)
 
   if (! enum_values)
     {
-      PyErr_SetString (gdbpyExc_RuntimeError,
+      PyErr_SetString (PyExc_RuntimeError,
 		       _("An enumeration is required for PARAM_ENUM."));
       return 0;
     }
 
   if (! PySequence_Check (enum_values))
     {
-      PyErr_SetString (gdbpyExc_RuntimeError,
+      PyErr_SetString (PyExc_RuntimeError,
 		       _("The enumeration is not a sequence."));
       return 0;
     }
@@ -685,7 +687,7 @@ compute_enum_values (parmpy_object *self, PyObject *enum_values)
     return 0;
   if (size == 0)
     {
-      PyErr_SetString (gdbpyExc_RuntimeError,
+      PyErr_SetString (PyExc_RuntimeError,
 		       _("The enumeration is empty."));
       return 0;
     }
@@ -701,7 +703,7 @@ compute_enum_values (parmpy_object *self, PyObject *enum_values)
 	return 0;
       if (! gdbpy_is_string (item.get ()))
 	{
-	  PyErr_SetString (gdbpyExc_RuntimeError,
+	  PyErr_SetString (PyExc_RuntimeError,
 			   _("The enumeration item not a string."));
 	  return 0;
 	}
@@ -747,7 +749,7 @@ parmpy_init (PyObject *self, PyObject *args, PyObject *kwds)
   PyObject *enum_values = NULL;
   struct cmd_list_element **set_list, **show_list;
 
-  if (! gdbpy_PyArg_ParseTuple (args, "sii|O", &name, &cmdtype, &parmclass,
+  if (! PyArg_ParseTuple (args, "sii|O", &name, &cmdtype, &parmclass,
 			  &enum_values))
     return -1;
 
@@ -758,7 +760,7 @@ parmpy_init (PyObject *self, PyObject *args, PyObject *kwds)
       && cmdtype != class_trace && cmdtype != class_obscure
       && cmdtype != class_maintenance)
     {
-      gdbpy_ErrFormat (gdbpyExc_RuntimeError, _("Invalid command class argument."));
+      PyErr_Format (PyExc_RuntimeError, _("Invalid command class argument."));
       return -1;
     }
 
@@ -770,14 +772,14 @@ parmpy_init (PyObject *self, PyObject *args, PyObject *kwds)
       && parmclass != var_zinteger && parmclass != var_zuinteger
       && parmclass != var_zuinteger_unlimited && parmclass != var_enum)
     {
-      PyErr_SetString (gdbpyExc_RuntimeError,
+      PyErr_SetString (PyExc_RuntimeError,
 		       _("Invalid parameter class argument."));
       return -1;
     }
 
   if (enum_values && parmclass != var_enum)
     {
-      PyErr_SetString (gdbpyExc_RuntimeError,
+      PyErr_SetString (PyExc_RuntimeError,
 		       _("Only PARAM_ENUM accepts a fourth argument."));
       return -1;
     }
@@ -847,10 +849,10 @@ gdbpy_initialize_parameters (void)
   if (PyType_Ready (&parmpy_object_type) < 0)
     return -1;
 
-  set_doc_cst = PyString_FromString ("set_doc");
+  set_doc_cst = PyUnicode_FromString ("set_doc");
   if (! set_doc_cst)
     return -1;
-  show_doc_cst = PyString_FromString ("show_doc");
+  show_doc_cst = PyUnicode_FromString ("show_doc");
   if (! show_doc_cst)
     return -1;
 
