@@ -1,6 +1,6 @@
 /* Solaris threads debugging interface.
 
-   Copyright (C) 1996-2022 Free Software Foundation, Inc.
+   Copyright (C) 1996-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -418,8 +418,8 @@ sol_thread_target::resume (ptid_t ptid, int step, enum gdb_signal signo)
       if (ptid.pid () == -2)		/* Inactive thread.  */
 	error (_("This version of Solaris can't start inactive threads."));
       if (info_verbose && ptid.pid () == -1)
-	warning (_("Specified thread %ld seems to have terminated"),
-		 save_ptid.tid ());
+	warning (_("Specified thread %s seems to have terminated"),
+		 pulongest (save_ptid.tid ()));
     }
 
   beneath ()->resume (ptid, step, signo);
@@ -440,8 +440,8 @@ sol_thread_target::wait (ptid_t ptid, struct target_waitstatus *ourstatus,
       if (ptid.pid () == -2)		/* Inactive thread.  */
 	error (_("This version of Solaris can't start inactive threads."));
       if (info_verbose && ptid.pid () == -1)
-	warning (_("Specified thread %ld seems to have terminated"),
-		 ptid_for_warning.tid ());
+	warning (_("Specified thread %s seems to have terminated"),
+		 pulongest (ptid_for_warning.tid ()));
     }
 
   ptid_t rtnval = beneath ()->wait (ptid, ourstatus, options);
@@ -645,7 +645,7 @@ check_for_thread_db (void)
       break;
 
     case TD_OK:
-      printf_unfiltered (_("[Thread debugging using libthread_db enabled]\n"));
+      gdb_printf (_("[Thread debugging using libthread_db enabled]\n"));
 
       /* The thread library was detected.  Activate the sol_thread target.  */
       current_inferior ()->push_target (&sol_thread_ops);
@@ -774,7 +774,7 @@ ps_pglobal_lookup (struct ps_prochandle *ph, const char *ld_object_name,
   if (!ms.minsym)
     return PS_NOSYM;
 
-  *ld_symbol_addr = BMSYMBOL_VALUE_ADDRESS (ms);
+  *ld_symbol_addr = ms.value_address ();
   return PS_OK;
 }
 
@@ -889,7 +889,7 @@ ps_plog (const char *fmt, ...)
 
   va_start (args, fmt);
 
-  vfprintf_filtered (gdb_stderr, fmt, args);
+  gdb_vprintf (gdb_stderr, fmt, args);
 }
 
 /* Get size of extra register set.  Currently a noop.  */
@@ -980,14 +980,14 @@ sol_thread_target::pid_to_str (ptid_t ptid)
       lwp = thread_to_lwp (ptid, -2);
 
       if (lwp.pid () == -1)
-	return string_printf ("Thread %ld (defunct)",
-			      ptid.tid ());
+	return string_printf ("Thread %s (defunct)",
+			      pulongest (ptid.tid ()));
       else if (lwp.pid () != -2)
-	return string_printf ("Thread %ld (LWP %ld)",
-			      ptid.tid (), lwp.lwp ());
+	return string_printf ("Thread %s (LWP %ld)",
+			      pulongest (ptid.tid ()), lwp.lwp ());
       else
-	return string_printf ("Thread %ld        ",
-			      ptid.tid ());
+	return string_printf ("Thread %s          ",
+			      pulongest (ptid.tid ()));
     }
   else if (ptid.lwp () != 0)
     return string_printf ("LWP    %ld        ", ptid.lwp ());
@@ -1050,32 +1050,32 @@ info_cb (const td_thrhandle_t *th, void *s)
   ret = p_td_thr_get_info (th, &ti);
   if (ret == TD_OK)
     {
-      printf_filtered ("%s thread #%d, lwp %d, ",
-		       ti.ti_type == TD_THR_SYSTEM ? "system" : "user  ",
-		       ti.ti_tid, ti.ti_lid);
+      gdb_printf ("%s thread #%d, lwp %d, ",
+		  ti.ti_type == TD_THR_SYSTEM ? "system" : "user  ",
+		  ti.ti_tid, ti.ti_lid);
       switch (ti.ti_state)
 	{
 	default:
 	case TD_THR_UNKNOWN:
-	  printf_filtered ("<unknown state>");
+	  gdb_printf ("<unknown state>");
 	  break;
 	case TD_THR_STOPPED:
-	  printf_filtered ("(stopped)");
+	  gdb_printf ("(stopped)");
 	  break;
 	case TD_THR_RUN:
-	  printf_filtered ("(run)    ");
+	  gdb_printf ("(run)    ");
 	  break;
 	case TD_THR_ACTIVE:
-	  printf_filtered ("(active) ");
+	  gdb_printf ("(active) ");
 	  break;
 	case TD_THR_ZOMBIE:
-	  printf_filtered ("(zombie) ");
+	  gdb_printf ("(zombie) ");
 	  break;
 	case TD_THR_SLEEP:
-	  printf_filtered ("(asleep) ");
+	  gdb_printf ("(asleep) ");
 	  break;
 	case TD_THR_STOPPED_ASLEEP:
-	  printf_filtered ("(stopped asleep)");
+	  gdb_printf ("(stopped asleep)");
 	  break;
 	}
       /* Print thr_create start function.  */
@@ -1084,10 +1084,10 @@ info_cb (const td_thrhandle_t *th, void *s)
 	  const struct bound_minimal_symbol msym
 	    = lookup_minimal_symbol_by_pc (ti.ti_startfunc);
 
-	  printf_filtered ("   startfunc=%s",
-			   msym.minsym
-			   ? msym.minsym->print_name ()
-			   : paddress (target_gdbarch (), ti.ti_startfunc));
+	  gdb_printf ("   startfunc=%s",
+		      msym.minsym
+		      ? msym.minsym->print_name ()
+		      : paddress (target_gdbarch (), ti.ti_startfunc));
 	}
 
       /* If thread is asleep, print function that went to sleep.  */
@@ -1096,13 +1096,13 @@ info_cb (const td_thrhandle_t *th, void *s)
 	  const struct bound_minimal_symbol msym
 	    = lookup_minimal_symbol_by_pc (ti.ti_pc);
 
-	  printf_filtered ("   sleepfunc=%s",
-			   msym.minsym
-			   ? msym.minsym->print_name ()
-			   : paddress (target_gdbarch (), ti.ti_pc));
+	  gdb_printf ("   sleepfunc=%s",
+		      msym.minsym
+		      ? msym.minsym->print_name ()
+		      : paddress (target_gdbarch (), ti.ti_pc));
 	}
 
-      printf_filtered ("\n");
+      gdb_printf ("\n");
     }
   else
     warning (_("info sol-thread: failed to get info for thread."));
@@ -1201,7 +1201,7 @@ _initialize_sol_thread ()
   return;
 
  die:
-  fprintf_unfiltered (gdb_stderr, "\
+  gdb_printf (gdb_stderr, "\
 [GDB will not be able to debug user-mode threads: %s]\n", dlerror ());
 
   if (dlhandle)
