@@ -1,6 +1,6 @@
 /* Multi-process/thread control for GDB, the GNU debugger.
 
-   Copyright (C) 1986-2022 Free Software Foundation, Inc.
+   Copyright (C) 1986-2023 Free Software Foundation, Inc.
 
    Contributed by Lynx Real-Time Systems, Inc.  Los Gatos, CA.
 
@@ -67,7 +67,7 @@ static void
 show_debug_threads (struct ui_file *file, int from_tty,
 		    struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Thread debugging is \"%s\".\n"), value);
+  gdb_printf (file, _("Thread debugging is \"%s\".\n"), value);
 }
 
 /* Definition of struct thread_info exported to gdbthread.h.  */
@@ -93,6 +93,13 @@ inferior_thread (void)
   gdb_assert (current_thread_ != nullptr);
   return current_thread_;
 }
+#ifdef NVIDIA_BUGFIX
+bool
+has_inferior_thread (void)
+{
+  return current_thread_ != nullptr;
+}
+#endif
 
 /* Delete the breakpoint pointed at by BP_P, if there's one.  */
 
@@ -301,7 +308,7 @@ add_thread_with_info (process_stratum_target *targ, ptid_t ptid,
   result->priv.reset (priv);
 
   if (print_thread_events)
-    printf_unfiltered (_("[New %s]\n"), target_pid_to_str (ptid).c_str ());
+    gdb_printf (_("[New %s]\n"), target_pid_to_str (ptid).c_str ());
 
   annotate_new_thread ();
   return result;
@@ -710,8 +717,7 @@ switch_to_thread_if_alive (thread_info *thr)
   /* Need to restore the cuda coords if this is a fake_pid_p thread.
    * This is necessary for cuda coredumps since we don't have a host
    * target available. */
-  cuda_coords_t cur_coords;
-  bool focus_is_device = !cuda_coords_get_current (&cur_coords);
+  bool focus_is_device = cuda_current_focus::isDevice ();
 #endif
   /* Switch inferior first, so that we're looking at the right target
      stack.  */
@@ -720,8 +726,9 @@ switch_to_thread_if_alive (thread_info *thr)
   if (thread_alive (thr))
     {
 #ifdef NVIDIA_CUDA_GDB
+      // Switch focus back to cuda thread
       if (focus_is_device && thr->inf->fake_pid_p)
-	cuda_coords_set_current_no_verify (&cur_coords);
+	cuda_current_focus::forceValid ();
 #endif
       switch_to_thread (thr);
       restore_thread.dont_restore ();
@@ -853,6 +860,10 @@ set_running_thread (struct thread_info *tp, bool running)
     started = true;
   tp->state = running ? THREAD_RUNNING : THREAD_STOPPED;
 
+  threads_debug_printf ("thread: %s, running? %d%s",
+			tp->ptid.to_string ().c_str (), running,
+			(started ? " (started)" : ""));
+
   if (!running)
     {
       /* If the thread is now marked stopped, remove it from
@@ -1064,8 +1075,7 @@ print_thread_info_1 (struct ui_out *uiout, const char *requested_threads,
 
 #ifdef NVIDIA_CUDA_GDB
   /* Switching to each thread will invalidate the cuda coordinates */
-  cuda_coords_t cur_coords;
-  bool focus_is_device = !cuda_coords_get_current (&cur_coords);
+  bool focus_is_device = cuda_current_focus::isDevice ();
 #endif
   update_thread_list ();
 
@@ -1189,7 +1199,7 @@ print_thread_info_1 (struct ui_out *uiout, const char *requested_threads,
 	   * This is necessary for cuda coredumps since we don't have a host
 	   * target available. */
 	  if (focus_is_device && tp->inf->fake_pid_p)
-	    cuda_coords_set_current_no_verify (&cur_coords);
+	    cuda_current_focus::forceValid ();
 #endif
 	  /* Switch to the thread (and inferior / target).  */
 	  switch_to_thread (tp);
@@ -1346,6 +1356,9 @@ info_threads_command_completer (struct cmd_list_element *ignore,
 void
 switch_to_thread_no_regs (struct thread_info *thread)
 {
+  gdb_assert (thread != nullptr);
+  threads_debug_printf ("thread = %s", thread->ptid.to_string ().c_str ());
+
   struct inferior *inf = thread->inf;
 
   set_current_program_space (inf->pspace);
@@ -1363,10 +1376,12 @@ switch_to_no_thread ()
   if (current_thread_ == nullptr)
     return;
 
+  threads_debug_printf ("thread = NONE");
+
   current_thread_ = nullptr;
   inferior_ptid = null_ptid;
 #ifdef NVIDIA_CUDA_GDB
-  cuda_coords_invalidate_current ();
+  cuda_current_focus::invalidate ();
 #endif
   reinit_frame_cache ();
 }
@@ -1379,7 +1394,13 @@ switch_to_no_thread ()
 static void
 switch_to_thread_1 (thread_info *thr, bool keep_cuda_focus)
 {
-  gdb_assert (thr != NULL);
+  /* FIXME: The original switch_to_thread code treats this as an assertion.
+   * We get into a weird situation at tear down intermittently where the CUDA
+   * thread is trying to restore focus, but the host thread has exited. This
+   * is mainly due to how we hijack the user thread for device focus tracking.
+   * Silently return in this case. */
+  if (thr == NULL)
+    return;
   /* Switch the program space as well, if we can infer it from the now
      current thread.  Otherwise, it's up to the caller to select the
      space it wants.  */
@@ -1391,7 +1412,7 @@ switch_to_thread_1 (thread_info *thr, bool keep_cuda_focus)
      the full switch from happening when changing focus from the
      device to a host thread (as the host thread ID may not have
      changed based on the user's switch). */
-  if (!cuda_focus_is_device () && is_current_thread (thr))
+  if (!cuda_current_focus::isDevice () && is_current_thread (thr))
     return; 
   /* If we switch the focus away from the CUDA device, we will end up
      with regular information from the host machine's threads.  If we
@@ -1400,7 +1421,7 @@ switch_to_thread_1 (thread_info *thr, bool keep_cuda_focus)
   /* CUDA - focus
    * Only invalidate if this isn't the fake_pid_p for cuda coredumps. */
   if (!keep_cuda_focus && !thr->inf->fake_pid_p)
-    cuda_coords_invalidate_current ();
+    cuda_current_focus::invalidate ();
   /* Following two calls is existing switch_to_thread code */
   switch_to_thread_no_regs (thr);
   reinit_frame_cache (); 
@@ -1486,16 +1507,21 @@ scoped_restore_current_thread::restore ()
 
 #ifdef NVIDIA_CUDA_GDB
   /* CUDA - focus */
-  if (m_cuda_coords.valid)
+  if (m_cuda_coords.valid ())
     {
       /* If the device currently is executing, we can't switch to the thread.
        * This will cause an error when we try calling into the debug API.
-       * Instead do a lightweight restore by forcing the coords without updating
-       * state. */
-      if (device_suspended (m_cuda_coords.dev))
-	switch_to_cuda_thread (&m_cuda_coords);
+       * Instead do a lightweight restore by forcing the coords without
+       * updating state. */
+      if (m_inf->pid != 0 && !m_inf->fake_pid_p
+          && cuda_state::device_suspended (
+              m_cuda_coords.physical ().dev ()))
+        {
+          if (const_cast<cuda_coords &> (m_cuda_coords).isValidOnDevice ())
+            switch_to_cuda_thread (m_cuda_coords);
+        }
       else
-	cuda_coords_set_current_no_verify (&m_cuda_coords);
+        cuda_current_focus::set (m_cuda_coords);
     }
 #endif
   /* The running state of the originally selected thread may have
@@ -1521,9 +1547,8 @@ scoped_restore_current_thread::scoped_restore_current_thread ()
 {
   m_inf = inferior_ref::new_reference (current_inferior ());
 #ifdef NVIDIA_CUDA_GDB
-  m_cuda_coords = CUDA_INVALID_COORDS;
   /* CUDA - get current focus */
-  cuda_coords_get_current (&m_cuda_coords);
+  m_cuda_coords = cuda_current_focus::get ();
 #endif
 
   m_lang = current_language->la_language;
@@ -1625,8 +1650,8 @@ thread_try_catch_cmd (thread_info *thr, gdb::optional<int> ada_task,
       if (!flags.silent || cmd_result.length () > 0)
 	{
 	  if (!flags.quiet)
-	    printf_filtered ("%s", thr_header.c_str ());
-	  printf_filtered ("%s", cmd_result.c_str ());
+	    gdb_printf ("%s", thr_header.c_str ());
+	  gdb_printf ("%s", cmd_result.c_str ());
 	}
     }
   catch (const gdb_exception_error &ex)
@@ -1634,9 +1659,9 @@ thread_try_catch_cmd (thread_info *thr, gdb::optional<int> ada_task,
       if (!flags.silent)
 	{
 	  if (!flags.quiet)
-	    printf_filtered ("%s", thr_header.c_str ());
+	    gdb_printf ("%s", thr_header.c_str ());
 	  if (flags.cont)
-	    printf_filtered ("%s\n", ex.what ());
+	    gdb_printf ("%s\n", ex.what ());
 	  else
 	    throw;
 	}
@@ -1949,7 +1974,7 @@ thread_command (const char *tidstr, int from_tty)
 
 #ifdef NVIDIA_CUDA_GDB
       /* CUDA - focus */
-      if (cuda_focus_is_device ())
+      if (cuda_current_focus::isDevice ())
         {
           printf_unfiltered (_("Focus not set on any host thread.\n"));
           return;
@@ -1960,13 +1985,13 @@ thread_command (const char *tidstr, int from_tty)
 	  struct thread_info *tp = inferior_thread ();
 
 	  if (tp->state == THREAD_EXITED)
-	    printf_filtered (_("[Current thread is %s (%s) (exited)]\n"),
-			     print_thread_id (tp),
-			     target_pid_to_str (inferior_ptid).c_str ());
+	    gdb_printf (_("[Current thread is %s (%s) (exited)]\n"),
+			print_thread_id (tp),
+			target_pid_to_str (inferior_ptid).c_str ());
 	  else
-	    printf_filtered (_("[Current thread is %s (%s)]\n"),
-			     print_thread_id (tp),
-			     target_pid_to_str (inferior_ptid).c_str ());
+	    gdb_printf (_("[Current thread is %s (%s)]\n"),
+			print_thread_id (tp),
+			target_pid_to_str (inferior_ptid).c_str ());
 	}
       else
 	error (_("No stack."));
@@ -2035,37 +2060,37 @@ thread_find_command (const char *arg, int from_tty)
 
       if (tp->name () != nullptr && re_exec (tp->name ()))
 	{
-	  printf_filtered (_("Thread %s has name '%s'\n"),
-			   print_thread_id (tp), tp->name ());
+	  gdb_printf (_("Thread %s has name '%s'\n"),
+		      print_thread_id (tp), tp->name ());
 	  match++;
 	}
 
       tmp = target_thread_name (tp);
       if (tmp != NULL && re_exec (tmp))
 	{
-	  printf_filtered (_("Thread %s has target name '%s'\n"),
-			   print_thread_id (tp), tmp);
+	  gdb_printf (_("Thread %s has target name '%s'\n"),
+		      print_thread_id (tp), tmp);
 	  match++;
 	}
 
       std::string name = target_pid_to_str (tp->ptid);
       if (!name.empty () && re_exec (name.c_str ()))
 	{
-	  printf_filtered (_("Thread %s has target id '%s'\n"),
-			   print_thread_id (tp), name.c_str ());
+	  gdb_printf (_("Thread %s has target id '%s'\n"),
+		      print_thread_id (tp), name.c_str ());
 	  match++;
 	}
 
       tmp = target_extra_thread_info (tp);
       if (tmp != NULL && re_exec (tmp))
 	{
-	  printf_filtered (_("Thread %s has extra info '%s'\n"),
-			   print_thread_id (tp), tmp);
+	  gdb_printf (_("Thread %s has extra info '%s'\n"),
+		      print_thread_id (tp), tmp);
 	  match++;
 	}
     }
   if (!match)
-    printf_filtered (_("No threads match '%s'\n"), arg);
+    gdb_printf (_("No threads match '%s'\n"), arg);
 }
 
 /* Print notices when new threads are attached and detached.  */
@@ -2074,9 +2099,9 @@ static void
 show_print_thread_events (struct ui_file *file, int from_tty,
 			  struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file,
-		    _("Printing of thread events is %s.\n"),
-		    value);
+  gdb_printf (file,
+	      _("Printing of thread events is %s.\n"),
+	      value);
 }
 
 /* See gdbthread.h.  */
@@ -2263,6 +2288,22 @@ global_thread_id_make_value (struct gdbarch *gdbarch, struct internalvar *var,
   return thread_num_make_value_helper (gdbarch, 1);
 }
 
+/* Return a new value for the number of non-exited threads in the current
+   inferior.  If there are no threads in the current inferior return a
+   value of 0.  */
+
+static struct value *
+inferior_thread_count_make_value (struct gdbarch *gdbarch,
+				  struct internalvar *var, void *ignore)
+{
+  int int_val = 0;
+
+  if (inferior_ptid != null_ptid)
+    int_val = current_inferior ()->non_exited_threads ().size ();
+
+  return value_from_longest (builtin_type (gdbarch)->builtin_int, int_val);
+}
+
 /* Commands with a prefix of `thread'.  */
 struct cmd_list_element *thread_cmd_list = NULL;
 
@@ -2282,6 +2323,14 @@ static const struct internalvar_funcs gthread_funcs =
   NULL,
 };
 
+/* Implementation of `_inferior_thread_count` convenience variable.  */
+
+static const struct internalvar_funcs inferior_thread_count_funcs =
+{
+  inferior_thread_count_make_value,
+  NULL,
+};
+
 void _initialize_thread ();
 void
 _initialize_thread ()
@@ -2398,4 +2447,6 @@ When on messages about thread creation and deletion are printed."),
 
   create_internalvar_type_lazy ("_thread", &thread_funcs, NULL);
   create_internalvar_type_lazy ("_gthread", &gthread_funcs, NULL);
+  create_internalvar_type_lazy ("_inferior_thread_count",
+				&inferior_thread_count_funcs, NULL);
 }
