@@ -1,6 +1,6 @@
 /* Top level stuff for GDB, the GNU debugger.
 
-   Copyright (C) 1986-2022 Free Software Foundation, Inc.
+   Copyright (C) 1986-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -61,9 +61,12 @@
 #include "gdbarch.h"
 #include "gdbsupport/pathstuff.h"
 #include "cli/cli-style.h"
+#include "pager.h"
 #ifdef NVIDIA_CUDA_GDB
 #include "cuda/cuda-gdb.h"
 #include "cuda/cuda-exceptions.h"
+#include "cuda/cuda-tdep.h"
+#include "cuda/cuda-version.h"
 #endif
 
 /* readline include files.  */
@@ -144,15 +147,11 @@ static void
 show_confirm (struct ui_file *file, int from_tty,
 	      struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Whether to confirm potentially "
-			    "dangerous operations is %s.\n"),
-		    value);
+  gdb_printf (file, _("Whether to confirm potentially "
+		      "dangerous operations is %s.\n"),
+	      value);
 }
 
-/* Current working directory.  */
-
-char *current_directory;
-
 /* The last command line executed on the console.  Used for command
    repetitions when the user enters an empty line.  */
 
@@ -307,29 +306,18 @@ unbuffer_stream (FILE *stream)
 /* See top.h.  */
 
 ui::ui (FILE *instream_, FILE *outstream_, FILE *errstream_)
-  : next (nullptr),
-    num (++highest_ui_num),
-    call_readline (nullptr),
-    input_handler (nullptr),
-    command_editing (0),
-    interp_info (nullptr),
-    async (0),
-    secondary_prompt_depth (0),
+  : num (++highest_ui_num),
     stdin_stream (instream_),
     instream (instream_),
     outstream (outstream_),
     errstream (errstream_),
     input_fd (fileno (instream)),
-    input_interactive_p (ISATTY (instream)),
-    prompt_state (PROMPT_NEEDED),
-    m_gdb_stdout (new stdio_file (outstream)),
+    m_input_interactive_p (ISATTY (instream)),
+    m_gdb_stdout (new pager_file (new stdio_file (outstream))),
     m_gdb_stdin (new stdio_file (instream)),
     m_gdb_stderr (new stderr_file (errstream)),
-    m_gdb_stdlog (m_gdb_stderr),
-    m_current_uiout (nullptr)
+    m_gdb_stdlog (new timestamped_file (m_gdb_stderr))
 {
-  buffer_init (&line_buffer);
-
   unbuffer_stream (instream_);
 
   if (ui_list == NULL)
@@ -424,7 +412,7 @@ new_ui_command (const char *args, int from_tty)
     ui.release ();
   }
 
-  printf_filtered ("New UI allocated\n");
+  gdb_printf ("New UI allocated\n");
 }
 
 /* Handler for SIGHUP.  */
@@ -471,11 +459,11 @@ read_command_file (FILE *stream)
 
   while (ui->instream != NULL && !feof (ui->instream))
     {
-      const char *command;
-
       /* Get a command-line.  This calls the readline package.  */
-      command = command_line_input (NULL, NULL);
-      if (command == NULL)
+      std::string command_buffer;
+      const char *command
+	= command_line_input (command_buffer, nullptr, nullptr);
+      if (command == nullptr)
 	break;
       command_handler (command);
     }
@@ -510,7 +498,7 @@ void
 check_frame_language_change (void)
 {
   static int warned = 0;
-  struct frame_info *frame;
+  frame_info_ptr frame;
 
   /* First make sure that a new frame has been selected, in case the
      command or the hooks changed the program state.  */
@@ -540,7 +528,7 @@ check_frame_language_change (void)
 	  && flang != language_unknown
 	  && flang != current_language->la_language)
 	{
-	  printf_filtered ("%s\n", _(lang_frame_mismatch_warn));
+	  gdb_printf ("%s\n", _(lang_frame_mismatch_warn));
 	  warned = 1;
 	}
     }
@@ -693,7 +681,7 @@ execute_command (const char *p, int from_tty)
 	  std::string prefixname = c->prefixname ();
           std::string prefixname_no_space
 	    = prefixname.substr (0, prefixname.length () - 1);
-	  printf_filtered
+	  gdb_printf
 	    ("\"%s\" must be followed by the name of a subcommand.\n",
 	     prefixname_no_space.c_str ());
 	  help_list (*c->subcommands, prefixname.c_str (), all_commands,
@@ -712,8 +700,14 @@ execute_command (const char *p, int from_tty)
 
       maybe_wait_sync_command_done (was_sync);
 
-      /* If this command has been post-hooked, run the hook last.  */
-      execute_cmd_post_hook (c);
+      /* If this command has been post-hooked, run the hook last.
+	 We need to lookup the command again since during its execution,
+	 a command may redefine itself.  In this case, C pointer
+	 becomes invalid so we need to look it up again.  */
+      const char *cmd2 = cmd_start;
+      c = lookup_cmd (&cmd2, cmdlist, "", nullptr, 1, 1);
+      if (c != nullptr)
+	execute_cmd_post_hook (c);
 
       if (repeat_arguments != NULL && cmd_start == saved_command_line)
 	{
@@ -745,8 +739,7 @@ execute_fn_to_ui_file (struct ui_file *file, std::function<void(void)> fn)
   scoped_restore save_async = make_scoped_restore (&current_ui->async, 0);
 
   {
-    current_uiout->redirect (file);
-    ui_out_redirect_pop redirect_popper (current_uiout);
+    ui_out_redirect_pop redirect_popper (current_uiout, file);
 
     scoped_restore save_stdout
       = make_scoped_restore (&gdb_stdout, file);
@@ -914,7 +907,7 @@ gdb_readline_no_editing (const char *prompt)
       /* Don't use a _filtered function here.  It causes the assumed
 	 character position to be off, since the newline we read from
 	 the user is not accounted for.  */
-      puts_unfiltered (prompt);
+      printf_unfiltered ("%s", prompt);
       gdb_flush (gdb_stdout);
     }
 
@@ -993,11 +986,11 @@ show_write_history_p (struct ui_file *file, int from_tty,
 		      struct cmd_list_element *c, const char *value)
 {
   if (!write_history_p || !history_filename.empty ())
-    fprintf_filtered (file, _("Saving of the history record on exit is %s.\n"),
-		      value);
+    gdb_printf (file, _("Saving of the history record on exit is %s.\n"),
+		value);
   else
-    fprintf_filtered (file, _("Saving of the history is disabled due to "
-			      "the value of 'history filename'.\n"));
+    gdb_printf (file, _("Saving of the history is disabled due to "
+			"the value of 'history filename'.\n"));
 }
 
 /* The variable associated with the "set/show history size"
@@ -1008,8 +1001,8 @@ static void
 show_history_size (struct ui_file *file, int from_tty,
 		   struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("The size of the command history is %s.\n"),
-		    value);
+  gdb_printf (file, _("The size of the command history is %s.\n"),
+	      value);
 }
 
 /* Variable associated with the "history remove-duplicates" option.
@@ -1020,10 +1013,10 @@ static void
 show_history_remove_duplicates (struct ui_file *file, int from_tty,
 				struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file,
-		    _("The number of history entries to look back at for "
-		      "duplicates is %s.\n"),
-		    value);
+  gdb_printf (file,
+	      _("The number of history entries to look back at for "
+		"duplicates is %s.\n"),
+	      value);
 }
 
 /* Implement 'show history filename'.  */
@@ -1032,12 +1025,12 @@ show_history_filename (struct ui_file *file, int from_tty,
 		       struct cmd_list_element *c, const char *value)
 {
   if (!history_filename.empty ())
-    fprintf_filtered (file, _("The filename in which to record "
-			      "the command history is \"%ps\".\n"),
-		      styled_string (file_name_style.style (), value));
+    gdb_printf (file, _("The filename in which to record "
+			"the command history is \"%ps\".\n"),
+		styled_string (file_name_style.style (), value));
   else
-    fprintf_filtered (file, _("There is no filename currently set for "
-			      "recording the command history in.\n"));
+    gdb_printf (file, _("There is no filename currently set for "
+			"recording the command history in.\n"));
 }
 
 /* This is like readline(), but it has some gdb-specific behavior.
@@ -1121,7 +1114,7 @@ public:
     current_ui->secondary_prompt_depth++;
 
     if (m_target_is_async_orig)
-      target_async (0);
+      target_async (false);
   }
 
   ~gdb_readline_wrapper_cleanup ()
@@ -1150,7 +1143,7 @@ public:
     saved_after_char_processing_hook = NULL;
 
     if (m_target_is_async_orig)
-      target_async (1);
+      target_async (true);
   }
 
   DISABLE_COPY_AND_ASSIGN (gdb_readline_wrapper_cleanup);
@@ -1343,23 +1336,23 @@ gdb_safe_append_history (void)
     }
 }
 
-/* Read one line from the command input stream `instream' into a local
-   static buffer.  The buffer is made bigger as necessary.  Returns
-   the address of the start of the line.
+/* Read one line from the command input stream `instream'.
 
-   NULL is returned for end of file.
+   CMD_LINE_BUFFER is a buffer that the function may use to store the result, if
+   it needs to be dynamically-allocated.  Otherwise, it is unused.string
+
+   Return nullptr for end of file.
 
    This routine either uses fancy command line editing or simple input
    as the user has requested.  */
 
 const char *
-command_line_input (const char *prompt_arg, const char *annotation_suffix)
+command_line_input (std::string &cmd_line_buffer, const char *prompt_arg,
+		    const char *annotation_suffix)
 {
-  static struct buffer cmd_line_buffer;
-  static int cmd_line_buffer_initialized;
   struct ui *ui = current_ui;
   const char *prompt = prompt_arg;
-  char *cmd;
+  const char *cmd;
   int from_tty = ui->instream == ui->stdin_stream;
 
   /* The annotation suffix must be non-NULL.  */
@@ -1384,15 +1377,6 @@ command_line_input (const char *prompt_arg, const char *annotation_suffix)
       prompt = local_prompt;
     }
 
-  if (!cmd_line_buffer_initialized)
-    {
-      buffer_init (&cmd_line_buffer);
-      cmd_line_buffer_initialized = 1;
-    }
-
-  /* Starting a new command line.  */
-  cmd_line_buffer.used_size = 0;
-
 #ifdef SIGTSTP
   if (job_control)
     signal (SIGTSTP, handle_sigtstp);
@@ -1405,7 +1389,6 @@ command_line_input (const char *prompt_arg, const char *annotation_suffix)
       /* Make sure that all output has been output.  Some machines may
 	 let you get away with leaving out some of the gdb_flush, but
 	 not all.  */
-      gdb_stdout->wrap_here (0);
       gdb_flush (gdb_stdout);
       gdb_flush (gdb_stderr);
 
@@ -1413,22 +1396,18 @@ command_line_input (const char *prompt_arg, const char *annotation_suffix)
 	++source_line_number;
 
       if (from_tty && annotation_level > 1)
-	{
-	  puts_unfiltered ("\n\032\032pre-");
-	  puts_unfiltered (annotation_suffix);
-	  puts_unfiltered ("\n");
-	}
+	printf_unfiltered ("\n\032\032pre-%s\n", annotation_suffix);
 
       /* Don't use fancy stuff if not talking to stdin.  */
       if (deprecated_readline_hook
 	  && from_tty
-	  && input_interactive_p (current_ui))
+	  && current_ui->input_interactive_p ())
 	{
 	  rl.reset ((*deprecated_readline_hook) (prompt));
 	}
       else if (command_editing_p
 	       && from_tty
-	       && input_interactive_p (current_ui))
+	       && current_ui->input_interactive_p ())
 	{
 	  rl.reset (gdb_readline_wrapper (prompt));
 	}
@@ -1437,7 +1416,7 @@ command_line_input (const char *prompt_arg, const char *annotation_suffix)
 	  rl.reset (gdb_readline_no_editing (prompt));
 	}
 
-      cmd = handle_line_of_input (&cmd_line_buffer, rl.get (),
+      cmd = handle_line_of_input (cmd_line_buffer, rl.get (),
 				  0, annotation_suffix);
       if (cmd == (char *) EOF)
 	{
@@ -1465,69 +1444,100 @@ command_line_input (const char *prompt_arg, const char *annotation_suffix)
 void
 print_gdb_version (struct ui_file *stream, bool interactive)
 {
-#ifdef NVIDIA_CUDA_GDB
-  /* CUDA - version information */
-  cuda_print_message_nvidia_version (stream);
-#endif
   /* From GNU coding standards, first line is meant to be easy for a
      program to parse, and is just canonical program name and version
      number, which starts after last space.  */
 
+#ifdef NVIDIA_CUDA_GDB
+  /* CUDA - cuda-gdb version information printed first. */
+  std::string v_str = string_printf ("NVIDIA (R) cuda-gdb %d.%d",
+                                     cuda_major_version (), cuda_minor_version ());
+  gdb_printf (stream, "%ps\n",
+	      styled_string (version_style.style (), v_str.c_str ()));
+
+  /* Print CUDA-GDB copyright notice as second line. */
+   gdb_printf (stream,
+	       "Portions Copyright (C) 2007-%s NVIDIA Corporation\n",
+	       cuda_current_year ());
+
+  /* Acknowledge upstream gdb version. */
+  std::string gdb_v_str = string_printf ("GNU gdb %s", version);
+  gdb_printf (stream, "Based on %s\n", gdb_v_str.c_str ());
+#else
   std::string v_str = string_printf ("GNU gdb %s%s", PKGVERSION, version);
-  fprintf_filtered (stream, "%ps\n",
-		    styled_string (version_style.style (), v_str.c_str ()));
+  gdb_printf (stream, "%ps\n",
+	      styled_string (version_style.style (), v_str.c_str ()));
+#endif
 
   /* Second line is a copyright notice.  */
 
-  fprintf_filtered (stream,
-		    "Copyright (C) 2022 Free Software Foundation, Inc.\n");
+  gdb_printf (stream,
+	      "Copyright (C) 2023 Free Software Foundation, Inc.\n");
 
   /* Following the copyright is a brief statement that the program is
      free software, that users are free to copy and change it on
      certain conditions, that it is covered by the GNU GPL, and that
      there is no warranty.  */
 
-  fprintf_filtered (stream, "\
-License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\
+  gdb_printf (stream, "\
+License GPLv3+: GNU GPL version 3 or later <%ps>\
 \nThis is free software: you are free to change and redistribute it.\n\
-There is NO WARRANTY, to the extent permitted by law.");
+There is NO WARRANTY, to the extent permitted by law.",
+		    styled_string (file_name_style.style (),
+				   "http://gnu.org/licenses/gpl.html"));
 
   if (!interactive)
     return;
 
-  fprintf_filtered (stream, ("\nType \"show copying\" and "
-			     "\"show warranty\" for details.\n"));
+  gdb_printf (stream, ("\nType \"show copying\" and "
+		       "\"show warranty\" for details.\n"));
 
   /* After the required info we print the configuration information.  */
 
-  fprintf_filtered (stream, "This GDB was configured as \"");
+#ifdef NVIDIA_CUDA_GDB
+  gdb_printf (stream, "This CUDA-GDB was configured as \"");
+#else
+  gdb_printf (stream, "This GDB was configured as \"");
+#endif
   if (strcmp (host_name, target_name) != 0)
     {
-      fprintf_filtered (stream, "--host=%s --target=%s",
-			host_name, target_name);
+      gdb_printf (stream, "--host=%s --target=%s",
+		  host_name, target_name);
     }
   else
     {
-      fprintf_filtered (stream, "%s", host_name);
+      gdb_printf (stream, "%s", host_name);
     }
-  fprintf_filtered (stream, "\".\n");
+  gdb_printf (stream, "\".\n");
 
-  fprintf_filtered (stream, _("Type \"show configuration\" "
-			      "for configuration details.\n"));
+  gdb_printf (stream, _("Type \"show configuration\" "
+			"for configuration details.\n"));
 
   if (REPORT_BUGS_TO[0])
     {
-      fprintf_filtered (stream,
-			_("For bug reporting instructions, please see:\n"));
-      fprintf_filtered (stream, "%s.\n", REPORT_BUGS_TO);
+      gdb_printf (stream,
+		  _("For bug reporting instructions, please see:\n"));
+      gdb_printf (stream, "%ps.\n",
+		  styled_string (file_name_style.style (),
+				 REPORT_BUGS_TO));
     }
-  fprintf_filtered (stream,
-		    _("Find the GDB manual and other documentation \
-resources online at:\n    <http://www.gnu.org/software/gdb/documentation/>."));
-  fprintf_filtered (stream, "\n\n");
-  fprintf_filtered (stream, _("For help, type \"help\".\n"));
-  fprintf_filtered (stream,
-		    _("Type \"apropos word\" to search for commands \
+#ifdef NVIDIA_CUDA_GDB
+  gdb_printf (stream,
+	      _("Find the CUDA-GDB manual and other documentation \
+resources online at:\n    <%ps>."),
+	      styled_string (file_name_style.style (),
+			     "https://docs.nvidia.com/cuda/cuda-gdb/index.html"));
+#else
+  gdb_printf (stream,
+	      _("Find the GDB manual and other documentation \
+resources online at:\n    <%ps>."),
+	      styled_string (file_name_style.style (),
+			     "http://www.gnu.org/software/gdb/documentation/"));
+#endif
+  gdb_printf (stream, "\n\n");
+  gdb_printf (stream, _("For help, type \"help\".\n"));
+  gdb_printf (stream,
+	      _("Type \"apropos word\" to search for commands \
 related to \"word\"."));
 }
 
@@ -1535,178 +1545,195 @@ related to \"word\"."));
 void
 print_gdb_configuration (struct ui_file *stream)
 {
-  fprintf_filtered (stream, _("\
+#ifdef NVIDIA_CUDA_GDB
+  gdb_printf (stream, _("\
+This CUDA-GDB was configured as follows:\n\
+   configure --host=%s --target=%s\n\
+"), host_name, target_name);
+#else
+  gdb_printf (stream, _("\
 This GDB was configured as follows:\n\
    configure --host=%s --target=%s\n\
 "), host_name, target_name);
+#endif
 
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-auto-load-dir=%s\n\
 	     --with-auto-load-safe-path=%s\n\
 "), AUTO_LOAD_DIR, AUTO_LOAD_SAFE_PATH);
 
 #if HAVE_LIBEXPAT
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-expat\n\
 "));
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-expat\n\
 "));
 #endif
 
   if (GDB_DATADIR[0])
-    fprintf_filtered (stream, _("\
+    gdb_printf (stream, _("\
 	     --with-gdb-datadir=%s%s\n\
 "), GDB_DATADIR, GDB_DATADIR_RELOCATABLE ? " (relocatable)" : "");
 
 #ifdef ICONV_BIN
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-iconv-bin=%s%s\n\
 "), ICONV_BIN, ICONV_BIN_RELOCATABLE ? " (relocatable)" : "");
 #endif
 
   if (JIT_READER_DIR[0])
-    fprintf_filtered (stream, _("\
+    gdb_printf (stream, _("\
 	     --with-jit-reader-dir=%s%s\n\
 "), JIT_READER_DIR, JIT_READER_DIR_RELOCATABLE ? " (relocatable)" : "");
 
 #if HAVE_LIBUNWIND_IA64_H
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-libunwind-ia64\n\
 "));
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-libunwind-ia64\n\
 "));
 #endif
 
 #if HAVE_LIBLZMA
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-lzma\n\
 "));
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-lzma\n\
 "));
 #endif
 
 #if HAVE_LIBBABELTRACE
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-babeltrace\n\
 "));
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-babeltrace\n\
 "));
 #endif
 
 #if HAVE_LIBIPT
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-intel-pt\n\
 "));
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-intel-pt\n\
 "));
 #endif
 
 #if HAVE_LIBMPFR
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-mpfr\n\
 "));
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-mpfr\n\
 "));
 #endif
 #if HAVE_LIBXXHASH
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-xxhash\n\
 "));
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-xxhash\n\
 "));
 #endif
 #ifdef WITH_PYTHON_PATH
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-python=%s%s\n\
 "), WITH_PYTHON_PATH, PYTHON_PATH_RELOCATABLE ? " (relocatable)" : "");
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-python\n\
 "));
 #endif
 #ifdef WITH_PYTHON_LIBDIR
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-python-libdir=%s%s\n\
 "), WITH_PYTHON_LIBDIR, PYTHON_LIBDIR_RELOCATABLE ? " (relocatable)" : "");
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-python-libdir\n\
 "));
 #endif
 
 #if HAVE_LIBDEBUGINFOD
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-debuginfod\n\
 "));
 #else
-   fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-debuginfod\n\
 "));
 #endif
 
 #if HAVE_GUILE
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-guile\n\
 "));
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --without-guile\n\
 "));
 #endif
 
 #if HAVE_SOURCE_HIGHLIGHT
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --enable-source-highlight\n\
 "));
 #else
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --disable-source-highlight\n\
 "));
 #endif
 
+#if CXX_STD_THREAD
+  gdb_printf (stream, _("\
+	     --enable-threading\n\
+"));
+#else
+  gdb_printf (stream, _("\
+	     --disable-threading\n\
+"));
+#endif
+
 #ifdef RELOC_SRCDIR
-  fprintf_filtered (stream, _("\
+  gdb_printf (stream, _("\
 	     --with-relocated-sources=%s\n\
 "), RELOC_SRCDIR);
 #endif
 
   if (DEBUGDIR[0])
-    fprintf_filtered (stream, _("\
+    gdb_printf (stream, _("\
 	     --with-separate-debug-dir=%s%s\n\
 "), DEBUGDIR, DEBUGDIR_RELOCATABLE ? " (relocatable)" : "");
 
   if (TARGET_SYSTEM_ROOT[0])
-    fprintf_filtered (stream, _("\
+    gdb_printf (stream, _("\
 	     --with-sysroot=%s%s\n\
 "), TARGET_SYSTEM_ROOT, TARGET_SYSTEM_ROOT_RELOCATABLE ? " (relocatable)" : "");
 
   if (SYSTEM_GDBINIT[0])
-    fprintf_filtered (stream, _("\
+    gdb_printf (stream, _("\
 	     --with-system-gdbinit=%s%s\n\
 "), SYSTEM_GDBINIT, SYSTEM_GDBINIT_RELOCATABLE ? " (relocatable)" : "");
 
   if (SYSTEM_GDBINIT_DIR[0])
-    fprintf_filtered (stream, _("\
+    gdb_printf (stream, _("\
 	     --with-system-gdbinit-dir=%s%s\n\
 "), SYSTEM_GDBINIT_DIR, SYSTEM_GDBINIT_DIR_RELOCATABLE ? " (relocatable)" : "");
 
   /* We assume "relocatable" will be printed at least once, thus we always
      print this text.  It's a reasonably safe assumption for now.  */
-  fprintf_filtered (stream, _("\n\
+  gdb_printf (stream, _("\n\
 (\"Relocatable\" means the directory can be moved with the GDB installation\n\
 tree, and GDB will still find it.)\n\
 "));
@@ -1775,13 +1802,13 @@ print_inferior_quit_action (inferior *inf, ui_file *out)
     return;
 
   if (inf->attach_flag)
-    fprintf_filtered (out,
-		      _("\tInferior %d [%s] will be detached.\n"), inf->num,
-		      target_pid_to_str (ptid_t (inf->pid)).c_str ());
+    gdb_printf (out,
+		_("\tInferior %d [%s] will be detached.\n"), inf->num,
+		target_pid_to_str (ptid_t (inf->pid)).c_str ());
   else
-    fprintf_filtered (out,
-		      _("\tInferior %d [%s] will be killed.\n"), inf->num,
-		      target_pid_to_str (ptid_t (inf->pid)).c_str ());
+    gdb_printf (out,
+		_("\tInferior %d [%s] will be killed.\n"), inf->num,
+		target_pid_to_str (ptid_t (inf->pid)).c_str ());
 }
 
 /* If necessary, make the user confirm that we should quit.  Return
@@ -1847,10 +1874,10 @@ quit_force (int *exit_arg, int from_tty)
   undo_terminal_modifications_before_exit ();
 
 #ifdef NVIDIA_CUDA_GDB
-  if (cuda_exception_is_valid (cuda_exception))
+  cuda_exception exp;
+  if (exp.valid ())
     {
       cuda_cleanup();
-      cuda_exception_reset (cuda_exception);
 
       if (target_has_execution ())
         target_kill ();
@@ -1874,10 +1901,9 @@ quit_force (int *exit_arg, int from_tty)
      them all out.  */
   for (inferior *inf : all_inferiors ())
     {
-      switch_to_inferior_no_thread (inf);
       try
 	{
-	  pop_all_targets ();
+	  inf->pop_all_targets ();
 	}
       catch (const gdb_exception &ex)
 	{
@@ -1896,7 +1922,7 @@ quit_force (int *exit_arg, int from_tty)
 	     any UI with a terminal, save history.  */
 	  for (ui *ui : all_uis ())
 	    {
-	      if (input_interactive_p (ui))
+	      if (ui->input_interactive_p ())
 		{
 		  save = 1;
 		  break;
@@ -1942,25 +1968,25 @@ show_interactive_mode (struct ui_file *file, int from_tty,
 		       const char *value)
 {
   if (interactive_mode == AUTO_BOOLEAN_AUTO)
-    fprintf_filtered (file, "Debugger's interactive mode "
-			    "is %s (currently %s).\n",
-		      value, input_interactive_p (current_ui) ? "on" : "off");
+    gdb_printf (file, "Debugger's interactive mode "
+		"is %s (currently %s).\n",
+		value, current_ui->input_interactive_p () ? "on" : "off");
   else
-    fprintf_filtered (file, "Debugger's interactive mode is %s.\n", value);
+    gdb_printf (file, "Debugger's interactive mode is %s.\n", value);
 }
 
 /* Returns whether GDB is running on an interactive terminal.  */
 
-int
-input_interactive_p (struct ui *ui)
+bool
+ui::input_interactive_p () const
 {
   if (batch_flag)
-    return 0;
+    return false;
 
   if (interactive_mode != AUTO_BOOLEAN_AUTO)
     return interactive_mode == AUTO_BOOLEAN_TRUE;
 
-  return ui->input_interactive_p;
+  return m_input_interactive_p;
 }
 
 static void
@@ -2018,8 +2044,8 @@ show_commands (const char *args, int from_tty)
        offset < num + Hist_print && offset < history_length;
        offset++)
     {
-      printf_filtered ("%5d  %s\n", history_base + offset,
-		       (history_get (history_base + offset))->line);
+      gdb_printf ("%5d  %s\n", history_base + offset,
+		  (history_get (history_base + offset))->line);
     }
 
   /* The next command we want to display is the next one that we haven't
@@ -2144,7 +2170,7 @@ static void
 show_prompt (struct ui_file *file, int from_tty,
 	     struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Gdb's prompt is \"%s\".\n"), value);
+  gdb_printf (file, _("Gdb's prompt is \"%s\".\n"), value);
 }
 
 /* "set editing" command.  */
@@ -2162,25 +2188,25 @@ static void
 show_editing (struct ui_file *file, int from_tty,
 	      struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Editing of command lines as "
-			    "they are typed is %s.\n"),
-		    current_ui->command_editing ? _("on") : _("off"));
+  gdb_printf (file, _("Editing of command lines as "
+		      "they are typed is %s.\n"),
+	      current_ui->command_editing ? _("on") : _("off"));
 }
 
 static void
 show_annotation_level (struct ui_file *file, int from_tty,
 		       struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Annotation_level is %s.\n"), value);
+  gdb_printf (file, _("Annotation_level is %s.\n"), value);
 }
 
 static void
 show_exec_done_display_p (struct ui_file *file, int from_tty,
 			  struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Notification of completion for "
-			    "asynchronous execution commands is %s.\n"),
-		    value);
+  gdb_printf (file, _("Notification of completion for "
+		      "asynchronous execution commands is %s.\n"),
+	      value);
 }
 
 /* New values of the "data-directory" parameter are staged here.
@@ -2211,9 +2237,9 @@ static void
 show_gdb_datadir (struct ui_file *file, int from_tty,
 		  struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("GDB's data directory is \"%ps\".\n"),
-		    styled_string (file_name_style.style (),
-				   gdb_datadir.c_str ()));
+  gdb_printf (file, _("GDB's data directory is \"%ps\".\n"),
+	      styled_string (file_name_style.style (),
+			     gdb_datadir.c_str ()));
 }
 
 /* Implement 'set history filename'.  */
@@ -2227,12 +2253,7 @@ set_history_filename (const char *args,
      that was read.  */
   if (!history_filename.empty ()
       && !IS_ABSOLUTE_PATH (history_filename.c_str ()))
-    {
-      gdb::unique_xmalloc_ptr<char> temp
-	(gdb_abspath (history_filename.c_str ()));
-
-      history_filename = temp.get ();
-    }
+    history_filename = gdb_abspath (history_filename.c_str ());
 }
 
 /* Whether we're in quiet startup mode.  */
@@ -2253,19 +2274,8 @@ static void
 show_startup_quiet (struct ui_file *file, int from_tty,
 	      struct cmd_list_element *c, const char *value)
 {
-  fprintf_filtered (file, _("Whether to start up quietly is %s.\n"),
-		    value);
-}
-
-static void
-init_gdb_version_vars (void)
-{
-  struct internalvar *major_version_var = create_internalvar ("_gdb_major");
-  struct internalvar *minor_version_var = create_internalvar ("_gdb_minor");
-  int vmajor = 0, vminor = 0, vrevision = 0;
-  sscanf (version, "%d.%d.%d", &vmajor, &vminor, &vrevision);
-  set_internalvar_integer (major_version_var, vmajor);
-  set_internalvar_integer (minor_version_var, vminor + (vrevision > 0));
+  gdb_printf (file, _("Whether to start up quietly is %s.\n"),
+	      value);
 }
 
 static void
@@ -2290,6 +2300,7 @@ init_main (void)
   rl_completion_display_matches_hook = cli_display_match_list;
   rl_readline_name = "gdb";
   rl_terminal_name = getenv ("TERM");
+  rl_deprep_term_function = gdb_rl_deprep_term_function;
 
   /* The name for this defun comes from Bash, where it originated.
      15 is Control-o, the same binding this function has in Bash.  */
@@ -2430,6 +2441,13 @@ Usage: new-ui INTERPRETER TTY\n\
 The first argument is the name of the interpreter to run.\n\
 The second argument is the terminal the UI runs on."), &cmdlist);
   set_cmd_completer (c, interpreter_completer);
+
+  struct internalvar *major_version_var = create_internalvar ("_gdb_major");
+  struct internalvar *minor_version_var = create_internalvar ("_gdb_minor");
+  int vmajor = 0, vminor = 0, vrevision = 0;
+  sscanf (version, "%d.%d.%d", &vmajor, &vminor, &vrevision);
+  set_internalvar_integer (major_version_var, vmajor);
+  set_internalvar_integer (minor_version_var, vminor + (vrevision > 0));
 }
 
 /* See top.h.  */
@@ -2448,8 +2466,6 @@ gdb_init ()
   make_final_cleanup (do_chdir_cleanup, xstrdup (current_directory));
 #endif
 
-  init_cmd_lists ();	    /* This needs to be done first.  */
-
   init_page_info ();
 
   /* Here is where we call all the _initialize_foo routines.  */
@@ -2479,9 +2495,6 @@ gdb_init ()
      during startup.  */
   set_language (language_c);
   expected_language = current_language;	/* Don't warn about the change.  */
-
-  /* Create $_gdb_major and $_gdb_minor convenience variables.  */
-  init_gdb_version_vars ();
 }
 
 void _initialize_top ();
@@ -2504,7 +2517,6 @@ _initialize_top ()
       const char *fname = ".gdb_history";
 #endif
 
-      gdb::unique_xmalloc_ptr<char> temp (gdb_abspath (fname));
-      history_filename = temp.get ();
+      history_filename = gdb_abspath (fname);
     }
 }
