@@ -1,6 +1,6 @@
 /* Perform arithmetic and other operations on values, for GDB.
 
-   Copyright (C) 1986-2022 Free Software Foundation, Inc.
+   Copyright (C) 1986-2023 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -62,7 +62,7 @@ find_size_for_pointer_math (struct type *ptr_type)
   struct type *ptr_target;
 
   gdb_assert (ptr_type->code () == TYPE_CODE_PTR);
-  ptr_target = check_typedef (TYPE_TARGET_TYPE (ptr_type));
+  ptr_target = check_typedef (ptr_type->target_type ());
 
   sz = type_length_units (ptr_target);
   if (sz == 0)
@@ -123,13 +123,13 @@ value_ptrdiff (struct value *arg1, struct value *arg2)
   gdb_assert (type1->code () == TYPE_CODE_PTR);
   gdb_assert (type2->code () == TYPE_CODE_PTR);
 
-  if (TYPE_LENGTH (check_typedef (TYPE_TARGET_TYPE (type1)))
-      != TYPE_LENGTH (check_typedef (TYPE_TARGET_TYPE (type2))))
+  if (check_typedef (type1->target_type ())->length ()
+      != check_typedef (type2->target_type ())->length ())
     error (_("First argument of `-' is a pointer and "
 	     "second argument is neither\n"
 	     "an integer nor a pointer of the same type."));
 
-  sz = type_length_units (check_typedef (TYPE_TARGET_TYPE (type1)));
+  sz = type_length_units (check_typedef (type1->target_type ()));
   if (sz == 0) 
     {
       warning (_("Type size unknown, assuming 1. "
@@ -208,7 +208,7 @@ value_subscripted_rvalue (struct value *array, LONGEST index,
 			  LONGEST lowerbound)
 {
   struct type *array_type = check_typedef (value_type (array));
-  struct type *elt_type = TYPE_TARGET_TYPE (array_type);
+  struct type *elt_type = array_type->target_type ();
   LONGEST elt_size = type_length_units (elt_type);
 
   /* Fetch the bit stride and convert it to a byte stride, assuming 8 bits
@@ -265,11 +265,11 @@ binop_types_user_defined_p (enum exp_opcode op,
 
   type1 = check_typedef (type1);
   if (TYPE_IS_REFERENCE (type1))
-    type1 = check_typedef (TYPE_TARGET_TYPE (type1));
+    type1 = check_typedef (type1->target_type ());
 
   type2 = check_typedef (type2);
   if (TYPE_IS_REFERENCE (type2))
-    type2 = check_typedef (TYPE_TARGET_TYPE (type2));
+    type2 = check_typedef (type2->target_type ());
 
   return (type1->code () == TYPE_CODE_STRUCT
 	  || type2->code () == TYPE_CODE_STRUCT);
@@ -303,7 +303,7 @@ unop_user_defined_p (enum exp_opcode op, struct value *arg1)
     return 0;
   type1 = check_typedef (value_type (arg1));
   if (TYPE_IS_REFERENCE (type1))
-    type1 = check_typedef (TYPE_TARGET_TYPE (type1));
+    type1 = check_typedef (type1->target_type ());
   return type1->code () == TYPE_CODE_STRUCT;
 }
 
@@ -533,8 +533,7 @@ value_x_binop (struct value *arg1, struct value *arg2, enum exp_opcode op,
 	{
 	  struct type *return_type;
 
-	  return_type
-	    = TYPE_TARGET_TYPE (check_typedef (value_type (argvec[0])));
+	  return_type = check_typedef (value_type (argvec[0]))->target_type ();
 	  return value_zero (return_type, VALUE_LVAL (arg1));
 	}
       return call_function_by_hand (argvec[0], NULL,
@@ -647,8 +646,7 @@ value_x_unop (struct value *arg1, enum exp_opcode op, enum noside noside)
 	{
 	  struct type *return_type;
 
-	  return_type
-	    = TYPE_TARGET_TYPE (check_typedef (value_type (argvec[0])));
+	  return_type = check_typedef (value_type (argvec[0]))->target_type ();
 	  return value_zero (return_type, VALUE_LVAL (arg1));
 	}
       return call_function_by_hand (argvec[0], NULL,
@@ -676,7 +674,7 @@ value_concat (struct value *arg1, struct value *arg2)
   struct type *elttype1 = type1;
   if (elttype1->code () == TYPE_CODE_ARRAY)
     {
-      elttype1 = TYPE_TARGET_TYPE (elttype1);
+      elttype1 = elttype1->target_type ();
       if (!get_array_bounds (type1, &low1, &high1))
 	error (_("could not determine array bounds on left-hand-side of "
 		 "array concatenation"));
@@ -691,7 +689,7 @@ value_concat (struct value *arg1, struct value *arg2)
   struct type *elttype2 = type2;
   if (elttype2->code () == TYPE_CODE_ARRAY)
     {
-      elttype2 = TYPE_TARGET_TYPE (elttype2);
+      elttype2 = elttype2->target_type ();
       if (!get_array_bounds (type2, &low2, &high2))
 	error (_("could not determine array bounds on right-hand-side of "
 		 "array concatenation"));
@@ -780,7 +778,7 @@ value_args_as_target_float (struct value *arg1, struct value *arg2,
   if (is_floating_type (type1))
     {
       *eff_type_x = type1;
-      memcpy (x, value_contents (arg1).data (), TYPE_LENGTH (type1));
+      memcpy (x, value_contents (arg1).data (), type1->length ());
     }
   else if (is_integral_type (type1))
     {
@@ -799,7 +797,7 @@ value_args_as_target_float (struct value *arg1, struct value *arg2,
   if (is_floating_type (type2))
     {
       *eff_type_y = type2;
-      memcpy (y, value_contents (arg2).data (), TYPE_LENGTH (type2));
+      memcpy (y, value_contents (arg2).data (), type2->length ());
     }
   else if (is_integral_type (type2))
     {
@@ -940,7 +938,7 @@ promotion_type (struct type *type1, struct type *type2)
 	result_type = type2;
       else if (!is_floating_type (type2))
 	result_type = type1;
-      else if (TYPE_LENGTH (type2) > TYPE_LENGTH (type1))
+      else if (type2->length () > type1->length ())
 	result_type = type2;
       else
 	result_type = type1;
@@ -948,9 +946,9 @@ promotion_type (struct type *type1, struct type *type2)
   else
     {
       /* Integer types.  */
-      if (TYPE_LENGTH (type1) > TYPE_LENGTH (type2))
+      if (type1->length () > type2->length ())
 	result_type = type1;
-      else if (TYPE_LENGTH (type2) > TYPE_LENGTH (type1))
+      else if (type2->length () > type1->length ())
 	result_type = type2;
       else if (type1->is_unsigned ())
 	result_type = type1;
@@ -1078,6 +1076,66 @@ complex_binop (struct value *arg1, struct value *arg2, enum exp_opcode op)
   return value_literal_complex (result_real, result_imag, result_type);
 }
 
+/* Return the type's length in bits.  */
+
+static int
+type_length_bits (type *type)
+{
+  int unit_size = gdbarch_addressable_memory_unit_size (type->arch ());
+  return unit_size * 8 * type->length ();
+}
+
+/* Check whether the RHS value of a shift is valid in C/C++ semantics.
+   SHIFT_COUNT is the shift amount, SHIFT_COUNT_TYPE is the type of
+   the shift count value, used to determine whether the type is
+   signed, and RESULT_TYPE is the result type.  This is used to avoid
+   both negative and too-large shift amounts, which are undefined, and
+   would crash a GDB built with UBSan.  Depending on the current
+   language, if the shift is not valid, this either warns and returns
+   false, or errors out.  Returns true if valid.  */
+
+static bool
+check_valid_shift_count (int op, type *result_type,
+			 type *shift_count_type, ULONGEST shift_count)
+{
+  if (!shift_count_type->is_unsigned () && (LONGEST) shift_count < 0)
+    {
+      auto error_or_warning = [] (const char *msg)
+      {
+	/* Shifts by a negative amount are always an error in Go.  Other
+	   languages are more permissive and their compilers just warn or
+	   have modes to disable the errors.  */
+	if (current_language->la_language == language_go)
+	  error (("%s"), msg);
+	else
+	  warning (("%s"), msg);
+      };
+
+      if (op == BINOP_RSH)
+	error_or_warning (_("right shift count is negative"));
+      else
+	error_or_warning (_("left shift count is negative"));
+      return false;
+    }
+
+  if (shift_count >= type_length_bits (result_type))
+    {
+      /* In Go, shifting by large amounts is defined.  Be silent and
+	 still return false, as the caller's error path does the right
+	 thing for Go.  */
+      if (current_language->la_language != language_go)
+	{
+	  if (op == BINOP_RSH)
+	    warning (_("right shift count >= width of type"));
+	  else
+	    warning (_("left shift count >= width of type"));
+	}
+      return false;
+    }
+
+  return true;
+}
+
 /* Perform a binary operation on two operands which have reasonable
    representations as integers or floats.  This includes booleans,
    characters, integers, or floats.
@@ -1118,8 +1176,8 @@ scalar_binop (struct value *arg1, struct value *arg2, enum exp_opcode op)
 
       struct type *eff_type_v1, *eff_type_v2;
       gdb::byte_vector v1, v2;
-      v1.resize (TYPE_LENGTH (result_type));
-      v2.resize (TYPE_LENGTH (result_type));
+      v1.resize (result_type->length ());
+      v2.resize (result_type->length ());
 
       value_args_as_target_float (arg1, arg2,
 				  v1.data (), &eff_type_v1,
@@ -1174,7 +1232,7 @@ scalar_binop (struct value *arg1, struct value *arg2, enum exp_opcode op)
 
       val = allocate_value (result_type);
       store_signed_integer (value_contents_raw (val).data (),
-			    TYPE_LENGTH (result_type),
+			    result_type->length (),
 			    type_byte_order (result_type),
 			    v);
     }
@@ -1249,11 +1307,17 @@ scalar_binop (struct value *arg1, struct value *arg2, enum exp_opcode op)
 	      break;
 
 	    case BINOP_LSH:
-	      v = v1 << v2;
+	      if (!check_valid_shift_count (op, result_type, type2, v2))
+		v = 0;
+	      else
+		v = v1 << v2;
 	      break;
 
 	    case BINOP_RSH:
-	      v = v1 >> v2;
+	      if (!check_valid_shift_count (op, result_type, type2, v2))
+		v = 0;
+	      else
+		v = v1 >> v2;
 	      break;
 
 	    case BINOP_BITWISE_AND:
@@ -1314,7 +1378,7 @@ scalar_binop (struct value *arg1, struct value *arg2, enum exp_opcode op)
 
 	  val = allocate_value (result_type);
 	  store_unsigned_integer (value_contents_raw (val).data (),
-				  TYPE_LENGTH (value_type (val)),
+				  value_type (val)->length (),
 				  type_byte_order (result_type),
 				  v);
 	}
@@ -1332,7 +1396,10 @@ scalar_binop (struct value *arg1, struct value *arg2, enum exp_opcode op)
 	      break;
 
 	    case BINOP_SUB:
-	      v = v1 - v2;
+	      /* Avoid runtime error: signed integer overflow: \
+		 0 - -9223372036854775808 cannot be represented in type
+		 'long int'.  */
+	      v = (ULONGEST)v1 - (ULONGEST)v2;
 	      break;
 
 	    case BINOP_MUL:
@@ -1378,11 +1445,40 @@ scalar_binop (struct value *arg1, struct value *arg2, enum exp_opcode op)
 	      break;
 
 	    case BINOP_LSH:
-	      v = v1 << v2;
+	      if (!check_valid_shift_count (op, result_type, type2, v2))
+		v = 0;
+	      else
+		{
+		  /* Cast to unsigned to avoid undefined behavior on
+		     signed shift overflow (unless C++20 or later),
+		     which would crash GDB when built with UBSan.
+		     Note we don't warn on left signed shift overflow,
+		     because starting with C++20, that is actually
+		     defined behavior.  Also, note GDB assumes 2's
+		     complement throughout.  */
+		  v = (ULONGEST) v1 << v2;
+		}
 	      break;
 
 	    case BINOP_RSH:
-	      v = v1 >> v2;
+	      if (!check_valid_shift_count (op, result_type, type2, v2))
+		{
+		  /* Pretend the too-large shift was decomposed in a
+		     number of smaller shifts.  An arithmetic signed
+		     right shift of a negative number always yields -1
+		     with such semantics.  This is the right thing to
+		     do for Go, and we might as well do it for
+		     languages where it is undefined.  Also, pretend a
+		     shift by a negative number was a shift by the
+		     negative number cast to unsigned, which is the
+		     same as shifting by a too-large number.  */
+		  if (v1 < 0)
+		    v = -1;
+		  else
+		    v = 0;
+		}
+	      else
+		v = v1 >> v2;
 	      break;
 
 	    case BINOP_BITWISE_AND:
@@ -1443,7 +1539,7 @@ scalar_binop (struct value *arg1, struct value *arg2, enum exp_opcode op)
 
 	  val = allocate_value (result_type);
 	  store_signed_integer (value_contents_raw (val).data (),
-				TYPE_LENGTH (value_type (val)),
+				value_type (val)->length (),
 				type_byte_order (result_type),
 				v);
 	}
@@ -1480,20 +1576,20 @@ value_vector_widen (struct value *scalar_value, struct type *vector_type)
   if (!get_array_bounds (vector_type, &low_bound, &high_bound))
     error (_("Could not determine the vector bounds"));
 
-  eltype = check_typedef (TYPE_TARGET_TYPE (vector_type));
+  eltype = check_typedef (vector_type->target_type ());
   elval = value_cast (eltype, scalar_value);
 
   scalar_type = check_typedef (value_type (scalar_value));
 
   /* If we reduced the length of the scalar then check we didn't loose any
      important bits.  */
-  if (TYPE_LENGTH (eltype) < TYPE_LENGTH (scalar_type)
+  if (eltype->length () < scalar_type->length ()
       && !value_equal (elval, scalar_value))
     error (_("conversion of scalar to vector involves truncation"));
 
   value *val = allocate_value (vector_type);
   gdb::array_view<gdb_byte> val_contents = value_contents_writeable (val);
-  int elt_len = TYPE_LENGTH (eltype);
+  int elt_len = eltype->length ();
 
   for (i = 0; i < high_bound - low_bound + 1; i++)
     /* Duplicate the contents of elval into the destination vector.  */
@@ -1528,19 +1624,19 @@ vector_binop (struct value *val1, struct value *val2, enum exp_opcode op)
       || !get_array_bounds (type2, &low_bound2, &high_bound2))
     error (_("Could not determine the vector bounds"));
 
-  eltype1 = check_typedef (TYPE_TARGET_TYPE (type1));
-  eltype2 = check_typedef (TYPE_TARGET_TYPE (type2));
-  elsize = TYPE_LENGTH (eltype1);
+  eltype1 = check_typedef (type1->target_type ());
+  eltype2 = check_typedef (type2->target_type ());
+  elsize = eltype1->length ();
 
   if (eltype1->code () != eltype2->code ()
-      || elsize != TYPE_LENGTH (eltype2)
+      || elsize != eltype2->length ()
       || eltype1->is_unsigned () != eltype2->is_unsigned ()
       || low_bound1 != low_bound2 || high_bound1 != high_bound2)
     error (_("Cannot perform operation on vectors with different types"));
 
   value *val = allocate_value (type1);
   gdb::array_view<gdb_byte> val_contents = value_contents_writeable (val);
-  value *mark = value_mark ();
+  scoped_value_mark mark;
   for (i = 0; i < high_bound1 - low_bound1 + 1; i++)
     {
       value *tmp = value_binop (value_subscript (val1, i),
@@ -1548,7 +1644,6 @@ vector_binop (struct value *val1, struct value *val2, enum exp_opcode op)
       copy (value_contents_all (tmp),
 	    val_contents.slice (i * elsize, elsize));
      }
-  value_free_to_mark (mark);
 
   return val;
 }
@@ -1605,7 +1700,7 @@ value_logical_not (struct value *arg1)
   if (is_floating_value (arg1))
     return target_float_is_zero (value_contents (arg1).data (), type1);
 
-  len = TYPE_LENGTH (type1);
+  len = type1->length ();
   p = value_contents (arg1).data ();
 
   while (--len >= 0)
@@ -1623,8 +1718,8 @@ value_logical_not (struct value *arg1)
 static int
 value_strcmp (struct value *arg1, struct value *arg2)
 {
-  int len1 = TYPE_LENGTH (value_type (arg1));
-  int len2 = TYPE_LENGTH (value_type (arg2));
+  int len1 = value_type (arg1)->length ();
+  int len2 = value_type (arg2)->length ();
   const gdb_byte *s1 = value_contents (arg1).data ();
   const gdb_byte *s2 = value_contents (arg2).data ();
   int i, len = len1 < len2 ? len1 : len2;
@@ -1679,8 +1774,8 @@ value_equal (struct value *arg1, struct value *arg2)
     {
       struct type *eff_type_v1, *eff_type_v2;
       gdb::byte_vector v1, v2;
-      v1.resize (std::max (TYPE_LENGTH (type1), TYPE_LENGTH (type2)));
-      v2.resize (std::max (TYPE_LENGTH (type1), TYPE_LENGTH (type2)));
+      v1.resize (std::max (type1->length (), type2->length ()));
+      v2.resize (std::max (type1->length (), type2->length ()));
 
       value_args_as_target_float (arg1, arg2,
 				  v1.data (), &eff_type_v1,
@@ -1698,8 +1793,8 @@ value_equal (struct value *arg1, struct value *arg2)
     return (CORE_ADDR) value_as_long (arg1) == value_as_address (arg2);
 
   else if (code1 == code2
-	   && ((len = (int) TYPE_LENGTH (type1))
-	       == (int) TYPE_LENGTH (type2)))
+	   && ((len = (int) type1->length ())
+	       == (int) type2->length ()))
     {
       p1 = value_contents (arg1).data ();
       p2 = value_contents (arg2).data ();
@@ -1731,10 +1826,10 @@ value_equal_contents (struct value *arg1, struct value *arg2)
   type2 = check_typedef (value_type (arg2));
 
   return (type1->code () == type2->code ()
-	  && TYPE_LENGTH (type1) == TYPE_LENGTH (type2)
+	  && type1->length () == type2->length ()
 	  && memcmp (value_contents (arg1).data (),
 		     value_contents (arg2).data (),
-		     TYPE_LENGTH (type1)) == 0);
+		     type1->length ()) == 0);
 }
 
 /* Simulate the C operator < by returning 1
@@ -1767,8 +1862,8 @@ value_less (struct value *arg1, struct value *arg2)
     {
       struct type *eff_type_v1, *eff_type_v2;
       gdb::byte_vector v1, v2;
-      v1.resize (std::max (TYPE_LENGTH (type1), TYPE_LENGTH (type2)));
-      v2.resize (std::max (TYPE_LENGTH (type1), TYPE_LENGTH (type2)));
+      v1.resize (std::max (type1->length (), type2->length ()));
+      v2.resize (std::max (type1->length (), type2->length ()));
 
       value_args_as_target_float (arg1, arg2,
 				  v1.data (), &eff_type_v1,
@@ -1828,7 +1923,7 @@ value_neg (struct value *arg1)
   else if (type->code () == TYPE_CODE_ARRAY && type->is_vector ())
     {
       struct value *val = allocate_value (type);
-      struct type *eltype = check_typedef (TYPE_TARGET_TYPE (type));
+      struct type *eltype = check_typedef (type->target_type ());
       int i;
       LONGEST low_bound, high_bound;
 
@@ -1836,7 +1931,7 @@ value_neg (struct value *arg1)
 	error (_("Could not determine the vector bounds"));
 
       gdb::array_view<gdb_byte> val_contents = value_contents_writeable (val);
-      int elt_len = TYPE_LENGTH (eltype);
+      int elt_len = eltype->length ();
 
       for (i = 0; i < high_bound - low_bound + 1; i++)
 	{
@@ -1872,7 +1967,7 @@ value_complement (struct value *arg1)
     val = value_from_longest (type, ~value_as_long (arg1));
   else if (type->code () == TYPE_CODE_ARRAY && type->is_vector ())
     {
-      struct type *eltype = check_typedef (TYPE_TARGET_TYPE (type));
+      struct type *eltype = check_typedef (type->target_type ());
       int i;
       LONGEST low_bound, high_bound;
 
@@ -1881,7 +1976,7 @@ value_complement (struct value *arg1)
 
       val = allocate_value (type);
       gdb::array_view<gdb_byte> val_contents = value_contents_writeable (val);
-      int elt_len = TYPE_LENGTH (eltype);
+      int elt_len = eltype->length ();
 
       for (i = 0; i < high_bound - low_bound + 1; i++)
 	{
@@ -1940,7 +2035,7 @@ value_in (struct value *element, struct value *set)
   struct type *eltype = check_typedef (value_type (element));
 
   if (eltype->code () == TYPE_CODE_RANGE)
-    eltype = TYPE_TARGET_TYPE (eltype);
+    eltype = eltype->target_type ();
   if (settype->code () != TYPE_CODE_SET)
     error (_("Second argument of 'IN' has wrong type"));
   if (eltype->code () != TYPE_CODE_INT
