@@ -46,6 +46,20 @@ extern struct target_waitstatus cuda_last_ws;
 #endif
 char *buf_head = NULL;
 
+static uint32_t cuda_debugapi_version_major;
+static uint32_t cuda_debugapi_version_minor;
+static uint32_t cuda_debugapi_version_revision;
+
+extern void
+cuda_gdbserver_set_api_version (uint32_t major, uint32_t minor, uint32_t revision);
+
+void
+cuda_gdbserver_set_api_version (uint32_t major, uint32_t minor, uint32_t revision)
+{
+  cuda_debugapi_version_major = major;
+  cuda_debugapi_version_minor = minor;
+  cuda_debugapi_version_revision = revision;
+}
 
 static char *
 append_string (const char *src, char *dest, bool sep)
@@ -96,7 +110,7 @@ extract_bin (char *src, unsigned char *dest, int size)
 
   p = extract_string (src);
   if (!p)
-    error ("The data in the cuda packet is not complete.\n");
+    error ("The data in the cuda packet is not complete (cuda-gdbserver).\n");
   hex2bin (p, dest, size);
   return p;
 }
@@ -393,7 +407,7 @@ cuda_process_notification_aliased_event_packet (char *buf)
 }
 
 static void
-cuda_process_single_step_warp_packet (char *buf)
+cuda_process_single_step_warp_packet65 (char *buf)
 {
   CUDBGResult res;
   char *p;
@@ -408,7 +422,7 @@ cuda_process_single_step_warp_packet (char *buf)
   extract_bin (NULL, (unsigned char *) &wp, sizeof (wp));
   extract_bin (NULL, (unsigned char *) &nsteps, sizeof (nsteps));
   extract_bin (NULL, (unsigned char *) &warp_mask, sizeof (warp_mask));
-  res = cudbgAPI->singleStepWarp (dev, sm, wp, nsteps, &warp_mask);
+  res = cudbgAPI->singleStepWarp65 (dev, sm, wp, nsteps, &warp_mask);
   p = append_bin ((unsigned char *) &res, buf, sizeof (res), true);
   p = append_bin ((unsigned char *) &warp_mask, p, sizeof (warp_mask), false);
 }
@@ -465,7 +479,16 @@ cuda_process_initialize_target_packet (char *buf)
   p = append_bin ((unsigned char *) &api_initialize_res, p, sizeof (api_initialize_res), true);
   p = append_bin ((unsigned char *) &cuda_initialized, p, sizeof (cuda_initialized), true);
   p = append_bin ((unsigned char *) &cuda_debugging_enabled, p, sizeof (cuda_debugging_enabled), true);
-  p = append_bin ((unsigned char *) &driver_is_compatible, p, sizeof (driver_is_compatible), false);
+  p = append_bin ((unsigned char *) &driver_is_compatible, p, sizeof (driver_is_compatible), true);
+
+  p = append_bin ((unsigned char *) &cuda_debugapi_version_major, p,
+		  sizeof (cuda_debugapi_version_major), true);
+
+  p = append_bin ((unsigned char *) &cuda_debugapi_version_minor, p,
+		  sizeof (cuda_debugapi_version_minor), true);
+
+  p = append_bin ((unsigned char *) &cuda_debugapi_version_revision, p,
+		  sizeof (cuda_debugapi_version_revision), false);
 }
 
 static void
@@ -1278,6 +1301,74 @@ cuda_process_api_read_device_exception_state (char *buf)
   xfree (value);
 }
 
+static void
+cuda_process_api_get_device_info_sizes (char *buf)
+{
+  CUDBGResult res;
+  uint32_t dev;
+  char *p;
+  CUDBGDeviceInfoSizes sizes;
+
+  extract_bin (NULL, (unsigned char*) &dev, sizeof (dev));
+
+  res = cudbgAPI->getDeviceInfoSizes (dev, &sizes);
+  p = append_bin ((unsigned char*) &res, buf, sizeof (res), true);
+  if (res == CUDBG_SUCCESS)
+    p = append_bin ((unsigned char*) &sizes, p, sizeof (CUDBGDeviceInfoSizes), false);
+}
+
+static void
+cuda_process_api_get_device_info (char *buf)
+{
+  CUDBGResult res;
+  char *p;
+  void *buffer;
+  uint32_t dev, length, data_length;
+  CUDBGDeviceInfoQueryType_t type;
+
+  extract_bin (NULL, (unsigned char*) &dev, sizeof (dev));
+  extract_bin (NULL, (unsigned char*) &type, sizeof (type));
+  extract_bin (NULL, (unsigned char*) &length, sizeof (length));
+
+  buffer = xmalloc (length);
+  res = cudbgAPI->getDeviceInfo (dev, type, buffer, length, &data_length);
+
+  if (res == CUDBG_SUCCESS)
+    {
+      p = append_bin ((unsigned char*) &res, buf, sizeof (res), true);
+      p = append_bin ((unsigned char*) &data_length, p, sizeof (data_length), true);
+      p = append_bin ((unsigned char*) buffer, p, data_length, false);
+    }
+  else
+    p = append_bin ((unsigned char*) &res, buf, sizeof (res), false);
+  xfree (buffer);
+}
+
+static void
+cuda_process_single_step_warp_packet (char *buf)
+{
+  CUDBGResult res;
+  char *p;
+  uint32_t dev;
+  uint32_t sm;
+  uint32_t wp;
+  uint32_t laneHint;
+  uint32_t nsteps;
+  uint32_t flags;
+  uint64_t warp_mask;
+
+  extract_bin (NULL, (unsigned char *) &dev, sizeof (dev));
+  extract_bin (NULL, (unsigned char *) &sm, sizeof (sm));
+  extract_bin (NULL, (unsigned char *) &wp, sizeof (wp));
+  extract_bin (NULL, (unsigned char *) &laneHint, sizeof (laneHint));
+  extract_bin (NULL, (unsigned char *) &nsteps, sizeof (nsteps));
+  extract_bin (NULL, (unsigned char *) &flags, sizeof (flags));
+  extract_bin (NULL, (unsigned char *) &warp_mask, sizeof (warp_mask));
+  res = cudbgAPI->singleStepWarp (dev, sm, wp, laneHint, nsteps, flags, &warp_mask);
+  p = append_bin ((unsigned char *) &res, buf, sizeof (res), true);
+  p = append_bin ((unsigned char *) &warp_mask, p, sizeof (warp_mask), false);
+}
+
 static void
 cuda_process_query_trace_message (char *buf)
 {
@@ -1327,8 +1418,8 @@ handle_cuda_packet (char *buf)
     case SUSPEND_DEVICE:
       cuda_process_suspend_device_packet (buf);
       break;
-    case SINGLE_STEP_WARP:
-      cuda_process_single_step_warp_packet (buf);
+    case SINGLE_STEP_WARP65:
+      cuda_process_single_step_warp_packet65 (buf);
       break;
     case SET_BREAKPOINT:
       cuda_process_set_breakpoint_packet (buf);
@@ -1532,6 +1623,15 @@ handle_cuda_packet (char *buf)
       cuda_process_update_cluster_idx_in_sm_packet (buf);
       break;
 #endif /* !__QNXHOST__ */
+    case GET_DEVICE_INFO_SIZES:
+      cuda_process_api_get_device_info_sizes (buf);
+      break;
+    case GET_DEVICE_INFO:
+      cuda_process_api_get_device_info (buf);
+      break;
+    case SINGLE_STEP_WARP:
+      cuda_process_single_step_warp_packet (buf);
+      break;
     default:
       error ("unknown cuda packet.\n");
       break;
