@@ -89,7 +89,7 @@ DEF_API_CALL(doNothing)()
 
 DEF_API_CALL(notSupported)()
 {
-	return CUDBG_ERROR_UNKNOWN;
+	return CUDBG_ERROR_NOT_SUPPORTED;
 }
 
 DEF_API_CALL(getElfImage32)(uint32_t dev, uint32_t sm, uint32_t wp,
@@ -1635,6 +1635,57 @@ DEF_API_CALL(getErrorStringEx)(char *buf, uint32_t bufSz, uint32_t *msgSz)
 	return CUDBG_SUCCESS;
 }
 
+DEF_API_CALL(getConstBankAddress123)(uint32_t dev, uint32_t sm, uint32_t wp, uint32_t bank, uint32_t offset, uint64_t* address)
+{
+	CUDBGResult rc;
+	uint64_t gridId;
+	CudbgGridTableEntry *gte;
+	CudbgConstBankTableEntry *cbte;
+
+	TRACE_FUNC("dev=%u sm=%u wp=%u bank=%u offset=%u",
+		   dev, sm, wp, bank, offset);
+
+	VERIFY_ARG(address);
+
+	if ((rc = getGridId(dev, sm, wp, &gridId)) != CUDBG_SUCCESS)
+		return rc;
+
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
+			"grid%llu_dev%u", gridId, dev);
+
+	GET_TABLE_ENTRY(cbte, NULL, CUDBG_ERROR_MISSING_DATA,
+			"grid%llu_dev%u_cbank%u", gte->gridId64, dev, bank);
+
+	if (offset >= cbte->size)
+		return CUDBG_ERROR_INVALID_MEMORY_ACCESS;
+
+	*address = cbte->addr + offset;
+
+	return CUDBG_SUCCESS;
+}
+
+DEF_API_CALL(getConstBankAddress)(uint32_t dev, uint64_t gridId64, uint32_t bank, uint64_t* address, uint32_t* size)
+{
+	CUDBGResult rc;
+	uint64_t gridId;
+	CudbgGridTableEntry *gte;
+	CudbgConstBankTableEntry *cbte;
+
+	TRACE_FUNC("dev=%u gridId64=%llu bank=%u",
+		   dev, gridId64, bank);
+
+	VERIFY_ARG(address);
+	VERIFY_ARG(size);
+
+	GET_TABLE_ENTRY(cbte, NULL, CUDBG_ERROR_MISSING_DATA,
+			"grid%llu_dev%u_cbank%u", gridId64, dev, bank);
+
+	*address = cbte->addr;
+	*size = cbte->size;
+
+	return CUDBG_SUCCESS;
+}
+
 static const struct CUDBGAPI_st cudbgCoreApi = {
     /* Initialization */
     API_CALL(doNothing),
@@ -1810,6 +1861,17 @@ static const struct CUDBGAPI_st cudbgCoreApi = {
 
     /* 12.2 Extensions */
     API_CALL(getErrorStringEx),
+
+    /* 12.3 Extensions */
+    API_CALL(notSupported), /* getLoadedFunctionInfo */
+    API_CALL(notSupported), /* generateCoredump */
+    API_CALL(getConstBankAddress123),
+
+    /* 12.4 Extensions */
+    API_CALL(notSupported), /* getDeviceInfoSizes */
+    API_CALL(notSupported), /* getDeviceInfo */
+    API_CALL(getConstBankAddress),
+    API_CALL(notSupported), /* singleStepWarp */
 };
 
 CUDBGAPI cuCoreGetApi(CudaCore *cc)
