@@ -753,6 +753,46 @@ static int cuCoreReadModuleTable(CudaCore *cc, Elf_Scn *scn)
 	return 0;
 }
 
+static int cuCoreReadConstBanksTable(CudaCore *cc, Elf_Scn *scn)
+{
+	CudbgGridTableEntry *gte;
+	CudbgDeviceTableEntry *dte;
+	CudbgConstBankTableEntry *cbte;
+	size_t cbte_count;
+	size_t cbte_size;
+	size_t cbte_grid_idx;
+	size_t cbte_gte_sh_off;
+
+	if (cuCoreGenericReadTable(cc->e, scn, &cbte_count, (void**)&cbte,
+				   &cbte_size, &cbte_gte_sh_off, &cbte_grid_idx))
+		return -1;
+
+	gte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
+				"grid_section%llu_offset%llu",
+				(unsigned long long)cbte_gte_sh_off,
+				(unsigned long long)cbte_grid_idx);
+	VERIFY(gte != NULL, -1, "Could not find Grid table entry");
+
+	dte = cuCoreGetMapEntry(&cc->tableEntriesMap,
+				NULL,
+				"grid_section%llu_offset%llu_dev",
+				(unsigned long long)cbte_gte_sh_off,
+				(unsigned long long)cbte_grid_idx);
+	VERIFY(dte != NULL, -1, "Could not find Device table entry by Grid");
+
+	for (; cbte_count-- > 0; cbte++)
+	{
+		if (cuCoreAddMapEntry(&cc->tableEntriesMap, cbte, cbte_size, false,
+				      "grid%llu_dev%u_cbank%u",
+				      (unsigned long long)gte->gridId64,
+				      dte->devId, cbte->bankId))
+			return -1;
+	}
+
+	return 0;
+}
+
 static int cuCoreReadSharedMemorySection(CudaCore *cc, Elf_Scn *scn)
 {
 	CudbgSmTableEntry *ste;
@@ -1148,6 +1188,8 @@ static int cuCoreProcessSection(CudaCore *cc, bool *processed, Elf_Scn *scn)
 		return cuCoreReadWarpInfo("uregs", cc, scn);
 	case CUDBG_SHT_DEV_UPRED:
 		return cuCoreReadWarpInfo("upred", cc, scn);
+	case CUDBG_SHT_CB_TABLE:
+		return cuCoreReadConstBanksTable(cc, scn);
 	default:
 		DPRINTF(5, "Found section of unknown type (0x%x)\n",
 			shdr->sh_type);
