@@ -1,5 +1,5 @@
 /* Calculate the size of physical memory.
-   Copyright (C) 2000-2019 Free Software Foundation, Inc.
+   Copyright (C) 2000-2022 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@
 #endif
 
 #if HAVE_SYS_SYSCTL_H
-#ifdef __ANDROID__
+#if defined(NVIDIA_CUDA_GDB) && defined(__ANDROID__)
 # include <linux/sysctl.h>
 #else
 # include <sys/sysctl.h>
