@@ -1,4 +1,4 @@
- /* Copyright 2010-2021 NVIDIA Corporation.  All rights reserved. 
+ /* Copyright 2010-2023 NVIDIA Corporation.  All rights reserved. 
   * 
   * NOTICE TO LICENSEE: 
   * 
@@ -61,13 +61,17 @@ extern "C" {
 #endif
 
 
-/** @defgroup signal_initialization Initialization
+/** 
+ * \page signal_initialization Initialization
+ * @defgroup signal_initialization Initialization
  * @ingroup npps
  * Functions that provide functionality of initialization signal like: set, zero or copy other signal.
  * @{
  */
 
-/** \defgroup signal_set Set
+/** 
+ * \section signal_set Set
+ * @defgroup signal_set Set
  * The set of set initialization operations available in the library.
  * @{
  *
@@ -91,10 +95,16 @@ extern "C" {
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_8u_Ctx(Npp8u nValue, Npp8u * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_8u_Ctx(Npp8u nValue, Npp8u * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 8-bit unsigned char, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_8u(Npp8u nValue, Npp8u * pDst, int nLength);
+nppsSet_8u(Npp8u nValue, Npp8u * pDst, size_t nLength);
 
 /** 
  * 8-bit signed char, vector set method.
@@ -105,10 +115,16 @@ nppsSet_8u(Npp8u nValue, Npp8u * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_8s_Ctx(Npp8s nValue, Npp8s * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_8s_Ctx(Npp8s nValue, Npp8s * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 8-bit signed char, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_8s(Npp8s nValue, Npp8s * pDst, int nLength);
+nppsSet_8s(Npp8s nValue, Npp8s * pDst, size_t nLength);
 
 /** 
  * 16-bit unsigned integer, vector set method.
@@ -119,10 +135,16 @@ nppsSet_8s(Npp8s nValue, Npp8s * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_16u_Ctx(Npp16u nValue, Npp16u * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_16u_Ctx(Npp16u nValue, Npp16u * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 16-bit unsigned integer, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_16u(Npp16u nValue, Npp16u * pDst, int nLength);
+nppsSet_16u(Npp16u nValue, Npp16u * pDst, size_t nLength);
 
 /** 
  * 16-bit signed integer, vector set method.
@@ -133,10 +155,16 @@ nppsSet_16u(Npp16u nValue, Npp16u * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_16s_Ctx(Npp16s nValue, Npp16s * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_16s_Ctx(Npp16s nValue, Npp16s * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 16-bit signed integer, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_16s(Npp16s nValue, Npp16s * pDst, int nLength);
+nppsSet_16s(Npp16s nValue, Npp16s * pDst, size_t nLength);
 
 /** 
  * 16-bit integer complex, vector set method.
@@ -147,10 +175,16 @@ nppsSet_16s(Npp16s nValue, Npp16s * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_16sc_Ctx(Npp16sc nValue, Npp16sc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_16sc_Ctx(Npp16sc nValue, Npp16sc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 16-bit integer complex, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_16sc(Npp16sc nValue, Npp16sc * pDst, int nLength);
+nppsSet_16sc(Npp16sc nValue, Npp16sc * pDst, size_t nLength);
 
 /** 
  * 32-bit unsigned integer, vector set method.
@@ -161,10 +195,16 @@ nppsSet_16sc(Npp16sc nValue, Npp16sc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_32u_Ctx(Npp32u nValue, Npp32u * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_32u_Ctx(Npp32u nValue, Npp32u * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit unsigned integer, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_32u(Npp32u nValue, Npp32u * pDst, int nLength);
+nppsSet_32u(Npp32u nValue, Npp32u * pDst, size_t nLength);
 
 /** 
  * 32-bit signed integer, vector set method.
@@ -175,10 +215,16 @@ nppsSet_32u(Npp32u nValue, Npp32u * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_32s_Ctx(Npp32s nValue, Npp32s * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_32s_Ctx(Npp32s nValue, Npp32s * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit signed integer, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_32s(Npp32s nValue, Npp32s * pDst, int nLength);
+nppsSet_32s(Npp32s nValue, Npp32s * pDst, size_t nLength);
 
 /** 
  * 32-bit integer complex, vector set method.
@@ -189,10 +235,16 @@ nppsSet_32s(Npp32s nValue, Npp32s * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_32sc_Ctx(Npp32sc nValue, Npp32sc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_32sc_Ctx(Npp32sc nValue, Npp32sc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit integer complex, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_32sc(Npp32sc nValue, Npp32sc * pDst, int nLength);
+nppsSet_32sc(Npp32sc nValue, Npp32sc * pDst, size_t nLength);
 
 /** 
  * 32-bit float, vector set method.
@@ -203,10 +255,16 @@ nppsSet_32sc(Npp32sc nValue, Npp32sc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_32f_Ctx(Npp32f nValue, Npp32f * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_32f_Ctx(Npp32f nValue, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit float, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_32f(Npp32f nValue, Npp32f * pDst, int nLength);
+nppsSet_32f(Npp32f nValue, Npp32f * pDst, size_t nLength);
 
 /** 
  * 32-bit float complex, vector set method.
@@ -217,10 +275,16 @@ nppsSet_32f(Npp32f nValue, Npp32f * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_32fc_Ctx(Npp32fc nValue, Npp32fc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_32fc_Ctx(Npp32fc nValue, Npp32fc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit float complex, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_32fc(Npp32fc nValue, Npp32fc * pDst, int nLength);
+nppsSet_32fc(Npp32fc nValue, Npp32fc * pDst, size_t nLength);
 
 /** 
  * 64-bit long long integer, vector set method.
@@ -231,10 +295,16 @@ nppsSet_32fc(Npp32fc nValue, Npp32fc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_64s_Ctx(Npp64s nValue, Npp64s * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_64s_Ctx(Npp64s nValue, Npp64s * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit long long integer, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_64s(Npp64s nValue, Npp64s * pDst, int nLength);
+nppsSet_64s(Npp64s nValue, Npp64s * pDst, size_t nLength);
 
 /** 
  * 64-bit long long integer complex, vector set method.
@@ -245,10 +315,16 @@ nppsSet_64s(Npp64s nValue, Npp64s * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_64sc_Ctx(Npp64sc nValue, Npp64sc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_64sc_Ctx(Npp64sc nValue, Npp64sc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit long long integer complex, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_64sc(Npp64sc nValue, Npp64sc * pDst, int nLength);
+nppsSet_64sc(Npp64sc nValue, Npp64sc * pDst, size_t nLength);
 
 /** 
  * 64-bit double, vector set method.
@@ -259,10 +335,16 @@ nppsSet_64sc(Npp64sc nValue, Npp64sc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_64f_Ctx(Npp64f nValue, Npp64f * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_64f_Ctx(Npp64f nValue, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit double, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_64f(Npp64f nValue, Npp64f * pDst, int nLength);
+nppsSet_64f(Npp64f nValue, Npp64f * pDst, size_t nLength);
 
 /** 
  * 64-bit double complex, vector set method.
@@ -273,15 +355,23 @@ nppsSet_64f(Npp64f nValue, Npp64f * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsSet_64fc_Ctx(Npp64fc nValue, Npp64fc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsSet_64fc_Ctx(Npp64fc nValue, Npp64fc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit double complex, vector set method.
+ * \param nValue Value used to initialize the vector pDst.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsSet_64fc(Npp64fc nValue, Npp64fc * pDst, int nLength);
+nppsSet_64fc(Npp64fc nValue, Npp64fc * pDst, size_t nLength);
 
 /** @} end of Signal Set */
 /** @} signal_set */
 
-/** \defgroup signal_zero Zero
+/** 
+ * \section signal_zero Zero
+ * @defgroup signal_zero Zero
  * The set of zero initialization operations available in the library.
  * @{
  *
@@ -302,10 +392,15 @@ nppsSet_64fc(Npp64fc nValue, Npp64fc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_8u_Ctx(Npp8u * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_8u_Ctx(Npp8u * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 8-bit unsigned char, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_8u(Npp8u * pDst, int nLength);
+nppsZero_8u(Npp8u * pDst, size_t nLength);
 
 /** 
  * 16-bit integer, vector zero method.
@@ -315,10 +410,15 @@ nppsZero_8u(Npp8u * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_16s_Ctx(Npp16s * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_16s_Ctx(Npp16s * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 16-bit integer, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_16s(Npp16s * pDst, int nLength);
+nppsZero_16s(Npp16s * pDst, size_t nLength);
 
 /** 
  * 16-bit integer complex, vector zero method.
@@ -328,10 +428,15 @@ nppsZero_16s(Npp16s * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_16sc_Ctx(Npp16sc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_16sc_Ctx(Npp16sc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 16-bit integer complex, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_16sc(Npp16sc * pDst, int nLength);
+nppsZero_16sc(Npp16sc * pDst, size_t nLength);
 
 /** 
  * 32-bit integer, vector zero method.
@@ -341,10 +446,15 @@ nppsZero_16sc(Npp16sc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_32s_Ctx(Npp32s * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_32s_Ctx(Npp32s * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit integer, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_32s(Npp32s * pDst, int nLength);
+nppsZero_32s(Npp32s * pDst, size_t nLength);
 
 /** 
  * 32-bit integer complex, vector zero method.
@@ -354,10 +464,15 @@ nppsZero_32s(Npp32s * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_32sc_Ctx(Npp32sc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_32sc_Ctx(Npp32sc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit integer complex, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_32sc(Npp32sc * pDst, int nLength);
+nppsZero_32sc(Npp32sc * pDst, size_t nLength);
 
 /** 
  * 32-bit float, vector zero method.
@@ -367,10 +482,15 @@ nppsZero_32sc(Npp32sc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_32f_Ctx(Npp32f * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_32f_Ctx(Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit float, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_32f(Npp32f * pDst, int nLength);
+nppsZero_32f(Npp32f * pDst, size_t nLength);
 
 /** 
  * 32-bit float complex, vector zero method.
@@ -380,10 +500,15 @@ nppsZero_32f(Npp32f * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_32fc_Ctx(Npp32fc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_32fc_Ctx(Npp32fc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit float complex, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_32fc(Npp32fc * pDst, int nLength);
+nppsZero_32fc(Npp32fc * pDst, size_t nLength);
 
 /** 
  * 64-bit long long integer, vector zero method.
@@ -393,10 +518,15 @@ nppsZero_32fc(Npp32fc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_64s_Ctx(Npp64s * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_64s_Ctx(Npp64s * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit long long integer, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_64s(Npp64s * pDst, int nLength);
+nppsZero_64s(Npp64s * pDst, size_t nLength);
 
 /** 
  * 64-bit long long integer complex, vector zero method.
@@ -406,10 +536,15 @@ nppsZero_64s(Npp64s * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_64sc_Ctx(Npp64sc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_64sc_Ctx(Npp64sc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit long long integer complex, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_64sc(Npp64sc * pDst, int nLength);
+nppsZero_64sc(Npp64sc * pDst, size_t nLength);
 
 /** 
  * 64-bit double, vector zero method.
@@ -419,10 +554,15 @@ nppsZero_64sc(Npp64sc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_64f_Ctx(Npp64f * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_64f_Ctx(Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit double, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_64f(Npp64f * pDst, int nLength);
+nppsZero_64f(Npp64f * pDst, size_t nLength);
 
 /** 
  * 64-bit double complex, vector zero method.
@@ -432,16 +572,23 @@ nppsZero_64f(Npp64f * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsZero_64fc_Ctx(Npp64fc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsZero_64fc_Ctx(Npp64fc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit double complex, vector zero method.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsZero_64fc(Npp64fc * pDst, int nLength);
+nppsZero_64fc(Npp64fc * pDst, size_t nLength);
 
 /** @} end of Zero */
 
 /** @} signal_zero */
 
-/** \defgroup signal_copy Copy
+/** 
+ * \section signal_copy Copy
+ * @defgroup signal_copy Copy
  * The set of copy initialization operations available in the library.
  * @{
  *
@@ -466,10 +613,16 @@ nppsZero_64fc(Npp64fc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsCopy_8u_Ctx(const Npp8u * pSrc, Npp8u * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsCopy_8u_Ctx(const Npp8u * pSrc, Npp8u * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 8-bit unsigned char, vector copy method
+ * \param pSrc \ref source_signal_pointer.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsCopy_8u(const Npp8u * pSrc, Npp8u * pDst, int nLength);
+nppsCopy_8u(const Npp8u * pSrc, Npp8u * pDst, size_t nLength);
 
 /** 
  * 16-bit signed short, vector copy method.
@@ -480,10 +633,16 @@ nppsCopy_8u(const Npp8u * pSrc, Npp8u * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsCopy_16s_Ctx(const Npp16s * pSrc, Npp16s * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsCopy_16s_Ctx(const Npp16s * pSrc, Npp16s * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 16-bit signed short, vector copy method.
+ * \param pSrc \ref source_signal_pointer.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsCopy_16s(const Npp16s * pSrc, Npp16s * pDst, int nLength);
+nppsCopy_16s(const Npp16s * pSrc, Npp16s * pDst, size_t nLength);
 
 /** 
  * 32-bit signed integer, vector copy method.
@@ -494,10 +653,16 @@ nppsCopy_16s(const Npp16s * pSrc, Npp16s * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsCopy_32s_Ctx(const Npp32s * pSrc, Npp32s * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsCopy_32s_Ctx(const Npp32s * pSrc, Npp32s * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit signed integer, vector copy method.
+ * \param pSrc \ref source_signal_pointer.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsCopy_32s(const Npp32s * pSrc, Npp32s * pDst, int nLength);
+nppsCopy_32s(const Npp32s * pSrc, Npp32s * pDst, size_t nLength);
 
 /** 
  * 32-bit float, vector copy method.
@@ -508,10 +673,16 @@ nppsCopy_32s(const Npp32s * pSrc, Npp32s * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsCopy_32f_Ctx(const Npp32f * pSrc, Npp32f * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsCopy_32f_Ctx(const Npp32f * pSrc, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit float, vector copy method.
+ * \param pSrc \ref source_signal_pointer.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsCopy_32f(const Npp32f * pSrc, Npp32f * pDst, int nLength);
+nppsCopy_32f(const Npp32f * pSrc, Npp32f * pDst, size_t nLength);
 
 /** 
  * 64-bit signed integer, vector copy method.
@@ -522,10 +693,16 @@ nppsCopy_32f(const Npp32f * pSrc, Npp32f * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsCopy_64s_Ctx(const Npp64s * pSrc, Npp64s * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsCopy_64s_Ctx(const Npp64s * pSrc, Npp64s * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit signed integer, vector copy method.
+ * \param pSrc \ref source_signal_pointer.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsCopy_64s(const Npp64s * pSrc, Npp64s * pDst, int nLength);
+nppsCopy_64s(const Npp64s * pSrc, Npp64s * pDst, size_t nLength);
 
 /** 
  * 16-bit complex short, vector copy method.
@@ -536,10 +713,16 @@ nppsCopy_64s(const Npp64s * pSrc, Npp64s * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsCopy_16sc_Ctx(const Npp16sc * pSrc, Npp16sc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsCopy_16sc_Ctx(const Npp16sc * pSrc, Npp16sc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 16-bit complex short, vector copy method.
+ * \param pSrc \ref source_signal_pointer.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsCopy_16sc(const Npp16sc * pSrc, Npp16sc * pDst, int nLength);
+nppsCopy_16sc(const Npp16sc * pSrc, Npp16sc * pDst, size_t nLength);
 
 /** 
  * 32-bit complex signed integer, vector copy method.
@@ -550,10 +733,16 @@ nppsCopy_16sc(const Npp16sc * pSrc, Npp16sc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsCopy_32sc_Ctx(const Npp32sc * pSrc, Npp32sc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsCopy_32sc_Ctx(const Npp32sc * pSrc, Npp32sc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit complex signed integer, vector copy method.
+ * \param pSrc \ref source_signal_pointer.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsCopy_32sc(const Npp32sc * pSrc, Npp32sc * pDst, int nLength);
+nppsCopy_32sc(const Npp32sc * pSrc, Npp32sc * pDst, size_t nLength);
 
 /** 
  * 32-bit complex float, vector copy method.
@@ -564,10 +753,16 @@ nppsCopy_32sc(const Npp32sc * pSrc, Npp32sc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsCopy_32fc_Ctx(const Npp32fc * pSrc, Npp32fc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsCopy_32fc_Ctx(const Npp32fc * pSrc, Npp32fc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 32-bit complex float, vector copy method.
+ * \param pSrc \ref source_signal_pointer.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsCopy_32fc(const Npp32fc * pSrc, Npp32fc * pDst, int nLength);
+nppsCopy_32fc(const Npp32fc * pSrc, Npp32fc * pDst, size_t nLength);
 
 /** 
  * 64-bit complex signed integer, vector copy method.
@@ -578,10 +773,16 @@ nppsCopy_32fc(const Npp32fc * pSrc, Npp32fc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsCopy_64sc_Ctx(const Npp64sc * pSrc, Npp64sc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsCopy_64sc_Ctx(const Npp64sc * pSrc, Npp64sc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit complex signed integer, vector copy method.
+ * \param pSrc \ref source_signal_pointer.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsCopy_64sc(const Npp64sc * pSrc, Npp64sc * pDst, int nLength);
+nppsCopy_64sc(const Npp64sc * pSrc, Npp64sc * pDst, size_t nLength);
 
 /** 
  * 64-bit complex double, vector copy method.
@@ -592,10 +793,16 @@ nppsCopy_64sc(const Npp64sc * pSrc, Npp64sc * pDst, int nLength);
  * \return \ref signal_data_error_codes, \ref length_error_codes.
  */
 NppStatus 
-nppsCopy_64fc_Ctx(const Npp64fc * pSrc, Npp64fc * pDst, int nLength, NppStreamContext nppStreamCtx);
-
+nppsCopy_64fc_Ctx(const Npp64fc * pSrc, Npp64fc * pDst, size_t nLength, NppStreamContext nppStreamCtx);
+/** 
+ * 64-bit complex double, vector copy method.
+ * \param pSrc \ref source_signal_pointer.
+ * \param pDst \ref destination_signal_pointer.
+ * \param nLength \ref length_specification. 
+ * \return \ref signal_data_error_codes, \ref length_error_codes.
+ */
 NppStatus 
-nppsCopy_64fc(const Npp64fc * pSrc, Npp64fc * pDst, int nLength);
+nppsCopy_64fc(const Npp64fc * pSrc, Npp64fc * pDst, size_t nLength);
 
 /** @} end of Copy */
 
