@@ -41,9 +41,28 @@ typedef enum {
   NVJITLINK_ERROR_INVALID_INPUT,
   NVJITLINK_ERROR_PTX_COMPILE,
   NVJITLINK_ERROR_NVVM_COMPILE,
-  NVJITLINK_ERROR_INTERNAL
+  NVJITLINK_ERROR_INTERNAL,
+  NVJITLINK_ERROR_THREADPOOL,
+  NVJITLINK_ERROR_UNRECOGNIZED_INPUT,
+#ifdef NEW_ERROR_CODES // These error codes will appear in a future CUDA release.
+  NVJITLINK_ERROR_NULL_INPUT,
+  NVJITLINK_ERROR_INCOMPATIBLE_OPTIONS,
+  NVJITLINK_ERROR_INCORRECT_INPUT_TYPE,
+  NVJITLINK_ERROR_ARCH_MISMATCH,
+  NVJITLINK_ERROR_OUTDATED_LIBRARY,
+  NVJITLINK_ERROR_MISSING_FATBIN
+#endif
 } nvJitLinkResult;
-  
+
+#ifndef NEW_ERROR_CODES // To avoid breaking compatibility, we map them to existing error codes for now.
+#define NVJITLINK_ERROR_NULL_INPUT NVJITLINK_ERROR_INVALID_INPUT
+#define NVJITLINK_ERROR_INCOMPATIBLE_OPTIONS NVJITLINK_ERROR_INVALID_INPUT
+#define NVJITLINK_ERROR_INCORRECT_INPUT_TYPE NVJITLINK_ERROR_INVALID_INPUT
+#define NVJITLINK_ERROR_ARCH_MISMATCH NVJITLINK_ERROR_INTERNAL
+#define NVJITLINK_ERROR_OUTDATED_LIBRARY NVJITLINK_ERROR_INTERNAL
+#define NVJITLINK_ERROR_MISSING_FATBIN NVJITLINK_ERROR_INVALID_INPUT
+#endif  
+
 /**
  *
  * \defgroup linking Linking
@@ -63,7 +82,8 @@ typedef enum {
   NVJITLINK_INPUT_LTOIR,
   NVJITLINK_INPUT_FATBIN,
   NVJITLINK_INPUT_OBJECT,
-  NVJITLINK_INPUT_LIBRARY
+  NVJITLINK_INPUT_LIBRARY,
+  NVJITLINK_INPUT_ANY = 10 // will dynamically determine one of above types
 } nvJitLinkInputType;
 
 /**
@@ -90,7 +110,7 @@ typedef enum {
  * - \c -ptx \n
  *   Emit ptx after linking instead of cubin; only supported with \c -lto
  * - \c -O<N\> \n
- *   Optimization level.
+ *   Optimization level. Only 0 and 3 are accepted.
  * - \c -g \n
  *   Generate debug information.
  * - \c -lineinfo \n
@@ -115,8 +135,18 @@ typedef enum {
  *   is not referenced in device code then it can be removed.
  * - \c -Xptxas=<opt\> \n
  *   Pass <opt\> to ptxas.  This option can be called multiple times.
- * - \c -Xnvvm=<opt\> \n
- *   Pass <opt\> to nvvm.  This option can be called multiple times.
+ * - \c -split-compile=<N\> \n
+ *   Split compilation maximum thread count. Use 0 to use all available processors.
+ *   Value of 1 disables split compilation (default).
+ * - \c -split-compile-extended=<N\> \n
+ *   [Experimental] A more aggressive form of split compilation.
+ *   Accepts a maximum thread count value. Use 0 to use all available processors.
+ *   Value of 1 disables extended split compilation (default).
+ * - \c -jump-table-density=<N\> \n
+ *   When doing LTO, specify the case density percentage in switch statements,
+ *   and use it as a minimal threshold to determine whether jump table(brx.idx 
+ *   instruction) will be used to implement a switch statement. Default
+ *   value is 101. The percentage ranges from 0 to 101 inclusively.
  */
 
 /**
@@ -132,7 +162,7 @@ typedef struct nvJitLink* nvJitLinkHandle; // opaque handle
 
 // For versioning we will have separate API version for each library version
 
-extern nvJitLinkResult __nvJitLinkCreate_12_2(
+extern nvJitLinkResult __nvJitLinkCreate_12_4(
   nvJitLinkHandle *handle,
   uint32_t numOptions,
   const char **options);
@@ -149,21 +179,23 @@ extern nvJitLinkResult __nvJitLinkCreate_12_2(
  *   - \link #nvJitLinkResult NVJITLINK_ERROR_UNRECOGNIZED_OPTION\endlink
  *   - \link #nvJitLinkResult NVJITLINK_ERROR_MISSING_ARCH\endlink
  *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  *
  * It supports options listed in \ref options.
  *
  * \see nvJitLinkDestroy
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkCreate(
   nvJitLinkHandle *handle,
   uint32_t numOptions,
   const char **options)
 {
-  return __nvJitLinkCreate_12_2 (handle, numOptions, options);
+  return __nvJitLinkCreate_12_4 (handle, numOptions, options);
 }
+#endif
  
-extern nvJitLinkResult __nvJitLinkDestroy_12_2 (nvJitLinkHandle *handle);
+extern nvJitLinkResult __nvJitLinkDestroy_12_4 (nvJitLinkHandle *handle);
 /**
  * \ingroup linking
  * \brief   nvJitLinkDestroy frees the memory associated with the given handle
@@ -172,17 +204,19 @@ extern nvJitLinkResult __nvJitLinkDestroy_12_2 (nvJitLinkHandle *handle);
  * \param    [in] handle      Address of nvJitLink handle.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  *
  * \see nvJitLinkCreate
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkDestroy (nvJitLinkHandle *handle)
 {
-  return __nvJitLinkDestroy_12_2 (handle);
+  return __nvJitLinkDestroy_12_4 (handle);
 }
+#endif
  
-extern nvJitLinkResult __nvJitLinkAddData_12_2(
+extern nvJitLinkResult __nvJitLinkAddData_12_4(
   nvJitLinkHandle handle,
   nvJitLinkInputType inputType,
   const void *data,
@@ -195,12 +229,14 @@ extern nvJitLinkResult __nvJitLinkAddData_12_2(
  * \param    [in] handle      nvJitLink handle.
  * \param    [in] inputType   kind of input.
  * \param    [in] data        pointer to data image in memory.
+ * \param    [in] size        size of the data.
  * \param    [in] name        name of input object.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkAddData(
   nvJitLinkHandle handle,
   nvJitLinkInputType inputType,
@@ -208,10 +244,11 @@ static inline nvJitLinkResult nvJitLinkAddData(
   size_t size,
   const char *name) // name can be null
 {
-  return __nvJitLinkAddData_12_2 (handle, inputType, data, size, name);
+  return __nvJitLinkAddData_12_4 (handle, inputType, data, size, name);
 }
+#endif
  
-extern nvJitLinkResult __nvJitLinkAddFile_12_2(
+extern nvJitLinkResult __nvJitLinkAddFile_12_4(
   nvJitLinkHandle handle,
   nvJitLinkInputType inputType,
   const char *fileName); // includes path to file
@@ -224,18 +261,20 @@ extern nvJitLinkResult __nvJitLinkAddFile_12_2(
  * \param    [in] fileName    name of file.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkAddFile(
   nvJitLinkHandle handle,
   nvJitLinkInputType inputType,
   const char *fileName) // includes path to file
 {
-  return __nvJitLinkAddFile_12_2 (handle, inputType, fileName);
+  return __nvJitLinkAddFile_12_4 (handle, inputType, fileName);
 }
+#endif
  
-extern nvJitLinkResult __nvJitLinkComplete_12_2 (nvJitLinkHandle handle);
+extern nvJitLinkResult __nvJitLinkComplete_12_4 (nvJitLinkHandle handle);
 /**
  * \ingroup linking
  * \brief   nvJitLinkComplete does the actual link. 
@@ -243,15 +282,17 @@ extern nvJitLinkResult __nvJitLinkComplete_12_2 (nvJitLinkHandle handle);
  * \param    [in] handle      nvJitLink handle.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkComplete (nvJitLinkHandle handle)
 {
-  return __nvJitLinkComplete_12_2 (handle);
+  return __nvJitLinkComplete_12_4 (handle);
 }
+#endif
  
-extern nvJitLinkResult __nvJitLinkGetLinkedCubinSize_12_2(
+extern nvJitLinkResult __nvJitLinkGetLinkedCubinSize_12_4(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -262,19 +303,21 @@ extern nvJitLinkResult __nvJitLinkGetLinkedCubinSize_12_2(
  * \param    [out] size       Size of the linked cubin.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  *
  * \see nvJitLinkGetLinkedCubin
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkGetLinkedCubinSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetLinkedCubinSize_12_2 (handle, size);
+  return __nvJitLinkGetLinkedCubinSize_12_4 (handle, size);
 }
+#endif
 
-extern nvJitLinkResult __nvJitLinkGetLinkedCubin_12_2(
+extern nvJitLinkResult __nvJitLinkGetLinkedCubin_12_4(
   nvJitLinkHandle handle,
   void *cubin);
 /**
@@ -285,20 +328,22 @@ extern nvJitLinkResult __nvJitLinkGetLinkedCubin_12_2(
  * \param    [out] cubin      The linked cubin.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  *
  * User is responsible for allocating enough space to hold the \p cubin.
  * \see nvJitLinkGetLinkedCubinSize
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkGetLinkedCubin(
   nvJitLinkHandle handle,
   void *cubin)
 {
-  return __nvJitLinkGetLinkedCubin_12_2 (handle, cubin);
+  return __nvJitLinkGetLinkedCubin_12_4 (handle, cubin);
 }
+#endif
  
-extern nvJitLinkResult __nvJitLinkGetLinkedPtxSize_12_2(
+extern nvJitLinkResult __nvJitLinkGetLinkedPtxSize_12_4(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -309,20 +354,22 @@ extern nvJitLinkResult __nvJitLinkGetLinkedPtxSize_12_2(
  * \param    [out] size       Size of the linked PTX.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  *
  * Linked PTX is only available when using the \c -lto option.
  * \see nvJitLinkGetLinkedPtx
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkGetLinkedPtxSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetLinkedPtxSize_12_2 (handle, size);
+  return __nvJitLinkGetLinkedPtxSize_12_4 (handle, size);
 }
+#endif
 
-extern nvJitLinkResult __nvJitLinkGetLinkedPtx_12_2(
+extern nvJitLinkResult __nvJitLinkGetLinkedPtx_12_4(
   nvJitLinkHandle handle,
   char *ptx);
 /**
@@ -333,21 +380,23 @@ extern nvJitLinkResult __nvJitLinkGetLinkedPtx_12_2(
  * \param    [out] ptx        The linked PTX.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  *
  * Linked PTX is only available when using the \c -lto option.
  * User is responsible for allocating enough space to hold the \p ptx.
  * \see nvJitLinkGetLinkedPtxSize
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkGetLinkedPtx(
   nvJitLinkHandle handle,
   char *ptx)
 {
-  return __nvJitLinkGetLinkedPtx_12_2 (handle, ptx);
+  return __nvJitLinkGetLinkedPtx_12_4 (handle, ptx);
 }
+#endif
  
-extern nvJitLinkResult __nvJitLinkGetErrorLogSize_12_2(
+extern nvJitLinkResult __nvJitLinkGetErrorLogSize_12_4(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -358,19 +407,21 @@ extern nvJitLinkResult __nvJitLinkGetErrorLogSize_12_2(
  * \param    [out] size       Size of the error log.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  *
  * \see nvJitLinkGetErrorLog
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkGetErrorLogSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetErrorLogSize_12_2 (handle, size);
+  return __nvJitLinkGetErrorLogSize_12_4 (handle, size);
 }
+#endif
 
-extern nvJitLinkResult __nvJitLinkGetErrorLog_12_2(
+extern nvJitLinkResult __nvJitLinkGetErrorLog_12_4(
   nvJitLinkHandle handle,
   char *log);
 /**
@@ -381,20 +432,22 @@ extern nvJitLinkResult __nvJitLinkGetErrorLog_12_2(
  * \param    [out] log        The error log.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  *
  * User is responsible for allocating enough space to hold the \p log.
  * \see nvJitLinkGetErrorLogSize
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkGetErrorLog(
   nvJitLinkHandle handle,
   char *log)
 {
-  return __nvJitLinkGetErrorLog_12_2 (handle, log);
+  return __nvJitLinkGetErrorLog_12_4 (handle, log);
 }
+#endif
 
-extern nvJitLinkResult __nvJitLinkGetInfoLogSize_12_2(
+extern nvJitLinkResult __nvJitLinkGetInfoLogSize_12_4(
   nvJitLinkHandle handle,
   size_t *size);
 /**
@@ -405,19 +458,21 @@ extern nvJitLinkResult __nvJitLinkGetInfoLogSize_12_2(
  * \param    [out] size       Size of the info log.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  *
  * \see nvJitLinkGetInfoLog
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkGetInfoLogSize(
   nvJitLinkHandle handle,
   size_t *size)
 {
-  return __nvJitLinkGetInfoLogSize_12_2 (handle, size);
+  return __nvJitLinkGetInfoLogSize_12_4 (handle, size);
 }
+#endif
 
-extern nvJitLinkResult __nvJitLinkGetInfoLog_12_2(
+extern nvJitLinkResult __nvJitLinkGetInfoLog_12_4(
   nvJitLinkHandle handle,
   char *log);
 /**
@@ -428,18 +483,36 @@ extern nvJitLinkResult __nvJitLinkGetInfoLog_12_2(
  * \param    [out] log        The info log.
  * \return
  *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INVALID_INPUT\endlink
- *   - \link #nvJitLinkResult NVRTC_ERROR_INTERNAL\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
  *
  * User is responsible for allocating enough space to hold the \p log.
  * \see nvJitLinkGetInfoLogSize
  */
+#ifndef NVJITLINK_NO_INLINE
 static inline nvJitLinkResult nvJitLinkGetInfoLog(
   nvJitLinkHandle handle,
   char *log)
 {
-  return __nvJitLinkGetInfoLog_12_2 (handle, log);
+  return __nvJitLinkGetInfoLog_12_4 (handle, log);
 }
+#endif
+
+/**
+ * \ingroup linking
+ * \brief   nvJitLinkVersion returns the current version of nvJitLink.
+ *
+ * \param    [out] major        The major version.
+ * \param    [out] minor        The minor version.
+ * \return
+ *   - \link #nvJitLinkResult NVJITLINK_SUCCESS \endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INVALID_INPUT\endlink
+ *   - \link #nvJitLinkResult NVJITLINK_ERROR_INTERNAL\endlink
+ *
+ */
+extern nvJitLinkResult nvJitLinkVersion(
+  unsigned int *major,
+  unsigned int *minor);
 
 #ifdef __cplusplus
 }
