@@ -68,10 +68,9 @@
 
 // nvjpeg version information
 #define NVJPEG_VER_MAJOR 12
-#define NVJPEG_VER_MINOR 1
+#define NVJPEG_VER_MINOR 3
 #define NVJPEG_VER_PATCH 1
-#define NVJPEG_VER_BUILD 14
-
+#define NVJPEG_VER_BUILD 117
 
 /* nvJPEG status enums, returned by nvJPEG API */
 typedef enum
@@ -566,6 +565,8 @@ nvjpegStatus_t NVJPEGAPI nvjpegBufferPinnedCreateV2(nvjpegHandle_t handle,
     nvjpegPinnedAllocatorV2_t* pinned_allocator,
     nvjpegBufferPinned_t* buffer);
 
+nvjpegStatus_t NVJPEGAPI nvjpegBufferPinnedResize(nvjpegBufferPinned_t buffer, size_t size, cudaStream_t stream);
+
 nvjpegStatus_t NVJPEGAPI nvjpegBufferPinnedDestroy(nvjpegBufferPinned_t buffer);
 
 struct nvjpegBufferDevice;
@@ -579,6 +580,8 @@ nvjpegStatus_t NVJPEGAPI nvjpegBufferDeviceCreateV2(nvjpegHandle_t handle,
     nvjpegDevAllocatorV2_t* device_allocator,
     nvjpegBufferDevice_t* buffer);
 
+nvjpegStatus_t NVJPEGAPI nvjpegBufferDeviceResize(nvjpegBufferDevice_t buffer, size_t size, cudaStream_t stream);
+
 nvjpegStatus_t NVJPEGAPI nvjpegBufferDeviceDestroy(nvjpegBufferDevice_t buffer);
 
 // retrieve buffer size and pointer - this allows reusing buffer when decode is not needed
