@@ -1,4 +1,4 @@
-# Copyright (c) 2023, NVIDIA CORPORATION. All rights reserved.
+# Copyright (c) 2024, NVIDIA CORPORATION. All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
 # are met:
@@ -23,6 +23,11 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 import NvRules
+from RequestedMetrics import MetricRequest, RequestedMetricsParser
+
+requested_metrics = [
+    MetricRequest("sass__inst_executed_per_opcode"),
+]
 
 
 def get_identifier():
@@ -37,11 +42,32 @@ def get_description():
 def get_section_identifier():
     return "InstructionStats"
 
+def get_parent_rules_identifiers():
+    return ["HighPipeUtilization"]
+
+def get_estimated_speedup(pipeline_utilization_pct, fused_instructions, non_fused_instructions):
+    # To calculate the speedup, assume we can convert non-fused to fused instructions,
+    # which have double the throughput.
+    # To get a global estimate weigh this with the FP pipeline utilization
+    # (in terms of active cycles).
+    all_instructions = non_fused_instructions + fused_instructions
+    improvement_local = 0.5 * (non_fused_instructions / all_instructions)
+
+    if pipeline_utilization_pct is not None:
+        speedup_type = NvRules.IFrontend.SpeedupType_GLOBAL
+        improvement_percent = improvement_local * pipeline_utilization_pct
+    else:
+        speedup_type = NvRules.IFrontend.SpeedupType_LOCAL
+        improvement_percent = improvement_local * 100
+
+    return speedup_type, improvement_percent
 
 def apply(handle):
     ctx = NvRules.get_context(handle)
     action = ctx.range_by_idx(0).action_by_idx(0)
     fe = ctx.frontend()
+    metrics = RequestedMetricsParser(handle, action).parse(requested_metrics)
+    parent_weights = fe.receive_dict_from_parent("HighPipeUtilization")
 
     fp_types = {
         32 : [ "FADD", "FMUL", "FFMA" ],
@@ -49,7 +75,7 @@ def apply(handle):
     }
 
     # the correlation IDs of sass__inst_executed_per_opcode are the opcode mnemonics
-    inst_per_opcode = action.metric_by_name("sass__inst_executed_per_opcode")
+    inst_per_opcode = metrics["sass__inst_executed_per_opcode"]
     num_opcodes = inst_per_opcode.num_instances()
     opcodes = inst_per_opcode.correlation_ids()
 
@@ -83,9 +109,20 @@ def apply(handle):
                 message += " By converting pairs of non-fused instructions to their @url:fused:https://docs.nvidia.com/cuda/floating-point/#cuda-and-floating-point@, higher-throughput equivalent, the achieved FP{} performance could be increased by up to {:.0f}%"\
                     " (relative to its current performance)."\
                     " Check the Source page to identify where this kernel executes FP{} instructions.".format(fp_type, 100. * ratio, fp_type)
-                msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_OPTIMIZATION, message)
+                message_title = "FP{} Non-Fused Instructions".format(fp_type)
+                msg_id = fe.message(NvRules.IFrontend.MsgType_MSG_OPTIMIZATION, message, message_title)
 
-                improvement_percent = (non_fused / (non_fused + fused)) / 2 * 100
-                fe.speedup(msg_id, NvRules.IFrontend.SpeedupType_LOCAL, improvement_percent)
+                pipeline_utilization_pct = None
+                parent_weight_name = "fp{}_pipeline_utilization_pct".format(fp_type)
+                if parent_weight_name in parent_weights:
+                    pipeline_utilization_pct = parent_weights[parent_weight_name]
+                speedup_type, speedup_value = get_estimated_speedup(pipeline_utilization_pct, fused, non_fused)
+                fe.speedup(msg_id, speedup_type, speedup_value)
 
-                fe.focus_metric(msg_id, "sass__inst_executed_per_opcode", non_fused, NvRules.IFrontend.Severity_SEVERITY_DEFAULT, "Decrease the number of non-fused floating-point instructions (FADD, FMUL, DADD, DMUL)")
+                fe.focus_metric(msg_id, "sass__inst_executed_per_opcode", non_fused, NvRules.IFrontend.Severity_SEVERITY_HIGH, "Decrease the number of non-fused floating-point instructions (FADD, FMUL, DADD, DMUL)")
+                if pipeline_utilization_pct is not None:
+                    if fp_type == 32:
+                        metric_name = "sm__pipe_fma_cycles_active.avg.pct_of_peak_sustained_active"
+                    else:
+                        metric_name = "sm__pipe_fp64_cycles_active.avg.pct_of_peak_sustained_active"
+                    fe.focus_metric(msg_id, metric_name, pipeline_utilization_pct, NvRules.IFrontend.Severity_SEVERITY_LOW, "The higher the utilization of the pipeline the more severe the issue becomes")
