@@ -1,4 +1,4 @@
-# Copyright (c) 2023, NVIDIA CORPORATION. All rights reserved.
+# Copyright (c) 2024, NVIDIA CORPORATION. All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
 # are met:
@@ -23,6 +23,14 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 import NvRules
+from RequestedMetrics import MetricRequest, RequestedMetricsParser
+
+requested_metrics = [
+    MetricRequest("smsp__pcsamp_sample_count"),
+    MetricRequest("smsp__pcsamp_interval_cycles"),
+    MetricRequest("gpc__cycles_elapsed.max"),
+]
+
 
 def get_identifier():
     return "PCSamplingData"
@@ -43,10 +51,11 @@ def apply(handle):
     ctx = NvRules.get_context(handle)
     action = ctx.range_by_idx(0).action_by_idx(0)
     fe = ctx.frontend()
+    metrics = RequestedMetricsParser(handle, action).parse(requested_metrics)
 
-    sample_count = action.metric_by_name("smsp__pcsamp_sample_count").as_uint64()
-    interval_cycles = action.metric_by_name("smsp__pcsamp_interval_cycles").as_double()
-    kernel_duration = action.metric_by_name("gpc__cycles_elapsed.max").as_double()
+    sample_count = metrics["smsp__pcsamp_sample_count"].value()
+    interval_cycles = metrics["smsp__pcsamp_interval_cycles"].value()
+    kernel_duration = metrics["gpc__cycles_elapsed.max"].value()
 
     if sample_count == 0:
         message = "Sampling metrics were enabled, but no samples could be collected for this kernel."
