@@ -1,4 +1,4 @@
-# Copyright (c) 2023, NVIDIA CORPORATION. All rights reserved.
+# Copyright (c) 2024, NVIDIA CORPORATION. All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
 # are met:
@@ -23,6 +23,14 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 import NvRules
+from RequestedMetrics import MetricRequest, RequestedMetricsParser
+
+requested_metrics = [
+    MetricRequest("sm__instruction_throughput.avg.pct_of_peak_sustained_active", "instruction_throughput"),
+    MetricRequest("sm__inst_issued.avg.pct_of_peak_sustained_active", "inst_issued_avg"),
+    MetricRequest("sm__inst_issued.max.pct_of_peak_sustained_active", "inst_issued_max"),
+]
+
 
 def get_identifier():
     return "SlowPipeLimiter"
@@ -37,36 +45,44 @@ def get_section_identifier():
     return "ComputeWorkloadAnalysis"
 
 
-def get_estimated_speedup(action):
+def get_parent_rules_identifiers():
+    return ["Compute"]
+
+
+def get_estimated_speedup(parent_weights, metrics):
     """Estimate potential speedup from decreasing the usage of slow pipes.
 
     The performance improvement is approximated as the relative part of instructions
     not issued.
-    In case "sm__throughput.avg.pct_of_peak_sustained_elapsed" was collected,
-    the above approximation can be improved by weighting it with the achieved throughput.
+    In case the compute (SM) throughput was collected,
+    the above approximation can be improved by weighing it with the achieved throughput.
 
     """
-    inst_issued_max = action["sm__inst_issued.max.pct_of_peak_sustained_active"].value()
-    inst_issued_avg = action["sm__inst_issued.avg.pct_of_peak_sustained_active"].value()
+    inst_issued_avg = metrics["inst_issued_avg"].value()
+    inst_issued_max = metrics["inst_issued_max"].value()
     improvement_local = (inst_issued_max - inst_issued_avg) / inst_issued_max
 
-    throughput_name = "sm__throughput.avg.pct_of_peak_sustained_elapsed"
-    if throughput_name in action:
-        throughput = action[throughput_name].value()
-        improvement_global = throughput * improvement_local
-        return NvRules.IFrontend.SpeedupType_GLOBAL, improvement_global * 100
+    compute_throughput_name = "compute_throughput_normalized"
+    if compute_throughput_name in parent_weights:
+        speedup_type = NvRules.IFrontend.SpeedupType_GLOBAL
+        improvement_percent = improvement_local * parent_weights[compute_throughput_name] * 100
     else:
-        return NvRules.IFrontend.SpeedupType_LOCAL, improvement_local * 100
+        speedup_type = NvRules.IFrontend.SpeedupType_LOCAL
+        improvement_percent = improvement_local * 100
+
+    return speedup_type, improvement_percent
 
 
 def apply(handle):
     ctx = NvRules.get_context(handle)
     action = ctx.range_by_idx(0).action_by_idx(0)
     fe = ctx.frontend()
+    metrics = RequestedMetricsParser(handle, action).parse(requested_metrics)
+    parent_weights = fe.receive_dict_from_parent("Compute")
 
-    sm_busy = action.metric_by_name("sm__instruction_throughput.avg.pct_of_peak_sustained_active").as_double()
-    inst_issued_avg = action.metric_by_name("sm__inst_issued.avg.pct_of_peak_sustained_active").as_double()
-    inst_issued_max = action.metric_by_name("sm__inst_issued.max.pct_of_peak_sustained_active").as_double()
+    sm_busy = metrics["instruction_throughput"].value()
+    inst_issued_avg = metrics["inst_issued_avg"].value()
+    inst_issued_max = metrics["inst_issued_max"].value()
 
     no_bound_threshold = 80
     issued_avg_threshold = 20
@@ -82,10 +98,10 @@ def apply(handle):
         " Try moving compute to other pipelines, e.g. from fp64 to fp32 or int."\
         "{}".format(inst_issued_avg, pipe_diff, inst_issued_max, doc_msg), "Slow Pipeline")
 
-        speedup_type, speedup_value = get_estimated_speedup(action)
+        speedup_type, speedup_value = get_estimated_speedup(parent_weights, metrics)
         fe.speedup(msg_id, speedup_type, speedup_value)
 
-        fe.focus_metric(msg_id, "sm__instruction_throughput.avg.pct_of_peak_sustained_active", sm_busy, NvRules.IFrontend.Severity_SEVERITY_LOW,
+        fe.focus_metric(msg_id, metrics["instruction_throughput"].name(), sm_busy, NvRules.IFrontend.Severity_SEVERITY_LOW,
             "The higher the instruction throughput the more likely is the impact of a slow pipeline")
-        fe.focus_metric(msg_id, "sm__inst_issued.avg.pct_of_peak_sustained_active", inst_issued_avg, NvRules.IFrontend.Severity_SEVERITY_HIGH,
+        fe.focus_metric(msg_id, metrics["inst_issued_avg"].name(), inst_issued_avg, NvRules.IFrontend.Severity_SEVERITY_HIGH,
             "Increase the average pipeline utilization towards the maximum utilization ({:.1f}%)".format(inst_issued_max))
