@@ -1,3 +1,13 @@
+# SPDX-FileCopyrightText: Copyright (c) 2020-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+# SPDX-License-Identifier: LicenseRef-NvidiaProprietary
+#
+# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
+# property and proprietary rights in and to this material, related
+# documentation and any modifications thereto. Any use, reproduction,
+# disclosure or distribution of this material and related documentation
+# without an express license agreement from NVIDIA CORPORATION or
+# its affiliates is strictly prohibited.
+
 import sys
 import inspect
 import os.path
@@ -97,40 +107,6 @@ class Report:
 
     DEFAULT_ROW_LIMIT = 50
 
-    MEM_KIND_STRS_CTE = """
-    MemKindStrs (id, name) AS (
-    VALUES
-        (0,     'Pageable'),
-        (1,     'Pinned'),
-        (2,     'Device'),
-        (3,     'Array'),
-        (4,     'Managed'),
-        (5,     'Device Static'),
-        (6,     'Managed Static'),
-        (7,     'Unknown')
-    ),
-"""
-
-    MEM_OPER_STRS_CTE = """
-    MemcpyOperStrs (id, name) AS (
-    VALUES
-        (0,     '[CUDA memcpy Unknown]'),
-        (1,     '[CUDA memcpy HtoD]'),
-        (2,     '[CUDA memcpy DtoH]'),
-        (3,     '[CUDA memcpy HtoA]'),
-        (4,     '[CUDA memcpy AtoH]'),
-        (5,     '[CUDA memcpy AtoA]'),
-        (6,     '[CUDA memcpy AtoD]'),
-        (7,     '[CUDA memcpy DtoA]'),
-        (8,     '[CUDA memcpy DtoD]'),
-        (9,     '[CUDA memcpy HtoH]'),
-        (10,    '[CUDA memcpy PtoP]'),
-        (11,    '[CUDA Unified Memory memcpy HtoD]'),
-        (12,    '[CUDA Unified Memory memcpy DtoH]'),
-        (13,    '[CUDA Unified Memory memcpy DtoD]')
-    ),
-"""
-
     _LOAD_TABLE_QUERY = """
         SELECT name
         FROM sqlite_master
@@ -176,7 +152,7 @@ class Report:
                 nvtx.eventType IN (59, 60, 70, 71) -- EVENT_TYPE_NVTX[T]_(PUSHPOP|STARTEND)_RANGE
             AND coalesce(nvtx.text, sid.value) || coalesce('@' || domain.text, '') == '{NVTX_RANGE}'
         ORDER BY 1
-        LIMIT 1
+        LIMIT 1 OFFSET {NVTX_IDX}
     """
 
     _boilerplate_statements = [
@@ -205,8 +181,9 @@ class Report:
         self._dbcur = None
         self._dbfile = dbfile
         self._args = []
-        self._parsed_args = None
         self._headers = []
+        self._parsed_args = None
+        self._parser = None
 
         if isinstance(args, argparse.Namespace):
             self._parsed_args = args
@@ -268,22 +245,35 @@ class Report:
 
         return cur.fetchone() != None
 
-    def setup(self):
-        for table, errmsg in self.table_checks.items():
-            if not self.table_exists(table):
-                return errmsg
+    def check_columns(self):
         for table, columns in self.table_col_checks.items():
             for col, errmsg in columns.items():
                 if not self.table_col_exists(table, col):
                     return errmsg
 
-        parser = self.ArgumentParser(allow_abbrev=False)
+    def setup(self):
+        for table, errmsg in self.table_checks.items():
+            if not self.table_exists(table):
+                return errmsg
+
+        errmsg = self.check_columns()
+        if errmsg != None:
+            return errmsg
+
+        self._parser = self.ArgumentParser(allow_abbrev=False)
         for opt in self._get_arg_options():
-            parser.add_optional_arg(*opt[0], **opt[1])
-        self._parsed_args = parser.parse_optional_dashless_args(self._args, self._parsed_args)
+            self._parser.add_optional_arg(*opt[0], **opt[1])
+        self._parsed_args = self._parser.parse_optional_dashless_args(self._args, self._parsed_args)
+
+        pid = getattr(self._parsed_args, 'process', -1)
+        # Negative value indicates no filtering.
+        pid = None if pid < 0 else pid
+
+        tid = getattr(self._parsed_args, 'thread', -1)
+        tid = None if tid < 0 else tid
 
         errmsg = self.filter_time_range(self._parsed_args.start,
-            self._parsed_args.end, self._parsed_args.nvtx)
+            self._parsed_args.end, self._parsed_args.nvtx, pid, tid)
         if errmsg != None:
             return errmsg
 
@@ -353,47 +343,79 @@ class Report:
         if self._dbcon == None:
             raise RuntimeError(f'Called {__name__}() with invalid database connection.')
 
-        if self.table_exists('NVTX_EVENTS'):
-            try:
-                cur = self._dbcon.execute(self._FIND_NVTX_RANGE_QUERY.format(
-                    NVTX_RANGE = nvtx))
-            except sqlite3.Error as err:
-                return (str(err), None, None, None)
+        if not self.table_exists('NVTX_EVENTS'):
+            errmsg = "{DBFILE} does not contain NV Tools Extension (NVTX) data.".format(
+                DBFILE = self.dbfile)
+            return (errmsg, None, None, None)
 
-            row = cur.fetchone()
-            if row:
-                return (None, row[0], row[1], row[2])
+        if not self.table_exists('StringIds'):
+            errmsg = "{DBFILE} file does not contain StringIds table.".format(
+                DBFILE = self.dbfile)
+            return (errmsg, None, None, None)
+
+        nvtx_range = nvtx
+        nvtx_idx = 0
+
+        # Find last slash (in case name contains a slash) and try to
+        # convert integer behind it.  If number won't convert, assume
+        # slash and anything trailing is part of the name.
+        slash_idx = nvtx.rfind("/")
+        if slash_idx != -1:
+            name = nvtx[:slash_idx]
+            numbStr = nvtx[slash_idx+1:]
+            numb = None
+            try:
+                numb = int(numbStr)
+                if numb < 0:
+                    numb = None
+            except:
+                pass
+            if numb is not None:
+                nvtx_range = name
+                nvtx_idx = numb
+
+        try:
+            cur = self._dbcon.execute(self._FIND_NVTX_RANGE_QUERY.format(
+                NVTX_RANGE = nvtx_range, NVTX_IDX = nvtx_idx))
+        except sqlite3.Error as err:
+            return (str(err), None, None, None)
+
+        row = cur.fetchone()
+        if row:
+            return (None, row[0], row[1], row[2])
 
         errmsg = "NVTX range '{NVTX_RANGE}' could not be found in {DBFILE}.".format(
             NVTX_RANGE = nvtx,
             DBFILE = self.dbfile)
-
         return (errmsg, None, None, None)
 
     # Filters tables according to start, end, and nvtx flags, if applicable.
     # Tables that should NOT be filtered (e.g. those used for correlation ID matching)
     # should be prefixed with 'main' in the query.
-    def filter_time_range(self, start, end, nvtx):
-        if start == None and end == None and nvtx == None:
+    def filter_time_range(self, start, end, nvtx, pid, tid):
+        if all(val is None for val in (start, end, nvtx, pid, tid)):
             return None
 
-        if nvtx != None:
-            err, nvtx_start, nvtx_end, globaltid = self._query_nvtx_filter_range(nvtx)
+        nvtx_globaltid = None
+        if nvtx is None:
+            start = 0 if start is None else start
+            # If no value is specified, set the end variable to the largest
+            # possible timestamp value.
+            end = 0x7FFFFFFFFFFFFFFF if end is None else end
+        else:
+            err, nvtx_start, nvtx_end, nvtx_globaltid = self._query_nvtx_filter_range(nvtx)
             if err != None:
                 return err
 
-            start = nvtx_start if start == None else start
-            end = nvtx_end if end == None else end
-            pid = (globaltid >> 24) & 0x00FFFFFF
-            tid = globaltid & 0x00FFFFFF
-        else:
-            start = 0 if start == None else start
-            end = 0x7FFFFFFFFFFFFFFF if end == None else end
-            globaltid = None
+            start = nvtx_start if start is None else start
+            end = nvtx_end if end is None else end
 
         if start > end:
             return "The start time cannot be greater than the end time."
 
+        if (pid is not None or tid is not None) and nvtx is not None:
+            return "The 'nvtx' option cannot be used with 'pid' or 'tid'."
+
         for table in self._tables:
             if not self.table_col_exists(table, 'start') or not self.table_col_exists(table, 'end'):
                 continue
@@ -401,18 +423,28 @@ class Report:
             statement = self._CREATE_FILTERED_VIEW_QUERY.format(
                     TABLE = table, START = start, END = end)
 
-            if globaltid is not None:
+            if pid is not None:
                 if self.table_col_exists(table, 'globalTid'):
-                    if pid != tid:
+                    statement += ' AND (globalTid >> 24) & 0x00FFFFFF == {PID}'.format(PID = pid)
+                elif self.table_col_exists(table, 'globalPid'):
+                    statement += ' AND (globalPid >> 24) & 0x00FFFFFF == {PID}'.format(PID = pid)
+
+            if tid is not None and self.table_col_exists(table, 'globalTid'):
+                statement += ' AND globalTid & 0x00FFFFFF == {TID}'.format(TID = tid)
+
+            if nvtx_globaltid is not None:
+                nvtx_pid = (nvtx_globaltid >> 24) & 0x00FFFFFF
+                nvtx_tid = nvtx_globaltid & 0x00FFFFFF
+
+                if self.table_col_exists(table, 'globalTid'):
+                    if nvtx_pid != nvtx_tid:
                         # NVTX Push/Pop range.
-                        statement += ' AND globalTid == {GLOBAL_TID}'.format(GLOBAL_TID = globaltid)
+                        statement += ' AND globalTid == {GLOBAL_TID}'.format(GLOBAL_TID = nvtx_globaltid)
                     else:
                         # NVTX Start/End range.
-                        statement += ' AND globalTid >> 24 == {GLOBAL_TID} >> 24'.format(GLOBAL_TID = globaltid)
+                        statement += ' AND globalTid >> 24 == {GLOBAL_TID} >> 24'.format(GLOBAL_TID = nvtx_globaltid)
                 elif self.table_col_exists(table, 'globalPid'):
-                    statement += ' AND globalPid >> 24 == {GLOBAL_TID} >> 24'.format(GLOBAL_TID = globaltid)
-                else:
-                    continue
+                    statement += ' AND globalPid >> 24 == {GLOBAL_TID} >> 24'.format(GLOBAL_TID = nvtx_globaltid)
 
             errmsg = self._execute_statement('DROP VIEW IF EXISTS temp.{TABLE}'.format(TABLE = table))
             if errmsg != None:
@@ -442,6 +474,10 @@ class Report:
     def dbcon(self):
         return self._dbcon
 
+    @property
+    def tables(self):
+        return self._tables
+
     @classmethod
     def get_script_name(klass):
         if klass.script_name == None:
@@ -573,7 +609,7 @@ class ExpertSystemsReport(Report):
         return self.message_noresult
 
     def MessageRowLimit(self, rows):
-        if self._row_limit == 0 or rows < self._row_limit:
+        if self._row_limit <= 0 or rows < self._row_limit:
             return ''
         if self._row_limit == 1:
             return 'Only the top result is displayed. More data may be available.'
