@@ -1,5 +1,15 @@
 #!/usr/bin/env python
 
+# SPDX-FileCopyrightText: Copyright (c) 2020-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+# SPDX-License-Identifier: LicenseRef-NvidiaProprietary
+#
+# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
+# property and proprietary rights in and to this material, related
+# documentation and any modifications thereto. Any use, reproduction,
+# disclosure or distribution of this material and related documentation
+# without an express license agreement from NVIDIA CORPORATION or
+# its affiliates is strictly prohibited.
+
 import nsysstats
 
 class CUDAAPIandGPUSummary(nsysstats.StatsReport):
@@ -10,7 +20,6 @@ class CUDAAPIandGPUSummary(nsysstats.StatsReport):
 
     query_stub = """
 WITH
-    {MEM_OPER_STRS_CTE}
     apigpu AS (
         {SUB_QUERY}
     ),
@@ -79,13 +88,13 @@ ORDER BY 2 DESC
 
     query_memcpy = """
         SELECT
-            mos.name AS name,
+            '[CUDA memcpy ' || mos.label || ']' AS name,
             mcpy.end - mcpy.start AS duration,
             'MEMORY_OPER' AS category
         FROM
             CUPTI_ACTIVITY_KIND_MEMCPY as mcpy
         JOIN
-            MemcpyOperStrs AS mos
+            ENUM_CUDA_MEMCPY_OPER AS mos
             ON mos.id == mcpy.copyKind
 """
 
@@ -128,6 +137,8 @@ ORDER BY 2 DESC
             sub_queries.append(self.query_kernel.format(NAME_COL_NAME = name_col_name))
 
         if self.table_exists('CUPTI_ACTIVITY_KIND_MEMCPY'):
+            if not self.table_exists('ENUM_CUDA_MEMCPY_OPER'):
+                return '{DBFILE} does not contain ENUM_CUDA_MEMCPY_OPER table.'
             sub_queries.append(self.query_memcpy)
 
         if self.table_exists('CUPTI_ACTIVITY_KIND_MEMSET'):
@@ -136,9 +147,7 @@ ORDER BY 2 DESC
         if len(sub_queries) == 0:
             return '{DBFILE} does not contain CUDA API, GPU kernel, nor memory operations data.'
 
-        self.query = self.query_stub.format(
-            MEM_OPER_STRS_CTE = self.MEM_OPER_STRS_CTE,
-            SUB_QUERY = self.query_union.join(sub_queries))
+        self.query = self.query_stub.format(SUB_QUERY = self.query_union.join(sub_queries))
 
 if __name__ == "__main__":
     CUDAAPIandGPUSummary.Main()
