@@ -1,5 +1,15 @@
 #!/usr/bin/env python
 
+# SPDX-FileCopyrightText: Copyright (c) 2020-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+# SPDX-License-Identifier: LicenseRef-NvidiaProprietary
+#
+# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
+# property and proprietary rights in and to this material, related
+# documentation and any modifications thereto. Any use, reproduction,
+# disclosure or distribution of this material and related documentation
+# without an express license agreement from NVIDIA CORPORATION or
+# its affiliates is strictly prohibited.
+
 import nsysstats
 
 class CUDAGPUTrace(nsysstats.StatsReport):
@@ -10,8 +20,6 @@ class CUDAGPUTrace(nsysstats.StatsReport):
 
     query_stub = """
 WITH
-    {MEM_KIND_STRS_CTE}
-    {MEM_OPER_STRS_CTE}
     recs AS (
         {GPU_SUB_QUERY}
     )
@@ -92,24 +100,24 @@ WITH
             NULL AS "ssmembytes",
             NULL AS "dsmembytes",
             bytes AS "bytes",
-            msrck.name AS "srcmemkind",
-            mdstk.name AS "dstmemkind",
+            msrck.label AS "srcmemkind",
+            mdstk.label AS "dstmemkind",
             NULL AS "memsetval",
             printf('%s (%d)', gpu.name, deviceId) AS "device",
             contextId AS "context",
             streamId AS "stream",
-            memopstr.name AS "name",
+            '[CUDA memcpy ' || memopstr.label || ']' AS "name",
             correlationId AS "correlation"
         FROM
             CUPTI_ACTIVITY_KIND_MEMCPY AS memcpy
         LEFT JOIN
-            MemcpyOperStrs AS memopstr
+            ENUM_CUDA_MEMCPY_OPER AS memopstr
             ON memcpy.copyKind == memopstr.id
         LEFT JOIN
-            MemKindStrs AS msrck
+            ENUM_CUDA_MEM_KIND AS msrck
             ON memcpy.srcKind == msrck.id
         LEFT JOIN
-            MemKindStrs AS mdstk
+            ENUM_CUDA_MEM_KIND AS mdstk
             ON memcpy.dstKind == mdstk.id
         LEFT JOIN
             TARGET_INFO_GPU AS gpu
@@ -130,7 +138,7 @@ WITH
             NULL AS "ssmembytes",
             NULL AS "dsmembytes",
             bytes AS "bytes",
-            mk.name AS "srcmemkind",
+            mk.label AS "srcmemkind",
             NULL AS "dstmemkind",
             value AS "memsetval",
             printf('%s (%d)', gpu.name, deviceId) AS "device",
@@ -141,7 +149,7 @@ WITH
         FROM
             CUPTI_ACTIVITY_KIND_MEMSET AS memset
         LEFT JOIN
-            MemKindStrs AS mk
+            ENUM_CUDA_MEM_KIND AS mk
             ON memset.memKind == mk.id
         LEFT JOIN
             TARGET_INFO_GPU AS gpu
@@ -175,18 +183,21 @@ WITH
             sub_queries.append(self.query_kernel.format(NAME_COL_NAME = name_col_name))
 
         if self.table_exists('CUPTI_ACTIVITY_KIND_MEMCPY'):
+            if not self.table_exists('ENUM_CUDA_MEMCPY_OPER'):
+                return '{DBFILE} does not contain ENUM_CUDA_MEMCPY_OPER table.'
+            if not self.table_exists('ENUM_CUDA_MEM_KIND'):
+                return '{DBFILE} does not contain ENUM_CUDA_MEM_KIND table.'
             sub_queries.append(self.query_memcpy)
 
         if self.table_exists('CUPTI_ACTIVITY_KIND_MEMSET'):
+            if not self.table_exists('ENUM_CUDA_MEM_KIND'):
+                return '{DBFILE} does not contain ENUM_CUDA_MEM_KIND table.'
             sub_queries.append(self.query_memset)
 
         if len(sub_queries) == 0:
             return "{DBFILE} does not contain GPU trace data."
 
-        self.query = self.query_stub.format(
-            MEM_OPER_STRS_CTE = self.MEM_OPER_STRS_CTE,
-            MEM_KIND_STRS_CTE = self.MEM_KIND_STRS_CTE,
-            GPU_SUB_QUERY = self.query_union.join(sub_queries))
+        self.query = self.query_stub.format(GPU_SUB_QUERY = self.query_union.join(sub_queries))
 
 if __name__ == "__main__":
     CUDAGPUTrace.Main()
